@@ -13,7 +13,6 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
-#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -21,6 +20,97 @@
 #include "parasail/internal_sse.h"
 
 
+static inline __m128i _mm_cmpeq_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]==B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    A.v[1] = (A.v[1]==B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    return A.m;
+}
+
+static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
+    a = _mm_andnot_si128(mask, a);
+    a = _mm_or_si128(a, _mm_and_si128(mask, b));
+    return a;
+}
+
+static inline __m128i _mm_cmpgt_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    A.v[1] = (A.v[1]>B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    return A.m;
+}
+
+static inline __m128i _mm_insert_epi64_rpl(__m128i a, int64_t i, const int imm) {
+    __m128i_64_t A;
+    A.m = a;
+    A.v[imm] = i;
+    return A.m;
+}
+
+static inline __m128i _mm_max_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
+    return A.m;
+}
+
+#if HAVE_SSE2_MM_SET_EPI64X
+#define _mm_set_epi64x_rpl _mm_set_epi64x
+#else
+static inline __m128i _mm_set_epi64x_rpl(int64_t e1, int64_t e0) {
+    __m128i_64_t A;
+    A.v[0] = e0;
+    A.v[1] = e1;
+    return A.m;
+}
+#endif
+
+static inline int64_t _mm_extract_epi64_rpl(__m128i a, const int imm) {
+    __m128i_64_t A;
+    A.m = a;
+    return A.v[imm];
+}
+
+static inline __m128i _mm_min_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
+    return A.m;
+}
+
+static inline __m128i _mm_cmplt_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]<B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    A.v[1] = (A.v[1]<B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    return A.m;
+}
+
+#if HAVE_SSE2_MM_SET1_EPI64X
+#define _mm_set1_epi64x_rpl _mm_set1_epi64x
+#else
+static inline __m128i _mm_set1_epi64x_rpl(int64_t i) {
+    __m128i_64_t A;
+    A.v[0] = i;
+    A.v[1] = i;
+    return A.m;
+}
+#endif
+
 
 #ifdef PARASAIL_TABLE
 static inline void arr_store_si128(
@@ -32,16 +122,10 @@ static inline void arr_store_si128(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int32_t)_mm_extract_epi32(vWH, 3);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int64_t)_mm_extract_epi64_rpl(vWH, 1);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int32_t)_mm_extract_epi32(vWH, 2);
-    }
-    if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
-        array[1LL*(i+2)*s2Len + (j-2)] = (int32_t)_mm_extract_epi32(vWH, 1);
-    }
-    if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
-        array[1LL*(i+3)*s2Len + (j-3)] = (int32_t)_mm_extract_epi32(vWH, 0);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int64_t)_mm_extract_epi64_rpl(vWH, 0);
     }
 }
 #endif
@@ -57,39 +141,27 @@ static inline void arr_store_rowcol(
         int32_t s2Len)
 {
     if (i+0 == s1Len-1 && 0 <= j-0 && j-0 < s2Len) {
-        row[j-0] = (int32_t)_mm_extract_epi32(vWH, 3);
+        row[j-0] = (int64_t)_mm_extract_epi64_rpl(vWH, 1);
     }
     if (j-0 == s2Len-1 && 0 <= i+0 && i+0 < s1Len) {
-        col[(i+0)] = (int32_t)_mm_extract_epi32(vWH, 3);
+        col[(i+0)] = (int64_t)_mm_extract_epi64_rpl(vWH, 1);
     }
     if (i+1 == s1Len-1 && 0 <= j-1 && j-1 < s2Len) {
-        row[j-1] = (int32_t)_mm_extract_epi32(vWH, 2);
+        row[j-1] = (int64_t)_mm_extract_epi64_rpl(vWH, 0);
     }
     if (j-1 == s2Len-1 && 0 <= i+1 && i+1 < s1Len) {
-        col[(i+1)] = (int32_t)_mm_extract_epi32(vWH, 2);
-    }
-    if (i+2 == s1Len-1 && 0 <= j-2 && j-2 < s2Len) {
-        row[j-2] = (int32_t)_mm_extract_epi32(vWH, 1);
-    }
-    if (j-2 == s2Len-1 && 0 <= i+2 && i+2 < s1Len) {
-        col[(i+2)] = (int32_t)_mm_extract_epi32(vWH, 1);
-    }
-    if (i+3 == s1Len-1 && 0 <= j-3 && j-3 < s2Len) {
-        row[j-3] = (int32_t)_mm_extract_epi32(vWH, 0);
-    }
-    if (j-3 == s2Len-1 && 0 <= i+3 && i+3 < s1Len) {
-        col[(i+3)] = (int32_t)_mm_extract_epi32(vWH, 0);
+        col[(i+1)] = (int64_t)_mm_extract_epi64_rpl(vWH, 0);
     }
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_nw_table_diag_sse41_128_32
+#define FNAME parasail_nw_table_diag_sse2_128_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_nw_rowcol_diag_sse41_128_32
+#define FNAME parasail_nw_rowcol_diag_sse2_128_64
 #else
-#define FNAME parasail_nw_diag_sse41_128_32
+#define FNAME parasail_nw_diag_sse2_128_64
 #endif
 #endif
 
@@ -105,21 +177,21 @@ parasail_result_t* FNAME(
     int32_t s1Len = 0;
     int32_t s1Len_PAD = 0;
     int32_t s2Len_PAD = 0;
-    int32_t * restrict s1 = NULL;
-    int32_t * restrict s2B = NULL;
-    int32_t * restrict _H_pr = NULL;
-    int32_t * restrict _F_pr = NULL;
-    int32_t * restrict s2 = NULL;
-    int32_t * restrict H_pr = NULL;
-    int32_t * restrict F_pr = NULL;
+    int64_t * restrict s1 = NULL;
+    int64_t * restrict s2B = NULL;
+    int64_t * restrict _H_pr = NULL;
+    int64_t * restrict _F_pr = NULL;
+    int64_t * restrict s2 = NULL;
+    int64_t * restrict H_pr = NULL;
+    int64_t * restrict F_pr = NULL;
     parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
-    int32_t score = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
+    int64_t score = 0;
     __m128i vNegLimit;
     __m128i vPosLimit;
     __m128i vSaturationCheckMin;
@@ -152,7 +224,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize stack variables */
-    N = 4; /* number of values in vector */
+    N = 2; /* number of values in vector */
     PAD = N-1;
     PAD2 = PAD*2;
     s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
@@ -162,32 +234,30 @@ parasail_result_t* FNAME(
     j = 0;
     end_query = s1Len-1;
     end_ref = s2Len-1;
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm_set1_epi64x_rpl(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi64x_rpl(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vNegInf = _mm_set1_epi32(NEG_LIMIT);
-    vOpen = _mm_set1_epi32(open);
-    vGap  = _mm_set1_epi32(gap);
-    vOne = _mm_set1_epi32(1);
-    vN = _mm_set1_epi32(N);
-    vGapN = _mm_set1_epi32(gap*N);
-    vNegOne = _mm_set1_epi32(-1);
-    vI = _mm_set_epi32(0,1,2,3);
-    vJreset = _mm_set_epi32(0,-1,-2,-3);
+    vNegInf = _mm_set1_epi64x_rpl(NEG_LIMIT);
+    vOpen = _mm_set1_epi64x_rpl(open);
+    vGap  = _mm_set1_epi64x_rpl(gap);
+    vOne = _mm_set1_epi64x_rpl(1);
+    vN = _mm_set1_epi64x_rpl(N);
+    vGapN = _mm_set1_epi64x_rpl(gap*N);
+    vNegOne = _mm_set1_epi64x_rpl(-1);
+    vI = _mm_set_epi64x_rpl(0,1);
+    vJreset = _mm_set_epi64x_rpl(0,-1);
     vMax = vNegInf;
-    vILimit = _mm_set1_epi32(s1Len);
-    vILimit1 = _mm_sub_epi32(vILimit, vOne);
-    vJLimit = _mm_set1_epi32(s2Len);
-    vJLimit1 = _mm_sub_epi32(vJLimit, vOne);
-    vIBoundary = _mm_set_epi32(
+    vILimit = _mm_set1_epi64x_rpl(s1Len);
+    vILimit1 = _mm_sub_epi64(vILimit, vOne);
+    vJLimit = _mm_set1_epi64x_rpl(s2Len);
+    vJLimit1 = _mm_sub_epi64(vJLimit, vOne);
+    vIBoundary = _mm_set_epi64x_rpl(
             -open-0*gap,
-            -open-1*gap,
-            -open-2*gap,
-            -open-3*gap);
+            -open-1*gap);
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -203,7 +273,7 @@ parasail_result_t* FNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_DIAG
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -212,9 +282,9 @@ parasail_result_t* FNAME(
 #endif
 
     /* initialize heap variables */
-    s2B= parasail_memalign_int32_t(16, s2Len+PAD2);
-    _H_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    _F_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    s2B= parasail_memalign_int64_t(16, s2Len+PAD2);
+    _H_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _F_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
     s2 = s2B+PAD; /* will allow later for negative indices */
     H_pr = _H_pr+PAD;
     F_pr = _F_pr+PAD;
@@ -226,7 +296,7 @@ parasail_result_t* FNAME(
 
     /* convert _s1 from char to int in range 0-23 */
     if (matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) {
-        s1 = parasail_memalign_int32_t(16, s1Len+PAD);
+        s1 = parasail_memalign_int64_t(16, s1Len+PAD);
         if (!s1) return NULL;
         for (i=0; i<s1Len; ++i) {
             s1[i] = matrix->mapper[(unsigned char)_s1[i]];
@@ -276,48 +346,44 @@ parasail_result_t* FNAME(
         __m128i vJ = vJreset;
         const int * const restrict matrow0 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+0] : ((i+0 >= s1Len) ? s1Len-1 : i+0))];
         const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
-        const int * const restrict matrow2 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+2] : ((i+2 >= s1Len) ? s1Len-1 : i+2))];
-        const int * const restrict matrow3 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+3] : ((i+3 >= s1Len) ? s1Len-1 : i+3))];
-        vNH = _mm_srli_si128(vNH, 4);
-        vNH = _mm_insert_epi32(vNH, H_pr[-1], 3);
-        vWH = _mm_srli_si128(vWH, 4);
-        vWH = _mm_insert_epi32(vWH, -open - i*gap, 3);
+        vNH = _mm_srli_si128(vNH, 8);
+        vNH = _mm_insert_epi64_rpl(vNH, H_pr[-1], 1);
+        vWH = _mm_srli_si128(vWH, 8);
+        vWH = _mm_insert_epi64_rpl(vWH, -open - i*gap, 1);
         H_pr[-1] = -open - (i+N)*gap;
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
             __m128i vMat;
             __m128i vNWH = vNH;
-            vNH = _mm_srli_si128(vWH, 4);
-            vNH = _mm_insert_epi32(vNH, H_pr[j], 3);
-            vF = _mm_srli_si128(vF, 4);
-            vF = _mm_insert_epi32(vF, F_pr[j], 3);
-            vF = _mm_max_epi32(
-                    _mm_sub_epi32(vNH, vOpen),
-                    _mm_sub_epi32(vF, vGap));
-            vE = _mm_max_epi32(
-                    _mm_sub_epi32(vWH, vOpen),
-                    _mm_sub_epi32(vE, vGap));
-            vMat = _mm_set_epi32(
+            vNH = _mm_srli_si128(vWH, 8);
+            vNH = _mm_insert_epi64_rpl(vNH, H_pr[j], 1);
+            vF = _mm_srli_si128(vF, 8);
+            vF = _mm_insert_epi64_rpl(vF, F_pr[j], 1);
+            vF = _mm_max_epi64_rpl(
+                    _mm_sub_epi64(vNH, vOpen),
+                    _mm_sub_epi64(vF, vGap));
+            vE = _mm_max_epi64_rpl(
+                    _mm_sub_epi64(vWH, vOpen),
+                    _mm_sub_epi64(vE, vGap));
+            vMat = _mm_set_epi64x_rpl(
                     matrow0[s2[j-0]],
-                    matrow1[s2[j-1]],
-                    matrow2[s2[j-2]],
-                    matrow3[s2[j-3]]
+                    matrow1[s2[j-1]]
                     );
-            vNWH = _mm_add_epi32(vNWH, vMat);
-            vWH = _mm_max_epi32(vNWH, vE);
-            vWH = _mm_max_epi32(vWH, vF);
+            vNWH = _mm_add_epi64(vNWH, vMat);
+            vWH = _mm_max_epi64_rpl(vNWH, vE);
+            vWH = _mm_max_epi64_rpl(vWH, vF);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                __m128i cond = _mm_cmpeq_epi32(vJ,vNegOne);
-                vWH = _mm_blendv_epi8(vWH, vIBoundary, cond);
-                vF = _mm_blendv_epi8(vF, vNegInf, cond);
-                vE = _mm_blendv_epi8(vE, vNegInf, cond);
+                __m128i cond = _mm_cmpeq_epi64_rpl(vJ,vNegOne);
+                vWH = _mm_blendv_epi8_rpl(vWH, vIBoundary, cond);
+                vF = _mm_blendv_epi8_rpl(vF, vNegInf, cond);
+                vE = _mm_blendv_epi8_rpl(vE, vNegInf, cond);
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWH);
+                vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWH);
             }
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vWH, i, s1Len, j, s2Len);
@@ -325,35 +391,35 @@ parasail_result_t* FNAME(
 #ifdef PARASAIL_ROWCOL
             arr_store_rowcol(result->rowcols->score_row, result->rowcols->score_col, vWH, i, s1Len, j, s2Len);
 #endif
-            H_pr[j-3] = (int32_t)_mm_extract_epi32(vWH,0);
-            F_pr[j-3] = (int32_t)_mm_extract_epi32(vF,0);
+            H_pr[j-1] = (int64_t)_mm_extract_epi64_rpl(vWH,0);
+            F_pr[j-1] = (int64_t)_mm_extract_epi64_rpl(vF,0);
             /* as minor diagonal vector passes across table, extract
                last table value at the i,j bound */
             {
-                __m128i cond_valid_I = _mm_cmpeq_epi32(vI, vILimit1);
-                __m128i cond_valid_J = _mm_cmpeq_epi32(vJ, vJLimit1);
+                __m128i cond_valid_I = _mm_cmpeq_epi64_rpl(vI, vILimit1);
+                __m128i cond_valid_J = _mm_cmpeq_epi64_rpl(vJ, vJLimit1);
                 __m128i cond_all = _mm_and_si128(cond_valid_I, cond_valid_J);
-                vMax = _mm_blendv_epi8(vMax, vWH, cond_all);
+                vMax = _mm_blendv_epi8_rpl(vMax, vWH, cond_all);
             }
-            vJ = _mm_add_epi32(vJ, vOne);
+            vJ = _mm_add_epi64(vJ, vOne);
         }
-        vI = _mm_add_epi32(vI, vN);
-        vIBoundary = _mm_sub_epi32(vIBoundary, vGapN);
+        vI = _mm_add_epi64(vI, vN);
+        vIBoundary = _mm_sub_epi64(vIBoundary, vGapN);
     }
 
     /* max in vMax */
     for (i=0; i<N; ++i) {
-        int32_t value;
-        value = (int32_t) _mm_extract_epi32(vMax, 3);
+        int64_t value;
+        value = (int64_t) _mm_extract_epi64_rpl(vMax, 1);
         if (value > score) {
             score = value;
         }
-        vMax = _mm_slli_si128(vMax, 4);
+        vMax = _mm_slli_si128(vMax, 8);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi64_rpl(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
