@@ -14,7 +14,6 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
-#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -22,78 +21,33 @@
 #include "parasail/internal_sse.h"
 
 
-static inline __m128i _mm_cmpgt_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]>B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    A.v[1] = (A.v[1]>B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    return A.m;
-}
-
-#if HAVE_SSE41_MM_INSERT_EPI64
-#define _mm_insert_epi64_rpl _mm_insert_epi64
-#else
-static inline __m128i _mm_insert_epi64_rpl(__m128i a, int64_t i, int imm) {
-    __m128i_64_t A;
+static inline __m128i _mm_insert_epi8_rpl(__m128i a, int8_t i, const int imm) {
+    __m128i_8_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
-#endif
 
-static inline __m128i _mm_max_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
-    return A.m;
+static inline __m128i _mm_max_epi8_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi8(a, b);
+    a = _mm_and_si128(a, mask);
+    b = _mm_andnot_si128(mask, b);
+    return _mm_or_si128(a, b);
 }
 
-#if HAVE_SSE41_MM_EXTRACT_EPI64
-#define _mm_extract_epi64_rpl _mm_extract_epi64
-#else
-static inline int64_t _mm_extract_epi64_rpl(__m128i a, int imm) {
-    __m128i_64_t A;
+static inline int8_t _mm_extract_epi8_rpl(__m128i a, const int imm) {
+    __m128i_8_t A;
     A.m = a;
     return A.v[imm];
 }
-#endif
 
-static inline __m128i _mm_min_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
-    return A.m;
+static inline __m128i _mm_min_epi8_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi8(b, a);
+    a = _mm_and_si128(a, mask);
+    b = _mm_andnot_si128(mask, b);
+    return _mm_or_si128(a, b);
 }
 
-static inline __m128i _mm_cmplt_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]<B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    A.v[1] = (A.v[1]<B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    return A.m;
-}
-
-#if HAVE_SSE2_MM_SET1_EPI64X
-#define _mm_set1_epi64x_rpl _mm_set1_epi64x
-#else
-static inline __m128i _mm_set1_epi64x_rpl(int64_t i) {
-    __m128i_64_t A;
-    A.v[0] = i;
-    A.v[1] = i;
-    return A.m;
-}
-#endif
-
 
 #ifdef PARASAIL_TABLE
 static inline void arr_store_si128(
@@ -104,8 +58,22 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)_mm_extract_epi64_rpl(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)_mm_extract_epi64_rpl(vH, 1);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  0);
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  1);
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  2);
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  3);
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  4);
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  5);
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  6);
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  7);
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  8);
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  9);
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 10);
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 11);
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 12);
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 13);
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 14);
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 15);
 }
 #endif
 
@@ -116,21 +84,35 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int64_t)_mm_extract_epi64_rpl(vH, 0);
-    col[1*seglen+t] = (int64_t)_mm_extract_epi64_rpl(vH, 1);
+    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  0);
+    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  1);
+    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  2);
+    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  3);
+    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  4);
+    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  5);
+    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  6);
+    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  7);
+    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  8);
+    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  9);
+    col[10*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 10);
+    col[11*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 11);
+    col[12*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 12);
+    col[13*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 13);
+    col[14*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 14);
+    col[15*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 15);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_nw_table_scan_sse41_128_64
-#define PNAME parasail_nw_table_scan_profile_sse41_128_64
+#define FNAME parasail_nw_table_scan_sse2_128_8
+#define PNAME parasail_nw_table_scan_profile_sse2_128_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_nw_rowcol_scan_sse41_128_64
-#define PNAME parasail_nw_rowcol_scan_profile_sse41_128_64
+#define FNAME parasail_nw_rowcol_scan_sse2_128_8
+#define PNAME parasail_nw_rowcol_scan_profile_sse2_128_8
 #else
-#define FNAME parasail_nw_scan_sse41_128_64
-#define PNAME parasail_nw_scan_profile_sse41_128_64
+#define FNAME parasail_nw_scan_sse2_128_8
+#define PNAME parasail_nw_scan_profile_sse2_128_8
 #endif
 #endif
 
@@ -155,7 +137,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_sse_128_64(s1, s1Len, matrix);
+    profile = parasail_profile_create_sse_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -183,16 +165,16 @@ parasail_result_t* PNAME(
     int32_t position = 0;
     __m128i* restrict pvP = NULL;
     __m128i* restrict pvE = NULL;
-    int64_t* restrict boundary = NULL;
+    int8_t* restrict boundary = NULL;
     __m128i* restrict pvHt = NULL;
     __m128i* restrict pvH = NULL;
     __m128i* restrict pvGapper = NULL;
     __m128i vGapO;
     __m128i vGapE;
-    int64_t NEG_LIMIT = 0;
-    int64_t POS_LIMIT = 0;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
     __m128i vZero;
-    int64_t score = 0;
+    int8_t score = 0;
     __m128i vNegLimit;
     __m128i vPosLimit;
     __m128i vSaturationCheckMin;
@@ -203,7 +185,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -219,25 +201,25 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 2; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP = (__m128i*)profile->profile64.score;
-    vGapO = _mm_set1_epi64x_rpl(open);
-    vGapE = _mm_set1_epi64x_rpl(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    pvP = (__m128i*)profile->profile8.score;
+    vGapO = _mm_set1_epi8(open);
+    vGapE = _mm_set1_epi8(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
     vZero = _mm_setzero_si128();
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi64x_rpl(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi64x_rpl(POS_LIMIT);
+    vNegLimit = _mm_set1_epi8(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi8(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vNegInfFront = vZero;
-    vNegInfFront = _mm_insert_epi64_rpl(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm_add_epi64(vNegInfFront,
-            _mm_slli_si128(_mm_set1_epi64x_rpl(-segLen*gap), 8));
+    vNegInfFront = _mm_insert_epi8_rpl(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm_adds_epi8(vNegInfFront,
+            _mm_slli_si128(_mm_set1_epi8(-segLen*gap), 1));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -253,7 +235,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_SCAN
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -263,7 +245,7 @@ parasail_result_t* PNAME(
 
     /* initialize heap variables */
     pvE = parasail_memalign___m128i(16, segLen);
-    boundary = parasail_memalign_int64_t(16, s2Len+1);
+    boundary = parasail_memalign_int8_t(16, s2Len+1);
     pvHt= parasail_memalign___m128i(16, segLen);
     pvH = parasail_memalign___m128i(16, segLen);
     pvGapper = parasail_memalign___m128i(16, segLen);
@@ -280,13 +262,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m128i_64_t h;
-            __m128i_64_t e;
+            __m128i_8_t h;
+            __m128i_8_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = -open-gap*(segNum*segLen+i);
-                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                h.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                e.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
             }
             _mm_store_si128(&pvH[index], h.m);
             _mm_store_si128(&pvE[index], e.m);
@@ -299,15 +281,15 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = -open-gap*(i-1);
-            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
+            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
         }
     }
 
     {
-        __m128i vGapper = _mm_sub_epi64(vZero,vGapO);
+        __m128i vGapper = _mm_subs_epi8(vZero,vGapO);
         for (i=segLen-1; i>=0; --i) {
             _mm_store_si128(pvGapper+i, vGapper);
-            vGapper = _mm_sub_epi64(vGapper, vGapE);
+            vGapper = _mm_subs_epi8(vGapper, vGapE);
         }
     }
 
@@ -325,49 +307,49 @@ parasail_result_t* PNAME(
         /* calculate Ht */
         /* calculate F and H first pass */
         vHp = _mm_load_si128(pvH+(segLen-1));
-        vHp = _mm_slli_si128(vHp, 8);
-        vHp = _mm_insert_epi64_rpl(vHp, boundary[j], 0);
+        vHp = _mm_slli_si128(vHp, 1);
+        vHp = _mm_insert_epi8_rpl(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = _mm_sub_epi64(vNegLimit, pvGapper[0]);
+        vHt = _mm_subs_epi8(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         for (i=0; i<segLen; ++i) {
             vH = _mm_load_si128(pvH+i);
             vE = _mm_load_si128(pvE+i);
             vW = _mm_load_si128(pvW+i);
-            vE = _mm_max_epi64_rpl(
-                    _mm_sub_epi64(vE, vGapE),
-                    _mm_sub_epi64(vH, vGapO));
-            vHp = _mm_add_epi64(vHp, vW);
-            vF = _mm_max_epi64_rpl(vF, _mm_add_epi64(vHt, pvGapper[i]));
-            vHt = _mm_max_epi64_rpl(vE, vHp);
+            vE = _mm_max_epi8_rpl(
+                    _mm_subs_epi8(vE, vGapE),
+                    _mm_subs_epi8(vH, vGapO));
+            vHp = _mm_adds_epi8(vHp, vW);
+            vF = _mm_max_epi8_rpl(vF, _mm_adds_epi8(vHt, pvGapper[i]));
+            vHt = _mm_max_epi8_rpl(vE, vHp);
             _mm_store_si128(pvE+i, vE);
             _mm_store_si128(pvHt+i, vHt);
             vHp = vH;
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm_slli_si128(vHt, 8);
-        vHt = _mm_insert_epi64_rpl(vHt, boundary[j+1], 0);
-        vF = _mm_max_epi64_rpl(vF, _mm_add_epi64(vHt, pvGapper[0]));
+        vHt = _mm_slli_si128(vHt, 1);
+        vHt = _mm_insert_epi8_rpl(vHt, boundary[j+1], 0);
+        vF = _mm_max_epi8_rpl(vF, _mm_adds_epi8(vHt, pvGapper[0]));
         for (i=0; i<segWidth-2; ++i) {
-            __m128i vFt = _mm_slli_si128(vF, 8);
-            vFt = _mm_add_epi64(vFt, vSegLenXgap);
-            vF = _mm_max_epi64_rpl(vF, vFt);
+            __m128i vFt = _mm_slli_si128(vF, 1);
+            vFt = _mm_adds_epi8(vFt, vSegLenXgap);
+            vF = _mm_max_epi8_rpl(vF, vFt);
         }
 
         /* calculate final H */
-        vF = _mm_slli_si128(vF, 8);
-        vF = _mm_add_epi64(vF, vNegInfFront);
-        vH = _mm_max_epi64_rpl(vHt, vF);
+        vF = _mm_slli_si128(vF, 1);
+        vF = _mm_adds_epi8(vF, vNegInfFront);
+        vH = _mm_max_epi8_rpl(vHt, vF);
         for (i=0; i<segLen; ++i) {
             vHt = _mm_load_si128(pvHt+i);
-            vF = _mm_max_epi64_rpl(
-                    _mm_sub_epi64(vF, vGapE),
-                    _mm_sub_epi64(vH, vGapO));
-            vH = _mm_max_epi64_rpl(vHt, vF);
+            vF = _mm_max_epi8_rpl(
+                    _mm_subs_epi8(vF, vGapE),
+                    _mm_subs_epi8(vH, vGapO));
+            vH = _mm_max_epi8_rpl(vHt, vF);
             _mm_store_si128(pvH+i, vH);
-            vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm_min_epi8_rpl(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vH);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
@@ -379,9 +361,9 @@ parasail_result_t* PNAME(
         {
             vH = _mm_load_si128(pvH + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 8);
+                vH = _mm_slli_si128(vH, 1);
             }
-            result->rowcols->score_row[j] = (int64_t) _mm_extract_epi64_rpl (vH, 1);
+            result->rowcols->score_row[j] = (int8_t) _mm_extract_epi8_rpl (vH, 15);
         }
 #endif
     }
@@ -397,14 +379,14 @@ parasail_result_t* PNAME(
     {
         __m128i vH = _mm_load_si128(pvH + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm_slli_si128(vH, 8);
+            vH = _mm_slli_si128(vH, 1);
         }
-        score = (int64_t) _mm_extract_epi64_rpl (vH, 1);
+        score = (int8_t) _mm_extract_epi8_rpl (vH, 15);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi64_rpl(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
