@@ -27,16 +27,10 @@ static inline void arr_store_si128(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int32_t)_mm_extract_epi32(vWH, 3);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int64_t)_mm_extract_epi64(vWH, 1);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int32_t)_mm_extract_epi32(vWH, 2);
-    }
-    if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
-        array[1LL*(i+2)*s2Len + (j-2)] = (int32_t)_mm_extract_epi32(vWH, 1);
-    }
-    if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
-        array[1LL*(i+3)*s2Len + (j-3)] = (int32_t)_mm_extract_epi32(vWH, 0);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int64_t)_mm_extract_epi64(vWH, 0);
     }
 }
 #endif
@@ -52,39 +46,27 @@ static inline void arr_store_rowcol(
         int32_t s2Len)
 {
     if (i+0 == s1Len-1 && 0 <= j-0 && j-0 < s2Len) {
-        row[j-0] = (int32_t)_mm_extract_epi32(vWH, 3);
+        row[j-0] = (int64_t)_mm_extract_epi64(vWH, 1);
     }
     if (j-0 == s2Len-1 && 0 <= i+0 && i+0 < s1Len) {
-        col[(i+0)] = (int32_t)_mm_extract_epi32(vWH, 3);
+        col[(i+0)] = (int64_t)_mm_extract_epi64(vWH, 1);
     }
     if (i+1 == s1Len-1 && 0 <= j-1 && j-1 < s2Len) {
-        row[j-1] = (int32_t)_mm_extract_epi32(vWH, 2);
+        row[j-1] = (int64_t)_mm_extract_epi64(vWH, 0);
     }
     if (j-1 == s2Len-1 && 0 <= i+1 && i+1 < s1Len) {
-        col[(i+1)] = (int32_t)_mm_extract_epi32(vWH, 2);
-    }
-    if (i+2 == s1Len-1 && 0 <= j-2 && j-2 < s2Len) {
-        row[j-2] = (int32_t)_mm_extract_epi32(vWH, 1);
-    }
-    if (j-2 == s2Len-1 && 0 <= i+2 && i+2 < s1Len) {
-        col[(i+2)] = (int32_t)_mm_extract_epi32(vWH, 1);
-    }
-    if (i+3 == s1Len-1 && 0 <= j-3 && j-3 < s2Len) {
-        row[j-3] = (int32_t)_mm_extract_epi32(vWH, 0);
-    }
-    if (j-3 == s2Len-1 && 0 <= i+3 && i+3 < s1Len) {
-        col[(i+3)] = (int32_t)_mm_extract_epi32(vWH, 0);
+        col[(i+1)] = (int64_t)_mm_extract_epi64(vWH, 0);
     }
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_nw_stats_table_diag_altivec_128_32
+#define FNAME parasail_nw_stats_table_diag_altivec_128_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_nw_stats_rowcol_diag_altivec_128_32
+#define FNAME parasail_nw_stats_rowcol_diag_altivec_128_64
 #else
-#define FNAME parasail_nw_stats_diag_altivec_128_32
+#define FNAME parasail_nw_stats_diag_altivec_128_64
 #endif
 #endif
 
@@ -100,36 +82,36 @@ parasail_result_t* FNAME(
     int32_t s1Len = 0;
     int32_t s1Len_PAD = 0;
     int32_t s2Len_PAD = 0;
-    int32_t * restrict s1 = NULL;
-    int32_t * restrict s2B = NULL;
-    int32_t * restrict _H_pr = NULL;
-    int32_t * restrict _HM_pr = NULL;
-    int32_t * restrict _HS_pr = NULL;
-    int32_t * restrict _HL_pr = NULL;
-    int32_t * restrict _F_pr = NULL;
-    int32_t * restrict _FM_pr = NULL;
-    int32_t * restrict _FS_pr = NULL;
-    int32_t * restrict _FL_pr = NULL;
-    int32_t * restrict s2 = NULL;
-    int32_t * restrict H_pr = NULL;
-    int32_t * restrict HM_pr = NULL;
-    int32_t * restrict HS_pr = NULL;
-    int32_t * restrict HL_pr = NULL;
-    int32_t * restrict F_pr = NULL;
-    int32_t * restrict FM_pr = NULL;
-    int32_t * restrict FS_pr = NULL;
-    int32_t * restrict FL_pr = NULL;
+    int64_t * restrict s1 = NULL;
+    int64_t * restrict s2B = NULL;
+    int64_t * restrict _H_pr = NULL;
+    int64_t * restrict _HM_pr = NULL;
+    int64_t * restrict _HS_pr = NULL;
+    int64_t * restrict _HL_pr = NULL;
+    int64_t * restrict _F_pr = NULL;
+    int64_t * restrict _FM_pr = NULL;
+    int64_t * restrict _FS_pr = NULL;
+    int64_t * restrict _FL_pr = NULL;
+    int64_t * restrict s2 = NULL;
+    int64_t * restrict H_pr = NULL;
+    int64_t * restrict HM_pr = NULL;
+    int64_t * restrict HS_pr = NULL;
+    int64_t * restrict HL_pr = NULL;
+    int64_t * restrict F_pr = NULL;
+    int64_t * restrict FM_pr = NULL;
+    int64_t * restrict FS_pr = NULL;
+    int64_t * restrict FL_pr = NULL;
     parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
-    int32_t score = 0;
-    int32_t matches = 0;
-    int32_t similar = 0;
-    int32_t length = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
+    int64_t score = 0;
+    int64_t matches = 0;
+    int64_t similar = 0;
+    int64_t length = 0;
     vec128i vNegLimit;
     vec128i vPosLimit;
     vec128i vSaturationCheckMin;
@@ -170,7 +152,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize stack variables */
-    N = 4; /* number of values in vector */
+    N = 2; /* number of values in vector */
     PAD = N-1;
     PAD2 = PAD*2;
     s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
@@ -180,40 +162,38 @@ parasail_result_t* FNAME(
     j = 0;
     end_query = s1Len-1;
     end_ref = s2Len-1;
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     score = NEG_LIMIT;
     matches = NEG_LIMIT;
     similar = NEG_LIMIT;
     length = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm_set1_epi64(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi64(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vNegInf = _mm_set1_epi32(NEG_LIMIT);
-    vOpen = _mm_set1_epi32(open);
-    vGap  = _mm_set1_epi32(gap);
-    vZero = _mm_set1_epi32(0);
-    vNegInf0 = _mm_insert_epi32(vZero, NEG_LIMIT, 3);
-    vOne = _mm_set1_epi32(1);
-    vN = _mm_set1_epi32(N);
-    vGapN = _mm_set1_epi32(gap*N);
-    vNegOne = _mm_set1_epi32(-1);
-    vI = _mm_set_epi32(0,1,2,3);
-    vJreset = _mm_set_epi32(0,-1,-2,-3);
+    vNegInf = _mm_set1_epi64(NEG_LIMIT);
+    vOpen = _mm_set1_epi64(open);
+    vGap  = _mm_set1_epi64(gap);
+    vZero = _mm_set1_epi64(0);
+    vNegInf0 = _mm_insert_epi64(vZero, NEG_LIMIT, 1);
+    vOne = _mm_set1_epi64(1);
+    vN = _mm_set1_epi64(N);
+    vGapN = _mm_set1_epi64(gap*N);
+    vNegOne = _mm_set1_epi64(-1);
+    vI = _mm_set_epi64(0,1);
+    vJreset = _mm_set_epi64(0,-1);
     vMaxH = vNegInf;
     vMaxM = vNegInf;
     vMaxS = vNegInf;
     vMaxL = vNegInf;
-    vILimit = _mm_set1_epi32(s1Len);
-    vILimit1 = _mm_sub_epi32(vILimit, vOne);
-    vJLimit = _mm_set1_epi32(s2Len);
-    vJLimit1 = _mm_sub_epi32(vJLimit, vOne);
-    vIBoundary = _mm_set_epi32(
+    vILimit = _mm_set1_epi64(s1Len);
+    vILimit1 = _mm_sub_epi64(vILimit, vOne);
+    vJLimit = _mm_set1_epi64(s2Len);
+    vJLimit1 = _mm_sub_epi64(vJLimit, vOne);
+    vIBoundary = _mm_set_epi64(
             -open-0*gap,
-            -open-1*gap,
-            -open-2*gap,
-            -open-3*gap);
+            -open-1*gap);
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -230,7 +210,7 @@ parasail_result_t* FNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_DIAG
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -239,16 +219,16 @@ parasail_result_t* FNAME(
 #endif
 
     /* initialize heap variables */
-    s1     = parasail_memalign_int32_t(16, s1Len+PAD);
-    s2B    = parasail_memalign_int32_t(16, s2Len+PAD2);
-    _H_pr  = parasail_memalign_int32_t(16, s2Len+PAD2);
-    _HM_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    _HS_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    _HL_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    _F_pr  = parasail_memalign_int32_t(16, s2Len+PAD2);
-    _FM_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    _FS_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    _FL_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    s1     = parasail_memalign_int64_t(16, s1Len+PAD);
+    s2B    = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _H_pr  = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _HM_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _HS_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _HL_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _F_pr  = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _FM_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _FS_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _FL_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
     s2 = s2B+PAD; /* will allow later for negative indices */
     H_pr = _H_pr+PAD;
     HM_pr = _HM_pr+PAD;
@@ -353,18 +333,14 @@ parasail_result_t* FNAME(
         vec128i vFS = vZero;
         vec128i vFL = vZero;
         vec128i vJ = vJreset;
-        vec128i vs1 = _mm_set_epi32(
+        vec128i vs1 = _mm_set_epi64(
                 s1[i+0],
-                s1[i+1],
-                s1[i+2],
-                s1[i+3]);
+                s1[i+1]);
         vec128i vs2 = vNegInf;
         const int * const restrict matrow0 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+0] : ((i+0 >= s1Len) ? s1Len-1 : i+0))];
         const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
-        const int * const restrict matrow2 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+2] : ((i+2 >= s1Len) ? s1Len-1 : i+2))];
-        const int * const restrict matrow3 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+3] : ((i+3 >= s1Len) ? s1Len-1 : i+3))];
-        vNH = _mm_insert_epi32(vNH, H_pr[-1], 3);
-        vWH = _mm_insert_epi32(vWH, -open - i*gap, 3);
+        vNH = _mm_insert_epi64(vNH, H_pr[-1], 1);
+        vWH = _mm_insert_epi64(vWH, -open - i*gap, 1);
         H_pr[-1] = -open - (i+N)*gap;
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
@@ -373,72 +349,70 @@ parasail_result_t* FNAME(
             vec128i vNWM = vNM;
             vec128i vNWS = vNS;
             vec128i vNWL = vNL;
-            vNH = _mm_srli_si128(vWH, 4);
-            vNH = _mm_insert_epi32(vNH, H_pr[j], 3);
-            vNM = _mm_srli_si128(vWM, 4);
-            vNM = _mm_insert_epi32(vNM, HM_pr[j], 3);
-            vNS = _mm_srli_si128(vWS, 4);
-            vNS = _mm_insert_epi32(vNS, HS_pr[j], 3);
-            vNL = _mm_srli_si128(vWL, 4);
-            vNL = _mm_insert_epi32(vNL, HL_pr[j], 3);
-            vF = _mm_srli_si128(vF, 4);
-            vF = _mm_insert_epi32(vF, F_pr[j], 3);
-            vFM = _mm_srli_si128(vFM, 4);
-            vFM = _mm_insert_epi32(vFM, FM_pr[j], 3);
-            vFS = _mm_srli_si128(vFS, 4);
-            vFS = _mm_insert_epi32(vFS, FS_pr[j], 3);
-            vFL = _mm_srli_si128(vFL, 4);
-            vFL = _mm_insert_epi32(vFL, FL_pr[j], 3);
-            vF_opn = _mm_sub_epi32(vNH, vOpen);
-            vF_ext = _mm_sub_epi32(vF, vGap);
-            vF = _mm_max_epi32(vF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi32(vF_opn, vF_ext);
+            vNH = _mm_srli_si128(vWH, 8);
+            vNH = _mm_insert_epi64(vNH, H_pr[j], 1);
+            vNM = _mm_srli_si128(vWM, 8);
+            vNM = _mm_insert_epi64(vNM, HM_pr[j], 1);
+            vNS = _mm_srli_si128(vWS, 8);
+            vNS = _mm_insert_epi64(vNS, HS_pr[j], 1);
+            vNL = _mm_srli_si128(vWL, 8);
+            vNL = _mm_insert_epi64(vNL, HL_pr[j], 1);
+            vF = _mm_srli_si128(vF, 8);
+            vF = _mm_insert_epi64(vF, F_pr[j], 1);
+            vFM = _mm_srli_si128(vFM, 8);
+            vFM = _mm_insert_epi64(vFM, FM_pr[j], 1);
+            vFS = _mm_srli_si128(vFS, 8);
+            vFS = _mm_insert_epi64(vFS, FS_pr[j], 1);
+            vFL = _mm_srli_si128(vFL, 8);
+            vFL = _mm_insert_epi64(vFL, FL_pr[j], 1);
+            vF_opn = _mm_sub_epi64(vNH, vOpen);
+            vF_ext = _mm_sub_epi64(vF, vGap);
+            vF = _mm_max_epi64(vF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi64(vF_opn, vF_ext);
             vFM = _mm_blendv_epi8(vFM, vNM, case1);
             vFS = _mm_blendv_epi8(vFS, vNS, case1);
             vFL = _mm_blendv_epi8(vFL, vNL, case1);
-            vFL = _mm_add_epi32(vFL, vOne);
-            vE_opn = _mm_sub_epi32(vWH, vOpen);
-            vE_ext = _mm_sub_epi32(vE, vGap);
-            vE = _mm_max_epi32(vE_opn, vE_ext);
-            case1 = _mm_cmpgt_epi32(vE_opn, vE_ext);
+            vFL = _mm_add_epi64(vFL, vOne);
+            vE_opn = _mm_sub_epi64(vWH, vOpen);
+            vE_ext = _mm_sub_epi64(vE, vGap);
+            vE = _mm_max_epi64(vE_opn, vE_ext);
+            case1 = _mm_cmpgt_epi64(vE_opn, vE_ext);
             vEM = _mm_blendv_epi8(vEM, vWM, case1);
             vES = _mm_blendv_epi8(vES, vWS, case1);
             vEL = _mm_blendv_epi8(vEL, vWL, case1);
-            vEL = _mm_add_epi32(vEL, vOne);
-            vs2 = _mm_srli_si128(vs2, 4);
-            vs2 = _mm_insert_epi32(vs2, s2[j], 3);
-            vMat = _mm_set_epi32(
+            vEL = _mm_add_epi64(vEL, vOne);
+            vs2 = _mm_srli_si128(vs2, 8);
+            vs2 = _mm_insert_epi64(vs2, s2[j], 1);
+            vMat = _mm_set_epi64(
                     matrow0[s2[j-0]],
-                    matrow1[s2[j-1]],
-                    matrow2[s2[j-2]],
-                    matrow3[s2[j-3]]
+                    matrow1[s2[j-1]]
                     );
-            vNWH = _mm_add_epi32(vNWH, vMat);
-            vWH = _mm_max_epi32(vNWH, vE);
-            vWH = _mm_max_epi32(vWH, vF);
-            case1 = _mm_cmpeq_epi32(vWH, vNWH);
-            case2 = _mm_cmpeq_epi32(vWH, vF);
+            vNWH = _mm_add_epi64(vNWH, vMat);
+            vWH = _mm_max_epi64(vNWH, vE);
+            vWH = _mm_max_epi64(vWH, vF);
+            case1 = _mm_cmpeq_epi64(vWH, vNWH);
+            case2 = _mm_cmpeq_epi64(vWH, vF);
             vWM = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEM, vFM, case2),
-                    _mm_add_epi32(vNWM,
+                    _mm_add_epi64(vNWM,
                         _mm_and_si128(
-                            _mm_cmpeq_epi32(vs1,vs2),
+                            _mm_cmpeq_epi64(vs1,vs2),
                             vOne)),
                     case1);
             vWS = _mm_blendv_epi8(
                     _mm_blendv_epi8(vES, vFS, case2),
-                    _mm_add_epi32(vNWS,
+                    _mm_add_epi64(vNWS,
                         _mm_and_si128(
-                            _mm_cmpgt_epi32(vMat,vZero),
+                            _mm_cmpgt_epi64(vMat,vZero),
                             vOne)),
                     case1);
             vWL = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEL, vFL, case2),
-                    _mm_add_epi32(vNWL, vOne), case1);
+                    _mm_add_epi64(vNWL, vOne), case1);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                vec128i cond = _mm_cmpeq_epi32(vJ,vNegOne);
+                vec128i cond = _mm_cmpeq_epi64(vJ,vNegOne);
                 vWH = _mm_blendv_epi8(vWH, vIBoundary, cond);
                 vWM = _mm_andnot_si128(cond, vWM);
                 vWS = _mm_andnot_si128(cond, vWS);
@@ -450,12 +424,12 @@ parasail_result_t* FNAME(
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWH);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWM);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWS);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWL);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWL);
+                vSaturationCheckMin = _mm_min_epi64(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vWH);
+                vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vWM);
+                vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vWS);
+                vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vWL);
+                vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vWL);
             }
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vWH, i, s1Len, j, s2Len);
@@ -469,50 +443,50 @@ parasail_result_t* FNAME(
             arr_store_rowcol(result->stats->rowcols->similar_row, result->stats->rowcols->similar_col, vWS, i, s1Len, j, s2Len);
             arr_store_rowcol(result->stats->rowcols->length_row,  result->stats->rowcols->length_col, vWL, i, s1Len, j, s2Len);
 #endif
-            H_pr[j-3] = (int32_t)_mm_extract_epi32(vWH,0);
-            HM_pr[j-3] = (int32_t)_mm_extract_epi32(vWM,0);
-            HS_pr[j-3] = (int32_t)_mm_extract_epi32(vWS,0);
-            HL_pr[j-3] = (int32_t)_mm_extract_epi32(vWL,0);
-            F_pr[j-3] = (int32_t)_mm_extract_epi32(vF,0);
-            FM_pr[j-3] = (int32_t)_mm_extract_epi32(vFM,0);
-            FS_pr[j-3] = (int32_t)_mm_extract_epi32(vFS,0);
-            FL_pr[j-3] = (int32_t)_mm_extract_epi32(vFL,0);
+            H_pr[j-1] = (int64_t)_mm_extract_epi64(vWH,0);
+            HM_pr[j-1] = (int64_t)_mm_extract_epi64(vWM,0);
+            HS_pr[j-1] = (int64_t)_mm_extract_epi64(vWS,0);
+            HL_pr[j-1] = (int64_t)_mm_extract_epi64(vWL,0);
+            F_pr[j-1] = (int64_t)_mm_extract_epi64(vF,0);
+            FM_pr[j-1] = (int64_t)_mm_extract_epi64(vFM,0);
+            FS_pr[j-1] = (int64_t)_mm_extract_epi64(vFS,0);
+            FL_pr[j-1] = (int64_t)_mm_extract_epi64(vFL,0);
             /* as minor diagonal vector passes across table, extract
                last table value at the i,j bound */
             {
-                vec128i cond_valid_I = _mm_cmpeq_epi32(vI, vILimit1);
-                vec128i cond_valid_J = _mm_cmpeq_epi32(vJ, vJLimit1);
+                vec128i cond_valid_I = _mm_cmpeq_epi64(vI, vILimit1);
+                vec128i cond_valid_J = _mm_cmpeq_epi64(vJ, vJLimit1);
                 vec128i cond_all = _mm_and_si128(cond_valid_I, cond_valid_J);
                 vMaxH = _mm_blendv_epi8(vMaxH, vWH, cond_all);
                 vMaxM = _mm_blendv_epi8(vMaxM, vWM, cond_all);
                 vMaxS = _mm_blendv_epi8(vMaxS, vWS, cond_all);
                 vMaxL = _mm_blendv_epi8(vMaxL, vWL, cond_all);
             }
-            vJ = _mm_add_epi32(vJ, vOne);
+            vJ = _mm_add_epi64(vJ, vOne);
         }
-        vI = _mm_add_epi32(vI, vN);
-        vIBoundary = _mm_sub_epi32(vIBoundary, vGapN);
+        vI = _mm_add_epi64(vI, vN);
+        vIBoundary = _mm_sub_epi64(vIBoundary, vGapN);
     }
 
     /* max in vMaxH */
     for (i=0; i<N; ++i) {
-        int32_t value;
-        value = (int32_t) _mm_extract_epi32(vMaxH, 3);
+        int64_t value;
+        value = (int64_t) _mm_extract_epi64(vMaxH, 1);
         if (value > score) {
             score = value;
-            matches = (int32_t) _mm_extract_epi32(vMaxM, 3);
-            similar = (int32_t) _mm_extract_epi32(vMaxS, 3);
-            length= (int32_t) _mm_extract_epi32(vMaxL, 3);
+            matches = (int64_t) _mm_extract_epi64(vMaxM, 1);
+            similar = (int64_t) _mm_extract_epi64(vMaxS, 1);
+            length= (int64_t) _mm_extract_epi64(vMaxL, 1);
         }
-        vMaxH = _mm_slli_si128(vMaxH, 4);
-        vMaxM = _mm_slli_si128(vMaxM, 4);
-        vMaxS = _mm_slli_si128(vMaxS, 4);
-        vMaxL = _mm_slli_si128(vMaxL, 4);
+        vMaxH = _mm_slli_si128(vMaxH, 8);
+        vMaxM = _mm_slli_si128(vMaxM, 8);
+        vMaxS = _mm_slli_si128(vMaxS, 8);
+        vMaxL = _mm_slli_si128(vMaxL, 8);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
