@@ -29,10 +29,22 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 3);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  0);
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  1);
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  2);
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  3);
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  4);
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  5);
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  6);
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  7);
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  8);
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  9);
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 10);
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 11);
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 12);
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 13);
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 14);
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 15);
 }
 #endif
 
@@ -43,23 +55,35 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int32_t)_mm_extract_epi32(vH, 0);
-    col[1*seglen+t] = (int32_t)_mm_extract_epi32(vH, 1);
-    col[2*seglen+t] = (int32_t)_mm_extract_epi32(vH, 2);
-    col[3*seglen+t] = (int32_t)_mm_extract_epi32(vH, 3);
+    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8(vH,  0);
+    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8(vH,  1);
+    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8(vH,  2);
+    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8(vH,  3);
+    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8(vH,  4);
+    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8(vH,  5);
+    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8(vH,  6);
+    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8(vH,  7);
+    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8(vH,  8);
+    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8(vH,  9);
+    col[10*seglen+t] = (int8_t)_mm_extract_epi8(vH, 10);
+    col[11*seglen+t] = (int8_t)_mm_extract_epi8(vH, 11);
+    col[12*seglen+t] = (int8_t)_mm_extract_epi8(vH, 12);
+    col[13*seglen+t] = (int8_t)_mm_extract_epi8(vH, 13);
+    col[14*seglen+t] = (int8_t)_mm_extract_epi8(vH, 14);
+    col[15*seglen+t] = (int8_t)_mm_extract_epi8(vH, 15);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_nw_stats_table_striped_altivec_128_32
-#define PNAME parasail_nw_stats_table_striped_profile_altivec_128_32
+#define FNAME parasail_nw_stats_table_striped_altivec_128_8
+#define PNAME parasail_nw_stats_table_striped_profile_altivec_128_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_nw_stats_rowcol_striped_altivec_128_32
-#define PNAME parasail_nw_stats_rowcol_striped_profile_altivec_128_32
+#define FNAME parasail_nw_stats_rowcol_striped_altivec_128_8
+#define PNAME parasail_nw_stats_rowcol_striped_profile_altivec_128_8
 #else
-#define FNAME parasail_nw_stats_striped_altivec_128_32
-#define PNAME parasail_nw_stats_striped_profile_altivec_128_32
+#define FNAME parasail_nw_stats_striped_altivec_128_8
+#define PNAME parasail_nw_stats_striped_profile_altivec_128_8
 #endif
 #endif
 
@@ -87,7 +111,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_altivec_128_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_altivec_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -128,17 +152,17 @@ parasail_result_t* PNAME(
     vec128i* restrict pvEM = NULL;
     vec128i* restrict pvES = NULL;
     vec128i* restrict pvEL = NULL;
-    int32_t* restrict boundary = NULL;
+    int8_t* restrict boundary = NULL;
     vec128i vGapO;
     vec128i vGapE;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
     vec128i vZero;
     vec128i vOne;
-    int32_t score = 0;
-    int32_t matches = 0;
-    int32_t similar = 0;
-    int32_t length = 0;
+    int8_t score = 0;
+    int8_t matches = 0;
+    int8_t similar = 0;
+    int8_t length = 0;
     vec128i vNegLimit;
     vec128i vPosLimit;
     vec128i vSaturationCheckMin;
@@ -147,7 +171,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -163,25 +187,25 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 4; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile  = (vec128i*)profile->profile32.score;
-    vProfileM = (vec128i*)profile->profile32.matches;
-    vProfileS = (vec128i*)profile->profile32.similar;
-    vGapO = _mm_set1_epi32(open);
-    vGapE = _mm_set1_epi32(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    vProfile  = (vec128i*)profile->profile8.score;
+    vProfileM = (vec128i*)profile->profile8.matches;
+    vProfileS = (vec128i*)profile->profile8.similar;
+    vGapO = _mm_set1_epi8(open);
+    vGapE = _mm_set1_epi8(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
     vZero = _mm_setzero_si128();
-    vOne = _mm_set1_epi32(1);
+    vOne = _mm_set1_epi8(1);
     score = NEG_LIMIT;
     matches = 0;
     similar = 0;
     length = 0;
-    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm_set1_epi8(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi8(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
 
@@ -200,7 +224,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -221,7 +245,7 @@ parasail_result_t* PNAME(
     pvEM      = parasail_memalign_vec128i(16, segLen);
     pvES      = parasail_memalign_vec128i(16, segLen);
     pvEL      = parasail_memalign_vec128i(16, segLen);
-    boundary  = parasail_memalign_int32_t(16, s2Len+1);
+    boundary  = parasail_memalign_int8_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -250,13 +274,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            vec128i_32_t h;
-            vec128i_32_t e;
+            vec128i_8_t h;
+            vec128i_8_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = -open-gap*(segNum*segLen+i);
-                h.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                h.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                e.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
             }
             _mm_store_si128(&pvHStore[index], h.m);
             _mm_store_si128(&pvE[index], e.m);
@@ -269,7 +293,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = -open-gap*(i-1);
-            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
+            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
         }
     }
 
@@ -302,18 +326,18 @@ parasail_result_t* PNAME(
         vFS = vZero;
         vFL = vOne;
 
-        /* load final segment of pvHStore and shift left by 4 bytes */
+        /* load final segment of pvHStore and shift left by 1 bytes */
         vH = _mm_load_si128(&pvHStore[segLen - 1]);
         vHM = _mm_load_si128(&pvHMStore[segLen - 1]);
         vHS = _mm_load_si128(&pvHSStore[segLen - 1]);
         vHL = _mm_load_si128(&pvHLStore[segLen - 1]);
-        vH = _mm_slli_si128(vH, 4);
-        vHM = _mm_slli_si128(vHM, 4);
-        vHS = _mm_slli_si128(vHS, 4);
-        vHL = _mm_slli_si128(vHL, 4);
+        vH = _mm_slli_si128(vH, 1);
+        vHM = _mm_slli_si128(vHM, 1);
+        vHS = _mm_slli_si128(vHS, 1);
+        vHL = _mm_slli_si128(vHL, 1);
 
         /* insert upper boundary condition */
-        vH = _mm_insert_epi32(vH, boundary[j], 0);
+        vH = _mm_insert_epi8(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -337,58 +361,58 @@ parasail_result_t* PNAME(
             vEL = _mm_load_si128(pvEL+ i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm_add_epi32(vH, _mm_load_si128(vP + i));
-            vH = _mm_max_epi32(vH_dag, vE);
-            vH = _mm_max_epi32(vH, vF);
+            vH_dag = _mm_adds_epi8(vH, _mm_load_si128(vP + i));
+            vH = _mm_max_epi8(vH_dag, vE);
+            vH = _mm_max_epi8(vH, vF);
             /* Save vH values. */
             _mm_store_si128(pvHStore + i, vH);
 
-            case1 = _mm_cmpeq_epi32(vH, vH_dag);
-            case2 = _mm_cmpeq_epi32(vH, vF);
+            case1 = _mm_cmpeq_epi8(vH, vH_dag);
+            case2 = _mm_cmpeq_epi8(vH, vF);
 
             /* calculate vM */
             vHM = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEM, vFM, case2),
-                    _mm_add_epi32(vHM, _mm_load_si128(vPM + i)),
+                    _mm_adds_epi8(vHM, _mm_load_si128(vPM + i)),
                     case1);
             _mm_store_si128(pvHMStore + i, vHM);
 
             /* calculate vS */
             vHS = _mm_blendv_epi8(
                     _mm_blendv_epi8(vES, vFS, case2),
-                    _mm_add_epi32(vHS, _mm_load_si128(vPS + i)),
+                    _mm_adds_epi8(vHS, _mm_load_si128(vPS + i)),
                     case1);
             _mm_store_si128(pvHSStore + i, vHS);
 
             /* calculate vL */
             vHL = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEL, vFL, case2),
-                    _mm_add_epi32(vHL, vOne),
+                    _mm_adds_epi8(vHL, vOne),
                     case1);
             _mm_store_si128(pvHLStore + i, vHL);
 
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-            vEF_opn = _mm_sub_epi32(vH, vGapO);
+            vEF_opn = _mm_subs_epi8(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm_sub_epi32(vE, vGapE);
-            vE = _mm_max_epi32(vEF_opn, vE_ext);
-            case1 = _mm_cmpgt_epi32(vEF_opn, vE_ext);
+            vE_ext = _mm_subs_epi8(vE, vGapE);
+            vE = _mm_max_epi8(vEF_opn, vE_ext);
+            case1 = _mm_cmpgt_epi8(vEF_opn, vE_ext);
             vEM = _mm_blendv_epi8(vEM, vHM, case1);
             vES = _mm_blendv_epi8(vES, vHS, case1);
             vEL = _mm_blendv_epi8(
-                    _mm_add_epi32(vEL, vOne),
-                    _mm_add_epi32(vHL, vOne),
+                    _mm_adds_epi8(vEL, vOne),
+                    _mm_adds_epi8(vHL, vOne),
                     case1);
             _mm_store_si128(pvE + i, vE);
             _mm_store_si128(pvEM + i, vEM);
@@ -396,14 +420,14 @@ parasail_result_t* PNAME(
             _mm_store_si128(pvEL + i, vEL);
 
             /* Update vF value. */
-            vF_ext = _mm_sub_epi32(vF, vGapE);
-            vF = _mm_max_epi32(vEF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi32(vEF_opn, vF_ext);
+            vF_ext = _mm_subs_epi8(vF, vGapE);
+            vF = _mm_max_epi8(vEF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi8(vEF_opn, vF_ext);
             vFM = _mm_blendv_epi8(vFM, vHM, case1);
             vFS = _mm_blendv_epi8(vFS, vHS, case1);
             vFL = _mm_blendv_epi8(
-                    _mm_add_epi32(vFL, vOne),
-                    _mm_add_epi32(vHL, vOne),
+                    _mm_adds_epi8(vFL, vOne),
+                    _mm_adds_epi8(vHL, vOne),
                     case1);
 
             /* Load the next vH. */
@@ -418,26 +442,26 @@ parasail_result_t* PNAME(
         for (k=0; k<segWidth; ++k) {
             vec128i vHp = _mm_load_si128(&pvHLoad[segLen - 1]);
             int64_t tmp = boundary[j+1]-open;
-            int32_t tmp2 = tmp < INT32_MIN ? INT32_MIN : tmp;
-            vHp = _mm_slli_si128(vHp, 4);
-            vF = _mm_slli_si128(vF, 4);
-            vFM = _mm_slli_si128(vFM, 4);
-            vFS = _mm_slli_si128(vFS, 4);
-            vFL = _mm_slli_si128(vFL, 4);
-            vHp = _mm_insert_epi32(vHp, boundary[j], 0);
-            vF = _mm_insert_epi32(vF, tmp2, 0);
-            vFL = _mm_insert_epi32(vFL, 1, 0);
+            int8_t tmp2 = tmp < INT8_MIN ? INT8_MIN : tmp;
+            vHp = _mm_slli_si128(vHp, 1);
+            vF = _mm_slli_si128(vF, 1);
+            vFM = _mm_slli_si128(vFM, 1);
+            vFS = _mm_slli_si128(vFS, 1);
+            vFL = _mm_slli_si128(vFL, 1);
+            vHp = _mm_insert_epi8(vHp, boundary[j], 0);
+            vF = _mm_insert_epi8(vF, tmp2, 0);
+            vFL = _mm_insert_epi8(vFL, 1, 0);
             for (i=0; i<segLen; ++i) {
                 vec128i case1;
                 vec128i case2;
                 vec128i cond;
 
-                vHp = _mm_add_epi32(vHp, _mm_load_si128(vP + i));
+                vHp = _mm_adds_epi8(vHp, _mm_load_si128(vP + i));
                 vH = _mm_load_si128(pvHStore + i);
-                vH = _mm_max_epi32(vH,vF);
+                vH = _mm_max_epi8(vH,vF);
                 _mm_store_si128(pvHStore + i, vH);
-                case1 = _mm_cmpeq_epi32(vH, vHp);
-                case2 = _mm_cmpeq_epi32(vH, vF);
+                case1 = _mm_cmpeq_epi8(vH, vHp);
+                case2 = _mm_cmpeq_epi8(vH, vF);
                 cond = _mm_andnot_si128(case1, case2);
 
                 /* calculate vM */
@@ -455,11 +479,11 @@ parasail_result_t* PNAME(
                 vHL = _mm_blendv_epi8(vHL, vFL, cond);
                 _mm_store_si128(pvHLStore + i, vHL);
 
-                vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHM);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHS);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHL);
+                vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vH);
+                vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHM);
+                vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHS);
+                vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
                 arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
                 arr_store_si128(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
@@ -467,21 +491,21 @@ parasail_result_t* PNAME(
                 arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
                 /* Update vF value. */
-                vEF_opn = _mm_sub_epi32(vH, vGapO);
-                vF_ext = _mm_sub_epi32(vF, vGapE);
+                vEF_opn = _mm_subs_epi8(vH, vGapO);
+                vF_ext = _mm_subs_epi8(vF, vGapE);
                 if (! _mm_movemask_epi8(
                             _mm_or_si128(
-                                _mm_cmpgt_epi32(vF_ext, vEF_opn),
-                                _mm_cmpeq_epi32(vF_ext, vEF_opn))))
+                                _mm_cmpgt_epi8(vF_ext, vEF_opn),
+                                _mm_cmpeq_epi8(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = _mm_max_epi32(vEF_opn, vF_ext);*/
+                /*vF = _mm_max_epi8(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                cond = _mm_cmpgt_epi32(vEF_opn, vF_ext);
+                cond = _mm_cmpgt_epi8(vEF_opn, vF_ext);
                 vFM = _mm_blendv_epi8(vFM, vHM, cond);
                 vFS = _mm_blendv_epi8(vFS, vHS, cond);
                 vFL = _mm_blendv_epi8(
-                        _mm_add_epi32(vFL, vOne),
-                        _mm_add_epi32(vHL, vOne),
+                        _mm_adds_epi8(vFL, vOne),
+                        _mm_adds_epi8(vHL, vOne),
                         cond);
                 vHp = _mm_load_si128(pvHLoad + i);
             }
@@ -498,15 +522,15 @@ end:
             vHS = _mm_load_si128(pvHSStore + offset);
             vHL = _mm_load_si128(pvHLStore + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128 (vH, 4);
-                vHM = _mm_slli_si128 (vHM, 4);
-                vHS = _mm_slli_si128 (vHS, 4);
-                vHL = _mm_slli_si128 (vHL, 4);
+                vH = _mm_slli_si128 (vH, 1);
+                vHM = _mm_slli_si128 (vHM, 1);
+                vHS = _mm_slli_si128 (vHS, 1);
+                vHL = _mm_slli_si128 (vHL, 1);
             }
-            result->stats->rowcols->score_row[j] = (int32_t) _mm_extract_epi32 (vH, 3);
-            result->stats->rowcols->matches_row[j] = (int32_t) _mm_extract_epi32 (vHM, 3);
-            result->stats->rowcols->similar_row[j] = (int32_t) _mm_extract_epi32 (vHS, 3);
-            result->stats->rowcols->length_row[j] = (int32_t) _mm_extract_epi32 (vHL, 3);
+            result->stats->rowcols->score_row[j] = (int8_t) _mm_extract_epi8 (vH, 15);
+            result->stats->rowcols->matches_row[j] = (int8_t) _mm_extract_epi8 (vHM, 15);
+            result->stats->rowcols->similar_row[j] = (int8_t) _mm_extract_epi8 (vHS, 15);
+            result->stats->rowcols->length_row[j] = (int8_t) _mm_extract_epi8 (vHL, 15);
         }
 #endif
     }
@@ -531,20 +555,20 @@ end:
         vec128i vHS = _mm_load_si128(pvHSStore + offset);
         vec128i vHL = _mm_load_si128(pvHLStore + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm_slli_si128 (vH, 4);
-            vHM = _mm_slli_si128 (vHM, 4);
-            vHS = _mm_slli_si128 (vHS, 4);
-            vHL = _mm_slli_si128 (vHL, 4);
+            vH = _mm_slli_si128 (vH, 1);
+            vHM = _mm_slli_si128 (vHM, 1);
+            vHS = _mm_slli_si128 (vHS, 1);
+            vHL = _mm_slli_si128 (vHL, 1);
         }
-        score = (int32_t) _mm_extract_epi32 (vH, 3);
-        matches = (int32_t) _mm_extract_epi32 (vHM, 3);
-        similar = (int32_t) _mm_extract_epi32 (vHS, 3);
-        length = (int32_t) _mm_extract_epi32 (vHL, 3);
+        score = (int8_t) _mm_extract_epi8 (vH, 15);
+        matches = (int8_t) _mm_extract_epi8 (vHM, 15);
+        similar = (int8_t) _mm_extract_epi8 (vHS, 15);
+        length = (int8_t) _mm_extract_epi8 (vHL, 15);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
