@@ -19,65 +19,28 @@
 #define SWAP(A,B) { __m256i* tmp = A; A = B; B = tmp; }
 
 
-#if HAVE_AVX2_MM256_INSERT_EPI64
-#define _mm256_insert_epi64_rpl _mm256_insert_epi64
+#if HAVE_AVX2_MM256_INSERT_EPI8
+#define _mm256_insert_epi8_rpl _mm256_insert_epi8
 #else
-static inline __m256i _mm256_insert_epi64_rpl(__m256i a, int64_t i, int imm) {
-    __m256i_64_t A;
+static inline __m256i _mm256_insert_epi8_rpl(__m256i a, int8_t i, int imm) {
+    __m256i_8_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_SET1_EPI64X
-#define _mm256_set1_epi64x_rpl _mm256_set1_epi64x
+#if HAVE_AVX2_MM256_EXTRACT_EPI8
+#define _mm256_extract_epi8_rpl _mm256_extract_epi8
 #else
-static inline __m256i _mm256_set1_epi64x_rpl(int64_t i) {
-    __m256i_64_t A;
-    A.v[0] = i;
-    A.v[1] = i;
-    A.v[2] = i;
-    A.v[3] = i;
-    return A.m;
-}
-#endif
-
-static inline __m256i _mm256_max_epi64_rpl(__m256i a, __m256i b) {
-    __m256i_64_t A;
-    __m256i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
-    A.v[2] = (A.v[2]>B.v[2]) ? A.v[2] : B.v[2];
-    A.v[3] = (A.v[3]>B.v[3]) ? A.v[3] : B.v[3];
-    return A.m;
-}
-
-#if HAVE_AVX2_MM256_EXTRACT_EPI64
-#define _mm256_extract_epi64_rpl _mm256_extract_epi64
-#else
-static inline int64_t _mm256_extract_epi64_rpl(__m256i a, int imm) {
-    __m256i_64_t A;
+static inline int8_t _mm256_extract_epi8_rpl(__m256i a, int imm) {
+    __m256i_8_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
-static inline __m256i _mm256_min_epi64_rpl(__m256i a, __m256i b) {
-    __m256i_64_t A;
-    __m256i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
-    A.v[2] = (A.v[2]<B.v[2]) ? A.v[2] : B.v[2];
-    A.v[3] = (A.v[3]<B.v[3]) ? A.v[3] : B.v[3];
-    return A.m;
-}
-
-#define _mm256_cmplt_epi64_rpl(a,b) _mm256_cmpgt_epi64(b,a)
+#define _mm256_cmplt_epi8_rpl(a,b) _mm256_cmpgt_epi8(b,a)
 
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
@@ -91,10 +54,38 @@ static inline void arr_store_si256(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 3);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  0);
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  1);
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  2);
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  3);
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  4);
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  5);
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  6);
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  7);
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  8);
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  9);
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 10);
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 11);
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 12);
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 13);
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 14);
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 15);
+    array[1LL*(16*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 16);
+    array[1LL*(17*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 17);
+    array[1LL*(18*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 18);
+    array[1LL*(19*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 19);
+    array[1LL*(20*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 20);
+    array[1LL*(21*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 21);
+    array[1LL*(22*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 22);
+    array[1LL*(23*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 23);
+    array[1LL*(24*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 24);
+    array[1LL*(25*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 25);
+    array[1LL*(26*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 26);
+    array[1LL*(27*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 27);
+    array[1LL*(28*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 28);
+    array[1LL*(29*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 29);
+    array[1LL*(30*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 30);
+    array[1LL*(31*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 31);
 }
 #endif
 
@@ -105,23 +96,51 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 0);
-    col[1*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 1);
-    col[2*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 2);
-    col[3*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 3);
+    col[ 0*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  0);
+    col[ 1*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  1);
+    col[ 2*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  2);
+    col[ 3*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  3);
+    col[ 4*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  4);
+    col[ 5*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  5);
+    col[ 6*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  6);
+    col[ 7*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  7);
+    col[ 8*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  8);
+    col[ 9*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  9);
+    col[10*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 10);
+    col[11*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 11);
+    col[12*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 12);
+    col[13*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 13);
+    col[14*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 14);
+    col[15*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 15);
+    col[16*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 16);
+    col[17*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 17);
+    col[18*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 18);
+    col[19*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 19);
+    col[20*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 20);
+    col[21*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 21);
+    col[22*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 22);
+    col[23*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 23);
+    col[24*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 24);
+    col[25*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 25);
+    col[26*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 26);
+    col[27*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 27);
+    col[28*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 28);
+    col[29*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 29);
+    col[30*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 30);
+    col[31*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 31);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_nw_stats_table_striped_avx2_256_64
-#define PNAME parasail_nw_stats_table_striped_profile_avx2_256_64
+#define FNAME parasail_nw_stats_table_striped_avx2_256_8
+#define PNAME parasail_nw_stats_table_striped_profile_avx2_256_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_nw_stats_rowcol_striped_avx2_256_64
-#define PNAME parasail_nw_stats_rowcol_striped_profile_avx2_256_64
+#define FNAME parasail_nw_stats_rowcol_striped_avx2_256_8
+#define PNAME parasail_nw_stats_rowcol_striped_profile_avx2_256_8
 #else
-#define FNAME parasail_nw_stats_striped_avx2_256_64
-#define PNAME parasail_nw_stats_striped_profile_avx2_256_64
+#define FNAME parasail_nw_stats_striped_avx2_256_8
+#define PNAME parasail_nw_stats_striped_profile_avx2_256_8
 #endif
 #endif
 
@@ -149,7 +168,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_avx_256_64(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_avx_256_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -190,17 +209,17 @@ parasail_result_t* PNAME(
     __m256i* restrict pvEM = NULL;
     __m256i* restrict pvES = NULL;
     __m256i* restrict pvEL = NULL;
-    int64_t* restrict boundary = NULL;
+    int8_t* restrict boundary = NULL;
     __m256i vGapO;
     __m256i vGapE;
-    int64_t NEG_LIMIT = 0;
-    int64_t POS_LIMIT = 0;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
     __m256i vZero;
     __m256i vOne;
-    int64_t score = 0;
-    int64_t matches = 0;
-    int64_t similar = 0;
-    int64_t length = 0;
+    int8_t score = 0;
+    int8_t matches = 0;
+    int8_t similar = 0;
+    int8_t length = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -209,7 +228,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -225,25 +244,25 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 4; /* number of values in vector unit */
+    segWidth = 32; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile  = (__m256i*)profile->profile64.score;
-    vProfileM = (__m256i*)profile->profile64.matches;
-    vProfileS = (__m256i*)profile->profile64.similar;
-    vGapO = _mm256_set1_epi64x_rpl(open);
-    vGapE = _mm256_set1_epi64x_rpl(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    vProfile  = (__m256i*)profile->profile8.score;
+    vProfileM = (__m256i*)profile->profile8.matches;
+    vProfileS = (__m256i*)profile->profile8.similar;
+    vGapO = _mm256_set1_epi8(open);
+    vGapE = _mm256_set1_epi8(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
     vZero = _mm256_setzero_si256();
-    vOne = _mm256_set1_epi64x_rpl(1);
+    vOne = _mm256_set1_epi8(1);
     score = NEG_LIMIT;
     matches = 0;
     similar = 0;
     length = 0;
-    vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi8(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi8(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
 
@@ -262,7 +281,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_32;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -283,7 +302,7 @@ parasail_result_t* PNAME(
     pvEM      = parasail_memalign___m256i(32, segLen);
     pvES      = parasail_memalign___m256i(32, segLen);
     pvEL      = parasail_memalign___m256i(32, segLen);
-    boundary  = parasail_memalign_int64_t(32, s2Len+1);
+    boundary  = parasail_memalign_int8_t(32, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -312,13 +331,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m256i_64_t h;
-            __m256i_64_t e;
+            __m256i_8_t h;
+            __m256i_8_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = -open-gap*(segNum*segLen+i);
-                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                h.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                e.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
             }
             _mm256_store_si256(&pvHStore[index], h.m);
             _mm256_store_si256(&pvE[index], e.m);
@@ -331,7 +350,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = -open-gap*(i-1);
-            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
+            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
         }
     }
 
@@ -364,18 +383,18 @@ parasail_result_t* PNAME(
         vFS = vZero;
         vFL = vOne;
 
-        /* load final segment of pvHStore and shift left by 8 bytes */
+        /* load final segment of pvHStore and shift left by 1 bytes */
         vH = _mm256_load_si256(&pvHStore[segLen - 1]);
         vHM = _mm256_load_si256(&pvHMStore[segLen - 1]);
         vHS = _mm256_load_si256(&pvHSStore[segLen - 1]);
         vHL = _mm256_load_si256(&pvHLStore[segLen - 1]);
-        vH = _mm256_slli_si256_rpl(vH, 8);
-        vHM = _mm256_slli_si256_rpl(vHM, 8);
-        vHS = _mm256_slli_si256_rpl(vHS, 8);
-        vHL = _mm256_slli_si256_rpl(vHL, 8);
+        vH = _mm256_slli_si256_rpl(vH, 1);
+        vHM = _mm256_slli_si256_rpl(vHM, 1);
+        vHS = _mm256_slli_si256_rpl(vHS, 1);
+        vHL = _mm256_slli_si256_rpl(vHL, 1);
 
         /* insert upper boundary condition */
-        vH = _mm256_insert_epi64_rpl(vH, boundary[j], 0);
+        vH = _mm256_insert_epi8_rpl(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -399,58 +418,58 @@ parasail_result_t* PNAME(
             vEL = _mm256_load_si256(pvEL+ i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm256_add_epi64(vH, _mm256_load_si256(vP + i));
-            vH = _mm256_max_epi64_rpl(vH_dag, vE);
-            vH = _mm256_max_epi64_rpl(vH, vF);
+            vH_dag = _mm256_adds_epi8(vH, _mm256_load_si256(vP + i));
+            vH = _mm256_max_epi8(vH_dag, vE);
+            vH = _mm256_max_epi8(vH, vF);
             /* Save vH values. */
             _mm256_store_si256(pvHStore + i, vH);
 
-            case1 = _mm256_cmpeq_epi64(vH, vH_dag);
-            case2 = _mm256_cmpeq_epi64(vH, vF);
+            case1 = _mm256_cmpeq_epi8(vH, vH_dag);
+            case2 = _mm256_cmpeq_epi8(vH, vF);
 
             /* calculate vM */
             vHM = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vEM, vFM, case2),
-                    _mm256_add_epi64(vHM, _mm256_load_si256(vPM + i)),
+                    _mm256_adds_epi8(vHM, _mm256_load_si256(vPM + i)),
                     case1);
             _mm256_store_si256(pvHMStore + i, vHM);
 
             /* calculate vS */
             vHS = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vES, vFS, case2),
-                    _mm256_add_epi64(vHS, _mm256_load_si256(vPS + i)),
+                    _mm256_adds_epi8(vHS, _mm256_load_si256(vPS + i)),
                     case1);
             _mm256_store_si256(pvHSStore + i, vHS);
 
             /* calculate vL */
             vHL = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vEL, vFL, case2),
-                    _mm256_add_epi64(vHL, vOne),
+                    _mm256_adds_epi8(vHL, vOne),
                     case1);
             _mm256_store_si256(pvHLStore + i, vHL);
 
-            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si256(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
             arr_store_si256(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
             arr_store_si256(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
             arr_store_si256(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-            vEF_opn = _mm256_sub_epi64(vH, vGapO);
+            vEF_opn = _mm256_subs_epi8(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm256_sub_epi64(vE, vGapE);
-            vE = _mm256_max_epi64_rpl(vEF_opn, vE_ext);
-            case1 = _mm256_cmpgt_epi64(vEF_opn, vE_ext);
+            vE_ext = _mm256_subs_epi8(vE, vGapE);
+            vE = _mm256_max_epi8(vEF_opn, vE_ext);
+            case1 = _mm256_cmpgt_epi8(vEF_opn, vE_ext);
             vEM = _mm256_blendv_epi8(vEM, vHM, case1);
             vES = _mm256_blendv_epi8(vES, vHS, case1);
             vEL = _mm256_blendv_epi8(
-                    _mm256_add_epi64(vEL, vOne),
-                    _mm256_add_epi64(vHL, vOne),
+                    _mm256_adds_epi8(vEL, vOne),
+                    _mm256_adds_epi8(vHL, vOne),
                     case1);
             _mm256_store_si256(pvE + i, vE);
             _mm256_store_si256(pvEM + i, vEM);
@@ -458,14 +477,14 @@ parasail_result_t* PNAME(
             _mm256_store_si256(pvEL + i, vEL);
 
             /* Update vF value. */
-            vF_ext = _mm256_sub_epi64(vF, vGapE);
-            vF = _mm256_max_epi64_rpl(vEF_opn, vF_ext);
-            case1 = _mm256_cmpgt_epi64(vEF_opn, vF_ext);
+            vF_ext = _mm256_subs_epi8(vF, vGapE);
+            vF = _mm256_max_epi8(vEF_opn, vF_ext);
+            case1 = _mm256_cmpgt_epi8(vEF_opn, vF_ext);
             vFM = _mm256_blendv_epi8(vFM, vHM, case1);
             vFS = _mm256_blendv_epi8(vFS, vHS, case1);
             vFL = _mm256_blendv_epi8(
-                    _mm256_add_epi64(vFL, vOne),
-                    _mm256_add_epi64(vHL, vOne),
+                    _mm256_adds_epi8(vFL, vOne),
+                    _mm256_adds_epi8(vHL, vOne),
                     case1);
 
             /* Load the next vH. */
@@ -480,26 +499,26 @@ parasail_result_t* PNAME(
         for (k=0; k<segWidth; ++k) {
             __m256i vHp = _mm256_load_si256(&pvHLoad[segLen - 1]);
             int64_t tmp = boundary[j+1]-open;
-            int64_t tmp2 = tmp < INT64_MIN ? INT64_MIN : tmp;
-            vHp = _mm256_slli_si256_rpl(vHp, 8);
-            vF = _mm256_slli_si256_rpl(vF, 8);
-            vFM = _mm256_slli_si256_rpl(vFM, 8);
-            vFS = _mm256_slli_si256_rpl(vFS, 8);
-            vFL = _mm256_slli_si256_rpl(vFL, 8);
-            vHp = _mm256_insert_epi64_rpl(vHp, boundary[j], 0);
-            vF = _mm256_insert_epi64_rpl(vF, tmp2, 0);
-            vFL = _mm256_insert_epi64_rpl(vFL, 1, 0);
+            int8_t tmp2 = tmp < INT8_MIN ? INT8_MIN : tmp;
+            vHp = _mm256_slli_si256_rpl(vHp, 1);
+            vF = _mm256_slli_si256_rpl(vF, 1);
+            vFM = _mm256_slli_si256_rpl(vFM, 1);
+            vFS = _mm256_slli_si256_rpl(vFS, 1);
+            vFL = _mm256_slli_si256_rpl(vFL, 1);
+            vHp = _mm256_insert_epi8_rpl(vHp, boundary[j], 0);
+            vF = _mm256_insert_epi8_rpl(vF, tmp2, 0);
+            vFL = _mm256_insert_epi8_rpl(vFL, 1, 0);
             for (i=0; i<segLen; ++i) {
                 __m256i case1;
                 __m256i case2;
                 __m256i cond;
 
-                vHp = _mm256_add_epi64(vHp, _mm256_load_si256(vP + i));
+                vHp = _mm256_adds_epi8(vHp, _mm256_load_si256(vP + i));
                 vH = _mm256_load_si256(pvHStore + i);
-                vH = _mm256_max_epi64_rpl(vH,vF);
+                vH = _mm256_max_epi8(vH,vF);
                 _mm256_store_si256(pvHStore + i, vH);
-                case1 = _mm256_cmpeq_epi64(vH, vHp);
-                case2 = _mm256_cmpeq_epi64(vH, vF);
+                case1 = _mm256_cmpeq_epi8(vH, vHp);
+                case2 = _mm256_cmpeq_epi8(vH, vF);
                 cond = _mm256_andnot_si256(case1, case2);
 
                 /* calculate vM */
@@ -517,11 +536,11 @@ parasail_result_t* PNAME(
                 vHL = _mm256_blendv_epi8(vHL, vFL, cond);
                 _mm256_store_si256(pvHLStore + i, vHL);
 
-                vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vH);
-                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHM);
-                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHS);
-                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHL);
+                vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vH);
+                vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHM);
+                vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHS);
+                vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
                 arr_store_si256(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
                 arr_store_si256(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
@@ -529,21 +548,21 @@ parasail_result_t* PNAME(
                 arr_store_si256(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
                 /* Update vF value. */
-                vEF_opn = _mm256_sub_epi64(vH, vGapO);
-                vF_ext = _mm256_sub_epi64(vF, vGapE);
+                vEF_opn = _mm256_subs_epi8(vH, vGapO);
+                vF_ext = _mm256_subs_epi8(vF, vGapE);
                 if (! _mm256_movemask_epi8(
                             _mm256_or_si256(
-                                _mm256_cmpgt_epi64(vF_ext, vEF_opn),
-                                _mm256_cmpeq_epi64(vF_ext, vEF_opn))))
+                                _mm256_cmpgt_epi8(vF_ext, vEF_opn),
+                                _mm256_cmpeq_epi8(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = _mm256_max_epi64_rpl(vEF_opn, vF_ext);*/
+                /*vF = _mm256_max_epi8(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                cond = _mm256_cmpgt_epi64(vEF_opn, vF_ext);
+                cond = _mm256_cmpgt_epi8(vEF_opn, vF_ext);
                 vFM = _mm256_blendv_epi8(vFM, vHM, cond);
                 vFS = _mm256_blendv_epi8(vFS, vHS, cond);
                 vFL = _mm256_blendv_epi8(
-                        _mm256_add_epi64(vFL, vOne),
-                        _mm256_add_epi64(vHL, vOne),
+                        _mm256_adds_epi8(vFL, vOne),
+                        _mm256_adds_epi8(vHL, vOne),
                         cond);
                 vHp = _mm256_load_si256(pvHLoad + i);
             }
@@ -560,15 +579,15 @@ end:
             vHS = _mm256_load_si256(pvHSStore + offset);
             vHL = _mm256_load_si256(pvHLStore + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm256_slli_si256_rpl (vH, 8);
-                vHM = _mm256_slli_si256_rpl (vHM, 8);
-                vHS = _mm256_slli_si256_rpl (vHS, 8);
-                vHL = _mm256_slli_si256_rpl (vHL, 8);
+                vH = _mm256_slli_si256_rpl (vH, 1);
+                vHM = _mm256_slli_si256_rpl (vHM, 1);
+                vHS = _mm256_slli_si256_rpl (vHS, 1);
+                vHL = _mm256_slli_si256_rpl (vHL, 1);
             }
-            result->stats->rowcols->score_row[j] = (int64_t) _mm256_extract_epi64_rpl (vH, 3);
-            result->stats->rowcols->matches_row[j] = (int64_t) _mm256_extract_epi64_rpl (vHM, 3);
-            result->stats->rowcols->similar_row[j] = (int64_t) _mm256_extract_epi64_rpl (vHS, 3);
-            result->stats->rowcols->length_row[j] = (int64_t) _mm256_extract_epi64_rpl (vHL, 3);
+            result->stats->rowcols->score_row[j] = (int8_t) _mm256_extract_epi8_rpl (vH, 31);
+            result->stats->rowcols->matches_row[j] = (int8_t) _mm256_extract_epi8_rpl (vHM, 31);
+            result->stats->rowcols->similar_row[j] = (int8_t) _mm256_extract_epi8_rpl (vHS, 31);
+            result->stats->rowcols->length_row[j] = (int8_t) _mm256_extract_epi8_rpl (vHL, 31);
         }
 #endif
     }
@@ -593,20 +612,20 @@ end:
         __m256i vHS = _mm256_load_si256(pvHSStore + offset);
         __m256i vHL = _mm256_load_si256(pvHLStore + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm256_slli_si256_rpl (vH, 8);
-            vHM = _mm256_slli_si256_rpl (vHM, 8);
-            vHS = _mm256_slli_si256_rpl (vHS, 8);
-            vHL = _mm256_slli_si256_rpl (vHL, 8);
+            vH = _mm256_slli_si256_rpl (vH, 1);
+            vHM = _mm256_slli_si256_rpl (vHM, 1);
+            vHS = _mm256_slli_si256_rpl (vHS, 1);
+            vHL = _mm256_slli_si256_rpl (vHL, 1);
         }
-        score = (int64_t) _mm256_extract_epi64_rpl (vH, 3);
-        matches = (int64_t) _mm256_extract_epi64_rpl (vHM, 3);
-        similar = (int64_t) _mm256_extract_epi64_rpl (vHS, 3);
-        length = (int64_t) _mm256_extract_epi64_rpl (vHL, 3);
+        score = (int8_t) _mm256_extract_epi8_rpl (vH, 31);
+        matches = (int8_t) _mm256_extract_epi8_rpl (vHM, 31);
+        similar = (int8_t) _mm256_extract_epi8_rpl (vHS, 31);
+        length = (int8_t) _mm256_extract_epi8_rpl (vHL, 31);
     }
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi8_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
