@@ -29,10 +29,8 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 3);
+    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)simde_mm_extract_epi64(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)simde_mm_extract_epi64(vH, 1);
 }
 #endif
 
@@ -43,23 +41,21 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 0);
-    col[1*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 1);
-    col[2*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 2);
-    col[3*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 3);
+    col[0*seglen+t] = (int64_t)simde_mm_extract_epi64(vH, 0);
+    col[1*seglen+t] = (int64_t)simde_mm_extract_epi64(vH, 1);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_nw_stats_table_striped_neon_128_32
-#define PNAME parasail_nw_stats_table_striped_profile_neon_128_32
+#define FNAME parasail_nw_stats_table_striped_neon_128_64
+#define PNAME parasail_nw_stats_table_striped_profile_neon_128_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_nw_stats_rowcol_striped_neon_128_32
-#define PNAME parasail_nw_stats_rowcol_striped_profile_neon_128_32
+#define FNAME parasail_nw_stats_rowcol_striped_neon_128_64
+#define PNAME parasail_nw_stats_rowcol_striped_profile_neon_128_64
 #else
-#define FNAME parasail_nw_stats_striped_neon_128_32
-#define PNAME parasail_nw_stats_striped_profile_neon_128_32
+#define FNAME parasail_nw_stats_striped_neon_128_64
+#define PNAME parasail_nw_stats_striped_profile_neon_128_64
 #endif
 #endif
 
@@ -87,7 +83,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_neon_128_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_neon_128_64(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -128,17 +124,17 @@ parasail_result_t* PNAME(
     simde__m128i* restrict pvEM = NULL;
     simde__m128i* restrict pvES = NULL;
     simde__m128i* restrict pvEL = NULL;
-    int32_t* restrict boundary = NULL;
+    int64_t* restrict boundary = NULL;
     simde__m128i vGapO;
     simde__m128i vGapE;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
     simde__m128i vZero;
     simde__m128i vOne;
-    int32_t score = 0;
-    int32_t matches = 0;
-    int32_t similar = 0;
-    int32_t length = 0;
+    int64_t score = 0;
+    int64_t matches = 0;
+    int64_t similar = 0;
+    int64_t length = 0;
     simde__m128i vNegLimit;
     simde__m128i vPosLimit;
     simde__m128i vSaturationCheckMin;
@@ -147,7 +143,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -163,25 +159,25 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 4; /* number of values in vector unit */
+    segWidth = 2; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile  = (simde__m128i*)profile->profile32.score;
-    vProfileM = (simde__m128i*)profile->profile32.matches;
-    vProfileS = (simde__m128i*)profile->profile32.similar;
-    vGapO = simde_mm_set1_epi32(open);
-    vGapE = simde_mm_set1_epi32(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    vProfile  = (simde__m128i*)profile->profile64.score;
+    vProfileM = (simde__m128i*)profile->profile64.matches;
+    vProfileS = (simde__m128i*)profile->profile64.similar;
+    vGapO = simde_mm_set1_epi64x(open);
+    vGapE = simde_mm_set1_epi64x(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     vZero = simde_mm_setzero_si128();
-    vOne = simde_mm_set1_epi32(1);
+    vOne = simde_mm_set1_epi64x(1);
     score = NEG_LIMIT;
     matches = 0;
     similar = 0;
     length = 0;
-    vNegLimit = simde_mm_set1_epi32(NEG_LIMIT);
-    vPosLimit = simde_mm_set1_epi32(POS_LIMIT);
+    vNegLimit = simde_mm_set1_epi64x(NEG_LIMIT);
+    vPosLimit = simde_mm_set1_epi64x(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
 
@@ -200,7 +196,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -221,7 +217,7 @@ parasail_result_t* PNAME(
     pvEM      = parasail_memalign_simde__m128i(16, segLen);
     pvES      = parasail_memalign_simde__m128i(16, segLen);
     pvEL      = parasail_memalign_simde__m128i(16, segLen);
-    boundary  = parasail_memalign_int32_t(16, s2Len+1);
+    boundary  = parasail_memalign_int64_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -254,9 +250,9 @@ parasail_result_t* PNAME(
             simde__m128i e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = -open-gap*(segNum*segLen+i);
-                h.i32[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                h.i64[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
                 tmp = tmp - open;
-                e.i32[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                e.i64[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
             }
             simde_mm_store_si128(&pvHStore[index], h);
             simde_mm_store_si128(&pvE[index], e);
@@ -269,7 +265,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = -open-gap*(i-1);
-            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
+            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
         }
     }
 
@@ -302,18 +298,18 @@ parasail_result_t* PNAME(
         vFS = vZero;
         vFL = vOne;
 
-        /* load final segment of pvHStore and shift left by 4 bytes */
+        /* load final segment of pvHStore and shift left by 8 bytes */
         vH = simde_mm_load_si128(&pvHStore[segLen - 1]);
         vHM = simde_mm_load_si128(&pvHMStore[segLen - 1]);
         vHS = simde_mm_load_si128(&pvHSStore[segLen - 1]);
         vHL = simde_mm_load_si128(&pvHLStore[segLen - 1]);
-        vH = simde_mm_slli_si128(vH, 4);
-        vHM = simde_mm_slli_si128(vHM, 4);
-        vHS = simde_mm_slli_si128(vHS, 4);
-        vHL = simde_mm_slli_si128(vHL, 4);
+        vH = simde_mm_slli_si128(vH, 8);
+        vHM = simde_mm_slli_si128(vHM, 8);
+        vHS = simde_mm_slli_si128(vHS, 8);
+        vHL = simde_mm_slli_si128(vHL, 8);
 
         /* insert upper boundary condition */
-        vH = simde_mm_insert_epi32(vH, boundary[j], 0);
+        vH = simde_mm_insert_epi64(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -337,58 +333,58 @@ parasail_result_t* PNAME(
             vEL = simde_mm_load_si128(pvEL+ i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = simde_mm_add_epi32(vH, simde_mm_load_si128(vP + i));
-            vH = simde_mm_max_epi32(vH_dag, vE);
-            vH = simde_mm_max_epi32(vH, vF);
+            vH_dag = simde_mm_add_epi64(vH, simde_mm_load_si128(vP + i));
+            vH = simde_mm_max_epi64(vH_dag, vE);
+            vH = simde_mm_max_epi64(vH, vF);
             /* Save vH values. */
             simde_mm_store_si128(pvHStore + i, vH);
 
-            case1 = simde_mm_cmpeq_epi32(vH, vH_dag);
-            case2 = simde_mm_cmpeq_epi32(vH, vF);
+            case1 = simde_mm_cmpeq_epi64(vH, vH_dag);
+            case2 = simde_mm_cmpeq_epi64(vH, vF);
 
             /* calculate vM */
             vHM = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vEM, vFM, case2),
-                    simde_mm_add_epi32(vHM, simde_mm_load_si128(vPM + i)),
+                    simde_mm_add_epi64(vHM, simde_mm_load_si128(vPM + i)),
                     case1);
             simde_mm_store_si128(pvHMStore + i, vHM);
 
             /* calculate vS */
             vHS = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vES, vFS, case2),
-                    simde_mm_add_epi32(vHS, simde_mm_load_si128(vPS + i)),
+                    simde_mm_add_epi64(vHS, simde_mm_load_si128(vPS + i)),
                     case1);
             simde_mm_store_si128(pvHSStore + i, vHS);
 
             /* calculate vL */
             vHL = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vEL, vFL, case2),
-                    simde_mm_add_epi32(vHL, vOne),
+                    simde_mm_add_epi64(vHL, vOne),
                     case1);
             simde_mm_store_si128(pvHLStore + i, vHL);
 
-            vSaturationCheckMin = simde_mm_min_epi32(vSaturationCheckMin, vH);
-            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vH);
-            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vH);
+            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vH);
+            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-            vEF_opn = simde_mm_sub_epi32(vH, vGapO);
+            vEF_opn = simde_mm_sub_epi64(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = simde_mm_sub_epi32(vE, vGapE);
-            vE = simde_mm_max_epi32(vEF_opn, vE_ext);
-            case1 = simde_mm_cmpgt_epi32(vEF_opn, vE_ext);
+            vE_ext = simde_mm_sub_epi64(vE, vGapE);
+            vE = simde_mm_max_epi64(vEF_opn, vE_ext);
+            case1 = simde_mm_cmpgt_epi64(vEF_opn, vE_ext);
             vEM = simde_mm_blendv_epi8(vEM, vHM, case1);
             vES = simde_mm_blendv_epi8(vES, vHS, case1);
             vEL = simde_mm_blendv_epi8(
-                    simde_mm_add_epi32(vEL, vOne),
-                    simde_mm_add_epi32(vHL, vOne),
+                    simde_mm_add_epi64(vEL, vOne),
+                    simde_mm_add_epi64(vHL, vOne),
                     case1);
             simde_mm_store_si128(pvE + i, vE);
             simde_mm_store_si128(pvEM + i, vEM);
@@ -396,14 +392,14 @@ parasail_result_t* PNAME(
             simde_mm_store_si128(pvEL + i, vEL);
 
             /* Update vF value. */
-            vF_ext = simde_mm_sub_epi32(vF, vGapE);
-            vF = simde_mm_max_epi32(vEF_opn, vF_ext);
-            case1 = simde_mm_cmpgt_epi32(vEF_opn, vF_ext);
+            vF_ext = simde_mm_sub_epi64(vF, vGapE);
+            vF = simde_mm_max_epi64(vEF_opn, vF_ext);
+            case1 = simde_mm_cmpgt_epi64(vEF_opn, vF_ext);
             vFM = simde_mm_blendv_epi8(vFM, vHM, case1);
             vFS = simde_mm_blendv_epi8(vFS, vHS, case1);
             vFL = simde_mm_blendv_epi8(
-                    simde_mm_add_epi32(vFL, vOne),
-                    simde_mm_add_epi32(vHL, vOne),
+                    simde_mm_add_epi64(vFL, vOne),
+                    simde_mm_add_epi64(vHL, vOne),
                     case1);
 
             /* Load the next vH. */
@@ -418,26 +414,26 @@ parasail_result_t* PNAME(
         for (k=0; k<segWidth; ++k) {
             simde__m128i vHp = simde_mm_load_si128(&pvHLoad[segLen - 1]);
             int64_t tmp = boundary[j+1]-open;
-            int32_t tmp2 = tmp < INT32_MIN ? INT32_MIN : tmp;
-            vHp = simde_mm_slli_si128(vHp, 4);
-            vF = simde_mm_slli_si128(vF, 4);
-            vFM = simde_mm_slli_si128(vFM, 4);
-            vFS = simde_mm_slli_si128(vFS, 4);
-            vFL = simde_mm_slli_si128(vFL, 4);
-            vHp = simde_mm_insert_epi32(vHp, boundary[j], 0);
-            vF = simde_mm_insert_epi32(vF, tmp2, 0);
-            vFL = simde_mm_insert_epi32(vFL, 1, 0);
+            int64_t tmp2 = tmp < INT64_MIN ? INT64_MIN : tmp;
+            vHp = simde_mm_slli_si128(vHp, 8);
+            vF = simde_mm_slli_si128(vF, 8);
+            vFM = simde_mm_slli_si128(vFM, 8);
+            vFS = simde_mm_slli_si128(vFS, 8);
+            vFL = simde_mm_slli_si128(vFL, 8);
+            vHp = simde_mm_insert_epi64(vHp, boundary[j], 0);
+            vF = simde_mm_insert_epi64(vF, tmp2, 0);
+            vFL = simde_mm_insert_epi64(vFL, 1, 0);
             for (i=0; i<segLen; ++i) {
                 simde__m128i case1;
                 simde__m128i case2;
                 simde__m128i cond;
 
-                vHp = simde_mm_add_epi32(vHp, simde_mm_load_si128(vP + i));
+                vHp = simde_mm_add_epi64(vHp, simde_mm_load_si128(vP + i));
                 vH = simde_mm_load_si128(pvHStore + i);
-                vH = simde_mm_max_epi32(vH,vF);
+                vH = simde_mm_max_epi64(vH,vF);
                 simde_mm_store_si128(pvHStore + i, vH);
-                case1 = simde_mm_cmpeq_epi32(vH, vHp);
-                case2 = simde_mm_cmpeq_epi32(vH, vF);
+                case1 = simde_mm_cmpeq_epi64(vH, vHp);
+                case2 = simde_mm_cmpeq_epi64(vH, vF);
                 cond = simde_mm_andnot_si128(case1, case2);
 
                 /* calculate vM */
@@ -455,11 +451,11 @@ parasail_result_t* PNAME(
                 vHL = simde_mm_blendv_epi8(vHL, vFL, cond);
                 simde_mm_store_si128(pvHLStore + i, vHL);
 
-                vSaturationCheckMin = simde_mm_min_epi32(vSaturationCheckMin, vH);
-                vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vH);
-                vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHM);
-                vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHS);
-                vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHL);
+                vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vH);
+                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vH);
+                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHM);
+                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHS);
+                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
                 arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
                 arr_store_si128(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
@@ -467,21 +463,21 @@ parasail_result_t* PNAME(
                 arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
                 /* Update vF value. */
-                vEF_opn = simde_mm_sub_epi32(vH, vGapO);
-                vF_ext = simde_mm_sub_epi32(vF, vGapE);
+                vEF_opn = simde_mm_sub_epi64(vH, vGapO);
+                vF_ext = simde_mm_sub_epi64(vF, vGapE);
                 if (! simde_mm_movemask_epi8(
                             simde_mm_or_si128(
-                                simde_mm_cmpgt_epi32(vF_ext, vEF_opn),
-                                simde_mm_cmpeq_epi32(vF_ext, vEF_opn))))
+                                simde_mm_cmpgt_epi64(vF_ext, vEF_opn),
+                                simde_mm_cmpeq_epi64(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = simde_mm_max_epi32(vEF_opn, vF_ext);*/
+                /*vF = simde_mm_max_epi64(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                cond = simde_mm_cmpgt_epi32(vEF_opn, vF_ext);
+                cond = simde_mm_cmpgt_epi64(vEF_opn, vF_ext);
                 vFM = simde_mm_blendv_epi8(vFM, vHM, cond);
                 vFS = simde_mm_blendv_epi8(vFS, vHS, cond);
                 vFL = simde_mm_blendv_epi8(
-                        simde_mm_add_epi32(vFL, vOne),
-                        simde_mm_add_epi32(vHL, vOne),
+                        simde_mm_add_epi64(vFL, vOne),
+                        simde_mm_add_epi64(vHL, vOne),
                         cond);
                 vHp = simde_mm_load_si128(pvHLoad + i);
             }
@@ -498,15 +494,15 @@ end:
             vHS = simde_mm_load_si128(pvHSStore + offset);
             vHL = simde_mm_load_si128(pvHLStore + offset);
             for (k=0; k<position; ++k) {
-                vH = simde_mm_slli_si128 (vH, 4);
-                vHM = simde_mm_slli_si128 (vHM, 4);
-                vHS = simde_mm_slli_si128 (vHS, 4);
-                vHL = simde_mm_slli_si128 (vHL, 4);
+                vH = simde_mm_slli_si128 (vH, 8);
+                vHM = simde_mm_slli_si128 (vHM, 8);
+                vHS = simde_mm_slli_si128 (vHS, 8);
+                vHL = simde_mm_slli_si128 (vHL, 8);
             }
-            result->stats->rowcols->score_row[j] = (int32_t) simde_mm_extract_epi32 (vH, 3);
-            result->stats->rowcols->matches_row[j] = (int32_t) simde_mm_extract_epi32 (vHM, 3);
-            result->stats->rowcols->similar_row[j] = (int32_t) simde_mm_extract_epi32 (vHS, 3);
-            result->stats->rowcols->length_row[j] = (int32_t) simde_mm_extract_epi32 (vHL, 3);
+            result->stats->rowcols->score_row[j] = (int64_t) simde_mm_extract_epi64 (vH, 1);
+            result->stats->rowcols->matches_row[j] = (int64_t) simde_mm_extract_epi64 (vHM, 1);
+            result->stats->rowcols->similar_row[j] = (int64_t) simde_mm_extract_epi64 (vHS, 1);
+            result->stats->rowcols->length_row[j] = (int64_t) simde_mm_extract_epi64 (vHL, 1);
         }
 #endif
     }
@@ -531,20 +527,20 @@ end:
         simde__m128i vHS = simde_mm_load_si128(pvHSStore + offset);
         simde__m128i vHL = simde_mm_load_si128(pvHLStore + offset);
         for (k=0; k<position; ++k) {
-            vH = simde_mm_slli_si128 (vH, 4);
-            vHM = simde_mm_slli_si128 (vHM, 4);
-            vHS = simde_mm_slli_si128 (vHS, 4);
-            vHL = simde_mm_slli_si128 (vHL, 4);
+            vH = simde_mm_slli_si128 (vH, 8);
+            vHM = simde_mm_slli_si128 (vHM, 8);
+            vHS = simde_mm_slli_si128 (vHS, 8);
+            vHL = simde_mm_slli_si128 (vHL, 8);
         }
-        score = (int32_t) simde_mm_extract_epi32 (vH, 3);
-        matches = (int32_t) simde_mm_extract_epi32 (vHM, 3);
-        similar = (int32_t) simde_mm_extract_epi32 (vHS, 3);
-        length = (int32_t) simde_mm_extract_epi32 (vHL, 3);
+        score = (int64_t) simde_mm_extract_epi64 (vH, 1);
+        matches = (int64_t) simde_mm_extract_epi64 (vHM, 1);
+        similar = (int64_t) simde_mm_extract_epi64 (vHS, 1);
+        length = (int64_t) simde_mm_extract_epi64 (vHL, 1);
     }
 
     if (simde_mm_movemask_epi8(simde_mm_or_si128(
-            simde_mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
-            simde_mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            simde_mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
+            simde_mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
