@@ -27,10 +27,22 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 3);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  0);
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  1);
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  2);
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  3);
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  4);
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  5);
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  6);
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  7);
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  8);
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  9);
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 10);
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 11);
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 12);
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 13);
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 14);
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 15);
 }
 #endif
 
@@ -41,23 +53,35 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int32_t)_mm_extract_epi32(vH, 0);
-    col[1*seglen+t] = (int32_t)_mm_extract_epi32(vH, 1);
-    col[2*seglen+t] = (int32_t)_mm_extract_epi32(vH, 2);
-    col[3*seglen+t] = (int32_t)_mm_extract_epi32(vH, 3);
+    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8(vH,  0);
+    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8(vH,  1);
+    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8(vH,  2);
+    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8(vH,  3);
+    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8(vH,  4);
+    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8(vH,  5);
+    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8(vH,  6);
+    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8(vH,  7);
+    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8(vH,  8);
+    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8(vH,  9);
+    col[10*seglen+t] = (int8_t)_mm_extract_epi8(vH, 10);
+    col[11*seglen+t] = (int8_t)_mm_extract_epi8(vH, 11);
+    col[12*seglen+t] = (int8_t)_mm_extract_epi8(vH, 12);
+    col[13*seglen+t] = (int8_t)_mm_extract_epi8(vH, 13);
+    col[14*seglen+t] = (int8_t)_mm_extract_epi8(vH, 14);
+    col[15*seglen+t] = (int8_t)_mm_extract_epi8(vH, 15);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_nw_table_striped_altivec_128_32
-#define PNAME parasail_nw_table_striped_profile_altivec_128_32
+#define FNAME parasail_nw_table_striped_altivec_128_8
+#define PNAME parasail_nw_table_striped_profile_altivec_128_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_nw_rowcol_striped_altivec_128_32
-#define PNAME parasail_nw_rowcol_striped_profile_altivec_128_32
+#define FNAME parasail_nw_rowcol_striped_altivec_128_8
+#define PNAME parasail_nw_rowcol_striped_profile_altivec_128_8
 #else
-#define FNAME parasail_nw_striped_altivec_128_32
-#define PNAME parasail_nw_striped_profile_altivec_128_32
+#define FNAME parasail_nw_striped_altivec_128_8
+#define PNAME parasail_nw_striped_profile_altivec_128_8
 #endif
 #endif
 
@@ -82,7 +106,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_altivec_128_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_altivec_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -112,12 +136,12 @@ parasail_result_t* PNAME(
     vec128i* restrict pvHStore = NULL;
     vec128i* restrict pvHLoad = NULL;
     vec128i* restrict pvE = NULL;
-    int32_t* restrict boundary = NULL;
+    int8_t* restrict boundary = NULL;
     vec128i vGapO;
     vec128i vGapE;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
-    int32_t score = 0;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
+    int8_t score = 0;
     vec128i vNegLimit;
     vec128i vPosLimit;
     vec128i vSaturationCheckMin;
@@ -126,7 +150,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -142,18 +166,18 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 4; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile = (vec128i*)profile->profile32.score;
-    vGapO = _mm_set1_epi32(open);
-    vGapE = _mm_set1_epi32(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    vProfile = (vec128i*)profile->profile8.score;
+    vGapO = _mm_set1_epi8(open);
+    vGapE = _mm_set1_epi8(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm_set1_epi8(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi8(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
 
@@ -171,7 +195,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_STRIPED
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -183,7 +207,7 @@ parasail_result_t* PNAME(
     pvHStore = parasail_memalign_vec128i(16, segLen);
     pvHLoad  = parasail_memalign_vec128i(16, segLen);
     pvE      = parasail_memalign_vec128i(16, segLen);
-    boundary = parasail_memalign_int32_t(16, s2Len+1);
+    boundary = parasail_memalign_int8_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -196,13 +220,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            vec128i_32_t h;
-            vec128i_32_t e;
+            vec128i_8_t h;
+            vec128i_8_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = -open-gap*(segNum*segLen+i);
-                h.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                h.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                e.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
             }
             _mm_store_si128(&pvHStore[index], h.m);
             _mm_store_si128(&pvE[index], e.m);
@@ -215,7 +239,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = -open-gap*(i-1);
-            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
+            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
         }
     }
 
@@ -227,7 +251,7 @@ parasail_result_t* PNAME(
         vec128i vF = vNegLimit;
 
         /* load final segment of pvHStore and shift left by 2 bytes */
-        vec128i vH = _mm_slli_si128(pvHStore[segLen - 1], 4);
+        vec128i vH = _mm_slli_si128(pvHStore[segLen - 1], 1);
 
         /* Correct part of the vProfile */
         const vec128i* vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -238,35 +262,35 @@ parasail_result_t* PNAME(
         pvHStore = pv;
 
         /* insert upper boundary condition */
-        vH = _mm_insert_epi32(vH, boundary[j], 0);
+        vH = _mm_insert_epi8(vH, boundary[j], 0);
 
         /* inner loop to process the query sequence */
         for (i=0; i<segLen; ++i) {
-            vH = _mm_add_epi32(vH, _mm_load_si128(vP + i));
+            vH = _mm_adds_epi8(vH, _mm_load_si128(vP + i));
             vE = _mm_load_si128(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH = _mm_max_epi32(vH, vE);
-            vH = _mm_max_epi32(vH, vF);
+            vH = _mm_max_epi8(vH, vE);
+            vH = _mm_max_epi8(vH, vF);
             /* Save vH values. */
             _mm_store_si128(pvHStore + i, vH);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vE);
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vE);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vF);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
 
             /* Update vE value. */
-            vH = _mm_sub_epi32(vH, vGapO);
-            vE = _mm_sub_epi32(vE, vGapE);
-            vE = _mm_max_epi32(vE, vH);
+            vH = _mm_subs_epi8(vH, vGapO);
+            vE = _mm_subs_epi8(vE, vGapE);
+            vE = _mm_max_epi8(vE, vH);
             _mm_store_si128(pvE + i, vE);
 
             /* Update vF value. */
-            vF = _mm_sub_epi32(vF, vGapE);
-            vF = _mm_max_epi32(vF, vH);
+            vF = _mm_subs_epi8(vF, vGapE);
+            vF = _mm_max_epi8(vF, vH);
 
             /* Load the next vH. */
             vH = _mm_load_si128(pvHLoad + i);
@@ -276,22 +300,22 @@ parasail_result_t* PNAME(
          * then deletion, so don't update E(i, i), learn from SWPS3 */
         for (k=0; k<segWidth; ++k) {
             int64_t tmp = boundary[j+1]-open;
-            int32_t tmp2 = tmp < INT32_MIN ? INT32_MIN : tmp;
-            vF = _mm_slli_si128(vF, 4);
-            vF = _mm_insert_epi32(vF, tmp2, 0);
+            int8_t tmp2 = tmp < INT8_MIN ? INT8_MIN : tmp;
+            vF = _mm_slli_si128(vF, 1);
+            vF = _mm_insert_epi8(vF, tmp2, 0);
             for (i=0; i<segLen; ++i) {
                 vH = _mm_load_si128(pvHStore + i);
-                vH = _mm_max_epi32(vH,vF);
+                vH = _mm_max_epi8(vH,vF);
                 _mm_store_si128(pvHStore + i, vH);
-                vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
+                vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vH);
 #ifdef PARASAIL_TABLE
                 arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-                vH = _mm_sub_epi32(vH, vGapO);
-                vF = _mm_sub_epi32(vF, vGapE);
-                if (! _mm_movemask_epi8(_mm_cmpgt_epi32(vF, vH))) goto end;
-                /*vF = _mm_max_epi32(vF, vH);*/
+                vH = _mm_subs_epi8(vH, vGapO);
+                vF = _mm_subs_epi8(vF, vGapE);
+                if (! _mm_movemask_epi8(_mm_cmpgt_epi8(vF, vH))) goto end;
+                /*vF = _mm_max_epi8(vF, vH);*/
             }
         }
 end:
@@ -303,9 +327,9 @@ end:
         {
             vH = _mm_load_si128(pvHStore + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 4);
+                vH = _mm_slli_si128(vH, 1);
             }
-            result->rowcols->score_row[j] = (int32_t) _mm_extract_epi32 (vH, 3);
+            result->rowcols->score_row[j] = (int8_t) _mm_extract_epi8 (vH, 15);
         }
 #endif
     }
@@ -321,14 +345,14 @@ end:
     {
         vec128i vH = _mm_load_si128(pvHStore + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm_slli_si128 (vH, 4);
+            vH = _mm_slli_si128 (vH, 1);
         }
-        score = (int32_t) _mm_extract_epi32 (vH, 3);
+        score = (int8_t) _mm_extract_epi8 (vH, 15);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
