@@ -17,28 +17,65 @@
 #include "parasail/internal_avx.h"
 
 
-#if HAVE_AVX2_MM256_INSERT_EPI32
-#define _mm256_insert_epi32_rpl _mm256_insert_epi32
+#if HAVE_AVX2_MM256_INSERT_EPI64
+#define _mm256_insert_epi64_rpl _mm256_insert_epi64
 #else
-static inline __m256i _mm256_insert_epi32_rpl(__m256i a, int32_t i, int imm) {
-    __m256i_32_t A;
+static inline __m256i _mm256_insert_epi64_rpl(__m256i a, int64_t i, int imm) {
+    __m256i_64_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_EXTRACT_EPI32
-#define _mm256_extract_epi32_rpl _mm256_extract_epi32
+#if HAVE_AVX2_MM256_SET1_EPI64X
+#define _mm256_set1_epi64x_rpl _mm256_set1_epi64x
 #else
-static inline int32_t _mm256_extract_epi32_rpl(__m256i a, int imm) {
-    __m256i_32_t A;
+static inline __m256i _mm256_set1_epi64x_rpl(int64_t i) {
+    __m256i_64_t A;
+    A.v[0] = i;
+    A.v[1] = i;
+    A.v[2] = i;
+    A.v[3] = i;
+    return A.m;
+}
+#endif
+
+static inline __m256i _mm256_max_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]>B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]>B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
+#if HAVE_AVX2_MM256_EXTRACT_EPI64
+#define _mm256_extract_epi64_rpl _mm256_extract_epi64
+#else
+static inline int64_t _mm256_extract_epi64_rpl(__m256i a, int imm) {
+    __m256i_64_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
-#define _mm256_cmplt_epi32_rpl(a,b) _mm256_cmpgt_epi32(b,a)
+static inline __m256i _mm256_min_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]<B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]<B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
+#define _mm256_cmplt_epi64_rpl(a,b) _mm256_cmpgt_epi64(b,a)
 
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
@@ -52,14 +89,10 @@ static inline void arr_store_si256(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 3);
-    array[1LL*(4*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 4);
-    array[1LL*(5*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 5);
-    array[1LL*(6*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 6);
-    array[1LL*(7*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 7);
+    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 3);
 }
 #endif
 
@@ -70,27 +103,23 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 0);
-    col[1*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 1);
-    col[2*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 2);
-    col[3*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 3);
-    col[4*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 4);
-    col[5*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 5);
-    col[6*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 6);
-    col[7*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 7);
+    col[0*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 0);
+    col[1*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 1);
+    col[2*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 2);
+    col[3*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 3);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_nw_table_striped_avx2_256_32
-#define PNAME parasail_nw_table_striped_profile_avx2_256_32
+#define FNAME parasail_nw_table_striped_avx2_256_64
+#define PNAME parasail_nw_table_striped_profile_avx2_256_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_nw_rowcol_striped_avx2_256_32
-#define PNAME parasail_nw_rowcol_striped_profile_avx2_256_32
+#define FNAME parasail_nw_rowcol_striped_avx2_256_64
+#define PNAME parasail_nw_rowcol_striped_profile_avx2_256_64
 #else
-#define FNAME parasail_nw_striped_avx2_256_32
-#define PNAME parasail_nw_striped_profile_avx2_256_32
+#define FNAME parasail_nw_striped_avx2_256_64
+#define PNAME parasail_nw_striped_profile_avx2_256_64
 #endif
 #endif
 
@@ -115,7 +144,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_avx_256_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_avx_256_64(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -145,12 +174,12 @@ parasail_result_t* PNAME(
     __m256i* restrict pvHStore = NULL;
     __m256i* restrict pvHLoad = NULL;
     __m256i* restrict pvE = NULL;
-    int32_t* restrict boundary = NULL;
+    int64_t* restrict boundary = NULL;
     __m256i vGapO;
     __m256i vGapE;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
-    int32_t score = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
+    int64_t score = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -159,7 +188,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -175,18 +204,18 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 8; /* number of values in vector unit */
+    segWidth = 4; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile = (__m256i*)profile->profile32.score;
-    vGapO = _mm256_set1_epi32(open);
-    vGapE = _mm256_set1_epi32(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    vProfile = (__m256i*)profile->profile64.score;
+    vGapO = _mm256_set1_epi64x_rpl(open);
+    vGapE = _mm256_set1_epi64x_rpl(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm256_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
 
@@ -204,7 +233,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_STRIPED
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -216,7 +245,7 @@ parasail_result_t* PNAME(
     pvHStore = parasail_memalign___m256i(32, segLen);
     pvHLoad  = parasail_memalign___m256i(32, segLen);
     pvE      = parasail_memalign___m256i(32, segLen);
-    boundary = parasail_memalign_int32_t(32, s2Len+1);
+    boundary = parasail_memalign_int64_t(32, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -229,13 +258,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m256i_32_t h;
-            __m256i_32_t e;
+            __m256i_64_t h;
+            __m256i_64_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = -open-gap*(segNum*segLen+i);
-                h.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                e.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
             }
             _mm256_store_si256(&pvHStore[index], h.m);
             _mm256_store_si256(&pvE[index], e.m);
@@ -248,7 +277,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = -open-gap*(i-1);
-            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
+            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
         }
     }
 
@@ -260,7 +289,7 @@ parasail_result_t* PNAME(
         __m256i vF = vNegLimit;
 
         /* load final segment of pvHStore and shift left by 2 bytes */
-        __m256i vH = _mm256_slli_si256_rpl(pvHStore[segLen - 1], 4);
+        __m256i vH = _mm256_slli_si256_rpl(pvHStore[segLen - 1], 8);
 
         /* Correct part of the vProfile */
         const __m256i* vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -271,35 +300,35 @@ parasail_result_t* PNAME(
         pvHStore = pv;
 
         /* insert upper boundary condition */
-        vH = _mm256_insert_epi32_rpl(vH, boundary[j], 0);
+        vH = _mm256_insert_epi64_rpl(vH, boundary[j], 0);
 
         /* inner loop to process the query sequence */
         for (i=0; i<segLen; ++i) {
-            vH = _mm256_add_epi32(vH, _mm256_load_si256(vP + i));
+            vH = _mm256_add_epi64(vH, _mm256_load_si256(vP + i));
             vE = _mm256_load_si256(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH = _mm256_max_epi32(vH, vE);
-            vH = _mm256_max_epi32(vH, vF);
+            vH = _mm256_max_epi64_rpl(vH, vE);
+            vH = _mm256_max_epi64_rpl(vH, vF);
             /* Save vH values. */
             _mm256_store_si256(pvHStore + i, vH);
-            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vH);
-            vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vE);
-            vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vE);
+            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vF);
 #ifdef PARASAIL_TABLE
             arr_store_si256(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
 
             /* Update vE value. */
-            vH = _mm256_sub_epi32(vH, vGapO);
-            vE = _mm256_sub_epi32(vE, vGapE);
-            vE = _mm256_max_epi32(vE, vH);
+            vH = _mm256_sub_epi64(vH, vGapO);
+            vE = _mm256_sub_epi64(vE, vGapE);
+            vE = _mm256_max_epi64_rpl(vE, vH);
             _mm256_store_si256(pvE + i, vE);
 
             /* Update vF value. */
-            vF = _mm256_sub_epi32(vF, vGapE);
-            vF = _mm256_max_epi32(vF, vH);
+            vF = _mm256_sub_epi64(vF, vGapE);
+            vF = _mm256_max_epi64_rpl(vF, vH);
 
             /* Load the next vH. */
             vH = _mm256_load_si256(pvHLoad + i);
@@ -309,22 +338,22 @@ parasail_result_t* PNAME(
          * then deletion, so don't update E(i, i), learn from SWPS3 */
         for (k=0; k<segWidth; ++k) {
             int64_t tmp = boundary[j+1]-open;
-            int32_t tmp2 = tmp < INT32_MIN ? INT32_MIN : tmp;
-            vF = _mm256_slli_si256_rpl(vF, 4);
-            vF = _mm256_insert_epi32_rpl(vF, tmp2, 0);
+            int64_t tmp2 = tmp < INT64_MIN ? INT64_MIN : tmp;
+            vF = _mm256_slli_si256_rpl(vF, 8);
+            vF = _mm256_insert_epi64_rpl(vF, tmp2, 0);
             for (i=0; i<segLen; ++i) {
                 vH = _mm256_load_si256(pvHStore + i);
-                vH = _mm256_max_epi32(vH,vF);
+                vH = _mm256_max_epi64_rpl(vH,vF);
                 _mm256_store_si256(pvHStore + i, vH);
-                vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vH);
+                vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vH);
 #ifdef PARASAIL_TABLE
                 arr_store_si256(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-                vH = _mm256_sub_epi32(vH, vGapO);
-                vF = _mm256_sub_epi32(vF, vGapE);
-                if (! _mm256_movemask_epi8(_mm256_cmpgt_epi32(vF, vH))) goto end;
-                /*vF = _mm256_max_epi32(vF, vH);*/
+                vH = _mm256_sub_epi64(vH, vGapO);
+                vF = _mm256_sub_epi64(vF, vGapE);
+                if (! _mm256_movemask_epi8(_mm256_cmpgt_epi64(vF, vH))) goto end;
+                /*vF = _mm256_max_epi64_rpl(vF, vH);*/
             }
         }
 end:
@@ -336,9 +365,9 @@ end:
         {
             vH = _mm256_load_si256(pvHStore + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm256_slli_si256_rpl(vH, 4);
+                vH = _mm256_slli_si256_rpl(vH, 8);
             }
-            result->rowcols->score_row[j] = (int32_t) _mm256_extract_epi32_rpl (vH, 7);
+            result->rowcols->score_row[j] = (int64_t) _mm256_extract_epi64_rpl (vH, 3);
         }
 #endif
     }
@@ -354,14 +383,14 @@ end:
     {
         __m256i vH = _mm256_load_si256(pvHStore + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm256_slli_si256_rpl (vH, 4);
+            vH = _mm256_slli_si256_rpl (vH, 8);
         }
-        score = (int32_t) _mm256_extract_epi32_rpl (vH, 7);
+        score = (int64_t) _mm256_extract_epi64_rpl (vH, 3);
     }
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi32_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
