@@ -17,65 +17,28 @@
 #include "parasail/internal_avx.h"
 
 
-#if HAVE_AVX2_MM256_INSERT_EPI64
-#define _mm256_insert_epi64_rpl _mm256_insert_epi64
+#if HAVE_AVX2_MM256_INSERT_EPI32
+#define _mm256_insert_epi32_rpl _mm256_insert_epi32
 #else
-static inline __m256i _mm256_insert_epi64_rpl(__m256i a, int64_t i, int imm) {
-    __m256i_64_t A;
+static inline __m256i _mm256_insert_epi32_rpl(__m256i a, int32_t i, int imm) {
+    __m256i_32_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_SET1_EPI64X
-#define _mm256_set1_epi64x_rpl _mm256_set1_epi64x
+#if HAVE_AVX2_MM256_EXTRACT_EPI32
+#define _mm256_extract_epi32_rpl _mm256_extract_epi32
 #else
-static inline __m256i _mm256_set1_epi64x_rpl(int64_t i) {
-    __m256i_64_t A;
-    A.v[0] = i;
-    A.v[1] = i;
-    A.v[2] = i;
-    A.v[3] = i;
-    return A.m;
-}
-#endif
-
-static inline __m256i _mm256_max_epi64_rpl(__m256i a, __m256i b) {
-    __m256i_64_t A;
-    __m256i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
-    A.v[2] = (A.v[2]>B.v[2]) ? A.v[2] : B.v[2];
-    A.v[3] = (A.v[3]>B.v[3]) ? A.v[3] : B.v[3];
-    return A.m;
-}
-
-#if HAVE_AVX2_MM256_EXTRACT_EPI64
-#define _mm256_extract_epi64_rpl _mm256_extract_epi64
-#else
-static inline int64_t _mm256_extract_epi64_rpl(__m256i a, int imm) {
-    __m256i_64_t A;
+static inline int32_t _mm256_extract_epi32_rpl(__m256i a, int imm) {
+    __m256i_32_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
-static inline __m256i _mm256_min_epi64_rpl(__m256i a, __m256i b) {
-    __m256i_64_t A;
-    __m256i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
-    A.v[2] = (A.v[2]<B.v[2]) ? A.v[2] : B.v[2];
-    A.v[3] = (A.v[3]<B.v[3]) ? A.v[3] : B.v[3];
-    return A.m;
-}
-
-#define _mm256_cmplt_epi64_rpl(a,b) _mm256_cmpgt_epi64(b,a)
+#define _mm256_cmplt_epi32_rpl(a,b) _mm256_cmpgt_epi32(b,a)
 
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
@@ -99,8 +62,8 @@ static inline __m256i arr_load(
     return _mm256_load_si256(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_nw_trace_scan_avx2_256_64
-#define PNAME parasail_nw_trace_scan_profile_avx2_256_64
+#define FNAME parasail_nw_trace_scan_avx2_256_32
+#define PNAME parasail_nw_trace_scan_profile_avx2_256_32
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -123,7 +86,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_avx_256_64(s1, s1Len, matrix);
+    profile = parasail_profile_create_avx_256_32(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -151,16 +114,16 @@ parasail_result_t* PNAME(
     int32_t position = 0;
     __m256i* restrict pvP = NULL;
     __m256i* restrict pvE = NULL;
-    int64_t* restrict boundary = NULL;
+    int32_t* restrict boundary = NULL;
     __m256i* restrict pvHt = NULL;
     __m256i* restrict pvH = NULL;
     __m256i* restrict pvGapper = NULL;
     __m256i vGapO;
     __m256i vGapE;
-    int64_t NEG_LIMIT = 0;
-    int64_t POS_LIMIT = 0;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
     __m256i vZero;
-    int64_t score = 0;
+    int32_t score = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -178,7 +141,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->profile32.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -194,32 +157,32 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 4; /* number of values in vector unit */
+    segWidth = 8; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP = (__m256i*)profile->profile64.score;
-    vGapO = _mm256_set1_epi64x_rpl(open);
-    vGapE = _mm256_set1_epi64x_rpl(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    pvP = (__m256i*)profile->profile32.score;
+    vGapO = _mm256_set1_epi32(open);
+    vGapE = _mm256_set1_epi32(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
     vZero = _mm256_setzero_si256();
     score = NEG_LIMIT;
-    vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi32(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi32(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vNegInfFront = vZero;
-    vTIns  = _mm256_set1_epi64x_rpl(PARASAIL_INS);
-    vTDel  = _mm256_set1_epi64x_rpl(PARASAIL_DEL);
-    vTDiag = _mm256_set1_epi64x_rpl(PARASAIL_DIAG);
-    vTDiagE = _mm256_set1_epi64x_rpl(PARASAIL_DIAG_E);
-    vTInsE = _mm256_set1_epi64x_rpl(PARASAIL_INS_E);
-    vTDiagF = _mm256_set1_epi64x_rpl(PARASAIL_DIAG_F);
-    vTDelF = _mm256_set1_epi64x_rpl(PARASAIL_DEL_F);
-    vNegInfFront = _mm256_insert_epi64_rpl(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm256_add_epi64(vNegInfFront,
-            _mm256_slli_si256_rpl(_mm256_set1_epi64x_rpl(-segLen*gap), 8));
+    vTIns  = _mm256_set1_epi32(PARASAIL_INS);
+    vTDel  = _mm256_set1_epi32(PARASAIL_DEL);
+    vTDiag = _mm256_set1_epi32(PARASAIL_DIAG);
+    vTDiagE = _mm256_set1_epi32(PARASAIL_DIAG_E);
+    vTInsE = _mm256_set1_epi32(PARASAIL_INS_E);
+    vTDiagF = _mm256_set1_epi32(PARASAIL_DIAG_F);
+    vTDelF = _mm256_set1_epi32(PARASAIL_DEL_F);
+    vNegInfFront = _mm256_insert_epi32_rpl(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm256_add_epi32(vNegInfFront,
+            _mm256_slli_si256_rpl(_mm256_set1_epi32(-segLen*gap), 4));
 
     /* initialize result */
     result = parasail_result_new_trace(segLen, s2Len, 32, sizeof(__m256i));
@@ -228,11 +191,11 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_8;
 
     /* initialize heap variables */
     pvE = parasail_memalign___m256i(32, segLen);
-    boundary = parasail_memalign_int64_t(32, s2Len+1);
+    boundary = parasail_memalign_int32_t(32, s2Len+1);
     pvHt= parasail_memalign___m256i(32, segLen);
     pvH = parasail_memalign___m256i(32, segLen);
     pvGapper = parasail_memalign___m256i(32, segLen);
@@ -249,13 +212,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m256i_64_t h;
-            __m256i_64_t e;
+            __m256i_32_t h;
+            __m256i_32_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = -open-gap*(segNum*segLen+i);
-                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                h.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                e.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
             }
             _mm256_store_si256(&pvH[index], h.m);
             _mm256_store_si256(&pvE[index], e.m);
@@ -268,15 +231,15 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = -open-gap*(i-1);
-            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
+            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
         }
     }
 
     {
-        __m256i vGapper = _mm256_sub_epi64(vZero,vGapO);
+        __m256i vGapper = _mm256_sub_epi32(vZero,vGapO);
         for (i=segLen-1; i>=0; --i) {
             _mm256_store_si256(pvGapper+i, vGapper);
-            vGapper = _mm256_sub_epi64(vGapper, vGapE);
+            vGapper = _mm256_sub_epi32(vGapper, vGapE);
         }
     }
 
@@ -304,27 +267,27 @@ parasail_result_t* PNAME(
         /* calculate Ht */
         /* calculate F and H first pass */
         vHp = _mm256_load_si256(pvH+(segLen-1));
-        vHp = _mm256_slli_si256_rpl(vHp, 8);
-        vHp = _mm256_insert_epi64_rpl(vHp, boundary[j], 0);
+        vHp = _mm256_slli_si256_rpl(vHp, 4);
+        vHp = _mm256_insert_epi32_rpl(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = _mm256_sub_epi64(vNegLimit, pvGapper[0]);
+        vHt = _mm256_sub_epi32(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         for (i=0; i<segLen; ++i) {
             vH = _mm256_load_si256(pvH+i);
             vE = _mm256_load_si256(pvE+i);
             vW = _mm256_load_si256(pvW+i);
             vGapper = _mm256_load_si256(pvGapper+i);
-            vE_opn = _mm256_sub_epi64(vH, vGapO);
-            vE_ext = _mm256_sub_epi64(vE, vGapE);
-            case1 = _mm256_cmpgt_epi64(vE_opn, vE_ext);
+            vE_opn = _mm256_sub_epi32(vH, vGapO);
+            vE_ext = _mm256_sub_epi32(vE, vGapE);
+            case1 = _mm256_cmpgt_epi32(vE_opn, vE_ext);
             vET = _mm256_blendv_epi8(vTInsE, vTDiagE, case1);
             arr_store(result->trace->trace_table, vET, i, segLen, j);
-            vE = _mm256_max_epi64_rpl(vE_opn, vE_ext);
-            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vE);
-            vGapper = _mm256_add_epi64(vHt, vGapper);
-            vF = _mm256_max_epi64_rpl(vF, vGapper);
-            vHp = _mm256_add_epi64(vHp, vW);
-            vHt = _mm256_max_epi64_rpl(vE, vHp);
+            vE = _mm256_max_epi32(vE_opn, vE_ext);
+            vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vE);
+            vGapper = _mm256_add_epi32(vHt, vGapper);
+            vF = _mm256_max_epi32(vF, vGapper);
+            vHp = _mm256_add_epi32(vHp, vW);
+            vHt = _mm256_max_epi32(vE, vHp);
             _mm256_store_si256(pvE+i, vE);
             _mm256_store_si256(pvHt+i, vHt);
             _mm256_store_si256(pvH+i, vHp);
@@ -332,33 +295,33 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm256_slli_si256_rpl(vHt, 8);
-        vHt = _mm256_insert_epi64_rpl(vHt, boundary[j+1], 0);
+        vHt = _mm256_slli_si256_rpl(vHt, 4);
+        vHt = _mm256_insert_epi32_rpl(vHt, boundary[j+1], 0);
         vGapper = _mm256_load_si256(pvGapper);
-        vGapper = _mm256_add_epi64(vHt, vGapper);
-        vF = _mm256_max_epi64_rpl(vF, vGapper);
+        vGapper = _mm256_add_epi32(vHt, vGapper);
+        vF = _mm256_max_epi32(vF, vGapper);
         for (i=0; i<segWidth-2; ++i) {
-            __m256i vFt = _mm256_slli_si256_rpl(vF, 8);
-            vFt = _mm256_add_epi64(vFt, vSegLenXgap);
-            vF = _mm256_max_epi64_rpl(vF, vFt);
+            __m256i vFt = _mm256_slli_si256_rpl(vF, 4);
+            vFt = _mm256_add_epi32(vFt, vSegLenXgap);
+            vF = _mm256_max_epi32(vF, vFt);
         }
 
         /* calculate final H */
-        vF = _mm256_slli_si256_rpl(vF, 8);
-        vF = _mm256_add_epi64(vF, vNegInfFront);
-        vH = _mm256_max_epi64_rpl(vF, vHt);
+        vF = _mm256_slli_si256_rpl(vF, 4);
+        vF = _mm256_add_epi32(vF, vNegInfFront);
+        vH = _mm256_max_epi32(vF, vHt);
         for (i=0; i<segLen; ++i) {
             vET = arr_load(result->trace->trace_table, i, segLen, j);
             vHp = _mm256_load_si256(pvH+i);
             vHt = _mm256_load_si256(pvHt+i);
-            vF_opn = _mm256_sub_epi64(vH, vGapO);
-            vF_ext = _mm256_sub_epi64(vF, vGapE);
-            vF = _mm256_max_epi64_rpl(vF_opn, vF_ext);
-            case1 = _mm256_cmpgt_epi64(vF_opn, vF_ext);
+            vF_opn = _mm256_sub_epi32(vH, vGapO);
+            vF_ext = _mm256_sub_epi32(vF, vGapE);
+            vF = _mm256_max_epi32(vF_opn, vF_ext);
+            case1 = _mm256_cmpgt_epi32(vF_opn, vF_ext);
             vFT = _mm256_blendv_epi8(vTDelF, vTDiagF, case1);
-            vH = _mm256_max_epi64_rpl(vHt, vF);
-            case1 = _mm256_cmpeq_epi64(vH, vHp);
-            case2 = _mm256_cmpeq_epi64(vH, vF);
+            vH = _mm256_max_epi32(vHt, vF);
+            case1 = _mm256_cmpeq_epi32(vH, vHp);
+            case2 = _mm256_cmpeq_epi32(vH, vF);
             vT = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vTIns, vTDel, case2),
                     vTDiag, case1);
@@ -366,9 +329,9 @@ parasail_result_t* PNAME(
             vT = _mm256_or_si256(vT, vFT);
             arr_store(result->trace->trace_table, vT, i, segLen, j);
             _mm256_store_si256(pvH+i, vH);
-            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vF);
-            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vH);
         }
     }
 
@@ -376,14 +339,14 @@ parasail_result_t* PNAME(
     {
         __m256i vH = _mm256_load_si256(pvH + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm256_slli_si256_rpl(vH, 8);
+            vH = _mm256_slli_si256_rpl(vH, 4);
         }
-        score = (int64_t) _mm256_extract_epi64_rpl (vH, 3);
+        score = (int32_t) _mm256_extract_epi32_rpl (vH, 7);
     }
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi32_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
