@@ -19,29 +19,66 @@
 #define SWAP(A,B) { __m256i* tmp = A; A = B; B = tmp; }
 
 
-#define _mm256_cmplt_epi16_rpl(a,b) _mm256_cmpgt_epi16(b,a)
-
-#if HAVE_AVX2_MM256_INSERT_EPI16
-#define _mm256_insert_epi16_rpl _mm256_insert_epi16
+#if HAVE_AVX2_MM256_INSERT_EPI64
+#define _mm256_insert_epi64_rpl _mm256_insert_epi64
 #else
-static inline __m256i _mm256_insert_epi16_rpl(__m256i a, int16_t i, int imm) {
-    __m256i_16_t A;
+static inline __m256i _mm256_insert_epi64_rpl(__m256i a, int64_t i, int imm) {
+    __m256i_64_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_EXTRACT_EPI16
-#define _mm256_extract_epi16_rpl _mm256_extract_epi16
+#if HAVE_AVX2_MM256_SET1_EPI64X
+#define _mm256_set1_epi64x_rpl _mm256_set1_epi64x
 #else
-static inline int16_t _mm256_extract_epi16_rpl(__m256i a, int imm) {
-    __m256i_16_t A;
+static inline __m256i _mm256_set1_epi64x_rpl(int64_t i) {
+    __m256i_64_t A;
+    A.v[0] = i;
+    A.v[1] = i;
+    A.v[2] = i;
+    A.v[3] = i;
+    return A.m;
+}
+#endif
+
+static inline __m256i _mm256_max_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]>B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]>B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
+#if HAVE_AVX2_MM256_EXTRACT_EPI64
+#define _mm256_extract_epi64_rpl _mm256_extract_epi64
+#else
+static inline int64_t _mm256_extract_epi64_rpl(__m256i a, int imm) {
+    __m256i_64_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
+static inline __m256i _mm256_min_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]<B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]<B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
+#define _mm256_cmplt_epi64_rpl(a,b) _mm256_cmpgt_epi64(b,a)
+
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
 
@@ -64,8 +101,8 @@ static inline __m256i arr_load(
     return _mm256_load_si256(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_nw_trace_striped_avx2_256_16
-#define PNAME parasail_nw_trace_striped_profile_avx2_256_16
+#define FNAME parasail_nw_trace_striped_avx2_256_64
+#define PNAME parasail_nw_trace_striped_profile_avx2_256_64
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -88,7 +125,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_avx_256_16(s1, s1Len, matrix);
+    profile = parasail_profile_create_avx_256_64(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -121,12 +158,12 @@ parasail_result_t* PNAME(
     __m256i* restrict pvEaStore = NULL;
     __m256i* restrict pvEaLoad = NULL;
     __m256i* restrict pvHT = NULL;
-    int16_t* restrict boundary = NULL;
+    int64_t* restrict boundary = NULL;
     __m256i vGapO;
     __m256i vGapE;
-    int16_t NEG_LIMIT = 0;
-    int16_t POS_LIMIT = 0;
-    int16_t score = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
+    int64_t score = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -144,7 +181,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile16.score);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -160,29 +197,29 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 16; /* number of values in vector unit */
+    segWidth = 4; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile = (__m256i*)profile->profile16.score;
-    vGapO = _mm256_set1_epi16(open);
-    vGapE = _mm256_set1_epi16(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    vProfile = (__m256i*)profile->profile64.score;
+    vGapO = _mm256_set1_epi64x_rpl(open);
+    vGapE = _mm256_set1_epi64x_rpl(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm256_set1_epi16(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi16(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vTIns  = _mm256_set1_epi16(PARASAIL_INS);
-    vTDel  = _mm256_set1_epi16(PARASAIL_DEL);
-    vTDiag = _mm256_set1_epi16(PARASAIL_DIAG);
-    vTDiagE = _mm256_set1_epi16(PARASAIL_DIAG_E);
-    vTInsE = _mm256_set1_epi16(PARASAIL_INS_E);
-    vTDiagF = _mm256_set1_epi16(PARASAIL_DIAG_F);
-    vTDelF = _mm256_set1_epi16(PARASAIL_DEL_F);
-    vTMask = _mm256_set1_epi16(PARASAIL_ZERO_MASK);
-    vFTMask = _mm256_set1_epi16(PARASAIL_F_MASK);
+    vTIns  = _mm256_set1_epi64x_rpl(PARASAIL_INS);
+    vTDel  = _mm256_set1_epi64x_rpl(PARASAIL_DEL);
+    vTDiag = _mm256_set1_epi64x_rpl(PARASAIL_DIAG);
+    vTDiagE = _mm256_set1_epi64x_rpl(PARASAIL_DIAG_E);
+    vTInsE = _mm256_set1_epi64x_rpl(PARASAIL_INS_E);
+    vTDiagF = _mm256_set1_epi64x_rpl(PARASAIL_DIAG_F);
+    vTDelF = _mm256_set1_epi64x_rpl(PARASAIL_DEL_F);
+    vTMask = _mm256_set1_epi64x_rpl(PARASAIL_ZERO_MASK);
+    vFTMask = _mm256_set1_epi64x_rpl(PARASAIL_F_MASK);
 
     /* initialize result */
     result = parasail_result_new_trace(segLen, s2Len, 32, sizeof(__m256i));
@@ -191,7 +228,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_16;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
 
     /* initialize heap variables */
     pvHStore = parasail_memalign___m256i(32, segLen);
@@ -200,7 +237,7 @@ parasail_result_t* PNAME(
     pvEaStore = parasail_memalign___m256i(32, segLen);
     pvEaLoad = parasail_memalign___m256i(32, segLen);
     pvHT = parasail_memalign___m256i(32, segLen);
-    boundary = parasail_memalign_int16_t(32, s2Len+1);
+    boundary = parasail_memalign_int64_t(32, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -216,13 +253,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m256i_16_t h;
-            __m256i_16_t e;
+            __m256i_64_t h;
+            __m256i_64_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = -open-gap*(segNum*segLen+i);
-                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                e.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
             }
             _mm256_store_si256(&pvHStore[index], h.m);
             _mm256_store_si256(&pvE[index], e.m);
@@ -236,7 +273,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = -open-gap*(i-1);
-            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
+            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
         }
     }
 
@@ -261,12 +298,12 @@ parasail_result_t* PNAME(
          * corrected in the Lazy_F loop.  */
         vF = vNegLimit;
 
-        /* load final segment of pvHStore and shift left by 2 bytes */
+        /* load final segment of pvHStore and shift left by 8 bytes */
         vH = _mm256_load_si256(&pvHStore[segLen - 1]);
-        vH = _mm256_slli_si256_rpl(vH, 2);
+        vH = _mm256_slli_si256_rpl(vH, 8);
 
         /* insert upper boundary condition */
-        vH = _mm256_insert_epi16_rpl(vH, boundary[j], 0);
+        vH = _mm256_insert_epi64_rpl(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -280,20 +317,20 @@ parasail_result_t* PNAME(
             vE = _mm256_load_si256(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm256_adds_epi16(vH, _mm256_load_si256(vP + i));
-            vH = _mm256_max_epi16(vH_dag, vE);
-            vH = _mm256_max_epi16(vH, vF);
+            vH_dag = _mm256_add_epi64(vH, _mm256_load_si256(vP + i));
+            vH = _mm256_max_epi64_rpl(vH_dag, vE);
+            vH = _mm256_max_epi64_rpl(vH, vF);
             /* Save vH values. */
             _mm256_store_si256(pvHStore + i, vH);
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vH);
-            vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vE);
-            vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vE);
+            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vF);
 
             {
                 __m256i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                __m256i case1 = _mm256_cmpeq_epi16(vH, vH_dag);
-                __m256i case2 = _mm256_cmpeq_epi16(vH, vF);
+                __m256i case1 = _mm256_cmpeq_epi64(vH, vH_dag);
+                __m256i case2 = _mm256_cmpeq_epi64(vH, vF);
                 __m256i vT = _mm256_blendv_epi8(
                         _mm256_blendv_epi8(vTIns, vTDel, case2),
                         vTDiag, case1);
@@ -302,30 +339,30 @@ parasail_result_t* PNAME(
                 arr_store(result->trace->trace_table, vT, i, segLen, j);
             }
 
-            vEF_opn = _mm256_subs_epi16(vH, vGapO);
+            vEF_opn = _mm256_sub_epi64(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm256_subs_epi16(vE, vGapE);
-            vE = _mm256_max_epi16(vEF_opn, vE_ext);
+            vE_ext = _mm256_sub_epi64(vE, vGapE);
+            vE = _mm256_max_epi64_rpl(vEF_opn, vE_ext);
             _mm256_store_si256(pvE + i, vE);
             {
                 __m256i vEa = _mm256_load_si256(pvEaLoad + i);
-                __m256i vEa_ext = _mm256_subs_epi16(vEa, vGapE);
-                vEa = _mm256_max_epi16(vEF_opn, vEa_ext);
+                __m256i vEa_ext = _mm256_sub_epi64(vEa, vGapE);
+                vEa = _mm256_max_epi64_rpl(vEF_opn, vEa_ext);
                 _mm256_store_si256(pvEaStore + i, vEa);
                 if (j+1<s2Len) {
-                    __m256i cond = _mm256_cmpgt_epi16(vEF_opn, vEa_ext);
+                    __m256i cond = _mm256_cmpgt_epi64(vEF_opn, vEa_ext);
                     __m256i vT = _mm256_blendv_epi8(vTInsE, vTDiagE, cond);
                     arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                 }
             }
 
             /* Update vF value. */
-            vF_ext = _mm256_subs_epi16(vF, vGapE);
-            vF = _mm256_max_epi16(vEF_opn, vF_ext);
+            vF_ext = _mm256_sub_epi64(vF, vGapE);
+            vF = _mm256_max_epi64_rpl(vEF_opn, vF_ext);
             if (i+1<segLen) {
                 __m256i vTAll = arr_load(result->trace->trace_table, i+1, segLen, j);
-                __m256i cond = _mm256_cmpgt_epi16(vEF_opn, vF_ext);
+                __m256i cond = _mm256_cmpgt_epi64(vEF_opn, vF_ext);
                 __m256i vT = _mm256_blendv_epi8(vTDelF, vTDiagF, cond);
                 vT = _mm256_or_si256(vT, vTAll);
                 arr_store(result->trace->trace_table, vT, i+1, segLen, j);
@@ -342,35 +379,35 @@ parasail_result_t* PNAME(
         vFa = vF;
         for (k=0; k<segWidth; ++k) {
             int64_t tmp = boundary[j+1]-open;
-            int16_t tmp2 = tmp < INT16_MIN ? INT16_MIN : tmp;
+            int64_t tmp2 = tmp < INT64_MIN ? INT64_MIN : tmp;
             __m256i vHp = _mm256_load_si256(&pvHLoad[segLen - 1]);
-            vHp = _mm256_slli_si256_rpl(vHp, 2);
-            vHp = _mm256_insert_epi16_rpl(vHp, boundary[j], 0);
-            vEF_opn = _mm256_slli_si256_rpl(vEF_opn, 2);
-            vEF_opn = _mm256_insert_epi16_rpl(vEF_opn, tmp2, 0);
-            vF_ext = _mm256_slli_si256_rpl(vF_ext, 2);
-            vF_ext = _mm256_insert_epi16_rpl(vF_ext, NEG_LIMIT, 0);
-            vF = _mm256_slli_si256_rpl(vF, 2);
-            vF = _mm256_insert_epi16_rpl(vF, tmp2, 0);
-            vFa_ext = _mm256_slli_si256_rpl(vFa_ext, 2);
-            vFa_ext = _mm256_insert_epi16_rpl(vFa_ext, NEG_LIMIT, 0);
-            vFa = _mm256_slli_si256_rpl(vFa, 2);
-            vFa = _mm256_insert_epi16_rpl(vFa, tmp2, 0);
+            vHp = _mm256_slli_si256_rpl(vHp, 8);
+            vHp = _mm256_insert_epi64_rpl(vHp, boundary[j], 0);
+            vEF_opn = _mm256_slli_si256_rpl(vEF_opn, 8);
+            vEF_opn = _mm256_insert_epi64_rpl(vEF_opn, tmp2, 0);
+            vF_ext = _mm256_slli_si256_rpl(vF_ext, 8);
+            vF_ext = _mm256_insert_epi64_rpl(vF_ext, NEG_LIMIT, 0);
+            vF = _mm256_slli_si256_rpl(vF, 8);
+            vF = _mm256_insert_epi64_rpl(vF, tmp2, 0);
+            vFa_ext = _mm256_slli_si256_rpl(vFa_ext, 8);
+            vFa_ext = _mm256_insert_epi64_rpl(vFa_ext, NEG_LIMIT, 0);
+            vFa = _mm256_slli_si256_rpl(vFa, 8);
+            vFa = _mm256_insert_epi64_rpl(vFa, tmp2, 0);
             for (i=0; i<segLen; ++i) {
                 vH = _mm256_load_si256(pvHStore + i);
-                vH = _mm256_max_epi16(vH,vF);
+                vH = _mm256_max_epi64_rpl(vH,vF);
                 _mm256_store_si256(pvHStore + i, vH);
-                vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vH);
+                vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vH);
                 {
                     __m256i vTAll;
                     __m256i vT;
                     __m256i case1;
                     __m256i case2;
                     __m256i cond;
-                    vHp = _mm256_adds_epi16(vHp, _mm256_load_si256(vP + i));
-                    case1 = _mm256_cmpeq_epi16(vH, vHp);
-                    case2 = _mm256_cmpeq_epi16(vH, vF);
+                    vHp = _mm256_add_epi64(vHp, _mm256_load_si256(vP + i));
+                    case1 = _mm256_cmpeq_epi64(vH, vHp);
+                    case2 = _mm256_cmpeq_epi64(vH, vF);
                     cond = _mm256_andnot_si256(case1,case2);
                     vTAll = arr_load(result->trace->trace_table, i, segLen, j);
                     vT = _mm256_load_si256(pvHT + i);
@@ -383,34 +420,34 @@ parasail_result_t* PNAME(
                 /* Update vF value. */
                 {
                     __m256i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                    __m256i cond = _mm256_cmpgt_epi16(vEF_opn, vFa_ext);
+                    __m256i cond = _mm256_cmpgt_epi64(vEF_opn, vFa_ext);
                     __m256i vT = _mm256_blendv_epi8(vTDelF, vTDiagF, cond);
                     vTAll = _mm256_and_si256(vTAll, vFTMask);
                     vTAll = _mm256_or_si256(vTAll, vT);
                     arr_store(result->trace->trace_table, vTAll, i, segLen, j);
                 }
-                vEF_opn = _mm256_subs_epi16(vH, vGapO);
-                vF_ext = _mm256_subs_epi16(vF, vGapE);
+                vEF_opn = _mm256_sub_epi64(vH, vGapO);
+                vF_ext = _mm256_sub_epi64(vF, vGapE);
                 {
                     __m256i vEa = _mm256_load_si256(pvEaLoad + i);
-                    __m256i vEa_ext = _mm256_subs_epi16(vEa, vGapE);
-                    vEa = _mm256_max_epi16(vEF_opn, vEa_ext);
+                    __m256i vEa_ext = _mm256_sub_epi64(vEa, vGapE);
+                    vEa = _mm256_max_epi64_rpl(vEF_opn, vEa_ext);
                     _mm256_store_si256(pvEaStore + i, vEa);
                     if (j+1<s2Len) {
-                        __m256i cond = _mm256_cmpgt_epi16(vEF_opn, vEa_ext);
+                        __m256i cond = _mm256_cmpgt_epi64(vEF_opn, vEa_ext);
                         __m256i vT = _mm256_blendv_epi8(vTInsE, vTDiagE, cond);
                         arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                     }
                 }
                 if (! _mm256_movemask_epi8(
                             _mm256_or_si256(
-                                _mm256_cmpgt_epi16(vF_ext, vEF_opn),
-                                _mm256_cmpeq_epi16(vF_ext, vEF_opn))))
+                                _mm256_cmpgt_epi64(vF_ext, vEF_opn),
+                                _mm256_cmpeq_epi64(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = _mm256_max_epi16(vEF_opn, vF_ext);*/
+                /*vF = _mm256_max_epi64_rpl(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                vFa_ext = _mm256_subs_epi16(vFa, vGapE);
-                vFa = _mm256_max_epi16(vEF_opn, vFa_ext);
+                vFa_ext = _mm256_sub_epi64(vFa, vGapE);
+                vFa = _mm256_max_epi64_rpl(vEF_opn, vFa_ext);
                 vHp = _mm256_load_si256(pvHLoad + i);
             }
         }
@@ -423,14 +460,14 @@ end:
     {
         __m256i vH = _mm256_load_si256(pvHStore + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm256_slli_si256_rpl (vH, 2);
+            vH = _mm256_slli_si256_rpl (vH, 8);
         }
-        score = (int16_t) _mm256_extract_epi16_rpl (vH, 15);
+        score = (int64_t) _mm256_extract_epi64_rpl (vH, 3);
     }
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi16_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
