@@ -14,6 +14,7 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
+#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -23,86 +24,6 @@
 #define SWAP(A,B) { __m128i* tmp = A; A = B; B = tmp; }
 
 
-static inline __m128i _mm_cmpeq_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]==B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    A.v[1] = (A.v[1]==B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    return A.m;
-}
-
-static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
-    a = _mm_andnot_si128(mask, a);
-    a = _mm_or_si128(a, _mm_and_si128(mask, b));
-    return a;
-}
-
-static inline __m128i _mm_cmpgt_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]>B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    A.v[1] = (A.v[1]>B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    return A.m;
-}
-
-static inline __m128i _mm_insert_epi64_rpl(__m128i a, int64_t i, const int imm) {
-    __m128i_64_t A;
-    A.m = a;
-    A.v[imm] = i;
-    return A.m;
-}
-
-static inline __m128i _mm_max_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
-    return A.m;
-}
-
-static inline int64_t _mm_extract_epi64_rpl(__m128i a, const int imm) {
-    __m128i_64_t A;
-    A.m = a;
-    return A.v[imm];
-}
-
-static inline __m128i _mm_min_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
-    return A.m;
-}
-
-static inline __m128i _mm_cmplt_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]<B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    A.v[1] = (A.v[1]<B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    return A.m;
-}
-
-#if HAVE_SSE2_MM_SET1_EPI64X
-#define _mm_set1_epi64x_rpl _mm_set1_epi64x
-#else
-static inline __m128i _mm_set1_epi64x_rpl(int64_t i) {
-    __m128i_64_t A;
-    A.v[0] = i;
-    A.v[1] = i;
-    return A.m;
-}
-#endif
-
 
 static inline void arr_store(
         __m128i *array,
@@ -123,8 +44,8 @@ static inline __m128i arr_load(
     return _mm_load_si128(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_nw_trace_striped_sse2_128_64
-#define PNAME parasail_nw_trace_striped_profile_sse2_128_64
+#define FNAME parasail_nw_trace_striped_sse41_128_16
+#define PNAME parasail_nw_trace_striped_profile_sse41_128_16
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -147,7 +68,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_sse_128_64(s1, s1Len, matrix);
+    profile = parasail_profile_create_sse_128_16(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -180,12 +101,12 @@ parasail_result_t* PNAME(
     __m128i* restrict pvEaStore = NULL;
     __m128i* restrict pvEaLoad = NULL;
     __m128i* restrict pvHT = NULL;
-    int64_t* restrict boundary = NULL;
+    int16_t* restrict boundary = NULL;
     __m128i vGapO;
     __m128i vGapE;
-    int64_t NEG_LIMIT = 0;
-    int64_t POS_LIMIT = 0;
-    int64_t score = 0;
+    int16_t NEG_LIMIT = 0;
+    int16_t POS_LIMIT = 0;
+    int16_t score = 0;
     __m128i vNegLimit;
     __m128i vPosLimit;
     __m128i vSaturationCheckMin;
@@ -203,7 +124,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->profile16.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -219,29 +140,29 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 2; /* number of values in vector unit */
+    segWidth = 8; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile = (__m128i*)profile->profile64.score;
-    vGapO = _mm_set1_epi64x_rpl(open);
-    vGapE = _mm_set1_epi64x_rpl(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    vProfile = (__m128i*)profile->profile16.score;
+    vGapO = _mm_set1_epi16(open);
+    vGapE = _mm_set1_epi16(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
+    POS_LIMIT = INT16_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi64x_rpl(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi64x_rpl(POS_LIMIT);
+    vNegLimit = _mm_set1_epi16(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi16(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vTIns  = _mm_set1_epi64x_rpl(PARASAIL_INS);
-    vTDel  = _mm_set1_epi64x_rpl(PARASAIL_DEL);
-    vTDiag = _mm_set1_epi64x_rpl(PARASAIL_DIAG);
-    vTDiagE = _mm_set1_epi64x_rpl(PARASAIL_DIAG_E);
-    vTInsE = _mm_set1_epi64x_rpl(PARASAIL_INS_E);
-    vTDiagF = _mm_set1_epi64x_rpl(PARASAIL_DIAG_F);
-    vTDelF = _mm_set1_epi64x_rpl(PARASAIL_DEL_F);
-    vTMask = _mm_set1_epi64x_rpl(PARASAIL_ZERO_MASK);
-    vFTMask = _mm_set1_epi64x_rpl(PARASAIL_F_MASK);
+    vTIns  = _mm_set1_epi16(PARASAIL_INS);
+    vTDel  = _mm_set1_epi16(PARASAIL_DEL);
+    vTDiag = _mm_set1_epi16(PARASAIL_DIAG);
+    vTDiagE = _mm_set1_epi16(PARASAIL_DIAG_E);
+    vTInsE = _mm_set1_epi16(PARASAIL_INS_E);
+    vTDiagF = _mm_set1_epi16(PARASAIL_DIAG_F);
+    vTDelF = _mm_set1_epi16(PARASAIL_DEL_F);
+    vTMask = _mm_set1_epi16(PARASAIL_ZERO_MASK);
+    vFTMask = _mm_set1_epi16(PARASAIL_F_MASK);
 
     /* initialize result */
     result = parasail_result_new_trace(segLen, s2Len, 16, sizeof(__m128i));
@@ -250,7 +171,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
+        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
 
     /* initialize heap variables */
     pvHStore = parasail_memalign___m128i(16, segLen);
@@ -259,7 +180,7 @@ parasail_result_t* PNAME(
     pvEaStore = parasail_memalign___m128i(16, segLen);
     pvEaLoad = parasail_memalign___m128i(16, segLen);
     pvHT = parasail_memalign___m128i(16, segLen);
-    boundary = parasail_memalign_int64_t(16, s2Len+1);
+    boundary = parasail_memalign_int16_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -275,13 +196,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m128i_64_t h;
-            __m128i_64_t e;
+            __m128i_16_t h;
+            __m128i_16_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = -open-gap*(segNum*segLen+i);
-                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                e.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
             }
             _mm_store_si128(&pvHStore[index], h.m);
             _mm_store_si128(&pvE[index], e.m);
@@ -295,7 +216,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = -open-gap*(i-1);
-            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
+            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
         }
     }
 
@@ -320,12 +241,12 @@ parasail_result_t* PNAME(
          * corrected in the Lazy_F loop.  */
         vF = vNegLimit;
 
-        /* load final segment of pvHStore and shift left by 8 bytes */
+        /* load final segment of pvHStore and shift left by 2 bytes */
         vH = _mm_load_si128(&pvHStore[segLen - 1]);
-        vH = _mm_slli_si128(vH, 8);
+        vH = _mm_slli_si128(vH, 2);
 
         /* insert upper boundary condition */
-        vH = _mm_insert_epi64_rpl(vH, boundary[j], 0);
+        vH = _mm_insert_epi16(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -339,53 +260,53 @@ parasail_result_t* PNAME(
             vE = _mm_load_si128(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm_add_epi64(vH, _mm_load_si128(vP + i));
-            vH = _mm_max_epi64_rpl(vH_dag, vE);
-            vH = _mm_max_epi64_rpl(vH, vF);
+            vH_dag = _mm_adds_epi16(vH, _mm_load_si128(vP + i));
+            vH = _mm_max_epi16(vH_dag, vE);
+            vH = _mm_max_epi16(vH, vF);
             /* Save vH values. */
             _mm_store_si128(pvHStore + i, vH);
-            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vH);
-            vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vE);
-            vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vE);
+            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vF);
 
             {
                 __m128i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                __m128i case1 = _mm_cmpeq_epi64_rpl(vH, vH_dag);
-                __m128i case2 = _mm_cmpeq_epi64_rpl(vH, vF);
-                __m128i vT = _mm_blendv_epi8_rpl(
-                        _mm_blendv_epi8_rpl(vTIns, vTDel, case2),
+                __m128i case1 = _mm_cmpeq_epi16(vH, vH_dag);
+                __m128i case2 = _mm_cmpeq_epi16(vH, vF);
+                __m128i vT = _mm_blendv_epi8(
+                        _mm_blendv_epi8(vTIns, vTDel, case2),
                         vTDiag, case1);
                 _mm_store_si128(pvHT + i, vT);
                 vT = _mm_or_si128(vT, vTAll);
                 arr_store(result->trace->trace_table, vT, i, segLen, j);
             }
 
-            vEF_opn = _mm_sub_epi64(vH, vGapO);
+            vEF_opn = _mm_subs_epi16(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm_sub_epi64(vE, vGapE);
-            vE = _mm_max_epi64_rpl(vEF_opn, vE_ext);
+            vE_ext = _mm_subs_epi16(vE, vGapE);
+            vE = _mm_max_epi16(vEF_opn, vE_ext);
             _mm_store_si128(pvE + i, vE);
             {
                 __m128i vEa = _mm_load_si128(pvEaLoad + i);
-                __m128i vEa_ext = _mm_sub_epi64(vEa, vGapE);
-                vEa = _mm_max_epi64_rpl(vEF_opn, vEa_ext);
+                __m128i vEa_ext = _mm_subs_epi16(vEa, vGapE);
+                vEa = _mm_max_epi16(vEF_opn, vEa_ext);
                 _mm_store_si128(pvEaStore + i, vEa);
                 if (j+1<s2Len) {
-                    __m128i cond = _mm_cmpgt_epi64_rpl(vEF_opn, vEa_ext);
-                    __m128i vT = _mm_blendv_epi8_rpl(vTInsE, vTDiagE, cond);
+                    __m128i cond = _mm_cmpgt_epi16(vEF_opn, vEa_ext);
+                    __m128i vT = _mm_blendv_epi8(vTInsE, vTDiagE, cond);
                     arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                 }
             }
 
             /* Update vF value. */
-            vF_ext = _mm_sub_epi64(vF, vGapE);
-            vF = _mm_max_epi64_rpl(vEF_opn, vF_ext);
+            vF_ext = _mm_subs_epi16(vF, vGapE);
+            vF = _mm_max_epi16(vEF_opn, vF_ext);
             if (i+1<segLen) {
                 __m128i vTAll = arr_load(result->trace->trace_table, i+1, segLen, j);
-                __m128i cond = _mm_cmpgt_epi64_rpl(vEF_opn, vF_ext);
-                __m128i vT = _mm_blendv_epi8_rpl(vTDelF, vTDiagF, cond);
+                __m128i cond = _mm_cmpgt_epi16(vEF_opn, vF_ext);
+                __m128i vT = _mm_blendv_epi8(vTDelF, vTDiagF, cond);
                 vT = _mm_or_si128(vT, vTAll);
                 arr_store(result->trace->trace_table, vT, i+1, segLen, j);
             }
@@ -401,39 +322,39 @@ parasail_result_t* PNAME(
         vFa = vF;
         for (k=0; k<segWidth; ++k) {
             int64_t tmp = boundary[j+1]-open;
-            int64_t tmp2 = tmp < INT64_MIN ? INT64_MIN : tmp;
+            int16_t tmp2 = tmp < INT16_MIN ? INT16_MIN : tmp;
             __m128i vHp = _mm_load_si128(&pvHLoad[segLen - 1]);
-            vHp = _mm_slli_si128(vHp, 8);
-            vHp = _mm_insert_epi64_rpl(vHp, boundary[j], 0);
-            vEF_opn = _mm_slli_si128(vEF_opn, 8);
-            vEF_opn = _mm_insert_epi64_rpl(vEF_opn, tmp2, 0);
-            vF_ext = _mm_slli_si128(vF_ext, 8);
-            vF_ext = _mm_insert_epi64_rpl(vF_ext, NEG_LIMIT, 0);
-            vF = _mm_slli_si128(vF, 8);
-            vF = _mm_insert_epi64_rpl(vF, tmp2, 0);
-            vFa_ext = _mm_slli_si128(vFa_ext, 8);
-            vFa_ext = _mm_insert_epi64_rpl(vFa_ext, NEG_LIMIT, 0);
-            vFa = _mm_slli_si128(vFa, 8);
-            vFa = _mm_insert_epi64_rpl(vFa, tmp2, 0);
+            vHp = _mm_slli_si128(vHp, 2);
+            vHp = _mm_insert_epi16(vHp, boundary[j], 0);
+            vEF_opn = _mm_slli_si128(vEF_opn, 2);
+            vEF_opn = _mm_insert_epi16(vEF_opn, tmp2, 0);
+            vF_ext = _mm_slli_si128(vF_ext, 2);
+            vF_ext = _mm_insert_epi16(vF_ext, NEG_LIMIT, 0);
+            vF = _mm_slli_si128(vF, 2);
+            vF = _mm_insert_epi16(vF, tmp2, 0);
+            vFa_ext = _mm_slli_si128(vFa_ext, 2);
+            vFa_ext = _mm_insert_epi16(vFa_ext, NEG_LIMIT, 0);
+            vFa = _mm_slli_si128(vFa, 2);
+            vFa = _mm_insert_epi16(vFa, tmp2, 0);
             for (i=0; i<segLen; ++i) {
                 vH = _mm_load_si128(pvHStore + i);
-                vH = _mm_max_epi64_rpl(vH,vF);
+                vH = _mm_max_epi16(vH,vF);
                 _mm_store_si128(pvHStore + i, vH);
-                vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vH);
+                vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
                 {
                     __m128i vTAll;
                     __m128i vT;
                     __m128i case1;
                     __m128i case2;
                     __m128i cond;
-                    vHp = _mm_add_epi64(vHp, _mm_load_si128(vP + i));
-                    case1 = _mm_cmpeq_epi64_rpl(vH, vHp);
-                    case2 = _mm_cmpeq_epi64_rpl(vH, vF);
+                    vHp = _mm_adds_epi16(vHp, _mm_load_si128(vP + i));
+                    case1 = _mm_cmpeq_epi16(vH, vHp);
+                    case2 = _mm_cmpeq_epi16(vH, vF);
                     cond = _mm_andnot_si128(case1,case2);
                     vTAll = arr_load(result->trace->trace_table, i, segLen, j);
                     vT = _mm_load_si128(pvHT + i);
-                    vT = _mm_blendv_epi8_rpl(vT, vTDel, cond);
+                    vT = _mm_blendv_epi8(vT, vTDel, cond);
                     _mm_store_si128(pvHT + i, vT);
                     vTAll = _mm_and_si128(vTAll, vTMask);
                     vTAll = _mm_or_si128(vTAll, vT);
@@ -442,34 +363,34 @@ parasail_result_t* PNAME(
                 /* Update vF value. */
                 {
                     __m128i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                    __m128i cond = _mm_cmpgt_epi64_rpl(vEF_opn, vFa_ext);
-                    __m128i vT = _mm_blendv_epi8_rpl(vTDelF, vTDiagF, cond);
+                    __m128i cond = _mm_cmpgt_epi16(vEF_opn, vFa_ext);
+                    __m128i vT = _mm_blendv_epi8(vTDelF, vTDiagF, cond);
                     vTAll = _mm_and_si128(vTAll, vFTMask);
                     vTAll = _mm_or_si128(vTAll, vT);
                     arr_store(result->trace->trace_table, vTAll, i, segLen, j);
                 }
-                vEF_opn = _mm_sub_epi64(vH, vGapO);
-                vF_ext = _mm_sub_epi64(vF, vGapE);
+                vEF_opn = _mm_subs_epi16(vH, vGapO);
+                vF_ext = _mm_subs_epi16(vF, vGapE);
                 {
                     __m128i vEa = _mm_load_si128(pvEaLoad + i);
-                    __m128i vEa_ext = _mm_sub_epi64(vEa, vGapE);
-                    vEa = _mm_max_epi64_rpl(vEF_opn, vEa_ext);
+                    __m128i vEa_ext = _mm_subs_epi16(vEa, vGapE);
+                    vEa = _mm_max_epi16(vEF_opn, vEa_ext);
                     _mm_store_si128(pvEaStore + i, vEa);
                     if (j+1<s2Len) {
-                        __m128i cond = _mm_cmpgt_epi64_rpl(vEF_opn, vEa_ext);
-                        __m128i vT = _mm_blendv_epi8_rpl(vTInsE, vTDiagE, cond);
+                        __m128i cond = _mm_cmpgt_epi16(vEF_opn, vEa_ext);
+                        __m128i vT = _mm_blendv_epi8(vTInsE, vTDiagE, cond);
                         arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                     }
                 }
                 if (! _mm_movemask_epi8(
                             _mm_or_si128(
-                                _mm_cmpgt_epi64_rpl(vF_ext, vEF_opn),
-                                _mm_cmpeq_epi64_rpl(vF_ext, vEF_opn))))
+                                _mm_cmpgt_epi16(vF_ext, vEF_opn),
+                                _mm_cmpeq_epi16(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = _mm_max_epi64_rpl(vEF_opn, vF_ext);*/
+                /*vF = _mm_max_epi16(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                vFa_ext = _mm_sub_epi64(vFa, vGapE);
-                vFa = _mm_max_epi64_rpl(vEF_opn, vFa_ext);
+                vFa_ext = _mm_subs_epi16(vFa, vGapE);
+                vFa = _mm_max_epi16(vEF_opn, vFa_ext);
                 vHp = _mm_load_si128(pvHLoad + i);
             }
         }
@@ -482,14 +403,14 @@ end:
     {
         __m128i vH = _mm_load_si128(pvHStore + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm_slli_si128 (vH, 8);
+            vH = _mm_slli_si128 (vH, 2);
         }
-        score = (int64_t) _mm_extract_epi64_rpl (vH, 1);
+        score = (int16_t) _mm_extract_epi16 (vH, 7);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi64_rpl(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
