@@ -16,8 +16,8 @@
 #include "parasail/memory.h"
 #include "parasail/internal_altivec.h"
 
-#define SG_SUFFIX _scan_altivec_128_64
-#define SG_SUFFIX_PROF _scan_profile_altivec_128_64
+#define SG_SUFFIX _scan_altivec_128_32
+#define SG_SUFFIX_PROF _scan_profile_altivec_128_32
 #include "sg_helper.h"
 
 
@@ -31,8 +31,10 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)_mm_extract_epi64(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)_mm_extract_epi64(vH, 1);
+    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 3);
 }
 #endif
 
@@ -43,21 +45,23 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int64_t)_mm_extract_epi64(vH, 0);
-    col[1*seglen+t] = (int64_t)_mm_extract_epi64(vH, 1);
+    col[0*seglen+t] = (int32_t)_mm_extract_epi32(vH, 0);
+    col[1*seglen+t] = (int32_t)_mm_extract_epi32(vH, 1);
+    col[2*seglen+t] = (int32_t)_mm_extract_epi32(vH, 2);
+    col[3*seglen+t] = (int32_t)_mm_extract_epi32(vH, 3);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_table_scan_altivec_128_64
-#define PNAME parasail_sg_flags_table_scan_profile_altivec_128_64
+#define FNAME parasail_sg_flags_table_scan_altivec_128_32
+#define PNAME parasail_sg_flags_table_scan_profile_altivec_128_32
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_rowcol_scan_altivec_128_64
-#define PNAME parasail_sg_flags_rowcol_scan_profile_altivec_128_64
+#define FNAME parasail_sg_flags_rowcol_scan_altivec_128_32
+#define PNAME parasail_sg_flags_rowcol_scan_profile_altivec_128_32
 #else
-#define FNAME parasail_sg_flags_scan_altivec_128_64
-#define PNAME parasail_sg_flags_scan_profile_altivec_128_64
+#define FNAME parasail_sg_flags_scan_altivec_128_32
+#define PNAME parasail_sg_flags_scan_profile_altivec_128_32
 #endif
 #endif
 
@@ -83,7 +87,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_altivec_128_64(s1, s1Len, matrix);
+    profile = parasail_profile_create_altivec_128_32(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -112,16 +116,16 @@ parasail_result_t* PNAME(
     int32_t position = 0;
     vec128i* restrict pvP = NULL;
     vec128i* restrict pvE = NULL;
-    int64_t* restrict boundary = NULL;
+    int32_t* restrict boundary = NULL;
     vec128i* restrict pvHt = NULL;
     vec128i* restrict pvH = NULL;
     vec128i* restrict pvGapper = NULL;
     vec128i vGapO;
     vec128i vGapE;
-    int64_t NEG_LIMIT = 0;
-    int64_t POS_LIMIT = 0;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
     vec128i vZero;
-    int64_t score = 0;
+    int32_t score = 0;
     vec128i vNegLimit;
     vec128i vPosLimit;
     vec128i vSaturationCheckMin;
@@ -134,7 +138,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->profile32.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -150,28 +154,28 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 2; /* number of values in vector unit */
+    segWidth = 4; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP = (vec128i*)profile->profile64.score;
-    vGapO = _mm_set1_epi64(open);
-    vGapE = _mm_set1_epi64(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    pvP = (vec128i*)profile->profile32.score;
+    vGapO = _mm_set1_epi32(open);
+    vGapE = _mm_set1_epi32(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
     vZero = _mm_setzero_si128();
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi64(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi64(POS_LIMIT);
+    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi32(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
-    vPosMask = _mm_cmpeq_epi64(_mm_set1_epi64(position),
-            _mm_set_epi64(0,1));
+    vPosMask = _mm_cmpeq_epi32(_mm_set1_epi32(position),
+            _mm_set_epi32(0,1,2,3));
     vNegInfFront = vZero;
-    vNegInfFront = _mm_insert_epi64(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm_add_epi64(vNegInfFront,
-            _mm_slli_si128(_mm_set1_epi64(-segLen*gap), 8));
+    vNegInfFront = _mm_insert_epi32(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm_add_epi32(vNegInfFront,
+            _mm_slli_si128(_mm_set1_epi32(-segLen*gap), 4));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -187,7 +191,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_SCAN
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -201,7 +205,7 @@ parasail_result_t* PNAME(
 
     /* initialize heap variables */
     pvE = parasail_memalign_vec128i(16, segLen);
-    boundary = parasail_memalign_int64_t(16, s2Len+1);
+    boundary = parasail_memalign_int32_t(16, s2Len+1);
     pvHt= parasail_memalign_vec128i(16, segLen);
     pvH = parasail_memalign_vec128i(16, segLen);
     pvGapper = parasail_memalign_vec128i(16, segLen);
@@ -218,13 +222,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            vec128i_64_t h;
-            vec128i_64_t e;
+            vec128i_32_t h;
+            vec128i_32_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                h.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                e.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
             }
             _mm_store_si128(&pvH[index], h.m);
             _mm_store_si128(&pvE[index], e.m);
@@ -237,17 +241,17 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
+            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
         }
     }
 
     {
-        vec128i vGapper = _mm_sub_epi64(vZero,vGapO);
+        vec128i vGapper = _mm_sub_epi32(vZero,vGapO);
         for (i=segLen-1; i>=0; --i) {
             _mm_store_si128(pvGapper+i, vGapper);
-            vGapper = _mm_sub_epi64(vGapper, vGapE);
+            vGapper = _mm_sub_epi32(vGapper, vGapE);
             /* long queries and/or large penalties will break the pseudo prefix scan */
-            vSaturationCheckMin = _mm_min_epi64(vSaturationCheckMin, vGapper);
+            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vGapper);
         }
     }
 
@@ -265,50 +269,50 @@ parasail_result_t* PNAME(
         /* calculate Ht */
         /* calculate F and H first pass */
         vHp = _mm_load_si128(pvH+(segLen-1));
-        vHp = _mm_slli_si128(vHp, 8);
-        vHp = _mm_insert_epi64(vHp, boundary[j], 0);
+        vHp = _mm_slli_si128(vHp, 4);
+        vHp = _mm_insert_epi32(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = _mm_sub_epi64(vNegLimit, pvGapper[0]);
+        vHt = _mm_sub_epi32(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         for (i=0; i<segLen; ++i) {
             vH = _mm_load_si128(pvH+i);
             vE = _mm_load_si128(pvE+i);
             vW = _mm_load_si128(pvW+i);
-            vE = _mm_max_epi64(
-                    _mm_sub_epi64(vE, vGapE),
-                    _mm_sub_epi64(vH, vGapO));
-            vHp = _mm_add_epi64(vHp, vW);
-            vF = _mm_max_epi64(vF, _mm_add_epi64(vHt, pvGapper[i]));
-            vHt = _mm_max_epi64(vE, vHp);
+            vE = _mm_max_epi32(
+                    _mm_sub_epi32(vE, vGapE),
+                    _mm_sub_epi32(vH, vGapO));
+            vHp = _mm_add_epi32(vHp, vW);
+            vF = _mm_max_epi32(vF, _mm_add_epi32(vHt, pvGapper[i]));
+            vHt = _mm_max_epi32(vE, vHp);
             _mm_store_si128(pvE+i, vE);
             _mm_store_si128(pvHt+i, vHt);
             vHp = vH;
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm_slli_si128(vHt, 8);
-        vHt = _mm_insert_epi64(vHt, boundary[j+1], 0);
-        vF = _mm_max_epi64(vF, _mm_add_epi64(vHt, pvGapper[0]));
+        vHt = _mm_slli_si128(vHt, 4);
+        vHt = _mm_insert_epi32(vHt, boundary[j+1], 0);
+        vF = _mm_max_epi32(vF, _mm_add_epi32(vHt, pvGapper[0]));
         for (i=0; i<segWidth-2; ++i) {
-            vec128i vFt = _mm_slli_si128(vF, 8);
-            vFt = _mm_add_epi64(vFt, vSegLenXgap);
-            vF = _mm_max_epi64(vF, vFt);
+            vec128i vFt = _mm_slli_si128(vF, 4);
+            vFt = _mm_add_epi32(vFt, vSegLenXgap);
+            vF = _mm_max_epi32(vF, vFt);
         }
 
         /* calculate final H */
-        vF = _mm_slli_si128(vF, 8);
-        vF = _mm_add_epi64(vF, vNegInfFront);
-        vH = _mm_max_epi64(vHt, vF);
+        vF = _mm_slli_si128(vF, 4);
+        vF = _mm_add_epi32(vF, vNegInfFront);
+        vH = _mm_max_epi32(vHt, vF);
         for (i=0; i<segLen; ++i) {
             vHt = _mm_load_si128(pvHt+i);
-            vF = _mm_max_epi64(
-                    _mm_sub_epi64(vF, vGapE),
-                    _mm_sub_epi64(vH, vGapO));
-            vH = _mm_max_epi64(vHt, vF);
+            vF = _mm_max_epi32(
+                    _mm_sub_epi32(vF, vGapE),
+                    _mm_sub_epi32(vH, vGapO));
+            vH = _mm_max_epi32(vHt, vF);
             _mm_store_si128(pvH+i, vH);
-            vSaturationCheckMin = _mm_min_epi64(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm_min_epi64(vSaturationCheckMin, vF);
-            vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
@@ -318,16 +322,16 @@ parasail_result_t* PNAME(
         {
             vec128i vCompare;
             vH = _mm_load_si128(pvH + offset);
-            vCompare = _mm_and_si128(vPosMask, _mm_cmpgt_epi64(vH, vMaxH));
-            vMaxH = _mm_max_epi64(vH, vMaxH);
+            vCompare = _mm_and_si128(vPosMask, _mm_cmpgt_epi32(vH, vMaxH));
+            vMaxH = _mm_max_epi32(vH, vMaxH);
             if (_mm_movemask_epi8(vCompare)) {
                 end_ref = j;
             }
 #ifdef PARASAIL_ROWCOL
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 8);
+                vH = _mm_slli_si128(vH, 4);
             }
-            result->rowcols->score_row[j] = (int64_t) _mm_extract_epi64 (vH, 1);
+            result->rowcols->score_row[j] = (int32_t) _mm_extract_epi32 (vH, 3);
 #endif
         }
     }
@@ -343,9 +347,9 @@ parasail_result_t* PNAME(
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm_slli_si128(vMaxH, 8);
+            vMaxH = _mm_slli_si128(vMaxH, 4);
         }
-        score = (int64_t) _mm_extract_epi64(vMaxH, 1);
+        score = (int32_t) _mm_extract_epi32(vMaxH, 3);
         end_query = s1Len-1;
     }
 
@@ -353,7 +357,7 @@ parasail_result_t* PNAME(
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int64_t *t = (int64_t*)pvH;
+        int32_t *t = (int32_t*)pvH;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -374,17 +378,17 @@ parasail_result_t* PNAME(
         {
             vec128i vH = _mm_load_si128(pvH + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 8);
+                vH = _mm_slli_si128(vH, 4);
             }
-            score = (int64_t) _mm_extract_epi64 (vH, 1);
+            score = (int32_t) _mm_extract_epi32 (vH, 3);
             end_ref = s2Len - 1;
             end_query = s1Len - 1;
         }
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
