@@ -16,8 +16,8 @@
 #include "parasail/memory.h"
 #include "parasail/internal_neon.h"
 
-#define SG_SUFFIX _scan_neon_128_16
-#define SG_SUFFIX_PROF _scan_profile_neon_128_16
+#define SG_SUFFIX _scan_neon_128_8
+#define SG_SUFFIX_PROF _scan_profile_neon_128_8
 #include "sg_helper.h"
 
 
@@ -31,14 +31,22 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 3);
-    array[1LL*(4*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 4);
-    array[1LL*(5*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 5);
-    array[1LL*(6*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 6);
-    array[1LL*(7*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 7);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  0);
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  1);
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  2);
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  3);
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  4);
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  5);
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  6);
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  7);
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  8);
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  9);
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 10);
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 11);
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 12);
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 13);
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 14);
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 15);
 }
 #endif
 
@@ -49,27 +57,35 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 0);
-    col[1*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 1);
-    col[2*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 2);
-    col[3*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 3);
-    col[4*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 4);
-    col[5*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 5);
-    col[6*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 6);
-    col[7*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 7);
+    col[ 0*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  0);
+    col[ 1*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  1);
+    col[ 2*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  2);
+    col[ 3*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  3);
+    col[ 4*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  4);
+    col[ 5*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  5);
+    col[ 6*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  6);
+    col[ 7*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  7);
+    col[ 8*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  8);
+    col[ 9*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  9);
+    col[10*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 10);
+    col[11*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 11);
+    col[12*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 12);
+    col[13*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 13);
+    col[14*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 14);
+    col[15*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 15);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_table_scan_neon_128_16
-#define PNAME parasail_sg_flags_table_scan_profile_neon_128_16
+#define FNAME parasail_sg_flags_table_scan_neon_128_8
+#define PNAME parasail_sg_flags_table_scan_profile_neon_128_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_rowcol_scan_neon_128_16
-#define PNAME parasail_sg_flags_rowcol_scan_profile_neon_128_16
+#define FNAME parasail_sg_flags_rowcol_scan_neon_128_8
+#define PNAME parasail_sg_flags_rowcol_scan_profile_neon_128_8
 #else
-#define FNAME parasail_sg_flags_scan_neon_128_16
-#define PNAME parasail_sg_flags_scan_profile_neon_128_16
+#define FNAME parasail_sg_flags_scan_neon_128_8
+#define PNAME parasail_sg_flags_scan_profile_neon_128_8
 #endif
 #endif
 
@@ -95,7 +111,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_neon_128_16(s1, s1Len, matrix);
+    profile = parasail_profile_create_neon_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -124,16 +140,16 @@ parasail_result_t* PNAME(
     int32_t position = 0;
     simde__m128i* restrict pvP = NULL;
     simde__m128i* restrict pvE = NULL;
-    int16_t* restrict boundary = NULL;
+    int8_t* restrict boundary = NULL;
     simde__m128i* restrict pvHt = NULL;
     simde__m128i* restrict pvH = NULL;
     simde__m128i* restrict pvGapper = NULL;
     simde__m128i vGapO;
     simde__m128i vGapE;
-    int16_t NEG_LIMIT = 0;
-    int16_t POS_LIMIT = 0;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
     simde__m128i vZero;
-    int16_t score = 0;
+    int8_t score = 0;
     simde__m128i vNegLimit;
     simde__m128i vPosLimit;
     simde__m128i vSaturationCheckMin;
@@ -146,7 +162,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile16.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -162,28 +178,28 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 8; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP = (simde__m128i*)profile->profile16.score;
-    vGapO = simde_mm_set1_epi16(open);
-    vGapE = simde_mm_set1_epi16(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    pvP = (simde__m128i*)profile->profile8.score;
+    vGapO = simde_mm_set1_epi8(open);
+    vGapE = simde_mm_set1_epi8(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
     vZero = simde_mm_setzero_si128();
     score = NEG_LIMIT;
-    vNegLimit = simde_mm_set1_epi16(NEG_LIMIT);
-    vPosLimit = simde_mm_set1_epi16(POS_LIMIT);
+    vNegLimit = simde_mm_set1_epi8(NEG_LIMIT);
+    vPosLimit = simde_mm_set1_epi8(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
-    vPosMask = simde_mm_cmpeq_epi16(simde_mm_set1_epi16(position),
-            simde_mm_set_epi16(0,1,2,3,4,5,6,7));
+    vPosMask = simde_mm_cmpeq_epi8(simde_mm_set1_epi8(position),
+            simde_mm_set_epi8(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15));
     vNegInfFront = vZero;
-    vNegInfFront = simde_mm_insert_epi16(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = simde_mm_adds_epi16(vNegInfFront,
-            simde_mm_slli_si128(simde_mm_set1_epi16(-segLen*gap), 2));
+    vNegInfFront = simde_mm_insert_epi8(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = simde_mm_adds_epi8(vNegInfFront,
+            simde_mm_slli_si128(simde_mm_set1_epi8(-segLen*gap), 1));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -199,7 +215,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_SCAN
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -213,7 +229,7 @@ parasail_result_t* PNAME(
 
     /* initialize heap variables */
     pvE = parasail_memalign_simde__m128i(16, segLen);
-    boundary = parasail_memalign_int16_t(16, s2Len+1);
+    boundary = parasail_memalign_int8_t(16, s2Len+1);
     pvHt= parasail_memalign_simde__m128i(16, segLen);
     pvH = parasail_memalign_simde__m128i(16, segLen);
     pvGapper = parasail_memalign_simde__m128i(16, segLen);
@@ -234,9 +250,9 @@ parasail_result_t* PNAME(
             simde__m128i e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.i16[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                h.i8[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
                 tmp = tmp - open;
-                e.i16[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                e.i8[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
             }
             simde_mm_store_si128(&pvH[index], h);
             simde_mm_store_si128(&pvE[index], e);
@@ -249,17 +265,17 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
+            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
         }
     }
 
     {
-        simde__m128i vGapper = simde_mm_subs_epi16(vZero,vGapO);
+        simde__m128i vGapper = simde_mm_subs_epi8(vZero,vGapO);
         for (i=segLen-1; i>=0; --i) {
             simde_mm_store_si128(pvGapper+i, vGapper);
-            vGapper = simde_mm_subs_epi16(vGapper, vGapE);
+            vGapper = simde_mm_subs_epi8(vGapper, vGapE);
             /* long queries and/or large penalties will break the pseudo prefix scan */
-            vSaturationCheckMin = simde_mm_min_epi16(vSaturationCheckMin, vGapper);
+            vSaturationCheckMin = simde_mm_min_epi8(vSaturationCheckMin, vGapper);
         }
     }
 
@@ -277,50 +293,50 @@ parasail_result_t* PNAME(
         /* calculate Ht */
         /* calculate F and H first pass */
         vHp = simde_mm_load_si128(pvH+(segLen-1));
-        vHp = simde_mm_slli_si128(vHp, 2);
-        vHp = simde_mm_insert_epi16(vHp, boundary[j], 0);
+        vHp = simde_mm_slli_si128(vHp, 1);
+        vHp = simde_mm_insert_epi8(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = simde_mm_subs_epi16(vNegLimit, pvGapper[0]);
+        vHt = simde_mm_subs_epi8(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         for (i=0; i<segLen; ++i) {
             vH = simde_mm_load_si128(pvH+i);
             vE = simde_mm_load_si128(pvE+i);
             vW = simde_mm_load_si128(pvW+i);
-            vE = simde_mm_max_epi16(
-                    simde_mm_subs_epi16(vE, vGapE),
-                    simde_mm_subs_epi16(vH, vGapO));
-            vHp = simde_mm_adds_epi16(vHp, vW);
-            vF = simde_mm_max_epi16(vF, simde_mm_adds_epi16(vHt, pvGapper[i]));
-            vHt = simde_mm_max_epi16(vE, vHp);
+            vE = simde_mm_max_epi8(
+                    simde_mm_subs_epi8(vE, vGapE),
+                    simde_mm_subs_epi8(vH, vGapO));
+            vHp = simde_mm_adds_epi8(vHp, vW);
+            vF = simde_mm_max_epi8(vF, simde_mm_adds_epi8(vHt, pvGapper[i]));
+            vHt = simde_mm_max_epi8(vE, vHp);
             simde_mm_store_si128(pvE+i, vE);
             simde_mm_store_si128(pvHt+i, vHt);
             vHp = vH;
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = simde_mm_slli_si128(vHt, 2);
-        vHt = simde_mm_insert_epi16(vHt, boundary[j+1], 0);
-        vF = simde_mm_max_epi16(vF, simde_mm_adds_epi16(vHt, pvGapper[0]));
+        vHt = simde_mm_slli_si128(vHt, 1);
+        vHt = simde_mm_insert_epi8(vHt, boundary[j+1], 0);
+        vF = simde_mm_max_epi8(vF, simde_mm_adds_epi8(vHt, pvGapper[0]));
         for (i=0; i<segWidth-2; ++i) {
-            simde__m128i vFt = simde_mm_slli_si128(vF, 2);
-            vFt = simde_mm_adds_epi16(vFt, vSegLenXgap);
-            vF = simde_mm_max_epi16(vF, vFt);
+            simde__m128i vFt = simde_mm_slli_si128(vF, 1);
+            vFt = simde_mm_adds_epi8(vFt, vSegLenXgap);
+            vF = simde_mm_max_epi8(vF, vFt);
         }
 
         /* calculate final H */
-        vF = simde_mm_slli_si128(vF, 2);
-        vF = simde_mm_adds_epi16(vF, vNegInfFront);
-        vH = simde_mm_max_epi16(vHt, vF);
+        vF = simde_mm_slli_si128(vF, 1);
+        vF = simde_mm_adds_epi8(vF, vNegInfFront);
+        vH = simde_mm_max_epi8(vHt, vF);
         for (i=0; i<segLen; ++i) {
             vHt = simde_mm_load_si128(pvHt+i);
-            vF = simde_mm_max_epi16(
-                    simde_mm_subs_epi16(vF, vGapE),
-                    simde_mm_subs_epi16(vH, vGapO));
-            vH = simde_mm_max_epi16(vHt, vF);
+            vF = simde_mm_max_epi8(
+                    simde_mm_subs_epi8(vF, vGapE),
+                    simde_mm_subs_epi8(vH, vGapO));
+            vH = simde_mm_max_epi8(vHt, vF);
             simde_mm_store_si128(pvH+i, vH);
-            vSaturationCheckMin = simde_mm_min_epi16(vSaturationCheckMin, vH);
-            vSaturationCheckMin = simde_mm_min_epi16(vSaturationCheckMin, vF);
-            vSaturationCheckMax = simde_mm_max_epi16(vSaturationCheckMax, vH);
+            vSaturationCheckMin = simde_mm_min_epi8(vSaturationCheckMin, vH);
+            vSaturationCheckMin = simde_mm_min_epi8(vSaturationCheckMin, vF);
+            vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vH);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
@@ -330,16 +346,16 @@ parasail_result_t* PNAME(
         {
             simde__m128i vCompare;
             vH = simde_mm_load_si128(pvH + offset);
-            vCompare = simde_mm_and_si128(vPosMask, simde_mm_cmpgt_epi16(vH, vMaxH));
-            vMaxH = simde_mm_max_epi16(vH, vMaxH);
+            vCompare = simde_mm_and_si128(vPosMask, simde_mm_cmpgt_epi8(vH, vMaxH));
+            vMaxH = simde_mm_max_epi8(vH, vMaxH);
             if (simde_mm_movemask_epi8(vCompare)) {
                 end_ref = j;
             }
 #ifdef PARASAIL_ROWCOL
             for (k=0; k<position; ++k) {
-                vH = simde_mm_slli_si128(vH, 2);
+                vH = simde_mm_slli_si128(vH, 1);
             }
-            result->rowcols->score_row[j] = (int16_t) simde_mm_extract_epi16 (vH, 7);
+            result->rowcols->score_row[j] = (int8_t) simde_mm_extract_epi8 (vH, 15);
 #endif
         }
     }
@@ -355,9 +371,9 @@ parasail_result_t* PNAME(
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = simde_mm_slli_si128(vMaxH, 2);
+            vMaxH = simde_mm_slli_si128(vMaxH, 1);
         }
-        score = (int16_t) simde_mm_extract_epi16(vMaxH, 7);
+        score = (int8_t) simde_mm_extract_epi8(vMaxH, 15);
         end_query = s1Len-1;
     }
 
@@ -365,7 +381,7 @@ parasail_result_t* PNAME(
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int16_t *t = (int16_t*)pvH;
+        int8_t *t = (int8_t*)pvH;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -386,17 +402,17 @@ parasail_result_t* PNAME(
         {
             simde__m128i vH = simde_mm_load_si128(pvH + offset);
             for (k=0; k<position; ++k) {
-                vH = simde_mm_slli_si128(vH, 2);
+                vH = simde_mm_slli_si128(vH, 1);
             }
-            score = (int16_t) simde_mm_extract_epi16 (vH, 7);
+            score = (int8_t) simde_mm_extract_epi8 (vH, 15);
             end_ref = s2Len - 1;
             end_query = s1Len - 1;
         }
     }
 
     if (simde_mm_movemask_epi8(simde_mm_or_si128(
-            simde_mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
-            simde_mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            simde_mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
+            simde_mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
