@@ -14,18 +14,44 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
-#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
 #include "parasail/memory.h"
 #include "parasail/internal_sse.h"
 
-#define SG_SUFFIX _scan_sse41_128_8
-#define SG_SUFFIX_PROF _scan_profile_sse41_128_8
+#define SG_SUFFIX _scan_sse2_128_8
+#define SG_SUFFIX_PROF _scan_profile_sse2_128_8
 #include "sg_helper.h"
 
 
+static inline __m128i _mm_insert_epi8_rpl(__m128i a, int8_t i, const int imm) {
+    __m128i_8_t A;
+    A.m = a;
+    A.v[imm] = i;
+    return A.m;
+}
+
+static inline __m128i _mm_max_epi8_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi8(a, b);
+    a = _mm_and_si128(a, mask);
+    b = _mm_andnot_si128(mask, b);
+    return _mm_or_si128(a, b);
+}
+
+static inline int8_t _mm_extract_epi8_rpl(__m128i a, const int imm) {
+    __m128i_8_t A;
+    A.m = a;
+    return A.v[imm];
+}
+
+static inline __m128i _mm_min_epi8_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi8(b, a);
+    a = _mm_and_si128(a, mask);
+    b = _mm_andnot_si128(mask, b);
+    return _mm_or_si128(a, b);
+}
+
 
 #ifdef PARASAIL_TABLE
 static inline void arr_store_si128(
@@ -36,22 +62,22 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  0);
-    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  1);
-    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  2);
-    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  3);
-    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  4);
-    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  5);
-    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  6);
-    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  7);
-    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  8);
-    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  9);
-    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 10);
-    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 11);
-    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 12);
-    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 13);
-    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 14);
-    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 15);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  0);
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  1);
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  2);
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  3);
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  4);
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  5);
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  6);
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  7);
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  8);
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  9);
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 10);
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 11);
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 12);
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 13);
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 14);
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 15);
 }
 #endif
 
@@ -62,35 +88,35 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8(vH,  0);
-    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8(vH,  1);
-    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8(vH,  2);
-    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8(vH,  3);
-    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8(vH,  4);
-    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8(vH,  5);
-    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8(vH,  6);
-    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8(vH,  7);
-    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8(vH,  8);
-    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8(vH,  9);
-    col[10*seglen+t] = (int8_t)_mm_extract_epi8(vH, 10);
-    col[11*seglen+t] = (int8_t)_mm_extract_epi8(vH, 11);
-    col[12*seglen+t] = (int8_t)_mm_extract_epi8(vH, 12);
-    col[13*seglen+t] = (int8_t)_mm_extract_epi8(vH, 13);
-    col[14*seglen+t] = (int8_t)_mm_extract_epi8(vH, 14);
-    col[15*seglen+t] = (int8_t)_mm_extract_epi8(vH, 15);
+    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  0);
+    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  1);
+    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  2);
+    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  3);
+    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  4);
+    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  5);
+    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  6);
+    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  7);
+    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  8);
+    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  9);
+    col[10*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 10);
+    col[11*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 11);
+    col[12*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 12);
+    col[13*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 13);
+    col[14*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 14);
+    col[15*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 15);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_table_scan_sse41_128_8
-#define PNAME parasail_sg_flags_table_scan_profile_sse41_128_8
+#define FNAME parasail_sg_flags_table_scan_sse2_128_8
+#define PNAME parasail_sg_flags_table_scan_profile_sse2_128_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_rowcol_scan_sse41_128_8
-#define PNAME parasail_sg_flags_rowcol_scan_profile_sse41_128_8
+#define FNAME parasail_sg_flags_rowcol_scan_sse2_128_8
+#define PNAME parasail_sg_flags_rowcol_scan_profile_sse2_128_8
 #else
-#define FNAME parasail_sg_flags_scan_sse41_128_8
-#define PNAME parasail_sg_flags_scan_profile_sse41_128_8
+#define FNAME parasail_sg_flags_scan_sse2_128_8
+#define PNAME parasail_sg_flags_scan_profile_sse2_128_8
 #endif
 #endif
 
@@ -202,7 +228,7 @@ parasail_result_t* PNAME(
     vPosMask = _mm_cmpeq_epi8(_mm_set1_epi8(position),
             _mm_set_epi8(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15));
     vNegInfFront = vZero;
-    vNegInfFront = _mm_insert_epi8(vNegInfFront, NEG_LIMIT, 0);
+    vNegInfFront = _mm_insert_epi8_rpl(vNegInfFront, NEG_LIMIT, 0);
     vSegLenXgap = _mm_adds_epi8(vNegInfFront,
             _mm_slli_si128(_mm_set1_epi8(-segLen*gap), 1));
 
@@ -280,7 +306,7 @@ parasail_result_t* PNAME(
             _mm_store_si128(pvGapper+i, vGapper);
             vGapper = _mm_subs_epi8(vGapper, vGapE);
             /* long queries and/or large penalties will break the pseudo prefix scan */
-            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vGapper);
+            vSaturationCheckMin = _mm_min_epi8_rpl(vSaturationCheckMin, vGapper);
         }
     }
 
@@ -299,7 +325,7 @@ parasail_result_t* PNAME(
         /* calculate F and H first pass */
         vHp = _mm_load_si128(pvH+(segLen-1));
         vHp = _mm_slli_si128(vHp, 1);
-        vHp = _mm_insert_epi8(vHp, boundary[j], 0);
+        vHp = _mm_insert_epi8_rpl(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         vHt = _mm_subs_epi8(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
@@ -307,12 +333,12 @@ parasail_result_t* PNAME(
             vH = _mm_load_si128(pvH+i);
             vE = _mm_load_si128(pvE+i);
             vW = _mm_load_si128(pvW+i);
-            vE = _mm_max_epi8(
+            vE = _mm_max_epi8_rpl(
                     _mm_subs_epi8(vE, vGapE),
                     _mm_subs_epi8(vH, vGapO));
             vHp = _mm_adds_epi8(vHp, vW);
-            vF = _mm_max_epi8(vF, _mm_adds_epi8(vHt, pvGapper[i]));
-            vHt = _mm_max_epi8(vE, vHp);
+            vF = _mm_max_epi8_rpl(vF, _mm_adds_epi8(vHt, pvGapper[i]));
+            vHt = _mm_max_epi8_rpl(vE, vHp);
             _mm_store_si128(pvE+i, vE);
             _mm_store_si128(pvHt+i, vHt);
             vHp = vH;
@@ -320,28 +346,28 @@ parasail_result_t* PNAME(
 
         /* pseudo prefix scan on F and H */
         vHt = _mm_slli_si128(vHt, 1);
-        vHt = _mm_insert_epi8(vHt, boundary[j+1], 0);
-        vF = _mm_max_epi8(vF, _mm_adds_epi8(vHt, pvGapper[0]));
+        vHt = _mm_insert_epi8_rpl(vHt, boundary[j+1], 0);
+        vF = _mm_max_epi8_rpl(vF, _mm_adds_epi8(vHt, pvGapper[0]));
         for (i=0; i<segWidth-2; ++i) {
             __m128i vFt = _mm_slli_si128(vF, 1);
             vFt = _mm_adds_epi8(vFt, vSegLenXgap);
-            vF = _mm_max_epi8(vF, vFt);
+            vF = _mm_max_epi8_rpl(vF, vFt);
         }
 
         /* calculate final H */
         vF = _mm_slli_si128(vF, 1);
         vF = _mm_adds_epi8(vF, vNegInfFront);
-        vH = _mm_max_epi8(vHt, vF);
+        vH = _mm_max_epi8_rpl(vHt, vF);
         for (i=0; i<segLen; ++i) {
             vHt = _mm_load_si128(pvHt+i);
-            vF = _mm_max_epi8(
+            vF = _mm_max_epi8_rpl(
                     _mm_subs_epi8(vF, vGapE),
                     _mm_subs_epi8(vH, vGapO));
-            vH = _mm_max_epi8(vHt, vF);
+            vH = _mm_max_epi8_rpl(vHt, vF);
             _mm_store_si128(pvH+i, vH);
-            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vF);
-            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm_min_epi8_rpl(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm_min_epi8_rpl(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vH);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
@@ -352,7 +378,7 @@ parasail_result_t* PNAME(
             __m128i vCompare;
             vH = _mm_load_si128(pvH + offset);
             vCompare = _mm_and_si128(vPosMask, _mm_cmpgt_epi8(vH, vMaxH));
-            vMaxH = _mm_max_epi8(vH, vMaxH);
+            vMaxH = _mm_max_epi8_rpl(vH, vMaxH);
             if (_mm_movemask_epi8(vCompare)) {
                 end_ref = j;
             }
@@ -360,7 +386,7 @@ parasail_result_t* PNAME(
             for (k=0; k<position; ++k) {
                 vH = _mm_slli_si128(vH, 1);
             }
-            result->rowcols->score_row[j] = (int8_t) _mm_extract_epi8 (vH, 15);
+            result->rowcols->score_row[j] = (int8_t) _mm_extract_epi8_rpl (vH, 15);
 #endif
         }
     }
@@ -378,7 +404,7 @@ parasail_result_t* PNAME(
         for (k=0; k<position; ++k) {
             vMaxH = _mm_slli_si128(vMaxH, 1);
         }
-        score = (int8_t) _mm_extract_epi8(vMaxH, 15);
+        score = (int8_t) _mm_extract_epi8_rpl(vMaxH, 15);
         end_query = s1Len-1;
     }
 
@@ -409,7 +435,7 @@ parasail_result_t* PNAME(
             for (k=0; k<position; ++k) {
                 vH = _mm_slli_si128(vH, 1);
             }
-            score = (int8_t) _mm_extract_epi8 (vH, 15);
+            score = (int8_t) _mm_extract_epi8_rpl (vH, 15);
             end_ref = s2Len - 1;
             end_query = s1Len - 1;
         }
