@@ -13,7 +13,6 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
-#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -21,10 +20,16 @@
 #include "parasail/internal_sse.h"
 
 #define SG_STATS
-#define SG_SUFFIX _diag_sse41_128_8
+#define SG_SUFFIX _diag_sse2_128_16
 #include "sg_helper.h"
 
 
+static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
+    a = _mm_andnot_si128(mask, a);
+    a = _mm_or_si128(a, _mm_and_si128(mask, b));
+    return a;
+}
+
 
 #ifdef PARASAIL_TABLE
 static inline void arr_store_si128(
@@ -36,52 +41,28 @@ static inline void arr_store_si128(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int8_t)_mm_extract_epi8(vWH, 15);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int16_t)_mm_extract_epi16(vWH, 7);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int8_t)_mm_extract_epi8(vWH, 14);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int16_t)_mm_extract_epi16(vWH, 6);
     }
     if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
-        array[1LL*(i+2)*s2Len + (j-2)] = (int8_t)_mm_extract_epi8(vWH, 13);
+        array[1LL*(i+2)*s2Len + (j-2)] = (int16_t)_mm_extract_epi16(vWH, 5);
     }
     if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
-        array[1LL*(i+3)*s2Len + (j-3)] = (int8_t)_mm_extract_epi8(vWH, 12);
+        array[1LL*(i+3)*s2Len + (j-3)] = (int16_t)_mm_extract_epi16(vWH, 4);
     }
     if (0 <= i+4 && i+4 < s1Len && 0 <= j-4 && j-4 < s2Len) {
-        array[1LL*(i+4)*s2Len + (j-4)] = (int8_t)_mm_extract_epi8(vWH, 11);
+        array[1LL*(i+4)*s2Len + (j-4)] = (int16_t)_mm_extract_epi16(vWH, 3);
     }
     if (0 <= i+5 && i+5 < s1Len && 0 <= j-5 && j-5 < s2Len) {
-        array[1LL*(i+5)*s2Len + (j-5)] = (int8_t)_mm_extract_epi8(vWH, 10);
+        array[1LL*(i+5)*s2Len + (j-5)] = (int16_t)_mm_extract_epi16(vWH, 2);
     }
     if (0 <= i+6 && i+6 < s1Len && 0 <= j-6 && j-6 < s2Len) {
-        array[1LL*(i+6)*s2Len + (j-6)] = (int8_t)_mm_extract_epi8(vWH, 9);
+        array[1LL*(i+6)*s2Len + (j-6)] = (int16_t)_mm_extract_epi16(vWH, 1);
     }
     if (0 <= i+7 && i+7 < s1Len && 0 <= j-7 && j-7 < s2Len) {
-        array[1LL*(i+7)*s2Len + (j-7)] = (int8_t)_mm_extract_epi8(vWH, 8);
-    }
-    if (0 <= i+8 && i+8 < s1Len && 0 <= j-8 && j-8 < s2Len) {
-        array[1LL*(i+8)*s2Len + (j-8)] = (int8_t)_mm_extract_epi8(vWH, 7);
-    }
-    if (0 <= i+9 && i+9 < s1Len && 0 <= j-9 && j-9 < s2Len) {
-        array[1LL*(i+9)*s2Len + (j-9)] = (int8_t)_mm_extract_epi8(vWH, 6);
-    }
-    if (0 <= i+10 && i+10 < s1Len && 0 <= j-10 && j-10 < s2Len) {
-        array[1LL*(i+10)*s2Len + (j-10)] = (int8_t)_mm_extract_epi8(vWH, 5);
-    }
-    if (0 <= i+11 && i+11 < s1Len && 0 <= j-11 && j-11 < s2Len) {
-        array[1LL*(i+11)*s2Len + (j-11)] = (int8_t)_mm_extract_epi8(vWH, 4);
-    }
-    if (0 <= i+12 && i+12 < s1Len && 0 <= j-12 && j-12 < s2Len) {
-        array[1LL*(i+12)*s2Len + (j-12)] = (int8_t)_mm_extract_epi8(vWH, 3);
-    }
-    if (0 <= i+13 && i+13 < s1Len && 0 <= j-13 && j-13 < s2Len) {
-        array[1LL*(i+13)*s2Len + (j-13)] = (int8_t)_mm_extract_epi8(vWH, 2);
-    }
-    if (0 <= i+14 && i+14 < s1Len && 0 <= j-14 && j-14 < s2Len) {
-        array[1LL*(i+14)*s2Len + (j-14)] = (int8_t)_mm_extract_epi8(vWH, 1);
-    }
-    if (0 <= i+15 && i+15 < s1Len && 0 <= j-15 && j-15 < s2Len) {
-        array[1LL*(i+15)*s2Len + (j-15)] = (int8_t)_mm_extract_epi8(vWH, 0);
+        array[1LL*(i+7)*s2Len + (j-7)] = (int16_t)_mm_extract_epi16(vWH, 0);
     }
 }
 #endif
@@ -97,111 +78,63 @@ static inline void arr_store_rowcol(
         int32_t s2Len)
 {
     if (i+0 == s1Len-1 && 0 <= j-0 && j-0 < s2Len) {
-        row[j-0] = (int8_t)_mm_extract_epi8(vWH, 15);
+        row[j-0] = (int16_t)_mm_extract_epi16(vWH, 7);
     }
     if (j-0 == s2Len-1 && 0 <= i+0 && i+0 < s1Len) {
-        col[(i+0)] = (int8_t)_mm_extract_epi8(vWH, 15);
+        col[(i+0)] = (int16_t)_mm_extract_epi16(vWH, 7);
     }
     if (i+1 == s1Len-1 && 0 <= j-1 && j-1 < s2Len) {
-        row[j-1] = (int8_t)_mm_extract_epi8(vWH, 14);
+        row[j-1] = (int16_t)_mm_extract_epi16(vWH, 6);
     }
     if (j-1 == s2Len-1 && 0 <= i+1 && i+1 < s1Len) {
-        col[(i+1)] = (int8_t)_mm_extract_epi8(vWH, 14);
+        col[(i+1)] = (int16_t)_mm_extract_epi16(vWH, 6);
     }
     if (i+2 == s1Len-1 && 0 <= j-2 && j-2 < s2Len) {
-        row[j-2] = (int8_t)_mm_extract_epi8(vWH, 13);
+        row[j-2] = (int16_t)_mm_extract_epi16(vWH, 5);
     }
     if (j-2 == s2Len-1 && 0 <= i+2 && i+2 < s1Len) {
-        col[(i+2)] = (int8_t)_mm_extract_epi8(vWH, 13);
+        col[(i+2)] = (int16_t)_mm_extract_epi16(vWH, 5);
     }
     if (i+3 == s1Len-1 && 0 <= j-3 && j-3 < s2Len) {
-        row[j-3] = (int8_t)_mm_extract_epi8(vWH, 12);
+        row[j-3] = (int16_t)_mm_extract_epi16(vWH, 4);
     }
     if (j-3 == s2Len-1 && 0 <= i+3 && i+3 < s1Len) {
-        col[(i+3)] = (int8_t)_mm_extract_epi8(vWH, 12);
+        col[(i+3)] = (int16_t)_mm_extract_epi16(vWH, 4);
     }
     if (i+4 == s1Len-1 && 0 <= j-4 && j-4 < s2Len) {
-        row[j-4] = (int8_t)_mm_extract_epi8(vWH, 11);
+        row[j-4] = (int16_t)_mm_extract_epi16(vWH, 3);
     }
     if (j-4 == s2Len-1 && 0 <= i+4 && i+4 < s1Len) {
-        col[(i+4)] = (int8_t)_mm_extract_epi8(vWH, 11);
+        col[(i+4)] = (int16_t)_mm_extract_epi16(vWH, 3);
     }
     if (i+5 == s1Len-1 && 0 <= j-5 && j-5 < s2Len) {
-        row[j-5] = (int8_t)_mm_extract_epi8(vWH, 10);
+        row[j-5] = (int16_t)_mm_extract_epi16(vWH, 2);
     }
     if (j-5 == s2Len-1 && 0 <= i+5 && i+5 < s1Len) {
-        col[(i+5)] = (int8_t)_mm_extract_epi8(vWH, 10);
+        col[(i+5)] = (int16_t)_mm_extract_epi16(vWH, 2);
     }
     if (i+6 == s1Len-1 && 0 <= j-6 && j-6 < s2Len) {
-        row[j-6] = (int8_t)_mm_extract_epi8(vWH, 9);
+        row[j-6] = (int16_t)_mm_extract_epi16(vWH, 1);
     }
     if (j-6 == s2Len-1 && 0 <= i+6 && i+6 < s1Len) {
-        col[(i+6)] = (int8_t)_mm_extract_epi8(vWH, 9);
+        col[(i+6)] = (int16_t)_mm_extract_epi16(vWH, 1);
     }
     if (i+7 == s1Len-1 && 0 <= j-7 && j-7 < s2Len) {
-        row[j-7] = (int8_t)_mm_extract_epi8(vWH, 8);
+        row[j-7] = (int16_t)_mm_extract_epi16(vWH, 0);
     }
     if (j-7 == s2Len-1 && 0 <= i+7 && i+7 < s1Len) {
-        col[(i+7)] = (int8_t)_mm_extract_epi8(vWH, 8);
-    }
-    if (i+8 == s1Len-1 && 0 <= j-8 && j-8 < s2Len) {
-        row[j-8] = (int8_t)_mm_extract_epi8(vWH, 7);
-    }
-    if (j-8 == s2Len-1 && 0 <= i+8 && i+8 < s1Len) {
-        col[(i+8)] = (int8_t)_mm_extract_epi8(vWH, 7);
-    }
-    if (i+9 == s1Len-1 && 0 <= j-9 && j-9 < s2Len) {
-        row[j-9] = (int8_t)_mm_extract_epi8(vWH, 6);
-    }
-    if (j-9 == s2Len-1 && 0 <= i+9 && i+9 < s1Len) {
-        col[(i+9)] = (int8_t)_mm_extract_epi8(vWH, 6);
-    }
-    if (i+10 == s1Len-1 && 0 <= j-10 && j-10 < s2Len) {
-        row[j-10] = (int8_t)_mm_extract_epi8(vWH, 5);
-    }
-    if (j-10 == s2Len-1 && 0 <= i+10 && i+10 < s1Len) {
-        col[(i+10)] = (int8_t)_mm_extract_epi8(vWH, 5);
-    }
-    if (i+11 == s1Len-1 && 0 <= j-11 && j-11 < s2Len) {
-        row[j-11] = (int8_t)_mm_extract_epi8(vWH, 4);
-    }
-    if (j-11 == s2Len-1 && 0 <= i+11 && i+11 < s1Len) {
-        col[(i+11)] = (int8_t)_mm_extract_epi8(vWH, 4);
-    }
-    if (i+12 == s1Len-1 && 0 <= j-12 && j-12 < s2Len) {
-        row[j-12] = (int8_t)_mm_extract_epi8(vWH, 3);
-    }
-    if (j-12 == s2Len-1 && 0 <= i+12 && i+12 < s1Len) {
-        col[(i+12)] = (int8_t)_mm_extract_epi8(vWH, 3);
-    }
-    if (i+13 == s1Len-1 && 0 <= j-13 && j-13 < s2Len) {
-        row[j-13] = (int8_t)_mm_extract_epi8(vWH, 2);
-    }
-    if (j-13 == s2Len-1 && 0 <= i+13 && i+13 < s1Len) {
-        col[(i+13)] = (int8_t)_mm_extract_epi8(vWH, 2);
-    }
-    if (i+14 == s1Len-1 && 0 <= j-14 && j-14 < s2Len) {
-        row[j-14] = (int8_t)_mm_extract_epi8(vWH, 1);
-    }
-    if (j-14 == s2Len-1 && 0 <= i+14 && i+14 < s1Len) {
-        col[(i+14)] = (int8_t)_mm_extract_epi8(vWH, 1);
-    }
-    if (i+15 == s1Len-1 && 0 <= j-15 && j-15 < s2Len) {
-        row[j-15] = (int8_t)_mm_extract_epi8(vWH, 0);
-    }
-    if (j-15 == s2Len-1 && 0 <= i+15 && i+15 < s1Len) {
-        col[(i+15)] = (int8_t)_mm_extract_epi8(vWH, 0);
+        col[(i+7)] = (int16_t)_mm_extract_epi16(vWH, 0);
     }
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_stats_table_diag_sse41_128_8
+#define FNAME parasail_sg_flags_stats_table_diag_sse2_128_16
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_stats_rowcol_diag_sse41_128_8
+#define FNAME parasail_sg_flags_stats_rowcol_diag_sse2_128_16
 #else
-#define FNAME parasail_sg_flags_stats_diag_sse41_128_8
+#define FNAME parasail_sg_flags_stats_diag_sse2_128_16
 #endif
 #endif
 
@@ -218,36 +151,36 @@ parasail_result_t* FNAME(
     int32_t s1Len = 0;
     int32_t s1Len_PAD = 0;
     int32_t s2Len_PAD = 0;
-    int8_t * restrict s1 = NULL;
-    int8_t * restrict s2B = NULL;
-    int8_t * restrict _H_pr = NULL;
-    int8_t * restrict _HM_pr = NULL;
-    int8_t * restrict _HS_pr = NULL;
-    int8_t * restrict _HL_pr = NULL;
-    int8_t * restrict _F_pr = NULL;
-    int8_t * restrict _FM_pr = NULL;
-    int8_t * restrict _FS_pr = NULL;
-    int8_t * restrict _FL_pr = NULL;
-    int8_t * restrict s2 = NULL;
-    int8_t * restrict H_pr = NULL;
-    int8_t * restrict HM_pr = NULL;
-    int8_t * restrict HS_pr = NULL;
-    int8_t * restrict HL_pr = NULL;
-    int8_t * restrict F_pr = NULL;
-    int8_t * restrict FM_pr = NULL;
-    int8_t * restrict FS_pr = NULL;
-    int8_t * restrict FL_pr = NULL;
+    int16_t * restrict s1 = NULL;
+    int16_t * restrict s2B = NULL;
+    int16_t * restrict _H_pr = NULL;
+    int16_t * restrict _HM_pr = NULL;
+    int16_t * restrict _HS_pr = NULL;
+    int16_t * restrict _HL_pr = NULL;
+    int16_t * restrict _F_pr = NULL;
+    int16_t * restrict _FM_pr = NULL;
+    int16_t * restrict _FS_pr = NULL;
+    int16_t * restrict _FL_pr = NULL;
+    int16_t * restrict s2 = NULL;
+    int16_t * restrict H_pr = NULL;
+    int16_t * restrict HM_pr = NULL;
+    int16_t * restrict HS_pr = NULL;
+    int16_t * restrict HL_pr = NULL;
+    int16_t * restrict F_pr = NULL;
+    int16_t * restrict FM_pr = NULL;
+    int16_t * restrict FS_pr = NULL;
+    int16_t * restrict FL_pr = NULL;
     parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    int8_t NEG_LIMIT = 0;
-    int8_t POS_LIMIT = 0;
-    int8_t score = 0;
-    int8_t matches = 0;
-    int8_t similar = 0;
-    int8_t length = 0;
+    int16_t NEG_LIMIT = 0;
+    int16_t POS_LIMIT = 0;
+    int16_t score = 0;
+    int16_t matches = 0;
+    int16_t similar = 0;
+    int16_t length = 0;
     __m128i vNegLimit;
     __m128i vPosLimit;
     __m128i vSaturationCheckMin;
@@ -297,7 +230,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize stack variables */
-    N = 16; /* number of values in vector */
+    N = 8; /* number of values in vector */
     PAD = N-1;
     PAD2 = PAD*2;
     s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
@@ -307,26 +240,26 @@ parasail_result_t* FNAME(
     j = 0;
     end_query = s1Len-1;
     end_ref = s2Len-1;
-    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
-    POS_LIMIT = INT8_MAX - matrix->max - 1;
+    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
+    POS_LIMIT = INT16_MAX - matrix->max - 1;
     score = NEG_LIMIT;
     matches = NEG_LIMIT;
     similar = NEG_LIMIT;
     length = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi8(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi8(POS_LIMIT);
+    vNegLimit = _mm_set1_epi16(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi16(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vNegInf = _mm_set1_epi8(NEG_LIMIT);
-    vOpen = _mm_set1_epi8(open);
-    vGap  = _mm_set1_epi8(gap);
-    vZero = _mm_set1_epi8(0);
-    vOne = _mm_set1_epi8(1);
-    vN = _mm_set1_epi8(N);
-    vGapN = s1_beg ? _mm_set1_epi8(0) : _mm_set1_epi8(gap*N);
-    vNegOne = _mm_set1_epi8(-1);
-    vI = _mm_set_epi8(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15);
-    vJreset = _mm_set_epi8(0,-1,-2,-3,-4,-5,-6,-7,-8,-9,-10,-11,-12,-13,-14,-15);
+    vNegInf = _mm_set1_epi16(NEG_LIMIT);
+    vOpen = _mm_set1_epi16(open);
+    vGap  = _mm_set1_epi16(gap);
+    vZero = _mm_set1_epi16(0);
+    vOne = _mm_set1_epi16(1);
+    vN = _mm_set1_epi16(N);
+    vGapN = s1_beg ? _mm_set1_epi16(0) : _mm_set1_epi16(gap*N);
+    vNegOne = _mm_set1_epi16(-1);
+    vI = _mm_set_epi16(0,1,2,3,4,5,6,7);
+    vJreset = _mm_set_epi16(0,-1,-2,-3,-4,-5,-6,-7);
     vMaxHRow = vNegInf;
     vMaxMRow = vNegInf;
     vMaxSRow = vNegInf;
@@ -341,11 +274,11 @@ parasail_result_t* FNAME(
     vLastValL = vNegInf;
     vEndI = vNegInf;
     vEndJ = vNegInf;
-    vILimit = _mm_set1_epi8(s1Len);
-    vILimit1 = _mm_subs_epi8(vILimit, vOne);
-    vJLimit = _mm_set1_epi8(s2Len);
-    vJLimit1 = _mm_subs_epi8(vJLimit, vOne);
-    vIBoundary = s1_beg ? _mm_set1_epi8(0) : _mm_set_epi8(
+    vILimit = _mm_set1_epi16(s1Len);
+    vILimit1 = _mm_subs_epi16(vILimit, vOne);
+    vJLimit = _mm_set1_epi16(s2Len);
+    vJLimit1 = _mm_subs_epi16(vJLimit, vOne);
+    vIBoundary = s1_beg ? _mm_set1_epi16(0) : _mm_set_epi16(
             -open-0*gap,
             -open-1*gap,
             -open-2*gap,
@@ -353,15 +286,7 @@ parasail_result_t* FNAME(
             -open-4*gap,
             -open-5*gap,
             -open-6*gap,
-            -open-7*gap,
-            -open-8*gap,
-            -open-9*gap,
-            -open-10*gap,
-            -open-11*gap,
-            -open-12*gap,
-            -open-13*gap,
-            -open-14*gap,
-            -open-15*gap);
+            -open-7*gap);
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -378,7 +303,7 @@ parasail_result_t* FNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_DIAG
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
+        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -391,16 +316,16 @@ parasail_result_t* FNAME(
 #endif
 
     /* initialize heap variables */
-    s1     = parasail_memalign_int8_t(16, s1Len+PAD);
-    s2B    = parasail_memalign_int8_t(16, s2Len+PAD2);
-    _H_pr  = parasail_memalign_int8_t(16, s2Len+PAD2);
-    _HM_pr = parasail_memalign_int8_t(16, s2Len+PAD2);
-    _HS_pr = parasail_memalign_int8_t(16, s2Len+PAD2);
-    _HL_pr = parasail_memalign_int8_t(16, s2Len+PAD2);
-    _F_pr  = parasail_memalign_int8_t(16, s2Len+PAD2);
-    _FM_pr = parasail_memalign_int8_t(16, s2Len+PAD2);
-    _FS_pr = parasail_memalign_int8_t(16, s2Len+PAD2);
-    _FL_pr = parasail_memalign_int8_t(16, s2Len+PAD2);
+    s1     = parasail_memalign_int16_t(16, s1Len+PAD);
+    s2B    = parasail_memalign_int16_t(16, s2Len+PAD2);
+    _H_pr  = parasail_memalign_int16_t(16, s2Len+PAD2);
+    _HM_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
+    _HS_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
+    _HL_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
+    _F_pr  = parasail_memalign_int16_t(16, s2Len+PAD2);
+    _FM_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
+    _FS_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
+    _FL_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
     s2 = s2B+PAD; /* will allow later for negative indices */
     H_pr = _H_pr+PAD;
     HM_pr = _HM_pr+PAD;
@@ -519,7 +444,7 @@ parasail_result_t* FNAME(
         __m128i vFS = vZero;
         __m128i vFL = vZero;
         __m128i vJ = vJreset;
-        __m128i vs1 = _mm_set_epi8(
+        __m128i vs1 = _mm_set_epi16(
                 s1[i+0],
                 s1[i+1],
                 s1[i+2],
@@ -527,15 +452,7 @@ parasail_result_t* FNAME(
                 s1[i+4],
                 s1[i+5],
                 s1[i+6],
-                s1[i+7],
-                s1[i+8],
-                s1[i+9],
-                s1[i+10],
-                s1[i+11],
-                s1[i+12],
-                s1[i+13],
-                s1[i+14],
-                s1[i+15]);
+                s1[i+7]);
         __m128i vs2 = vNegInf;
         const int * const restrict matrow0 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+0] : ((i+0 >= s1Len) ? s1Len-1 : i+0))];
         const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
@@ -545,20 +462,12 @@ parasail_result_t* FNAME(
         const int * const restrict matrow5 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+5] : ((i+5 >= s1Len) ? s1Len-1 : i+5))];
         const int * const restrict matrow6 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+6] : ((i+6 >= s1Len) ? s1Len-1 : i+6))];
         const int * const restrict matrow7 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+7] : ((i+7 >= s1Len) ? s1Len-1 : i+7))];
-        const int * const restrict matrow8 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+8] : ((i+8 >= s1Len) ? s1Len-1 : i+8))];
-        const int * const restrict matrow9 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+9] : ((i+9 >= s1Len) ? s1Len-1 : i+9))];
-        const int * const restrict matrow10 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+10] : ((i+10 >= s1Len) ? s1Len-1 : i+10))];
-        const int * const restrict matrow11 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+11] : ((i+11 >= s1Len) ? s1Len-1 : i+11))];
-        const int * const restrict matrow12 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+12] : ((i+12 >= s1Len) ? s1Len-1 : i+12))];
-        const int * const restrict matrow13 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+13] : ((i+13 >= s1Len) ? s1Len-1 : i+13))];
-        const int * const restrict matrow14 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+14] : ((i+14 >= s1Len) ? s1Len-1 : i+14))];
-        const int * const restrict matrow15 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+15] : ((i+15 >= s1Len) ? s1Len-1 : i+15))];
-        __m128i vIltLimit = _mm_cmplt_epi8(vI, vILimit);
-        __m128i vIeqLimit1 = _mm_cmpeq_epi8(vI, vILimit1);
-        vNH = _mm_srli_si128(vNH, 1);
-        vNH = _mm_insert_epi8(vNH, H_pr[-1], 15);
-        vWH = _mm_srli_si128(vWH, 1);
-        vWH = _mm_insert_epi8(vWH, s1_beg ? 0 : (-open - i*gap), 15);
+        __m128i vIltLimit = _mm_cmplt_epi16(vI, vILimit);
+        __m128i vIeqLimit1 = _mm_cmpeq_epi16(vI, vILimit1);
+        vNH = _mm_srli_si128(vNH, 2);
+        vNH = _mm_insert_epi16(vNH, H_pr[-1], 7);
+        vWH = _mm_srli_si128(vWH, 2);
+        vWH = _mm_insert_epi16(vWH, s1_beg ? 0 : (-open - i*gap), 7);
         H_pr[-1] = -open - (i+N)*gap;
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
@@ -567,41 +476,41 @@ parasail_result_t* FNAME(
             __m128i vNWM = vNM;
             __m128i vNWS = vNS;
             __m128i vNWL = vNL;
-            vNH = _mm_srli_si128(vWH, 1);
-            vNH = _mm_insert_epi8(vNH, H_pr[j], 15);
-            vNM = _mm_srli_si128(vWM, 1);
-            vNM = _mm_insert_epi8(vNM, HM_pr[j], 15);
-            vNS = _mm_srli_si128(vWS, 1);
-            vNS = _mm_insert_epi8(vNS, HS_pr[j], 15);
-            vNL = _mm_srli_si128(vWL, 1);
-            vNL = _mm_insert_epi8(vNL, HL_pr[j], 15);
-            vF = _mm_srli_si128(vF, 1);
-            vF = _mm_insert_epi8(vF, F_pr[j], 15);
-            vFM = _mm_srli_si128(vFM, 1);
-            vFM = _mm_insert_epi8(vFM, FM_pr[j], 15);
-            vFS = _mm_srli_si128(vFS, 1);
-            vFS = _mm_insert_epi8(vFS, FS_pr[j], 15);
-            vFL = _mm_srli_si128(vFL, 1);
-            vFL = _mm_insert_epi8(vFL, FL_pr[j], 15);
-            vF_opn = _mm_subs_epi8(vNH, vOpen);
-            vF_ext = _mm_subs_epi8(vF, vGap);
-            vF = _mm_max_epi8(vF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi8(vF_opn, vF_ext);
-            vFM = _mm_blendv_epi8(vFM, vNM, case1);
-            vFS = _mm_blendv_epi8(vFS, vNS, case1);
-            vFL = _mm_blendv_epi8(vFL, vNL, case1);
-            vFL = _mm_adds_epi8(vFL, vOne);
-            vE_opn = _mm_subs_epi8(vWH, vOpen);
-            vE_ext = _mm_subs_epi8(vE, vGap);
-            vE = _mm_max_epi8(vE_opn, vE_ext);
-            case1 = _mm_cmpgt_epi8(vE_opn, vE_ext);
-            vEM = _mm_blendv_epi8(vEM, vWM, case1);
-            vES = _mm_blendv_epi8(vES, vWS, case1);
-            vEL = _mm_blendv_epi8(vEL, vWL, case1);
-            vEL = _mm_adds_epi8(vEL, vOne);
-            vs2 = _mm_srli_si128(vs2, 1);
-            vs2 = _mm_insert_epi8(vs2, s2[j], 15);
-            vMat = _mm_set_epi8(
+            vNH = _mm_srli_si128(vWH, 2);
+            vNH = _mm_insert_epi16(vNH, H_pr[j], 7);
+            vNM = _mm_srli_si128(vWM, 2);
+            vNM = _mm_insert_epi16(vNM, HM_pr[j], 7);
+            vNS = _mm_srli_si128(vWS, 2);
+            vNS = _mm_insert_epi16(vNS, HS_pr[j], 7);
+            vNL = _mm_srli_si128(vWL, 2);
+            vNL = _mm_insert_epi16(vNL, HL_pr[j], 7);
+            vF = _mm_srli_si128(vF, 2);
+            vF = _mm_insert_epi16(vF, F_pr[j], 7);
+            vFM = _mm_srli_si128(vFM, 2);
+            vFM = _mm_insert_epi16(vFM, FM_pr[j], 7);
+            vFS = _mm_srli_si128(vFS, 2);
+            vFS = _mm_insert_epi16(vFS, FS_pr[j], 7);
+            vFL = _mm_srli_si128(vFL, 2);
+            vFL = _mm_insert_epi16(vFL, FL_pr[j], 7);
+            vF_opn = _mm_subs_epi16(vNH, vOpen);
+            vF_ext = _mm_subs_epi16(vF, vGap);
+            vF = _mm_max_epi16(vF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi16(vF_opn, vF_ext);
+            vFM = _mm_blendv_epi8_rpl(vFM, vNM, case1);
+            vFS = _mm_blendv_epi8_rpl(vFS, vNS, case1);
+            vFL = _mm_blendv_epi8_rpl(vFL, vNL, case1);
+            vFL = _mm_adds_epi16(vFL, vOne);
+            vE_opn = _mm_subs_epi16(vWH, vOpen);
+            vE_ext = _mm_subs_epi16(vE, vGap);
+            vE = _mm_max_epi16(vE_opn, vE_ext);
+            case1 = _mm_cmpgt_epi16(vE_opn, vE_ext);
+            vEM = _mm_blendv_epi8_rpl(vEM, vWM, case1);
+            vES = _mm_blendv_epi8_rpl(vES, vWS, case1);
+            vEL = _mm_blendv_epi8_rpl(vEL, vWL, case1);
+            vEL = _mm_adds_epi16(vEL, vOne);
+            vs2 = _mm_srli_si128(vs2, 2);
+            vs2 = _mm_insert_epi16(vs2, s2[j], 7);
+            vMat = _mm_set_epi16(
                     matrow0[s2[j-0]],
                     matrow1[s2[j-1]],
                     matrow2[s2[j-2]],
@@ -609,59 +518,51 @@ parasail_result_t* FNAME(
                     matrow4[s2[j-4]],
                     matrow5[s2[j-5]],
                     matrow6[s2[j-6]],
-                    matrow7[s2[j-7]],
-                    matrow8[s2[j-8]],
-                    matrow9[s2[j-9]],
-                    matrow10[s2[j-10]],
-                    matrow11[s2[j-11]],
-                    matrow12[s2[j-12]],
-                    matrow13[s2[j-13]],
-                    matrow14[s2[j-14]],
-                    matrow15[s2[j-15]]
+                    matrow7[s2[j-7]]
                     );
-            vNWH = _mm_adds_epi8(vNWH, vMat);
-            vWH = _mm_max_epi8(vNWH, vE);
-            vWH = _mm_max_epi8(vWH, vF);
-            case1 = _mm_cmpeq_epi8(vWH, vNWH);
-            case2 = _mm_cmpeq_epi8(vWH, vF);
-            vWM = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vEM, vFM, case2),
-                    _mm_adds_epi8(vNWM,
+            vNWH = _mm_adds_epi16(vNWH, vMat);
+            vWH = _mm_max_epi16(vNWH, vE);
+            vWH = _mm_max_epi16(vWH, vF);
+            case1 = _mm_cmpeq_epi16(vWH, vNWH);
+            case2 = _mm_cmpeq_epi16(vWH, vF);
+            vWM = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vEM, vFM, case2),
+                    _mm_adds_epi16(vNWM,
                         _mm_and_si128(
-                            _mm_cmpeq_epi8(vs1,vs2),
+                            _mm_cmpeq_epi16(vs1,vs2),
                             vOne)),
                     case1);
-            vWS = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vES, vFS, case2),
-                    _mm_adds_epi8(vNWS,
+            vWS = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vES, vFS, case2),
+                    _mm_adds_epi16(vNWS,
                         _mm_and_si128(
-                            _mm_cmpgt_epi8(vMat,vZero),
+                            _mm_cmpgt_epi16(vMat,vZero),
                             vOne)),
                     case1);
-            vWL = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vEL, vFL, case2),
-                    _mm_adds_epi8(vNWL, vOne), case1);
+            vWL = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vEL, vFL, case2),
+                    _mm_adds_epi16(vNWL, vOne), case1);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                __m128i cond = _mm_cmpeq_epi8(vJ,vNegOne);
-                vWH = _mm_blendv_epi8(vWH, vIBoundary, cond);
+                __m128i cond = _mm_cmpeq_epi16(vJ,vNegOne);
+                vWH = _mm_blendv_epi8_rpl(vWH, vIBoundary, cond);
                 vWM = _mm_andnot_si128(cond, vWM);
                 vWS = _mm_andnot_si128(cond, vWS);
                 vWL = _mm_andnot_si128(cond, vWL);
-                vE = _mm_blendv_epi8(vE, vNegInf, cond);
+                vE = _mm_blendv_epi8_rpl(vE, vNegInf, cond);
                 vEM = _mm_andnot_si128(cond, vEM);
                 vES = _mm_andnot_si128(cond, vES);
                 vEL = _mm_andnot_si128(cond, vEL);
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vWH);
-                vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vWM);
-                vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vWS);
-                vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vWL);
-                vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vWL);
+                vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWH);
+                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWM);
+                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWS);
+                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWL);
+                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWL);
             }
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vWH, i, s1Len, j, s2Len);
@@ -675,77 +576,77 @@ parasail_result_t* FNAME(
             arr_store_rowcol(result->stats->rowcols->similar_row, result->stats->rowcols->similar_col, vWS, i, s1Len, j, s2Len);
             arr_store_rowcol(result->stats->rowcols->length_row,  result->stats->rowcols->length_col, vWL, i, s1Len, j, s2Len);
 #endif
-            H_pr[j-15] = (int8_t)_mm_extract_epi8(vWH,0);
-            HM_pr[j-15] = (int8_t)_mm_extract_epi8(vWM,0);
-            HS_pr[j-15] = (int8_t)_mm_extract_epi8(vWS,0);
-            HL_pr[j-15] = (int8_t)_mm_extract_epi8(vWL,0);
-            F_pr[j-15] = (int8_t)_mm_extract_epi8(vF,0);
-            FM_pr[j-15] = (int8_t)_mm_extract_epi8(vFM,0);
-            FS_pr[j-15] = (int8_t)_mm_extract_epi8(vFS,0);
-            FL_pr[j-15] = (int8_t)_mm_extract_epi8(vFL,0);
+            H_pr[j-7] = (int16_t)_mm_extract_epi16(vWH,0);
+            HM_pr[j-7] = (int16_t)_mm_extract_epi16(vWM,0);
+            HS_pr[j-7] = (int16_t)_mm_extract_epi16(vWS,0);
+            HL_pr[j-7] = (int16_t)_mm_extract_epi16(vWL,0);
+            F_pr[j-7] = (int16_t)_mm_extract_epi16(vF,0);
+            FM_pr[j-7] = (int16_t)_mm_extract_epi16(vFM,0);
+            FS_pr[j-7] = (int16_t)_mm_extract_epi16(vFS,0);
+            FL_pr[j-7] = (int16_t)_mm_extract_epi16(vFL,0);
             /* as minor diagonal vector passes across the i or j limit
              * boundary, extract the last value of the column or row */
             {
-                __m128i vJeqLimit1 = _mm_cmpeq_epi8(vJ, vJLimit1);
-                __m128i vJgtNegOne = _mm_cmpgt_epi8(vJ, vNegOne);
-                __m128i vJltLimit = _mm_cmplt_epi8(vJ, vJLimit);
+                __m128i vJeqLimit1 = _mm_cmpeq_epi16(vJ, vJLimit1);
+                __m128i vJgtNegOne = _mm_cmpgt_epi16(vJ, vNegOne);
+                __m128i vJltLimit = _mm_cmplt_epi16(vJ, vJLimit);
                 __m128i cond_j = _mm_and_si128(vIltLimit, vJeqLimit1);
                 __m128i cond_i = _mm_and_si128(vIeqLimit1,
                         _mm_and_si128(vJgtNegOne, vJltLimit));
-                __m128i cond_max_row = _mm_cmpgt_epi8(vWH, vMaxHRow);
-                __m128i cond_max_col = _mm_cmpgt_epi8(vWH, vMaxHCol);
+                __m128i cond_max_row = _mm_cmpgt_epi16(vWH, vMaxHRow);
+                __m128i cond_max_col = _mm_cmpgt_epi16(vWH, vMaxHCol);
                 __m128i cond_last_val = _mm_and_si128(vIeqLimit1, vJeqLimit1);
                 __m128i cond_all_row = _mm_and_si128(cond_max_row, cond_i);
                 __m128i cond_all_col = _mm_and_si128(cond_max_col, cond_j);
-                vMaxHRow = _mm_blendv_epi8(vMaxHRow, vWH, cond_all_row);
-                vMaxMRow = _mm_blendv_epi8(vMaxMRow, vWM, cond_all_row);
-                vMaxSRow = _mm_blendv_epi8(vMaxSRow, vWS, cond_all_row);
-                vMaxLRow = _mm_blendv_epi8(vMaxLRow, vWL, cond_all_row);
-                vMaxHCol = _mm_blendv_epi8(vMaxHCol, vWH, cond_all_col);
-                vMaxMCol = _mm_blendv_epi8(vMaxMCol, vWM, cond_all_col);
-                vMaxSCol = _mm_blendv_epi8(vMaxSCol, vWS, cond_all_col);
-                vMaxLCol = _mm_blendv_epi8(vMaxLCol, vWL, cond_all_col);
-                vLastValH = _mm_blendv_epi8(vLastValH, vWH, cond_last_val);
-                vLastValM = _mm_blendv_epi8(vLastValM, vWM, cond_last_val);
-                vLastValS = _mm_blendv_epi8(vLastValS, vWS, cond_last_val);
-                vLastValL = _mm_blendv_epi8(vLastValL, vWL, cond_last_val);
-                vEndI = _mm_blendv_epi8(vEndI, vI, cond_all_col);
-                vEndJ = _mm_blendv_epi8(vEndJ, vJ, cond_all_row);
+                vMaxHRow = _mm_blendv_epi8_rpl(vMaxHRow, vWH, cond_all_row);
+                vMaxMRow = _mm_blendv_epi8_rpl(vMaxMRow, vWM, cond_all_row);
+                vMaxSRow = _mm_blendv_epi8_rpl(vMaxSRow, vWS, cond_all_row);
+                vMaxLRow = _mm_blendv_epi8_rpl(vMaxLRow, vWL, cond_all_row);
+                vMaxHCol = _mm_blendv_epi8_rpl(vMaxHCol, vWH, cond_all_col);
+                vMaxMCol = _mm_blendv_epi8_rpl(vMaxMCol, vWM, cond_all_col);
+                vMaxSCol = _mm_blendv_epi8_rpl(vMaxSCol, vWS, cond_all_col);
+                vMaxLCol = _mm_blendv_epi8_rpl(vMaxLCol, vWL, cond_all_col);
+                vLastValH = _mm_blendv_epi8_rpl(vLastValH, vWH, cond_last_val);
+                vLastValM = _mm_blendv_epi8_rpl(vLastValM, vWM, cond_last_val);
+                vLastValS = _mm_blendv_epi8_rpl(vLastValS, vWS, cond_last_val);
+                vLastValL = _mm_blendv_epi8_rpl(vLastValL, vWL, cond_last_val);
+                vEndI = _mm_blendv_epi8_rpl(vEndI, vI, cond_all_col);
+                vEndJ = _mm_blendv_epi8_rpl(vEndJ, vJ, cond_all_row);
             }
-            vJ = _mm_adds_epi8(vJ, vOne);
+            vJ = _mm_adds_epi16(vJ, vOne);
         }
-        vI = _mm_adds_epi8(vI, vN);
-        vIBoundary = _mm_subs_epi8(vIBoundary, vGapN);
+        vI = _mm_adds_epi16(vI, vN);
+        vIBoundary = _mm_subs_epi16(vIBoundary, vGapN);
     }
 
     /* alignment ending position */
     {
-        int8_t max_rowh = NEG_LIMIT;
-        int8_t max_rowm = NEG_LIMIT;
-        int8_t max_rows = NEG_LIMIT;
-        int8_t max_rowl = NEG_LIMIT;
-        int8_t max_colh = NEG_LIMIT;
-        int8_t max_colm = NEG_LIMIT;
-        int8_t max_cols = NEG_LIMIT;
-        int8_t max_coll = NEG_LIMIT;
-        int8_t last_valh = NEG_LIMIT;
-        int8_t last_valm = NEG_LIMIT;
-        int8_t last_vals = NEG_LIMIT;
-        int8_t last_vall = NEG_LIMIT;
-        int8_t *rh = (int8_t*)&vMaxHRow;
-        int8_t *rm = (int8_t*)&vMaxMRow;
-        int8_t *rs = (int8_t*)&vMaxSRow;
-        int8_t *rl = (int8_t*)&vMaxLRow;
-        int8_t *ch = (int8_t*)&vMaxHCol;
-        int8_t *cm = (int8_t*)&vMaxMCol;
-        int8_t *cs = (int8_t*)&vMaxSCol;
-        int8_t *cl = (int8_t*)&vMaxLCol;
-        int8_t *lh = (int8_t*)&vLastValH;
-        int8_t *lm = (int8_t*)&vLastValM;
-        int8_t *ls = (int8_t*)&vLastValS;
-        int8_t *ll = (int8_t*)&vLastValL;
-        int8_t *i = (int8_t*)&vEndI;
-        int8_t *j = (int8_t*)&vEndJ;
+        int16_t max_rowh = NEG_LIMIT;
+        int16_t max_rowm = NEG_LIMIT;
+        int16_t max_rows = NEG_LIMIT;
+        int16_t max_rowl = NEG_LIMIT;
+        int16_t max_colh = NEG_LIMIT;
+        int16_t max_colm = NEG_LIMIT;
+        int16_t max_cols = NEG_LIMIT;
+        int16_t max_coll = NEG_LIMIT;
+        int16_t last_valh = NEG_LIMIT;
+        int16_t last_valm = NEG_LIMIT;
+        int16_t last_vals = NEG_LIMIT;
+        int16_t last_vall = NEG_LIMIT;
+        int16_t *rh = (int16_t*)&vMaxHRow;
+        int16_t *rm = (int16_t*)&vMaxMRow;
+        int16_t *rs = (int16_t*)&vMaxSRow;
+        int16_t *rl = (int16_t*)&vMaxLRow;
+        int16_t *ch = (int16_t*)&vMaxHCol;
+        int16_t *cm = (int16_t*)&vMaxMCol;
+        int16_t *cs = (int16_t*)&vMaxSCol;
+        int16_t *cl = (int16_t*)&vMaxLCol;
+        int16_t *lh = (int16_t*)&vLastValH;
+        int16_t *lm = (int16_t*)&vLastValM;
+        int16_t *ls = (int16_t*)&vLastValS;
+        int16_t *ll = (int16_t*)&vLastValL;
+        int16_t *i = (int16_t*)&vEndI;
+        int16_t *j = (int16_t*)&vEndJ;
         int32_t k;
         for (k=0; k<N; ++k, ++rh, ++rm, ++rs, ++rl, ++ch, ++cm, ++cs, ++cl, ++lh, ++lm, ++ls, ++ll, ++i, ++j) {
             if (*ch > max_colh || (*ch == max_colh && *i < end_query)) {
@@ -810,8 +711,8 @@ parasail_result_t* FNAME(
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
