@@ -17,8 +17,8 @@
 #include "parasail/internal_altivec.h"
 
 #define SG_STATS
-#define SG_SUFFIX _scan_altivec_128_8
-#define SG_SUFFIX_PROF _scan_profile_altivec_128_8
+#define SG_SUFFIX _scan_altivec_128_16
+#define SG_SUFFIX_PROF _scan_profile_altivec_128_16
 #include "sg_helper.h"
 
 
@@ -32,22 +32,14 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  0);
-    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  1);
-    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  2);
-    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  3);
-    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  4);
-    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  5);
-    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  6);
-    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  7);
-    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  8);
-    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  9);
-    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 10);
-    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 11);
-    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 12);
-    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 13);
-    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 14);
-    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 15);
+    array[1LL*(0*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 3);
+    array[1LL*(4*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 4);
+    array[1LL*(5*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 5);
+    array[1LL*(6*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 6);
+    array[1LL*(7*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 7);
 }
 #endif
 
@@ -58,35 +50,27 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8(vH,  0);
-    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8(vH,  1);
-    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8(vH,  2);
-    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8(vH,  3);
-    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8(vH,  4);
-    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8(vH,  5);
-    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8(vH,  6);
-    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8(vH,  7);
-    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8(vH,  8);
-    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8(vH,  9);
-    col[10*seglen+t] = (int8_t)_mm_extract_epi8(vH, 10);
-    col[11*seglen+t] = (int8_t)_mm_extract_epi8(vH, 11);
-    col[12*seglen+t] = (int8_t)_mm_extract_epi8(vH, 12);
-    col[13*seglen+t] = (int8_t)_mm_extract_epi8(vH, 13);
-    col[14*seglen+t] = (int8_t)_mm_extract_epi8(vH, 14);
-    col[15*seglen+t] = (int8_t)_mm_extract_epi8(vH, 15);
+    col[0*seglen+t] = (int16_t)_mm_extract_epi16(vH, 0);
+    col[1*seglen+t] = (int16_t)_mm_extract_epi16(vH, 1);
+    col[2*seglen+t] = (int16_t)_mm_extract_epi16(vH, 2);
+    col[3*seglen+t] = (int16_t)_mm_extract_epi16(vH, 3);
+    col[4*seglen+t] = (int16_t)_mm_extract_epi16(vH, 4);
+    col[5*seglen+t] = (int16_t)_mm_extract_epi16(vH, 5);
+    col[6*seglen+t] = (int16_t)_mm_extract_epi16(vH, 6);
+    col[7*seglen+t] = (int16_t)_mm_extract_epi16(vH, 7);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_stats_table_scan_altivec_128_8
-#define PNAME parasail_sg_flags_stats_table_scan_profile_altivec_128_8
+#define FNAME parasail_sg_flags_stats_table_scan_altivec_128_16
+#define PNAME parasail_sg_flags_stats_table_scan_profile_altivec_128_16
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_stats_rowcol_scan_altivec_128_8
-#define PNAME parasail_sg_flags_stats_rowcol_scan_profile_altivec_128_8
+#define FNAME parasail_sg_flags_stats_rowcol_scan_altivec_128_16
+#define PNAME parasail_sg_flags_stats_rowcol_scan_profile_altivec_128_16
 #else
-#define FNAME parasail_sg_flags_stats_scan_altivec_128_8
-#define PNAME parasail_sg_flags_stats_scan_profile_altivec_128_8
+#define FNAME parasail_sg_flags_stats_scan_altivec_128_16
+#define PNAME parasail_sg_flags_stats_scan_profile_altivec_128_16
 #endif
 #endif
 
@@ -115,7 +99,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_altivec_128_8(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_altivec_128_16(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -159,17 +143,17 @@ parasail_result_t* PNAME(
     vec128i* restrict pvHLMax = NULL;
     vec128i* restrict pvGapper = NULL;
     vec128i* restrict pvGapperL = NULL;
-    int8_t* restrict boundary = NULL;
+    int16_t* restrict boundary = NULL;
     vec128i vGapO;
     vec128i vGapE;
-    int8_t NEG_LIMIT = 0;
-    int8_t POS_LIMIT = 0;
+    int16_t NEG_LIMIT = 0;
+    int16_t POS_LIMIT = 0;
     vec128i vZero;
     vec128i vOne;
-    int8_t score = 0;
-    int8_t matches = 0;
-    int8_t similar = 0;
-    int8_t length = 0;
+    int16_t score = 0;
+    int16_t matches = 0;
+    int16_t similar = 0;
+    int16_t length = 0;
     vec128i vNegLimit;
     vec128i vPosLimit;
     vec128i vSaturationCheckMin;
@@ -186,7 +170,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile8.score);
+    PARASAIL_CHECK_NULL(profile->profile16.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -202,38 +186,38 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 16; /* number of values in vector unit */
+    segWidth = 8; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP  = (vec128i*)profile->profile8.score;
-    pvPm = (vec128i*)profile->profile8.matches;
-    pvPs = (vec128i*)profile->profile8.similar;
-    vGapO = _mm_set1_epi8(open);
-    vGapE = _mm_set1_epi8(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
-    POS_LIMIT = INT8_MAX - matrix->max - 1;
+    pvP  = (vec128i*)profile->profile16.score;
+    pvPm = (vec128i*)profile->profile16.matches;
+    pvPs = (vec128i*)profile->profile16.similar;
+    vGapO = _mm_set1_epi16(open);
+    vGapE = _mm_set1_epi16(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
+    POS_LIMIT = INT16_MAX - matrix->max - 1;
     vZero = _mm_setzero_si128();
-    vOne = _mm_set1_epi8(1);
+    vOne = _mm_set1_epi16(1);
     score = NEG_LIMIT;
     matches = 0;
     similar = 0;
     length = 0;
-    vNegLimit = _mm_set1_epi8(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi8(POS_LIMIT);
+    vNegLimit = _mm_set1_epi16(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi16(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
     vMaxM = vNegLimit;
     vMaxS = vNegLimit;
     vMaxL = vNegLimit;
-    vPosMask = _mm_cmpeq_epi8(_mm_set1_epi8(position),
-            _mm_set_epi8(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15));
+    vPosMask = _mm_cmpeq_epi16(_mm_set1_epi16(position),
+            _mm_set_epi16(0,1,2,3,4,5,6,7));
     vNegInfFront = vZero;
-    vSegLen = _mm_slli_si128(_mm_set1_epi8(segLen), 1);
-    vNegInfFront = _mm_insert_epi8(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm_adds_epi8(vNegInfFront,
-            _mm_slli_si128(_mm_set1_epi8(-segLen*gap), 1));
+    vSegLen = _mm_slli_si128(_mm_set1_epi16(segLen), 2);
+    vNegInfFront = _mm_insert_epi16(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm_adds_epi16(vNegInfFront,
+            _mm_slli_si128(_mm_set1_epi16(-segLen*gap), 2));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -250,7 +234,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
+        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -277,7 +261,7 @@ parasail_result_t* PNAME(
     pvHLMax = parasail_memalign_vec128i(16, segLen);
     pvGapper = parasail_memalign_vec128i(16, segLen);
     pvGapperL = parasail_memalign_vec128i(16, segLen);
-    boundary = parasail_memalign_int8_t(16, s2Len+1);
+    boundary = parasail_memalign_int16_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvE) return NULL;
@@ -304,13 +288,13 @@ parasail_result_t* PNAME(
     parasail_memset_vec128i(pvES, vZero, segLen);
     parasail_memset_vec128i(pvEL, vZero, segLen);
     {
-        vec128i vGapper = _mm_subs_epi8(vZero,vGapO);
+        vec128i vGapper = _mm_subs_epi16(vZero,vGapO);
         vec128i vGapperL = vOne;
         for (i=segLen-1; i>=0; --i) {
             _mm_store_si128(pvGapper+i, vGapper);
             _mm_store_si128(pvGapperL+i, vGapperL);
-            vGapper = _mm_subs_epi8(vGapper, vGapE);
-            vGapperL = _mm_adds_epi8(vGapperL, vOne);
+            vGapper = _mm_subs_epi16(vGapper, vGapE);
+            vGapperL = _mm_adds_epi16(vGapperL, vOne);
         }
     }
 
@@ -319,10 +303,10 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            vec128i_8_t h;
+            vec128i_16_t h;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
+                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
             }
             _mm_store_si128(&pvH[index], h.m);
             ++index;
@@ -334,7 +318,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
+            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
         }
     }
 
@@ -382,15 +366,15 @@ parasail_result_t* PNAME(
         vHpM = _mm_load_si128(pvHM+(segLen-1));
         vHpS = _mm_load_si128(pvHS+(segLen-1));
         vHpL = _mm_load_si128(pvHL+(segLen-1));
-        vHp = _mm_slli_si128(vHp, 1);
-        vHpM = _mm_slli_si128(vHpM, 1);
-        vHpS = _mm_slli_si128(vHpS, 1);
-        vHpL = _mm_slli_si128(vHpL, 1);
-        vHp = _mm_insert_epi8(vHp, boundary[j], 0);
+        vHp = _mm_slli_si128(vHp, 2);
+        vHpM = _mm_slli_si128(vHpM, 2);
+        vHpS = _mm_slli_si128(vHpS, 2);
+        vHpL = _mm_slli_si128(vHpL, 2);
+        vHp = _mm_insert_epi16(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWM= pvPm+ matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWS= pvPs+ matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = _mm_subs_epi8(vNegLimit, pvGapper[0]);
+        vHt = _mm_subs_epi16(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         vFM = vZero;
         vFS = vZero;
@@ -409,30 +393,30 @@ parasail_result_t* PNAME(
             vWS = _mm_load_si128(pvWS+i);
             vGapper = _mm_load_si128(pvGapper+i);
             vGapperL = _mm_load_si128(pvGapperL+i);
-            vE_opn = _mm_subs_epi8(vH, vGapO);
-            vE_ext = _mm_subs_epi8(vE, vGapE);
-            case1 = _mm_cmpgt_epi8(vE_opn, vE_ext);
-            vE = _mm_max_epi8(vE_opn, vE_ext);
+            vE_opn = _mm_subs_epi16(vH, vGapO);
+            vE_ext = _mm_subs_epi16(vE, vGapE);
+            case1 = _mm_cmpgt_epi16(vE_opn, vE_ext);
+            vE = _mm_max_epi16(vE_opn, vE_ext);
             vEM = _mm_blendv_epi8(vEM, vHM, case1);
             vES = _mm_blendv_epi8(vES, vHS, case1);
             vEL = _mm_blendv_epi8(vEL, vHL, case1);
-            vEL = _mm_adds_epi8(vEL, vOne);
-            vGapper = _mm_adds_epi8(vHt, vGapper);
+            vEL = _mm_adds_epi16(vEL, vOne);
+            vGapper = _mm_adds_epi16(vHt, vGapper);
             case1 = _mm_or_si128(
-                    _mm_cmpgt_epi8(vF, vGapper),
-                    _mm_cmpeq_epi8(vF, vGapper));
-            vF = _mm_max_epi8(vF, vGapper);
+                    _mm_cmpgt_epi16(vF, vGapper),
+                    _mm_cmpeq_epi16(vF, vGapper));
+            vF = _mm_max_epi16(vF, vGapper);
             vFM = _mm_blendv_epi8(vHtM, vFM, case1);
             vFS = _mm_blendv_epi8(vHtS, vFS, case1);
             vFL = _mm_blendv_epi8(
-                    _mm_adds_epi8(vHtL, vGapperL),
+                    _mm_adds_epi16(vHtL, vGapperL),
                     vFL, case1);
-            vHp = _mm_adds_epi8(vHp, vW);
-            vHpM = _mm_adds_epi8(vHpM, vWM);
-            vHpS = _mm_adds_epi8(vHpS, vWS);
-            vHpL = _mm_adds_epi8(vHpL, vOne);
-            case1 = _mm_cmpgt_epi8(vE, vHp);
-            vHt = _mm_max_epi8(vE, vHp);
+            vHp = _mm_adds_epi16(vHp, vW);
+            vHpM = _mm_adds_epi16(vHpM, vWM);
+            vHpS = _mm_adds_epi16(vHpS, vWS);
+            vHpL = _mm_adds_epi16(vHpL, vOne);
+            case1 = _mm_cmpgt_epi16(vE, vHp);
+            vHt = _mm_max_epi16(vE, vHp);
             vHtM = _mm_blendv_epi8(vHpM, vEM, case1);
             vHtS = _mm_blendv_epi8(vHpS, vES, case1);
             vHtL = _mm_blendv_epi8(vHpL, vEL, case1);
@@ -451,50 +435,50 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm_slli_si128(vHt, 1);
-        vHtM = _mm_slli_si128(vHtM, 1);
-        vHtS = _mm_slli_si128(vHtS, 1);
-        vHtL = _mm_slli_si128(vHtL, 1);
-        vHt = _mm_insert_epi8(vHt, boundary[j+1], 0);
+        vHt = _mm_slli_si128(vHt, 2);
+        vHtM = _mm_slli_si128(vHtM, 2);
+        vHtS = _mm_slli_si128(vHtS, 2);
+        vHtL = _mm_slli_si128(vHtL, 2);
+        vHt = _mm_insert_epi16(vHt, boundary[j+1], 0);
         vGapper = _mm_load_si128(pvGapper);
         vGapperL = _mm_load_si128(pvGapperL);
-        vGapper = _mm_adds_epi8(vHt, vGapper);
+        vGapper = _mm_adds_epi16(vHt, vGapper);
         case1 = _mm_or_si128(
-                _mm_cmpgt_epi8(vGapper, vF),
-                _mm_cmpeq_epi8(vGapper, vF));
-        vF = _mm_max_epi8(vF, vGapper);
+                _mm_cmpgt_epi16(vGapper, vF),
+                _mm_cmpeq_epi16(vGapper, vF));
+        vF = _mm_max_epi16(vF, vGapper);
         vFM = _mm_blendv_epi8(vFM, vHtM, case1);
         vFS = _mm_blendv_epi8(vFS, vHtS, case1);
         vFL = _mm_blendv_epi8(
                 vFL,
-                _mm_adds_epi8(vHtL, vGapperL),
+                _mm_adds_epi16(vHtL, vGapperL),
                 case1);
         for (i=0; i<segWidth-2; ++i) {
-            vec128i vFt = _mm_slli_si128(vF, 1);
-            vec128i vFtM = _mm_slli_si128(vFM, 1);
-            vec128i vFtS = _mm_slli_si128(vFS, 1);
-            vec128i vFtL = _mm_slli_si128(vFL, 1);
-            vFt = _mm_adds_epi8(vFt, vSegLenXgap);
+            vec128i vFt = _mm_slli_si128(vF, 2);
+            vec128i vFtM = _mm_slli_si128(vFM, 2);
+            vec128i vFtS = _mm_slli_si128(vFS, 2);
+            vec128i vFtL = _mm_slli_si128(vFL, 2);
+            vFt = _mm_adds_epi16(vFt, vSegLenXgap);
             case1 = _mm_or_si128(
-                    _mm_cmpgt_epi8(vFt, vF),
-                    _mm_cmpeq_epi8(vFt, vF));
-            vF = _mm_max_epi8(vF, vFt);
+                    _mm_cmpgt_epi16(vFt, vF),
+                    _mm_cmpeq_epi16(vFt, vF));
+            vF = _mm_max_epi16(vF, vFt);
             vFM = _mm_blendv_epi8(vFM, vFtM, case1);
             vFS = _mm_blendv_epi8(vFS, vFtS, case1);
             vFL = _mm_blendv_epi8(
                     vFL,
-                    _mm_adds_epi8(vFtL, vSegLen),
+                    _mm_adds_epi16(vFtL, vSegLen),
                     case1);
         }
 
         /* calculate final H */
-        vF = _mm_slli_si128(vF, 1);
-        vFM = _mm_slli_si128(vFM, 1);
-        vFS = _mm_slli_si128(vFS, 1);
-        vFL = _mm_slli_si128(vFL, 1);
-        vF = _mm_adds_epi8(vF, vNegInfFront);
-        case1 = _mm_cmpgt_epi8(vF, vHt);
-        vH = _mm_max_epi8(vF, vHt);
+        vF = _mm_slli_si128(vF, 2);
+        vFM = _mm_slli_si128(vFM, 2);
+        vFS = _mm_slli_si128(vFS, 2);
+        vFL = _mm_slli_si128(vFL, 2);
+        vF = _mm_adds_epi16(vF, vNegInfFront);
+        case1 = _mm_cmpgt_epi16(vF, vHt);
+        vH = _mm_max_epi16(vF, vHt);
         vHM = _mm_blendv_epi8(vHtM, vFM, case1);
         vHS = _mm_blendv_epi8(vHtS, vFS, case1);
         vHL = _mm_blendv_epi8(vHtL, vFL, case1);
@@ -507,18 +491,18 @@ parasail_result_t* PNAME(
             vEM = _mm_load_si128(pvEM+i);
             vES = _mm_load_si128(pvES+i);
             vEL = _mm_load_si128(pvEL+i);
-            vF_opn = _mm_subs_epi8(vH, vGapO);
-            vF_ext = _mm_subs_epi8(vF, vGapE);
-            vF = _mm_max_epi8(vF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi8(vF_opn, vF_ext);
+            vF_opn = _mm_subs_epi16(vH, vGapO);
+            vF_ext = _mm_subs_epi16(vF, vGapE);
+            vF = _mm_max_epi16(vF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi16(vF_opn, vF_ext);
             vFM = _mm_blendv_epi8(vFM, vHM, case1);
             vFS = _mm_blendv_epi8(vFS, vHS, case1);
             vFL = _mm_blendv_epi8(vFL, vHL, case1);
-            vFL = _mm_adds_epi8(vFL, vOne);
-            vH = _mm_max_epi8(vHp, vE);
-            vH = _mm_max_epi8(vH, vF);
-            case1 = _mm_cmpeq_epi8(vH, vHp);
-            case2 = _mm_cmpeq_epi8(vH, vF);
+            vFL = _mm_adds_epi16(vFL, vOne);
+            vH = _mm_max_epi16(vHp, vE);
+            vH = _mm_max_epi16(vH, vF);
+            case1 = _mm_cmpeq_epi16(vH, vHp);
+            case2 = _mm_cmpeq_epi16(vH, vF);
             vHM = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEM, vFM, case2),
                     vHpM, case1);
@@ -532,11 +516,11 @@ parasail_result_t* PNAME(
             _mm_store_si128(pvHM+i, vHM);
             _mm_store_si128(pvHS+i, vHS);
             _mm_store_si128(pvHL+i, vHL);
-            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
@@ -551,8 +535,8 @@ parasail_result_t* PNAME(
             vHM = _mm_load_si128(pvHM + offset);
             vHS = _mm_load_si128(pvHS + offset);
             vHL = _mm_load_si128(pvHL + offset);
-            cond_max = _mm_cmpgt_epi8(vH, vMaxH);
-            vMaxH = _mm_max_epi8(vMaxH, vH);
+            cond_max = _mm_cmpgt_epi16(vH, vMaxH);
+            vMaxH = _mm_max_epi16(vMaxH, vH);
             vMaxM = _mm_blendv_epi8(vMaxM, vHM, cond_max);
             vMaxS = _mm_blendv_epi8(vMaxS, vHS, cond_max);
             vMaxL = _mm_blendv_epi8(vMaxL, vHL, cond_max);
@@ -561,15 +545,15 @@ parasail_result_t* PNAME(
             }
 #ifdef PARASAIL_ROWCOL
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 1);
-                vHM = _mm_slli_si128(vHM, 1);
-                vHS = _mm_slli_si128(vHS, 1);
-                vHL = _mm_slli_si128(vHL, 1);
+                vH = _mm_slli_si128(vH, 2);
+                vHM = _mm_slli_si128(vHM, 2);
+                vHS = _mm_slli_si128(vHS, 2);
+                vHL = _mm_slli_si128(vHL, 2);
             }
-            result->stats->rowcols->score_row[j] = (int8_t) _mm_extract_epi8 (vH, 15);
-            result->stats->rowcols->matches_row[j] = (int8_t) _mm_extract_epi8 (vHM, 15);
-            result->stats->rowcols->similar_row[j] = (int8_t) _mm_extract_epi8 (vHS, 15);
-            result->stats->rowcols->length_row[j] = (int8_t) _mm_extract_epi8 (vHL, 15);
+            result->stats->rowcols->score_row[j] = (int16_t) _mm_extract_epi16 (vH, 7);
+            result->stats->rowcols->matches_row[j] = (int16_t) _mm_extract_epi16 (vHM, 7);
+            result->stats->rowcols->similar_row[j] = (int16_t) _mm_extract_epi16 (vHS, 7);
+            result->stats->rowcols->length_row[j] = (int16_t) _mm_extract_epi16 (vHL, 7);
 #endif
         }
     }
@@ -591,26 +575,26 @@ parasail_result_t* PNAME(
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm_slli_si128(vMaxH, 1);
-            vMaxM = _mm_slli_si128(vMaxM, 1);
-            vMaxS = _mm_slli_si128(vMaxS, 1);
-            vMaxL = _mm_slli_si128(vMaxL, 1);
+            vMaxH = _mm_slli_si128(vMaxH, 2);
+            vMaxM = _mm_slli_si128(vMaxM, 2);
+            vMaxS = _mm_slli_si128(vMaxS, 2);
+            vMaxL = _mm_slli_si128(vMaxL, 2);
         }
         end_query = s1Len-1;
-        score = (int8_t) _mm_extract_epi8(vMaxH, 15);
-        matches = (int8_t) _mm_extract_epi8(vMaxM, 15);
-        similar = (int8_t) _mm_extract_epi8(vMaxS, 15);
-        length = (int8_t) _mm_extract_epi8(vMaxL, 15);
+        score = (int16_t) _mm_extract_epi16(vMaxH, 7);
+        matches = (int16_t) _mm_extract_epi16(vMaxM, 7);
+        similar = (int16_t) _mm_extract_epi16(vMaxS, 7);
+        length = (int16_t) _mm_extract_epi16(vMaxL, 7);
     }
 
     /* max of last column */
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int8_t *t = (int8_t*)pvH;
-        int8_t *m = (int8_t*)pvHM;
-        int8_t *s = (int8_t*)pvHS;
-        int8_t *l = (int8_t*)pvHL;
+        int16_t *t = (int16_t*)pvH;
+        int16_t *m = (int16_t*)pvHM;
+        int16_t *s = (int16_t*)pvHS;
+        int16_t *l = (int16_t*)pvHL;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -640,22 +624,22 @@ parasail_result_t* PNAME(
         vec128i vHS = _mm_load_si128(pvHS + offset);
         vec128i vHL = _mm_load_si128(pvHL + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm_slli_si128(vH, 1);
-            vHM = _mm_slli_si128(vHM, 1);
-            vHS = _mm_slli_si128(vHS, 1);
-            vHL = _mm_slli_si128(vHL, 1);
+            vH = _mm_slli_si128(vH, 2);
+            vHM = _mm_slli_si128(vHM, 2);
+            vHS = _mm_slli_si128(vHS, 2);
+            vHL = _mm_slli_si128(vHL, 2);
         }
         end_ref = s2Len - 1;
         end_query = s1Len - 1;
-        score = (int8_t) _mm_extract_epi8 (vH, 15);
-        matches = (int8_t) _mm_extract_epi8 (vHM, 15);
-        similar = (int8_t) _mm_extract_epi8 (vHS, 15);
-        length = (int8_t) _mm_extract_epi8 (vHL, 15);
+        score = (int16_t) _mm_extract_epi16 (vH, 7);
+        matches = (int16_t) _mm_extract_epi16 (vHM, 7);
+        similar = (int16_t) _mm_extract_epi16 (vHS, 7);
+        length = (int16_t) _mm_extract_epi16 (vHL, 7);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
