@@ -17,33 +17,83 @@
 #include "parasail/internal_avx.h"
 
 #define SG_STATS
-#define SG_SUFFIX _scan_avx2_256_32
-#define SG_SUFFIX_PROF _scan_profile_avx2_256_32
+#define SG_SUFFIX _scan_avx2_256_64
+#define SG_SUFFIX_PROF _scan_profile_avx2_256_64
 #include "sg_helper.h"
 
 
-#if HAVE_AVX2_MM256_INSERT_EPI32
-#define _mm256_insert_epi32_rpl _mm256_insert_epi32
+#if HAVE_AVX2_MM256_INSERT_EPI64
+#define _mm256_insert_epi64_rpl _mm256_insert_epi64
 #else
-static inline __m256i _mm256_insert_epi32_rpl(__m256i a, int32_t i, int imm) {
-    __m256i_32_t A;
+static inline __m256i _mm256_insert_epi64_rpl(__m256i a, int64_t i, int imm) {
+    __m256i_64_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_EXTRACT_EPI32
-#define _mm256_extract_epi32_rpl _mm256_extract_epi32
+#if HAVE_AVX2_MM256_SET1_EPI64X
+#define _mm256_set1_epi64x_rpl _mm256_set1_epi64x
 #else
-static inline int32_t _mm256_extract_epi32_rpl(__m256i a, int imm) {
-    __m256i_32_t A;
+static inline __m256i _mm256_set1_epi64x_rpl(int64_t i) {
+    __m256i_64_t A;
+    A.v[0] = i;
+    A.v[1] = i;
+    A.v[2] = i;
+    A.v[3] = i;
+    return A.m;
+}
+#endif
+
+static inline __m256i _mm256_max_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]>B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]>B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
+#if HAVE_AVX2_MM256_SET_EPI64X
+#define _mm256_set_epi64x_rpl _mm256_set_epi64x
+#else
+static inline __m256i _mm256_set_epi64x_rpl(int64_t e3, int64_t e2, int64_t e1, int64_t e0) {
+    __m256i_64_t A;
+    A.v[0] = e0;
+    A.v[1] = e1;
+    A.v[2] = e2;
+    A.v[3] = e3;
+    return A.m;
+}
+#endif
+
+#if HAVE_AVX2_MM256_EXTRACT_EPI64
+#define _mm256_extract_epi64_rpl _mm256_extract_epi64
+#else
+static inline int64_t _mm256_extract_epi64_rpl(__m256i a, int imm) {
+    __m256i_64_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
-#define _mm256_cmplt_epi32_rpl(a,b) _mm256_cmpgt_epi32(b,a)
+static inline __m256i _mm256_min_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]<B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]<B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
+#define _mm256_cmplt_epi64_rpl(a,b) _mm256_cmpgt_epi64(b,a)
 
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
@@ -57,14 +107,10 @@ static inline void arr_store_si256(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 3);
-    array[1LL*(4*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 4);
-    array[1LL*(5*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 5);
-    array[1LL*(6*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 6);
-    array[1LL*(7*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 7);
+    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 3);
 }
 #endif
 
@@ -75,27 +121,23 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 0);
-    col[1*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 1);
-    col[2*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 2);
-    col[3*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 3);
-    col[4*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 4);
-    col[5*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 5);
-    col[6*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 6);
-    col[7*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 7);
+    col[0*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 0);
+    col[1*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 1);
+    col[2*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 2);
+    col[3*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 3);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_stats_table_scan_avx2_256_32
-#define PNAME parasail_sg_flags_stats_table_scan_profile_avx2_256_32
+#define FNAME parasail_sg_flags_stats_table_scan_avx2_256_64
+#define PNAME parasail_sg_flags_stats_table_scan_profile_avx2_256_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_stats_rowcol_scan_avx2_256_32
-#define PNAME parasail_sg_flags_stats_rowcol_scan_profile_avx2_256_32
+#define FNAME parasail_sg_flags_stats_rowcol_scan_avx2_256_64
+#define PNAME parasail_sg_flags_stats_rowcol_scan_profile_avx2_256_64
 #else
-#define FNAME parasail_sg_flags_stats_scan_avx2_256_32
-#define PNAME parasail_sg_flags_stats_scan_profile_avx2_256_32
+#define FNAME parasail_sg_flags_stats_scan_avx2_256_64
+#define PNAME parasail_sg_flags_stats_scan_profile_avx2_256_64
 #endif
 #endif
 
@@ -124,7 +166,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_avx_256_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_avx_256_64(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -168,17 +210,17 @@ parasail_result_t* PNAME(
     __m256i* restrict pvHLMax = NULL;
     __m256i* restrict pvGapper = NULL;
     __m256i* restrict pvGapperL = NULL;
-    int32_t* restrict boundary = NULL;
+    int64_t* restrict boundary = NULL;
     __m256i vGapO;
     __m256i vGapE;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
     __m256i vZero;
     __m256i vOne;
-    int32_t score = 0;
-    int32_t matches = 0;
-    int32_t similar = 0;
-    int32_t length = 0;
+    int64_t score = 0;
+    int64_t matches = 0;
+    int64_t similar = 0;
+    int64_t length = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -195,7 +237,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -211,38 +253,38 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 8; /* number of values in vector unit */
+    segWidth = 4; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP  = (__m256i*)profile->profile32.score;
-    pvPm = (__m256i*)profile->profile32.matches;
-    pvPs = (__m256i*)profile->profile32.similar;
-    vGapO = _mm256_set1_epi32(open);
-    vGapE = _mm256_set1_epi32(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    pvP  = (__m256i*)profile->profile64.score;
+    pvPm = (__m256i*)profile->profile64.matches;
+    pvPs = (__m256i*)profile->profile64.similar;
+    vGapO = _mm256_set1_epi64x_rpl(open);
+    vGapE = _mm256_set1_epi64x_rpl(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     vZero = _mm256_setzero_si256();
-    vOne = _mm256_set1_epi32(1);
+    vOne = _mm256_set1_epi64x_rpl(1);
     score = NEG_LIMIT;
     matches = 0;
     similar = 0;
     length = 0;
-    vNegLimit = _mm256_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
     vMaxM = vNegLimit;
     vMaxS = vNegLimit;
     vMaxL = vNegLimit;
-    vPosMask = _mm256_cmpeq_epi32(_mm256_set1_epi32(position),
-            _mm256_set_epi32(0,1,2,3,4,5,6,7));
+    vPosMask = _mm256_cmpeq_epi64(_mm256_set1_epi64x_rpl(position),
+            _mm256_set_epi64x_rpl(0,1,2,3));
     vNegInfFront = vZero;
-    vSegLen = _mm256_slli_si256_rpl(_mm256_set1_epi32(segLen), 4);
-    vNegInfFront = _mm256_insert_epi32_rpl(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm256_add_epi32(vNegInfFront,
-            _mm256_slli_si256_rpl(_mm256_set1_epi32(-segLen*gap), 4));
+    vSegLen = _mm256_slli_si256_rpl(_mm256_set1_epi64x_rpl(segLen), 8);
+    vNegInfFront = _mm256_insert_epi64_rpl(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm256_add_epi64(vNegInfFront,
+            _mm256_slli_si256_rpl(_mm256_set1_epi64x_rpl(-segLen*gap), 8));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -259,7 +301,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -286,7 +328,7 @@ parasail_result_t* PNAME(
     pvHLMax = parasail_memalign___m256i(32, segLen);
     pvGapper = parasail_memalign___m256i(32, segLen);
     pvGapperL = parasail_memalign___m256i(32, segLen);
-    boundary = parasail_memalign_int32_t(32, s2Len+1);
+    boundary = parasail_memalign_int64_t(32, s2Len+1);
 
     /* validate heap variables */
     if (!pvE) return NULL;
@@ -313,13 +355,13 @@ parasail_result_t* PNAME(
     parasail_memset___m256i(pvES, vZero, segLen);
     parasail_memset___m256i(pvEL, vZero, segLen);
     {
-        __m256i vGapper = _mm256_sub_epi32(vZero,vGapO);
+        __m256i vGapper = _mm256_sub_epi64(vZero,vGapO);
         __m256i vGapperL = vOne;
         for (i=segLen-1; i>=0; --i) {
             _mm256_store_si256(pvGapper+i, vGapper);
             _mm256_store_si256(pvGapperL+i, vGapperL);
-            vGapper = _mm256_sub_epi32(vGapper, vGapE);
-            vGapperL = _mm256_add_epi32(vGapperL, vOne);
+            vGapper = _mm256_sub_epi64(vGapper, vGapE);
+            vGapperL = _mm256_add_epi64(vGapperL, vOne);
         }
     }
 
@@ -328,10 +370,10 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m256i_32_t h;
+            __m256i_64_t h;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
             }
             _mm256_store_si256(&pvH[index], h.m);
             ++index;
@@ -343,7 +385,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
+            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
         }
     }
 
@@ -391,15 +433,15 @@ parasail_result_t* PNAME(
         vHpM = _mm256_load_si256(pvHM+(segLen-1));
         vHpS = _mm256_load_si256(pvHS+(segLen-1));
         vHpL = _mm256_load_si256(pvHL+(segLen-1));
-        vHp = _mm256_slli_si256_rpl(vHp, 4);
-        vHpM = _mm256_slli_si256_rpl(vHpM, 4);
-        vHpS = _mm256_slli_si256_rpl(vHpS, 4);
-        vHpL = _mm256_slli_si256_rpl(vHpL, 4);
-        vHp = _mm256_insert_epi32_rpl(vHp, boundary[j], 0);
+        vHp = _mm256_slli_si256_rpl(vHp, 8);
+        vHpM = _mm256_slli_si256_rpl(vHpM, 8);
+        vHpS = _mm256_slli_si256_rpl(vHpS, 8);
+        vHpL = _mm256_slli_si256_rpl(vHpL, 8);
+        vHp = _mm256_insert_epi64_rpl(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWM= pvPm+ matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWS= pvPs+ matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = _mm256_sub_epi32(vNegLimit, pvGapper[0]);
+        vHt = _mm256_sub_epi64(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         vFM = vZero;
         vFS = vZero;
@@ -418,30 +460,30 @@ parasail_result_t* PNAME(
             vWS = _mm256_load_si256(pvWS+i);
             vGapper = _mm256_load_si256(pvGapper+i);
             vGapperL = _mm256_load_si256(pvGapperL+i);
-            vE_opn = _mm256_sub_epi32(vH, vGapO);
-            vE_ext = _mm256_sub_epi32(vE, vGapE);
-            case1 = _mm256_cmpgt_epi32(vE_opn, vE_ext);
-            vE = _mm256_max_epi32(vE_opn, vE_ext);
+            vE_opn = _mm256_sub_epi64(vH, vGapO);
+            vE_ext = _mm256_sub_epi64(vE, vGapE);
+            case1 = _mm256_cmpgt_epi64(vE_opn, vE_ext);
+            vE = _mm256_max_epi64_rpl(vE_opn, vE_ext);
             vEM = _mm256_blendv_epi8(vEM, vHM, case1);
             vES = _mm256_blendv_epi8(vES, vHS, case1);
             vEL = _mm256_blendv_epi8(vEL, vHL, case1);
-            vEL = _mm256_add_epi32(vEL, vOne);
-            vGapper = _mm256_add_epi32(vHt, vGapper);
+            vEL = _mm256_add_epi64(vEL, vOne);
+            vGapper = _mm256_add_epi64(vHt, vGapper);
             case1 = _mm256_or_si256(
-                    _mm256_cmpgt_epi32(vF, vGapper),
-                    _mm256_cmpeq_epi32(vF, vGapper));
-            vF = _mm256_max_epi32(vF, vGapper);
+                    _mm256_cmpgt_epi64(vF, vGapper),
+                    _mm256_cmpeq_epi64(vF, vGapper));
+            vF = _mm256_max_epi64_rpl(vF, vGapper);
             vFM = _mm256_blendv_epi8(vHtM, vFM, case1);
             vFS = _mm256_blendv_epi8(vHtS, vFS, case1);
             vFL = _mm256_blendv_epi8(
-                    _mm256_add_epi32(vHtL, vGapperL),
+                    _mm256_add_epi64(vHtL, vGapperL),
                     vFL, case1);
-            vHp = _mm256_add_epi32(vHp, vW);
-            vHpM = _mm256_add_epi32(vHpM, vWM);
-            vHpS = _mm256_add_epi32(vHpS, vWS);
-            vHpL = _mm256_add_epi32(vHpL, vOne);
-            case1 = _mm256_cmpgt_epi32(vE, vHp);
-            vHt = _mm256_max_epi32(vE, vHp);
+            vHp = _mm256_add_epi64(vHp, vW);
+            vHpM = _mm256_add_epi64(vHpM, vWM);
+            vHpS = _mm256_add_epi64(vHpS, vWS);
+            vHpL = _mm256_add_epi64(vHpL, vOne);
+            case1 = _mm256_cmpgt_epi64(vE, vHp);
+            vHt = _mm256_max_epi64_rpl(vE, vHp);
             vHtM = _mm256_blendv_epi8(vHpM, vEM, case1);
             vHtS = _mm256_blendv_epi8(vHpS, vES, case1);
             vHtL = _mm256_blendv_epi8(vHpL, vEL, case1);
@@ -460,50 +502,50 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm256_slli_si256_rpl(vHt, 4);
-        vHtM = _mm256_slli_si256_rpl(vHtM, 4);
-        vHtS = _mm256_slli_si256_rpl(vHtS, 4);
-        vHtL = _mm256_slli_si256_rpl(vHtL, 4);
-        vHt = _mm256_insert_epi32_rpl(vHt, boundary[j+1], 0);
+        vHt = _mm256_slli_si256_rpl(vHt, 8);
+        vHtM = _mm256_slli_si256_rpl(vHtM, 8);
+        vHtS = _mm256_slli_si256_rpl(vHtS, 8);
+        vHtL = _mm256_slli_si256_rpl(vHtL, 8);
+        vHt = _mm256_insert_epi64_rpl(vHt, boundary[j+1], 0);
         vGapper = _mm256_load_si256(pvGapper);
         vGapperL = _mm256_load_si256(pvGapperL);
-        vGapper = _mm256_add_epi32(vHt, vGapper);
+        vGapper = _mm256_add_epi64(vHt, vGapper);
         case1 = _mm256_or_si256(
-                _mm256_cmpgt_epi32(vGapper, vF),
-                _mm256_cmpeq_epi32(vGapper, vF));
-        vF = _mm256_max_epi32(vF, vGapper);
+                _mm256_cmpgt_epi64(vGapper, vF),
+                _mm256_cmpeq_epi64(vGapper, vF));
+        vF = _mm256_max_epi64_rpl(vF, vGapper);
         vFM = _mm256_blendv_epi8(vFM, vHtM, case1);
         vFS = _mm256_blendv_epi8(vFS, vHtS, case1);
         vFL = _mm256_blendv_epi8(
                 vFL,
-                _mm256_add_epi32(vHtL, vGapperL),
+                _mm256_add_epi64(vHtL, vGapperL),
                 case1);
         for (i=0; i<segWidth-2; ++i) {
-            __m256i vFt = _mm256_slli_si256_rpl(vF, 4);
-            __m256i vFtM = _mm256_slli_si256_rpl(vFM, 4);
-            __m256i vFtS = _mm256_slli_si256_rpl(vFS, 4);
-            __m256i vFtL = _mm256_slli_si256_rpl(vFL, 4);
-            vFt = _mm256_add_epi32(vFt, vSegLenXgap);
+            __m256i vFt = _mm256_slli_si256_rpl(vF, 8);
+            __m256i vFtM = _mm256_slli_si256_rpl(vFM, 8);
+            __m256i vFtS = _mm256_slli_si256_rpl(vFS, 8);
+            __m256i vFtL = _mm256_slli_si256_rpl(vFL, 8);
+            vFt = _mm256_add_epi64(vFt, vSegLenXgap);
             case1 = _mm256_or_si256(
-                    _mm256_cmpgt_epi32(vFt, vF),
-                    _mm256_cmpeq_epi32(vFt, vF));
-            vF = _mm256_max_epi32(vF, vFt);
+                    _mm256_cmpgt_epi64(vFt, vF),
+                    _mm256_cmpeq_epi64(vFt, vF));
+            vF = _mm256_max_epi64_rpl(vF, vFt);
             vFM = _mm256_blendv_epi8(vFM, vFtM, case1);
             vFS = _mm256_blendv_epi8(vFS, vFtS, case1);
             vFL = _mm256_blendv_epi8(
                     vFL,
-                    _mm256_add_epi32(vFtL, vSegLen),
+                    _mm256_add_epi64(vFtL, vSegLen),
                     case1);
         }
 
         /* calculate final H */
-        vF = _mm256_slli_si256_rpl(vF, 4);
-        vFM = _mm256_slli_si256_rpl(vFM, 4);
-        vFS = _mm256_slli_si256_rpl(vFS, 4);
-        vFL = _mm256_slli_si256_rpl(vFL, 4);
-        vF = _mm256_add_epi32(vF, vNegInfFront);
-        case1 = _mm256_cmpgt_epi32(vF, vHt);
-        vH = _mm256_max_epi32(vF, vHt);
+        vF = _mm256_slli_si256_rpl(vF, 8);
+        vFM = _mm256_slli_si256_rpl(vFM, 8);
+        vFS = _mm256_slli_si256_rpl(vFS, 8);
+        vFL = _mm256_slli_si256_rpl(vFL, 8);
+        vF = _mm256_add_epi64(vF, vNegInfFront);
+        case1 = _mm256_cmpgt_epi64(vF, vHt);
+        vH = _mm256_max_epi64_rpl(vF, vHt);
         vHM = _mm256_blendv_epi8(vHtM, vFM, case1);
         vHS = _mm256_blendv_epi8(vHtS, vFS, case1);
         vHL = _mm256_blendv_epi8(vHtL, vFL, case1);
@@ -516,18 +558,18 @@ parasail_result_t* PNAME(
             vEM = _mm256_load_si256(pvEM+i);
             vES = _mm256_load_si256(pvES+i);
             vEL = _mm256_load_si256(pvEL+i);
-            vF_opn = _mm256_sub_epi32(vH, vGapO);
-            vF_ext = _mm256_sub_epi32(vF, vGapE);
-            vF = _mm256_max_epi32(vF_opn, vF_ext);
-            case1 = _mm256_cmpgt_epi32(vF_opn, vF_ext);
+            vF_opn = _mm256_sub_epi64(vH, vGapO);
+            vF_ext = _mm256_sub_epi64(vF, vGapE);
+            vF = _mm256_max_epi64_rpl(vF_opn, vF_ext);
+            case1 = _mm256_cmpgt_epi64(vF_opn, vF_ext);
             vFM = _mm256_blendv_epi8(vFM, vHM, case1);
             vFS = _mm256_blendv_epi8(vFS, vHS, case1);
             vFL = _mm256_blendv_epi8(vFL, vHL, case1);
-            vFL = _mm256_add_epi32(vFL, vOne);
-            vH = _mm256_max_epi32(vHp, vE);
-            vH = _mm256_max_epi32(vH, vF);
-            case1 = _mm256_cmpeq_epi32(vH, vHp);
-            case2 = _mm256_cmpeq_epi32(vH, vF);
+            vFL = _mm256_add_epi64(vFL, vOne);
+            vH = _mm256_max_epi64_rpl(vHp, vE);
+            vH = _mm256_max_epi64_rpl(vH, vF);
+            case1 = _mm256_cmpeq_epi64(vH, vHp);
+            case2 = _mm256_cmpeq_epi64(vH, vF);
             vHM = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vEM, vFM, case2),
                     vHpM, case1);
@@ -541,11 +583,11 @@ parasail_result_t* PNAME(
             _mm256_store_si256(pvHM+i, vHM);
             _mm256_store_si256(pvHS+i, vHS);
             _mm256_store_si256(pvHL+i, vHL);
-            vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si256(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
             arr_store_si256(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
@@ -560,8 +602,8 @@ parasail_result_t* PNAME(
             vHM = _mm256_load_si256(pvHM + offset);
             vHS = _mm256_load_si256(pvHS + offset);
             vHL = _mm256_load_si256(pvHL + offset);
-            cond_max = _mm256_cmpgt_epi32(vH, vMaxH);
-            vMaxH = _mm256_max_epi32(vMaxH, vH);
+            cond_max = _mm256_cmpgt_epi64(vH, vMaxH);
+            vMaxH = _mm256_max_epi64_rpl(vMaxH, vH);
             vMaxM = _mm256_blendv_epi8(vMaxM, vHM, cond_max);
             vMaxS = _mm256_blendv_epi8(vMaxS, vHS, cond_max);
             vMaxL = _mm256_blendv_epi8(vMaxL, vHL, cond_max);
@@ -570,15 +612,15 @@ parasail_result_t* PNAME(
             }
 #ifdef PARASAIL_ROWCOL
             for (k=0; k<position; ++k) {
-                vH = _mm256_slli_si256_rpl(vH, 4);
-                vHM = _mm256_slli_si256_rpl(vHM, 4);
-                vHS = _mm256_slli_si256_rpl(vHS, 4);
-                vHL = _mm256_slli_si256_rpl(vHL, 4);
+                vH = _mm256_slli_si256_rpl(vH, 8);
+                vHM = _mm256_slli_si256_rpl(vHM, 8);
+                vHS = _mm256_slli_si256_rpl(vHS, 8);
+                vHL = _mm256_slli_si256_rpl(vHL, 8);
             }
-            result->stats->rowcols->score_row[j] = (int32_t) _mm256_extract_epi32_rpl (vH, 7);
-            result->stats->rowcols->matches_row[j] = (int32_t) _mm256_extract_epi32_rpl (vHM, 7);
-            result->stats->rowcols->similar_row[j] = (int32_t) _mm256_extract_epi32_rpl (vHS, 7);
-            result->stats->rowcols->length_row[j] = (int32_t) _mm256_extract_epi32_rpl (vHL, 7);
+            result->stats->rowcols->score_row[j] = (int64_t) _mm256_extract_epi64_rpl (vH, 3);
+            result->stats->rowcols->matches_row[j] = (int64_t) _mm256_extract_epi64_rpl (vHM, 3);
+            result->stats->rowcols->similar_row[j] = (int64_t) _mm256_extract_epi64_rpl (vHS, 3);
+            result->stats->rowcols->length_row[j] = (int64_t) _mm256_extract_epi64_rpl (vHL, 3);
 #endif
         }
     }
@@ -600,26 +642,26 @@ parasail_result_t* PNAME(
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm256_slli_si256_rpl(vMaxH, 4);
-            vMaxM = _mm256_slli_si256_rpl(vMaxM, 4);
-            vMaxS = _mm256_slli_si256_rpl(vMaxS, 4);
-            vMaxL = _mm256_slli_si256_rpl(vMaxL, 4);
+            vMaxH = _mm256_slli_si256_rpl(vMaxH, 8);
+            vMaxM = _mm256_slli_si256_rpl(vMaxM, 8);
+            vMaxS = _mm256_slli_si256_rpl(vMaxS, 8);
+            vMaxL = _mm256_slli_si256_rpl(vMaxL, 8);
         }
         end_query = s1Len-1;
-        score = (int32_t) _mm256_extract_epi32_rpl(vMaxH, 7);
-        matches = (int32_t) _mm256_extract_epi32_rpl(vMaxM, 7);
-        similar = (int32_t) _mm256_extract_epi32_rpl(vMaxS, 7);
-        length = (int32_t) _mm256_extract_epi32_rpl(vMaxL, 7);
+        score = (int64_t) _mm256_extract_epi64_rpl(vMaxH, 3);
+        matches = (int64_t) _mm256_extract_epi64_rpl(vMaxM, 3);
+        similar = (int64_t) _mm256_extract_epi64_rpl(vMaxS, 3);
+        length = (int64_t) _mm256_extract_epi64_rpl(vMaxL, 3);
     }
 
     /* max of last column */
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int32_t *t = (int32_t*)pvH;
-        int32_t *m = (int32_t*)pvHM;
-        int32_t *s = (int32_t*)pvHS;
-        int32_t *l = (int32_t*)pvHL;
+        int64_t *t = (int64_t*)pvH;
+        int64_t *m = (int64_t*)pvHM;
+        int64_t *s = (int64_t*)pvHS;
+        int64_t *l = (int64_t*)pvHL;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -649,22 +691,22 @@ parasail_result_t* PNAME(
         __m256i vHS = _mm256_load_si256(pvHS + offset);
         __m256i vHL = _mm256_load_si256(pvHL + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm256_slli_si256_rpl(vH, 4);
-            vHM = _mm256_slli_si256_rpl(vHM, 4);
-            vHS = _mm256_slli_si256_rpl(vHS, 4);
-            vHL = _mm256_slli_si256_rpl(vHL, 4);
+            vH = _mm256_slli_si256_rpl(vH, 8);
+            vHM = _mm256_slli_si256_rpl(vHM, 8);
+            vHS = _mm256_slli_si256_rpl(vHS, 8);
+            vHL = _mm256_slli_si256_rpl(vHL, 8);
         }
         end_ref = s2Len - 1;
         end_query = s1Len - 1;
-        score = (int32_t) _mm256_extract_epi32_rpl (vH, 7);
-        matches = (int32_t) _mm256_extract_epi32_rpl (vHM, 7);
-        similar = (int32_t) _mm256_extract_epi32_rpl (vHS, 7);
-        length = (int32_t) _mm256_extract_epi32_rpl (vHL, 7);
+        score = (int64_t) _mm256_extract_epi64_rpl (vH, 3);
+        matches = (int64_t) _mm256_extract_epi64_rpl (vHM, 3);
+        similar = (int64_t) _mm256_extract_epi64_rpl (vHS, 3);
+        length = (int64_t) _mm256_extract_epi64_rpl (vHL, 3);
     }
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi32_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
