@@ -11,8 +11,8 @@
 #include <stdlib.h>
 
 #define SG_STATS
-#define SG_SUFFIX _striped_avx2_256_16
-#define SG_SUFFIX_PROF _striped_profile_avx2_256_16
+#define SG_SUFFIX _striped_avx2_256_32
+#define SG_SUFFIX_PROF _striped_profile_avx2_256_32
 #include "sg_helper.h"
 
 #include <immintrin.h>
@@ -24,29 +24,29 @@
 #define SWAP(A,B) { __m256i* tmp = A; A = B; B = tmp; }
 
 
-#define _mm256_cmplt_epi16_rpl(a,b) _mm256_cmpgt_epi16(b,a)
-
-#if HAVE_AVX2_MM256_INSERT_EPI16
-#define _mm256_insert_epi16_rpl _mm256_insert_epi16
+#if HAVE_AVX2_MM256_INSERT_EPI32
+#define _mm256_insert_epi32_rpl _mm256_insert_epi32
 #else
-static inline __m256i _mm256_insert_epi16_rpl(__m256i a, int16_t i, int imm) {
-    __m256i_16_t A;
+static inline __m256i _mm256_insert_epi32_rpl(__m256i a, int32_t i, int imm) {
+    __m256i_32_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_EXTRACT_EPI16
-#define _mm256_extract_epi16_rpl _mm256_extract_epi16
+#if HAVE_AVX2_MM256_EXTRACT_EPI32
+#define _mm256_extract_epi32_rpl _mm256_extract_epi32
 #else
-static inline int16_t _mm256_extract_epi16_rpl(__m256i a, int imm) {
-    __m256i_16_t A;
+static inline int32_t _mm256_extract_epi32_rpl(__m256i a, int imm) {
+    __m256i_32_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
+#define _mm256_cmplt_epi32_rpl(a,b) _mm256_cmpgt_epi32(b,a)
+
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
 
@@ -59,22 +59,14 @@ static inline void arr_store_si256(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*( 0*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  0);
-    array[1LL*( 1*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  1);
-    array[1LL*( 2*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  2);
-    array[1LL*( 3*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  3);
-    array[1LL*( 4*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  4);
-    array[1LL*( 5*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  5);
-    array[1LL*( 6*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  6);
-    array[1LL*( 7*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  7);
-    array[1LL*( 8*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  8);
-    array[1LL*( 9*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  9);
-    array[1LL*(10*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 10);
-    array[1LL*(11*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 11);
-    array[1LL*(12*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 12);
-    array[1LL*(13*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 13);
-    array[1LL*(14*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 14);
-    array[1LL*(15*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 15);
+    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 3);
+    array[1LL*(4*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 4);
+    array[1LL*(5*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 5);
+    array[1LL*(6*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 6);
+    array[1LL*(7*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 7);
 }
 #endif
 
@@ -85,35 +77,27 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[ 0*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  0);
-    col[ 1*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  1);
-    col[ 2*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  2);
-    col[ 3*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  3);
-    col[ 4*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  4);
-    col[ 5*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  5);
-    col[ 6*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  6);
-    col[ 7*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  7);
-    col[ 8*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  8);
-    col[ 9*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  9);
-    col[10*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 10);
-    col[11*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 11);
-    col[12*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 12);
-    col[13*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 13);
-    col[14*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 14);
-    col[15*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 15);
+    col[0*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 0);
+    col[1*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 1);
+    col[2*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 2);
+    col[3*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 3);
+    col[4*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 4);
+    col[5*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 5);
+    col[6*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 6);
+    col[7*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 7);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_stats_table_striped_avx2_256_16
-#define PNAME parasail_sg_flags_stats_table_striped_profile_avx2_256_16
+#define FNAME parasail_sg_flags_stats_table_striped_avx2_256_32
+#define PNAME parasail_sg_flags_stats_table_striped_profile_avx2_256_32
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_stats_rowcol_striped_avx2_256_16
-#define PNAME parasail_sg_flags_stats_rowcol_striped_profile_avx2_256_16
+#define FNAME parasail_sg_flags_stats_rowcol_striped_avx2_256_32
+#define PNAME parasail_sg_flags_stats_rowcol_striped_profile_avx2_256_32
 #else
-#define FNAME parasail_sg_flags_stats_striped_avx2_256_16
-#define PNAME parasail_sg_flags_stats_striped_profile_avx2_256_16
+#define FNAME parasail_sg_flags_stats_striped_avx2_256_32
+#define PNAME parasail_sg_flags_stats_striped_profile_avx2_256_32
 #endif
 #endif
 
@@ -123,9 +107,31 @@ parasail_result_t* FNAME(
         const int open, const int gap, const parasail_matrix_t *matrix,
         int s1_beg, int s1_end, int s2_beg, int s2_end)
 {
-    parasail_profile_t *profile = parasail_profile_create_stats_avx_256_16(s1, s1Len, matrix);
-    parasail_result_t *result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
+    /* declare local variables */
+    parasail_profile_t *profile = NULL;
+    parasail_result_t *result = NULL;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+    PARASAIL_CHECK_NULL(matrix);
+    if (matrix->type == PARASAIL_MATRIX_TYPE_PSSM) {
+        PARASAIL_CHECK_NULL_PSSM_STATS(s1);
+    }
+    else {
+        PARASAIL_CHECK_NULL(s1);
+        PARASAIL_CHECK_GT0(s1Len);
+    }
+
+    /* initialize local variables */
+    profile = parasail_profile_create_stats_avx_256_32(s1, s1Len, matrix);
+    if (!profile) return NULL;
+    result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
+
     parasail_profile_free(profile);
+
     return result;
 }
 
@@ -135,63 +141,157 @@ parasail_result_t* PNAME(
         const int open, const int gap,
         int s1_beg, int s1_end, int s2_beg, int s2_end)
 {
+    /* declare local variables */
     int32_t i = 0;
     int32_t j = 0;
     int32_t k = 0;
-    const int s1Len = profile->s1Len;
-    int32_t end_query = s1Len-1;
-    int32_t end_ref = s2Len-1;
-    const parasail_matrix_t *matrix = profile->matrix;
-    const int32_t segWidth = 16; /* number of values in vector unit */
-    const int32_t segLen = (s1Len + segWidth - 1) / segWidth;
-    const int32_t offset = (s1Len - 1) % segLen;
-    const int32_t position = (segWidth - 1) - (s1Len - 1) / segLen;
-    __m256i* const restrict vProfile  = (__m256i*)profile->profile16.score;
-    __m256i* const restrict vProfileM = (__m256i*)profile->profile16.matches;
-    __m256i* const restrict vProfileS = (__m256i*)profile->profile16.similar;
-    __m256i* restrict pvHStore        = parasail_memalign___m256i(32, segLen);
-    __m256i* restrict pvHLoad         = parasail_memalign___m256i(32, segLen);
-    __m256i* restrict pvHMStore       = parasail_memalign___m256i(32, segLen);
-    __m256i* restrict pvHMLoad        = parasail_memalign___m256i(32, segLen);
-    __m256i* restrict pvHSStore       = parasail_memalign___m256i(32, segLen);
-    __m256i* restrict pvHSLoad        = parasail_memalign___m256i(32, segLen);
-    __m256i* restrict pvHLStore       = parasail_memalign___m256i(32, segLen);
-    __m256i* restrict pvHLLoad        = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvE       = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvEM      = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvES      = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvEL      = parasail_memalign___m256i(32, segLen);
-    int16_t* const restrict boundary = parasail_memalign_int16_t(32, s2Len+1);
-    const __m256i vGapO = _mm256_set1_epi16(open);
-    const __m256i vGapE = _mm256_set1_epi16(gap);
-    const int16_t NEG_LIMIT = (-open < matrix->min ?
-        INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    const int16_t POS_LIMIT = INT16_MAX - matrix->max - 1;
-    const __m256i vZero = _mm256_setzero_si256();
-    const __m256i vOne = _mm256_set1_epi16(1);
-    int16_t score = NEG_LIMIT;
-    int16_t matches = 0;
-    int16_t similar = 0;
-    int16_t length = 0;
-    __m256i vNegLimit = _mm256_set1_epi16(NEG_LIMIT);
-    __m256i vPosLimit = _mm256_set1_epi16(POS_LIMIT);
-    __m256i vSaturationCheckMin = vPosLimit;
-    __m256i vSaturationCheckMax = vNegLimit;
-    __m256i vMaxH = vNegLimit;
-    __m256i vMaxHM = vNegLimit;
-    __m256i vMaxHS = vNegLimit;
-    __m256i vMaxHL = vNegLimit;
-    __m256i vPosMask = _mm256_cmpeq_epi16(_mm256_set1_epi16(position),
-            _mm256_set_epi16(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15));
+    int32_t s1Len = 0;
+    int32_t end_query = 0;
+    int32_t end_ref = 0;
+    const parasail_matrix_t *matrix = NULL;
+    int32_t segWidth = 0;
+    int32_t segLen = 0;
+    int32_t offset = 0;
+    int32_t position = 0;
+    __m256i* restrict vProfile = NULL;
+    __m256i* restrict vProfileM = NULL;
+    __m256i* restrict vProfileS = NULL;
+    __m256i* restrict pvHStore = NULL;
+    __m256i* restrict pvHLoad = NULL;
+    __m256i* restrict pvHMStore = NULL;
+    __m256i* restrict pvHMLoad = NULL;
+    __m256i* restrict pvHSStore = NULL;
+    __m256i* restrict pvHSLoad = NULL;
+    __m256i* restrict pvHLStore = NULL;
+    __m256i* restrict pvHLLoad = NULL;
+    __m256i* restrict pvE = NULL;
+    __m256i* restrict pvEM = NULL;
+    __m256i* restrict pvES = NULL;
+    __m256i* restrict pvEL = NULL;
+    int32_t* restrict boundary = NULL;
+    __m256i vGapO;
+    __m256i vGapE;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
+    __m256i vZero;
+    __m256i vOne;
+    int32_t score = 0;
+    int32_t matches = 0;
+    int32_t similar = 0;
+    int32_t length = 0;
+    __m256i vNegLimit;
+    __m256i vPosLimit;
+    __m256i vSaturationCheckMin;
+    __m256i vSaturationCheckMax;
+    __m256i vMaxH;
+    __m256i vMaxHM;
+    __m256i vMaxHS;
+    __m256i vMaxHL;
+    __m256i vPosMask;
+    parasail_result_t *result = NULL;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(profile);
+    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->matrix);
+    PARASAIL_CHECK_GT0(profile->s1Len);
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+
+    /* initialize stack variables */
+    i = 0;
+    j = 0;
+    k = 0;
+    s1Len = profile->s1Len;
+    end_query = s1Len-1;
+    end_ref = s2Len-1;
+    matrix = profile->matrix;
+    segWidth = 8; /* number of values in vector unit */
+    segLen = (s1Len + segWidth - 1) / segWidth;
+    offset = (s1Len - 1) % segLen;
+    position = (segWidth - 1) - (s1Len - 1) / segLen;
+    vProfile  = (__m256i*)profile->profile32.score;
+    vProfileM = (__m256i*)profile->profile32.matches;
+    vProfileS = (__m256i*)profile->profile32.similar;
+    vGapO = _mm256_set1_epi32(open);
+    vGapE = _mm256_set1_epi32(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    vZero = _mm256_setzero_si256();
+    vOne = _mm256_set1_epi32(1);
+    score = NEG_LIMIT;
+    matches = 0;
+    similar = 0;
+    length = 0;
+    vNegLimit = _mm256_set1_epi32(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi32(POS_LIMIT);
+    vSaturationCheckMin = vPosLimit;
+    vSaturationCheckMax = vNegLimit;
+    vMaxH = vNegLimit;
+    vMaxHM = vNegLimit;
+    vMaxHS = vNegLimit;
+    vMaxHL = vNegLimit;
+    vPosMask = _mm256_cmpeq_epi32(_mm256_set1_epi32(position),
+            _mm256_set_epi32(0,1,2,3,4,5,6,7));
+
+    /* initialize result */
 #ifdef PARASAIL_TABLE
-    parasail_result_t *result = parasail_result_new_table3(segLen*segWidth, s2Len);
+    result = parasail_result_new_table3(segLen*segWidth, s2Len);
 #else
 #ifdef PARASAIL_ROWCOL
-    parasail_result_t *result = parasail_result_new_rowcol3(segLen*segWidth, s2Len);
+    result = parasail_result_new_rowcol3(segLen*segWidth, s2Len);
 #else
-    parasail_result_t *result = parasail_result_new_stats();
+    result = parasail_result_new_stats();
 #endif
 #endif
+    if (!result) return NULL;
+
+    /* set known flags */
+    result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
+        | PARASAIL_FLAG_STATS
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_8;
+    result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
+    result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
+    result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
+    result->flag |= s2_end ? PARASAIL_FLAG_SG_S2_END : 0;
+#ifdef PARASAIL_TABLE
+    result->flag |= PARASAIL_FLAG_TABLE;
+#endif
+#ifdef PARASAIL_ROWCOL
+    result->flag |= PARASAIL_FLAG_ROWCOL;
+#endif
+
+    /* initialize heap variables */
+    pvHStore  = parasail_memalign___m256i(32, segLen);
+    pvHLoad   = parasail_memalign___m256i(32, segLen);
+    pvHMStore = parasail_memalign___m256i(32, segLen);
+    pvHMLoad  = parasail_memalign___m256i(32, segLen);
+    pvHSStore = parasail_memalign___m256i(32, segLen);
+    pvHSLoad  = parasail_memalign___m256i(32, segLen);
+    pvHLStore = parasail_memalign___m256i(32, segLen);
+    pvHLLoad  = parasail_memalign___m256i(32, segLen);
+    pvE       = parasail_memalign___m256i(32, segLen);
+    pvEM      = parasail_memalign___m256i(32, segLen);
+    pvES      = parasail_memalign___m256i(32, segLen);
+    pvEL      = parasail_memalign___m256i(32, segLen);
+    boundary  = parasail_memalign_int32_t(32, s2Len+1);
+
+    /* validate heap variables */
+    if (!pvHStore) return NULL;
+    if (!pvHLoad) return NULL;
+    if (!pvHMStore) return NULL;
+    if (!pvHMLoad) return NULL;
+    if (!pvHSStore) return NULL;
+    if (!pvHSLoad) return NULL;
+    if (!pvHLStore) return NULL;
+    if (!pvHLLoad) return NULL;
+    if (!pvE) return NULL;
+    if (!pvEM) return NULL;
+    if (!pvES) return NULL;
+    if (!pvEL) return NULL;
+    if (!boundary) return NULL;
 
     parasail_memset___m256i(pvHMStore, vZero, segLen);
     parasail_memset___m256i(pvHSStore, vZero, segLen);
@@ -205,13 +305,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m256i_16_t h;
-            __m256i_16_t e;
+            __m256i_32_t h;
+            __m256i_32_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                h.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                e.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
             }
             _mm256_store_si256(&pvHStore[index], h.m);
             _mm256_store_si256(&pvE[index], e.m);
@@ -224,7 +324,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
+            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
         }
     }
 
@@ -257,18 +357,18 @@ parasail_result_t* PNAME(
         vFS = vZero;
         vFL = vOne;
 
-        /* load final segment of pvHStore and shift left by 2 bytes */
+        /* load final segment of pvHStore and shift left by 4 bytes */
         vH = _mm256_load_si256(&pvHStore[segLen - 1]);
         vHM = _mm256_load_si256(&pvHMStore[segLen - 1]);
         vHS = _mm256_load_si256(&pvHSStore[segLen - 1]);
         vHL = _mm256_load_si256(&pvHLStore[segLen - 1]);
-        vH = _mm256_slli_si256_rpl(vH, 2);
-        vHM = _mm256_slli_si256_rpl(vHM, 2);
-        vHS = _mm256_slli_si256_rpl(vHS, 2);
-        vHL = _mm256_slli_si256_rpl(vHL, 2);
+        vH = _mm256_slli_si256_rpl(vH, 4);
+        vHM = _mm256_slli_si256_rpl(vHM, 4);
+        vHS = _mm256_slli_si256_rpl(vHS, 4);
+        vHL = _mm256_slli_si256_rpl(vHL, 4);
 
         /* insert upper boundary condition */
-        vH = _mm256_insert_epi16_rpl(vH, boundary[j], 0);
+        vH = _mm256_insert_epi32_rpl(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -292,58 +392,58 @@ parasail_result_t* PNAME(
             vEL = _mm256_load_si256(pvEL+ i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm256_add_epi16(vH, _mm256_load_si256(vP + i));
-            vH = _mm256_max_epi16(vH_dag, vE);
-            vH = _mm256_max_epi16(vH, vF);
+            vH_dag = _mm256_add_epi32(vH, _mm256_load_si256(vP + i));
+            vH = _mm256_max_epi32(vH_dag, vE);
+            vH = _mm256_max_epi32(vH, vF);
             /* Save vH values. */
             _mm256_store_si256(pvHStore + i, vH);
 
-            case1 = _mm256_cmpeq_epi16(vH, vH_dag);
-            case2 = _mm256_cmpeq_epi16(vH, vF);
+            case1 = _mm256_cmpeq_epi32(vH, vH_dag);
+            case2 = _mm256_cmpeq_epi32(vH, vF);
 
             /* calculate vM */
             vHM = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vEM, vFM, case2),
-                    _mm256_add_epi16(vHM, _mm256_load_si256(vPM + i)),
+                    _mm256_add_epi32(vHM, _mm256_load_si256(vPM + i)),
                     case1);
             _mm256_store_si256(pvHMStore + i, vHM);
 
             /* calculate vS */
             vHS = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vES, vFS, case2),
-                    _mm256_add_epi16(vHS, _mm256_load_si256(vPS + i)),
+                    _mm256_add_epi32(vHS, _mm256_load_si256(vPS + i)),
                     case1);
             _mm256_store_si256(pvHSStore + i, vHS);
 
             /* calculate vL */
             vHL = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vEL, vFL, case2),
-                    _mm256_add_epi16(vHL, vOne),
+                    _mm256_add_epi32(vHL, vOne),
                     case1);
             _mm256_store_si256(pvHLStore + i, vHL);
 
-            vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si256(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
             arr_store_si256(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
             arr_store_si256(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
             arr_store_si256(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-            vEF_opn = _mm256_sub_epi16(vH, vGapO);
+            vEF_opn = _mm256_sub_epi32(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm256_sub_epi16(vE, vGapE);
-            vE = _mm256_max_epi16(vEF_opn, vE_ext);
-            case1 = _mm256_cmpgt_epi16(vEF_opn, vE_ext);
+            vE_ext = _mm256_sub_epi32(vE, vGapE);
+            vE = _mm256_max_epi32(vEF_opn, vE_ext);
+            case1 = _mm256_cmpgt_epi32(vEF_opn, vE_ext);
             vEM = _mm256_blendv_epi8(vEM, vHM, case1);
             vES = _mm256_blendv_epi8(vES, vHS, case1);
             vEL = _mm256_blendv_epi8(
-                    _mm256_add_epi16(vEL, vOne),
-                    _mm256_add_epi16(vHL, vOne),
+                    _mm256_add_epi32(vEL, vOne),
+                    _mm256_add_epi32(vHL, vOne),
                     case1);
             _mm256_store_si256(pvE + i, vE);
             _mm256_store_si256(pvEM + i, vEM);
@@ -351,14 +451,14 @@ parasail_result_t* PNAME(
             _mm256_store_si256(pvEL + i, vEL);
 
             /* Update vF value. */
-            vF_ext = _mm256_sub_epi16(vF, vGapE);
-            vF = _mm256_max_epi16(vEF_opn, vF_ext);
-            case1 = _mm256_cmpgt_epi16(vEF_opn, vF_ext);
+            vF_ext = _mm256_sub_epi32(vF, vGapE);
+            vF = _mm256_max_epi32(vEF_opn, vF_ext);
+            case1 = _mm256_cmpgt_epi32(vEF_opn, vF_ext);
             vFM = _mm256_blendv_epi8(vFM, vHM, case1);
             vFS = _mm256_blendv_epi8(vFS, vHS, case1);
             vFL = _mm256_blendv_epi8(
-                    _mm256_add_epi16(vFL, vOne),
-                    _mm256_add_epi16(vHL, vOne),
+                    _mm256_add_epi32(vFL, vOne),
+                    _mm256_add_epi32(vHL, vOne),
                     case1);
 
             /* Load the next vH. */
@@ -373,26 +473,26 @@ parasail_result_t* PNAME(
         for (k=0; k<segWidth; ++k) {
             __m256i vHp = _mm256_load_si256(&pvHLoad[segLen - 1]);
             int64_t tmp = s2_beg ? -open : (boundary[j+1]-open);
-            int16_t tmp2 = tmp < INT16_MIN ? INT16_MIN : tmp;
-            vHp = _mm256_slli_si256_rpl(vHp, 2);
-            vF = _mm256_slli_si256_rpl(vF, 2);
-            vFM = _mm256_slli_si256_rpl(vFM, 2);
-            vFS = _mm256_slli_si256_rpl(vFS, 2);
-            vFL = _mm256_slli_si256_rpl(vFL, 2);
-            vHp = _mm256_insert_epi16_rpl(vHp, boundary[j], 0);
-            vF = _mm256_insert_epi16_rpl(vF, tmp2, 0);
-            vFL = _mm256_insert_epi16_rpl(vFL, 1, 0);
+            int32_t tmp2 = tmp < INT32_MIN ? INT32_MIN : tmp;
+            vHp = _mm256_slli_si256_rpl(vHp, 4);
+            vF = _mm256_slli_si256_rpl(vF, 4);
+            vFM = _mm256_slli_si256_rpl(vFM, 4);
+            vFS = _mm256_slli_si256_rpl(vFS, 4);
+            vFL = _mm256_slli_si256_rpl(vFL, 4);
+            vHp = _mm256_insert_epi32_rpl(vHp, boundary[j], 0);
+            vF = _mm256_insert_epi32_rpl(vF, tmp2, 0);
+            vFL = _mm256_insert_epi32_rpl(vFL, 1, 0);
             for (i=0; i<segLen; ++i) {
                 __m256i case1;
                 __m256i case2;
                 __m256i cond;
 
-                vHp = _mm256_add_epi16(vHp, _mm256_load_si256(vP + i));
+                vHp = _mm256_add_epi32(vHp, _mm256_load_si256(vP + i));
                 vH = _mm256_load_si256(pvHStore + i);
-                vH = _mm256_max_epi16(vH,vF);
+                vH = _mm256_max_epi32(vH,vF);
                 _mm256_store_si256(pvHStore + i, vH);
-                case1 = _mm256_cmpeq_epi16(vH, vHp);
-                case2 = _mm256_cmpeq_epi16(vH, vF);
+                case1 = _mm256_cmpeq_epi32(vH, vHp);
+                case2 = _mm256_cmpeq_epi32(vH, vF);
                 cond = _mm256_andnot_si256(case1, case2);
 
                 /* calculate vM */
@@ -410,11 +510,11 @@ parasail_result_t* PNAME(
                 vHL = _mm256_blendv_epi8(vHL, vFL, cond);
                 _mm256_store_si256(pvHLStore + i, vHL);
 
-                vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vH);
-                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vHM);
-                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vHS);
-                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vHL);
+                vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vH);
+                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHM);
+                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHS);
+                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
                 arr_store_si256(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
                 arr_store_si256(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
@@ -422,21 +522,21 @@ parasail_result_t* PNAME(
                 arr_store_si256(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
                 /* Update vF value. */
-                vEF_opn = _mm256_sub_epi16(vH, vGapO);
-                vF_ext = _mm256_sub_epi16(vF, vGapE);
+                vEF_opn = _mm256_sub_epi32(vH, vGapO);
+                vF_ext = _mm256_sub_epi32(vF, vGapE);
                 if (! _mm256_movemask_epi8(
                             _mm256_or_si256(
-                                _mm256_cmpgt_epi16(vF_ext, vEF_opn),
-                                _mm256_cmpeq_epi16(vF_ext, vEF_opn))))
+                                _mm256_cmpgt_epi32(vF_ext, vEF_opn),
+                                _mm256_cmpeq_epi32(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = _mm256_max_epi16(vEF_opn, vF_ext);*/
+                /*vF = _mm256_max_epi32(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                cond = _mm256_cmpgt_epi16(vEF_opn, vF_ext);
+                cond = _mm256_cmpgt_epi32(vEF_opn, vF_ext);
                 vFM = _mm256_blendv_epi8(vFM, vHM, cond);
                 vFS = _mm256_blendv_epi8(vFS, vHS, cond);
                 vFL = _mm256_blendv_epi8(
-                        _mm256_add_epi16(vFL, vOne),
-                        _mm256_add_epi16(vHL, vOne),
+                        _mm256_add_epi32(vFL, vOne),
+                        _mm256_add_epi32(vHL, vOne),
                         cond);
                 vHp = _mm256_load_si256(pvHLoad + i);
             }
@@ -452,7 +552,7 @@ end:
             vHM = _mm256_load_si256(pvHMStore + offset);
             vHS = _mm256_load_si256(pvHSStore + offset);
             vHL = _mm256_load_si256(pvHLStore + offset);
-            cond_max = _mm256_cmpgt_epi16(vH, vMaxH);
+            cond_max = _mm256_cmpgt_epi32(vH, vMaxH);
             vMaxH = _mm256_blendv_epi8(vMaxH, vH, cond_max);
             vMaxHM = _mm256_blendv_epi8(vMaxHM, vHM, cond_max);
             vMaxHS = _mm256_blendv_epi8(vMaxHS, vHS, cond_max);
@@ -463,15 +563,15 @@ end:
         }
 #ifdef PARASAIL_ROWCOL
         for (k=0; k<position; ++k) {
-            vH = _mm256_slli_si256_rpl(vH, 2);
-            vHM = _mm256_slli_si256_rpl(vHM, 2);
-            vHS = _mm256_slli_si256_rpl(vHS, 2);
-            vHL = _mm256_slli_si256_rpl(vHL, 2);
+            vH = _mm256_slli_si256_rpl(vH, 4);
+            vHM = _mm256_slli_si256_rpl(vHM, 4);
+            vHS = _mm256_slli_si256_rpl(vHS, 4);
+            vHL = _mm256_slli_si256_rpl(vHL, 4);
         }
-        result->stats->rowcols->score_row[j] = (int16_t) _mm256_extract_epi16_rpl (vH, 15);
-        result->stats->rowcols->matches_row[j] = (int16_t) _mm256_extract_epi16_rpl (vHM, 15);
-        result->stats->rowcols->similar_row[j] = (int16_t) _mm256_extract_epi16_rpl (vHS, 15);
-        result->stats->rowcols->length_row[j] = (int16_t) _mm256_extract_epi16_rpl (vHL, 15);
+        result->stats->rowcols->score_row[j] = (int32_t) _mm256_extract_epi32_rpl (vH, 7);
+        result->stats->rowcols->matches_row[j] = (int32_t) _mm256_extract_epi32_rpl (vHM, 7);
+        result->stats->rowcols->similar_row[j] = (int32_t) _mm256_extract_epi32_rpl (vHS, 7);
+        result->stats->rowcols->length_row[j] = (int32_t) _mm256_extract_epi32_rpl (vHL, 7);
 #endif
     }
 
@@ -492,26 +592,26 @@ end:
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm256_slli_si256_rpl(vMaxH, 2);
-            vMaxHM = _mm256_slli_si256_rpl(vMaxHM, 2);
-            vMaxHS = _mm256_slli_si256_rpl(vMaxHS, 2);
-            vMaxHL = _mm256_slli_si256_rpl(vMaxHL, 2);
+            vMaxH = _mm256_slli_si256_rpl(vMaxH, 4);
+            vMaxHM = _mm256_slli_si256_rpl(vMaxHM, 4);
+            vMaxHS = _mm256_slli_si256_rpl(vMaxHS, 4);
+            vMaxHL = _mm256_slli_si256_rpl(vMaxHL, 4);
         }
         end_query = s1Len-1;
-        score = (int16_t) _mm256_extract_epi16_rpl(vMaxH, 15);
-        matches = (int16_t) _mm256_extract_epi16_rpl(vMaxHM, 15);
-        similar = (int16_t) _mm256_extract_epi16_rpl(vMaxHS, 15);
-        length = (int16_t) _mm256_extract_epi16_rpl(vMaxHL, 15);
+        score = (int32_t) _mm256_extract_epi32_rpl(vMaxH, 7);
+        matches = (int32_t) _mm256_extract_epi32_rpl(vMaxHM, 7);
+        similar = (int32_t) _mm256_extract_epi32_rpl(vMaxHS, 7);
+        length = (int32_t) _mm256_extract_epi32_rpl(vMaxHL, 7);
     }
 
     /* max of last column */
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int16_t *t = (int16_t*)pvHStore;
-        int16_t *m = (int16_t*)pvHMStore;
-        int16_t *s = (int16_t*)pvHSStore;
-        int16_t *l = (int16_t*)pvHLStore;
+        int32_t *t = (int32_t*)pvHStore;
+        int32_t *m = (int32_t*)pvHMStore;
+        int32_t *s = (int32_t*)pvHSStore;
+        int32_t *l = (int32_t*)pvHLStore;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -541,22 +641,22 @@ end:
         __m256i vHS = _mm256_load_si256(pvHSStore + offset);
         __m256i vHL = _mm256_load_si256(pvHLStore + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm256_slli_si256_rpl(vH, 2);
-            vHM = _mm256_slli_si256_rpl(vHM, 2);
-            vHS = _mm256_slli_si256_rpl(vHS, 2);
-            vHL = _mm256_slli_si256_rpl(vHL, 2);
+            vH = _mm256_slli_si256_rpl(vH, 4);
+            vHM = _mm256_slli_si256_rpl(vHM, 4);
+            vHS = _mm256_slli_si256_rpl(vHS, 4);
+            vHL = _mm256_slli_si256_rpl(vHL, 4);
         }
         end_ref = s2Len - 1;
         end_query = s1Len - 1;
-        score = (int16_t) _mm256_extract_epi16_rpl (vH, 15);
-        matches = (int16_t) _mm256_extract_epi16_rpl (vHM, 15);
-        similar = (int16_t) _mm256_extract_epi16_rpl (vHS, 15);
-        length = (int16_t) _mm256_extract_epi16_rpl (vHL, 15);
+        score = (int32_t) _mm256_extract_epi32_rpl (vH, 7);
+        matches = (int32_t) _mm256_extract_epi32_rpl (vHM, 7);
+        similar = (int32_t) _mm256_extract_epi32_rpl (vHS, 7);
+        length = (int32_t) _mm256_extract_epi32_rpl (vHL, 7);
     }
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi16_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi32_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
@@ -572,19 +672,6 @@ end:
     result->stats->matches = matches;
     result->stats->similar = similar;
     result->stats->length = length;
-    result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
-        | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_16;
-    result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
-    result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
-    result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
-    result->flag |= s2_end ? PARASAIL_FLAG_SG_S2_END : 0;
-#ifdef PARASAIL_TABLE
-    result->flag |= PARASAIL_FLAG_TABLE;
-#endif
-#ifdef PARASAIL_ROWCOL
-    result->flag |= PARASAIL_FLAG_ROWCOL;
-#endif
 
     parasail_free(boundary);
     parasail_free(pvEL);
