@@ -16,8 +16,8 @@
 #include "parasail/memory.h"
 #include "parasail/internal_altivec.h"
 
-#define SG_SUFFIX _striped_altivec_128_16
-#define SG_SUFFIX_PROF _striped_profile_altivec_128_16
+#define SG_SUFFIX _striped_altivec_128_64
+#define SG_SUFFIX_PROF _striped_profile_altivec_128_64
 #include "sg_helper.h"
 
 
@@ -31,14 +31,8 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 3);
-    array[1LL*(4*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 4);
-    array[1LL*(5*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 5);
-    array[1LL*(6*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 6);
-    array[1LL*(7*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 7);
+    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)_mm_extract_epi64(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)_mm_extract_epi64(vH, 1);
 }
 #endif
 
@@ -49,27 +43,21 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int16_t)_mm_extract_epi16(vH, 0);
-    col[1*seglen+t] = (int16_t)_mm_extract_epi16(vH, 1);
-    col[2*seglen+t] = (int16_t)_mm_extract_epi16(vH, 2);
-    col[3*seglen+t] = (int16_t)_mm_extract_epi16(vH, 3);
-    col[4*seglen+t] = (int16_t)_mm_extract_epi16(vH, 4);
-    col[5*seglen+t] = (int16_t)_mm_extract_epi16(vH, 5);
-    col[6*seglen+t] = (int16_t)_mm_extract_epi16(vH, 6);
-    col[7*seglen+t] = (int16_t)_mm_extract_epi16(vH, 7);
+    col[0*seglen+t] = (int64_t)_mm_extract_epi64(vH, 0);
+    col[1*seglen+t] = (int64_t)_mm_extract_epi64(vH, 1);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_table_striped_altivec_128_16
-#define PNAME parasail_sg_flags_table_striped_profile_altivec_128_16
+#define FNAME parasail_sg_flags_table_striped_altivec_128_64
+#define PNAME parasail_sg_flags_table_striped_profile_altivec_128_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_rowcol_striped_altivec_128_16
-#define PNAME parasail_sg_flags_rowcol_striped_profile_altivec_128_16
+#define FNAME parasail_sg_flags_rowcol_striped_altivec_128_64
+#define PNAME parasail_sg_flags_rowcol_striped_profile_altivec_128_64
 #else
-#define FNAME parasail_sg_flags_striped_altivec_128_16
-#define PNAME parasail_sg_flags_striped_profile_altivec_128_16
+#define FNAME parasail_sg_flags_striped_altivec_128_64
+#define PNAME parasail_sg_flags_striped_profile_altivec_128_64
 #endif
 #endif
 
@@ -95,7 +83,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_altivec_128_16(s1, s1Len, matrix);
+    profile = parasail_profile_create_altivec_128_64(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -126,12 +114,12 @@ parasail_result_t* PNAME(
     vec128i* restrict pvHStore = NULL;
     vec128i* restrict pvHLoad = NULL;
     vec128i* restrict pvE = NULL;
-    int16_t* restrict boundary = NULL;
+    int64_t* restrict boundary = NULL;
     vec128i vGapO;
     vec128i vGapE;
-    int16_t NEG_LIMIT = 0;
-    int16_t POS_LIMIT = 0;
-    int16_t score = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
+    int64_t score = 0;
     vec128i vNegLimit;
     vec128i vPosLimit;
     vec128i vSaturationCheckMin;
@@ -142,7 +130,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile16.score);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -158,23 +146,23 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 8; /* number of values in vector unit */
+    segWidth = 2; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile = (vec128i*)profile->profile16.score;
-    vGapO = _mm_set1_epi16(open);
-    vGapE = _mm_set1_epi16(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    vProfile = (vec128i*)profile->profile64.score;
+    vGapO = _mm_set1_epi64(open);
+    vGapE = _mm_set1_epi64(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi16(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi16(POS_LIMIT);
+    vNegLimit = _mm_set1_epi64(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi64(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
-    vPosMask = _mm_cmpeq_epi16(_mm_set1_epi16(position),
-            _mm_set_epi16(0,1,2,3,4,5,6,7));
+    vPosMask = _mm_cmpeq_epi64(_mm_set1_epi64(position),
+            _mm_set_epi64(0,1));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -190,7 +178,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -206,7 +194,7 @@ parasail_result_t* PNAME(
     pvHStore = parasail_memalign_vec128i(16, segLen);
     pvHLoad =  parasail_memalign_vec128i(16, segLen);
     pvE      = parasail_memalign_vec128i(16, segLen);
-    boundary = parasail_memalign_int16_t(16, s2Len+1);
+    boundary = parasail_memalign_int64_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -219,13 +207,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            vec128i_16_t h;
-            vec128i_16_t e;
+            vec128i_64_t h;
+            vec128i_64_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                e.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
             }
             _mm_store_si128(&pvHStore[index], h.m);
             _mm_store_si128(&pvE[index], e.m);
@@ -238,7 +226,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
+            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
         }
     }
 
@@ -250,7 +238,7 @@ parasail_result_t* PNAME(
         vec128i vF = vNegLimit;
 
         /* load final segment of pvHStore and shift left by 2 bytes */
-        vec128i vH = _mm_slli_si128(pvHStore[segLen - 1], 2);
+        vec128i vH = _mm_slli_si128(pvHStore[segLen - 1], 8);
 
         /* Correct part of the vProfile */
         const vec128i* vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -261,35 +249,35 @@ parasail_result_t* PNAME(
         pvHStore = pv;
 
         /* insert upper boundary condition */
-        vH = _mm_insert_epi16(vH, boundary[j], 0);
+        vH = _mm_insert_epi64(vH, boundary[j], 0);
 
         /* inner loop to process the query sequence */
         for (i=0; i<segLen; ++i) {
-            vH = _mm_adds_epi16(vH, _mm_load_si128(vP + i));
+            vH = _mm_add_epi64(vH, _mm_load_si128(vP + i));
             vE = _mm_load_si128(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH = _mm_max_epi16(vH, vE);
-            vH = _mm_max_epi16(vH, vF);
+            vH = _mm_max_epi64(vH, vE);
+            vH = _mm_max_epi64(vH, vF);
             /* Save vH values. */
             _mm_store_si128(pvHStore + i, vH);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
-            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vE);
-            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm_min_epi64(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm_min_epi64(vSaturationCheckMin, vE);
+            vSaturationCheckMin = _mm_min_epi64(vSaturationCheckMin, vF);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
 
             /* Update vE value. */
-            vH = _mm_subs_epi16(vH, vGapO);
-            vE = _mm_subs_epi16(vE, vGapE);
-            vE = _mm_max_epi16(vE, vH);
+            vH = _mm_sub_epi64(vH, vGapO);
+            vE = _mm_sub_epi64(vE, vGapE);
+            vE = _mm_max_epi64(vE, vH);
             _mm_store_si128(pvE + i, vE);
 
             /* Update vF value. */
-            vF = _mm_subs_epi16(vF, vGapE);
-            vF = _mm_max_epi16(vF, vH);
+            vF = _mm_sub_epi64(vF, vGapE);
+            vF = _mm_max_epi64(vF, vH);
 
             /* Load the next vH. */
             vH = _mm_load_si128(pvHLoad + i);
@@ -299,22 +287,22 @@ parasail_result_t* PNAME(
          * then deletion, so don't update E(i, i), learn from SWPS3 */
         for (k=0; k<segWidth; ++k) {
             int64_t tmp = s2_beg ? -open : (boundary[j+1]-open);
-            int16_t tmp2 = tmp < INT16_MIN ? INT16_MIN : tmp;
-            vF = _mm_slli_si128(vF, 2);
-            vF = _mm_insert_epi16(vF, tmp2, 0);
+            int64_t tmp2 = tmp < INT64_MIN ? INT64_MIN : tmp;
+            vF = _mm_slli_si128(vF, 8);
+            vF = _mm_insert_epi64(vF, tmp2, 0);
             for (i=0; i<segLen; ++i) {
                 vH = _mm_load_si128(pvHStore + i);
-                vH = _mm_max_epi16(vH,vF);
+                vH = _mm_max_epi64(vH,vF);
                 _mm_store_si128(pvHStore + i, vH);
-                vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
+                vSaturationCheckMin = _mm_min_epi64(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vH);
 #ifdef PARASAIL_TABLE
                 arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-                vH = _mm_subs_epi16(vH, vGapO);
-                vF = _mm_subs_epi16(vF, vGapE);
-                if (! _mm_movemask_epi8(_mm_cmpgt_epi16(vF, vH))) goto end;
-                /*vF = _mm_max_epi16(vF, vH);*/
+                vH = _mm_sub_epi64(vH, vGapO);
+                vF = _mm_sub_epi64(vF, vGapE);
+                if (! _mm_movemask_epi8(_mm_cmpgt_epi64(vF, vH))) goto end;
+                /*vF = _mm_max_epi64(vF, vH);*/
             }
         }
 end:
@@ -322,16 +310,16 @@ end:
             /* extract vector containing last value from the column */
             vec128i vCompare;
             vH = _mm_load_si128(pvHStore + offset);
-            vCompare = _mm_and_si128(vPosMask, _mm_cmpgt_epi16(vH, vMaxH));
-            vMaxH = _mm_max_epi16(vH, vMaxH);
+            vCompare = _mm_and_si128(vPosMask, _mm_cmpgt_epi64(vH, vMaxH));
+            vMaxH = _mm_max_epi64(vH, vMaxH);
             if (_mm_movemask_epi8(vCompare)) {
                 end_ref = j;
             }
 #ifdef PARASAIL_ROWCOL
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 2);
+                vH = _mm_slli_si128(vH, 8);
             }
-            result->rowcols->score_row[j] = (int16_t) _mm_extract_epi16 (vH, 7);
+            result->rowcols->score_row[j] = (int64_t) _mm_extract_epi64 (vH, 1);
 #endif
         }
     }
@@ -347,9 +335,9 @@ end:
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm_slli_si128(vMaxH, 2);
+            vMaxH = _mm_slli_si128(vMaxH, 8);
         }
-        score = (int16_t) _mm_extract_epi16(vMaxH, 7);
+        score = (int64_t) _mm_extract_epi64(vMaxH, 1);
         end_query = s1Len-1;
     }
 
@@ -357,7 +345,7 @@ end:
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int16_t *t = (int16_t*)pvHStore;
+        int64_t *t = (int64_t*)pvHStore;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -378,17 +366,17 @@ end:
         {
             vec128i vH = _mm_load_si128(pvHStore + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 2);
+                vH = _mm_slli_si128(vH, 8);
             }
-            score = (int16_t) _mm_extract_epi16 (vH, 7);
+            score = (int64_t) _mm_extract_epi64 (vH, 1);
             end_ref = s2Len - 1;
             end_query = s1Len - 1;
         }
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
