@@ -16,33 +16,83 @@
 #include "parasail/memory.h"
 #include "parasail/internal_avx.h"
 
-#define SG_SUFFIX _striped_avx2_256_8
-#define SG_SUFFIX_PROF _striped_profile_avx2_256_8
+#define SG_SUFFIX _striped_avx2_256_64
+#define SG_SUFFIX_PROF _striped_profile_avx2_256_64
 #include "sg_helper.h"
 
 
-#if HAVE_AVX2_MM256_INSERT_EPI8
-#define _mm256_insert_epi8_rpl _mm256_insert_epi8
+#if HAVE_AVX2_MM256_INSERT_EPI64
+#define _mm256_insert_epi64_rpl _mm256_insert_epi64
 #else
-static inline __m256i _mm256_insert_epi8_rpl(__m256i a, int8_t i, int imm) {
-    __m256i_8_t A;
+static inline __m256i _mm256_insert_epi64_rpl(__m256i a, int64_t i, int imm) {
+    __m256i_64_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_EXTRACT_EPI8
-#define _mm256_extract_epi8_rpl _mm256_extract_epi8
+#if HAVE_AVX2_MM256_SET1_EPI64X
+#define _mm256_set1_epi64x_rpl _mm256_set1_epi64x
 #else
-static inline int8_t _mm256_extract_epi8_rpl(__m256i a, int imm) {
-    __m256i_8_t A;
+static inline __m256i _mm256_set1_epi64x_rpl(int64_t i) {
+    __m256i_64_t A;
+    A.v[0] = i;
+    A.v[1] = i;
+    A.v[2] = i;
+    A.v[3] = i;
+    return A.m;
+}
+#endif
+
+static inline __m256i _mm256_max_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]>B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]>B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
+#if HAVE_AVX2_MM256_SET_EPI64X
+#define _mm256_set_epi64x_rpl _mm256_set_epi64x
+#else
+static inline __m256i _mm256_set_epi64x_rpl(int64_t e3, int64_t e2, int64_t e1, int64_t e0) {
+    __m256i_64_t A;
+    A.v[0] = e0;
+    A.v[1] = e1;
+    A.v[2] = e2;
+    A.v[3] = e3;
+    return A.m;
+}
+#endif
+
+#if HAVE_AVX2_MM256_EXTRACT_EPI64
+#define _mm256_extract_epi64_rpl _mm256_extract_epi64
+#else
+static inline int64_t _mm256_extract_epi64_rpl(__m256i a, int imm) {
+    __m256i_64_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
-#define _mm256_cmplt_epi8_rpl(a,b) _mm256_cmpgt_epi8(b,a)
+static inline __m256i _mm256_min_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]<B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]<B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
+#define _mm256_cmplt_epi64_rpl(a,b) _mm256_cmpgt_epi64(b,a)
 
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
@@ -56,38 +106,10 @@ static inline void arr_store_si256(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  0);
-    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  1);
-    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  2);
-    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  3);
-    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  4);
-    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  5);
-    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  6);
-    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  7);
-    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  8);
-    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  9);
-    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 10);
-    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 11);
-    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 12);
-    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 13);
-    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 14);
-    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 15);
-    array[1LL*(16*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 16);
-    array[1LL*(17*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 17);
-    array[1LL*(18*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 18);
-    array[1LL*(19*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 19);
-    array[1LL*(20*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 20);
-    array[1LL*(21*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 21);
-    array[1LL*(22*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 22);
-    array[1LL*(23*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 23);
-    array[1LL*(24*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 24);
-    array[1LL*(25*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 25);
-    array[1LL*(26*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 26);
-    array[1LL*(27*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 27);
-    array[1LL*(28*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 28);
-    array[1LL*(29*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 29);
-    array[1LL*(30*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 30);
-    array[1LL*(31*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 31);
+    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 3);
 }
 #endif
 
@@ -98,51 +120,23 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[ 0*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  0);
-    col[ 1*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  1);
-    col[ 2*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  2);
-    col[ 3*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  3);
-    col[ 4*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  4);
-    col[ 5*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  5);
-    col[ 6*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  6);
-    col[ 7*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  7);
-    col[ 8*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  8);
-    col[ 9*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  9);
-    col[10*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 10);
-    col[11*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 11);
-    col[12*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 12);
-    col[13*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 13);
-    col[14*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 14);
-    col[15*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 15);
-    col[16*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 16);
-    col[17*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 17);
-    col[18*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 18);
-    col[19*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 19);
-    col[20*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 20);
-    col[21*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 21);
-    col[22*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 22);
-    col[23*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 23);
-    col[24*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 24);
-    col[25*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 25);
-    col[26*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 26);
-    col[27*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 27);
-    col[28*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 28);
-    col[29*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 29);
-    col[30*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 30);
-    col[31*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 31);
+    col[0*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 0);
+    col[1*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 1);
+    col[2*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 2);
+    col[3*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 3);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_table_striped_avx2_256_8
-#define PNAME parasail_sg_flags_table_striped_profile_avx2_256_8
+#define FNAME parasail_sg_flags_table_striped_avx2_256_64
+#define PNAME parasail_sg_flags_table_striped_profile_avx2_256_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_rowcol_striped_avx2_256_8
-#define PNAME parasail_sg_flags_rowcol_striped_profile_avx2_256_8
+#define FNAME parasail_sg_flags_rowcol_striped_avx2_256_64
+#define PNAME parasail_sg_flags_rowcol_striped_profile_avx2_256_64
 #else
-#define FNAME parasail_sg_flags_striped_avx2_256_8
-#define PNAME parasail_sg_flags_striped_profile_avx2_256_8
+#define FNAME parasail_sg_flags_striped_avx2_256_64
+#define PNAME parasail_sg_flags_striped_profile_avx2_256_64
 #endif
 #endif
 
@@ -168,7 +162,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_avx_256_8(s1, s1Len, matrix);
+    profile = parasail_profile_create_avx_256_64(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -199,12 +193,12 @@ parasail_result_t* PNAME(
     __m256i* restrict pvHStore = NULL;
     __m256i* restrict pvHLoad = NULL;
     __m256i* restrict pvE = NULL;
-    int8_t* restrict boundary = NULL;
+    int64_t* restrict boundary = NULL;
     __m256i vGapO;
     __m256i vGapE;
-    int8_t NEG_LIMIT = 0;
-    int8_t POS_LIMIT = 0;
-    int8_t score = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
+    int64_t score = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -215,7 +209,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile8.score);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -231,23 +225,23 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 32; /* number of values in vector unit */
+    segWidth = 4; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile = (__m256i*)profile->profile8.score;
-    vGapO = _mm256_set1_epi8(open);
-    vGapE = _mm256_set1_epi8(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
-    POS_LIMIT = INT8_MAX - matrix->max - 1;
+    vProfile = (__m256i*)profile->profile64.score;
+    vGapO = _mm256_set1_epi64x_rpl(open);
+    vGapE = _mm256_set1_epi64x_rpl(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm256_set1_epi8(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi8(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
-    vPosMask = _mm256_cmpeq_epi8(_mm256_set1_epi8(position),
-            _mm256_set_epi8(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31));
+    vPosMask = _mm256_cmpeq_epi64(_mm256_set1_epi64x_rpl(position),
+            _mm256_set_epi64x_rpl(0,1,2,3));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -263,7 +257,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
-        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_32;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -279,7 +273,7 @@ parasail_result_t* PNAME(
     pvHStore = parasail_memalign___m256i(32, segLen);
     pvHLoad =  parasail_memalign___m256i(32, segLen);
     pvE      = parasail_memalign___m256i(32, segLen);
-    boundary = parasail_memalign_int8_t(32, s2Len+1);
+    boundary = parasail_memalign_int64_t(32, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -292,13 +286,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m256i_8_t h;
-            __m256i_8_t e;
+            __m256i_64_t h;
+            __m256i_64_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
+                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
+                e.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
             }
             _mm256_store_si256(&pvHStore[index], h.m);
             _mm256_store_si256(&pvE[index], e.m);
@@ -311,7 +305,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
+            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
         }
     }
 
@@ -323,7 +317,7 @@ parasail_result_t* PNAME(
         __m256i vF = vNegLimit;
 
         /* load final segment of pvHStore and shift left by 2 bytes */
-        __m256i vH = _mm256_slli_si256_rpl(pvHStore[segLen - 1], 1);
+        __m256i vH = _mm256_slli_si256_rpl(pvHStore[segLen - 1], 8);
 
         /* Correct part of the vProfile */
         const __m256i* vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -334,35 +328,35 @@ parasail_result_t* PNAME(
         pvHStore = pv;
 
         /* insert upper boundary condition */
-        vH = _mm256_insert_epi8_rpl(vH, boundary[j], 0);
+        vH = _mm256_insert_epi64_rpl(vH, boundary[j], 0);
 
         /* inner loop to process the query sequence */
         for (i=0; i<segLen; ++i) {
-            vH = _mm256_adds_epi8(vH, _mm256_load_si256(vP + i));
+            vH = _mm256_add_epi64(vH, _mm256_load_si256(vP + i));
             vE = _mm256_load_si256(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH = _mm256_max_epi8(vH, vE);
-            vH = _mm256_max_epi8(vH, vF);
+            vH = _mm256_max_epi64_rpl(vH, vE);
+            vH = _mm256_max_epi64_rpl(vH, vF);
             /* Save vH values. */
             _mm256_store_si256(pvHStore + i, vH);
-            vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vH);
-            vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vE);
-            vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vE);
+            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vF);
 #ifdef PARASAIL_TABLE
             arr_store_si256(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
 
             /* Update vE value. */
-            vH = _mm256_subs_epi8(vH, vGapO);
-            vE = _mm256_subs_epi8(vE, vGapE);
-            vE = _mm256_max_epi8(vE, vH);
+            vH = _mm256_sub_epi64(vH, vGapO);
+            vE = _mm256_sub_epi64(vE, vGapE);
+            vE = _mm256_max_epi64_rpl(vE, vH);
             _mm256_store_si256(pvE + i, vE);
 
             /* Update vF value. */
-            vF = _mm256_subs_epi8(vF, vGapE);
-            vF = _mm256_max_epi8(vF, vH);
+            vF = _mm256_sub_epi64(vF, vGapE);
+            vF = _mm256_max_epi64_rpl(vF, vH);
 
             /* Load the next vH. */
             vH = _mm256_load_si256(pvHLoad + i);
@@ -372,22 +366,22 @@ parasail_result_t* PNAME(
          * then deletion, so don't update E(i, i), learn from SWPS3 */
         for (k=0; k<segWidth; ++k) {
             int64_t tmp = s2_beg ? -open : (boundary[j+1]-open);
-            int8_t tmp2 = tmp < INT8_MIN ? INT8_MIN : tmp;
-            vF = _mm256_slli_si256_rpl(vF, 1);
-            vF = _mm256_insert_epi8_rpl(vF, tmp2, 0);
+            int64_t tmp2 = tmp < INT64_MIN ? INT64_MIN : tmp;
+            vF = _mm256_slli_si256_rpl(vF, 8);
+            vF = _mm256_insert_epi64_rpl(vF, tmp2, 0);
             for (i=0; i<segLen; ++i) {
                 vH = _mm256_load_si256(pvHStore + i);
-                vH = _mm256_max_epi8(vH,vF);
+                vH = _mm256_max_epi64_rpl(vH,vF);
                 _mm256_store_si256(pvHStore + i, vH);
-                vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vH);
+                vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vH);
 #ifdef PARASAIL_TABLE
                 arr_store_si256(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-                vH = _mm256_subs_epi8(vH, vGapO);
-                vF = _mm256_subs_epi8(vF, vGapE);
-                if (! _mm256_movemask_epi8(_mm256_cmpgt_epi8(vF, vH))) goto end;
-                /*vF = _mm256_max_epi8(vF, vH);*/
+                vH = _mm256_sub_epi64(vH, vGapO);
+                vF = _mm256_sub_epi64(vF, vGapE);
+                if (! _mm256_movemask_epi8(_mm256_cmpgt_epi64(vF, vH))) goto end;
+                /*vF = _mm256_max_epi64_rpl(vF, vH);*/
             }
         }
 end:
@@ -395,16 +389,16 @@ end:
             /* extract vector containing last value from the column */
             __m256i vCompare;
             vH = _mm256_load_si256(pvHStore + offset);
-            vCompare = _mm256_and_si256(vPosMask, _mm256_cmpgt_epi8(vH, vMaxH));
-            vMaxH = _mm256_max_epi8(vH, vMaxH);
+            vCompare = _mm256_and_si256(vPosMask, _mm256_cmpgt_epi64(vH, vMaxH));
+            vMaxH = _mm256_max_epi64_rpl(vH, vMaxH);
             if (_mm256_movemask_epi8(vCompare)) {
                 end_ref = j;
             }
 #ifdef PARASAIL_ROWCOL
             for (k=0; k<position; ++k) {
-                vH = _mm256_slli_si256_rpl(vH, 1);
+                vH = _mm256_slli_si256_rpl(vH, 8);
             }
-            result->rowcols->score_row[j] = (int8_t) _mm256_extract_epi8_rpl (vH, 31);
+            result->rowcols->score_row[j] = (int64_t) _mm256_extract_epi64_rpl (vH, 3);
 #endif
         }
     }
@@ -420,9 +414,9 @@ end:
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm256_slli_si256_rpl(vMaxH, 1);
+            vMaxH = _mm256_slli_si256_rpl(vMaxH, 8);
         }
-        score = (int8_t) _mm256_extract_epi8_rpl(vMaxH, 31);
+        score = (int64_t) _mm256_extract_epi64_rpl(vMaxH, 3);
         end_query = s1Len-1;
     }
 
@@ -430,7 +424,7 @@ end:
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int8_t *t = (int8_t*)pvHStore;
+        int64_t *t = (int64_t*)pvHStore;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -451,17 +445,17 @@ end:
         {
             __m256i vH = _mm256_load_si256(pvHStore + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm256_slli_si256_rpl(vH, 1);
+                vH = _mm256_slli_si256_rpl(vH, 8);
             }
-            score = (int8_t) _mm256_extract_epi8_rpl (vH, 31);
+            score = (int64_t) _mm256_extract_epi64_rpl (vH, 3);
             end_ref = s2Len - 1;
             end_query = s1Len - 1;
         }
     }
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi8_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
