@@ -20,11 +20,38 @@
 #include "parasail/memory.h"
 #include "parasail/internal_sse.h"
 
-#define SG_SUFFIX _striped_sse2_128_16
-#define SG_SUFFIX_PROF _striped_profile_sse2_128_16
+#define SG_SUFFIX _striped_sse2_128_8
+#define SG_SUFFIX_PROF _striped_profile_sse2_128_8
 #include "sg_helper.h"
 
 
+static inline __m128i _mm_insert_epi8_rpl(__m128i a, int8_t i, const int imm) {
+    __m128i_8_t A;
+    A.m = a;
+    A.v[imm] = i;
+    return A.m;
+}
+
+static inline __m128i _mm_max_epi8_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi8(a, b);
+    a = _mm_and_si128(a, mask);
+    b = _mm_andnot_si128(mask, b);
+    return _mm_or_si128(a, b);
+}
+
+static inline int8_t _mm_extract_epi8_rpl(__m128i a, const int imm) {
+    __m128i_8_t A;
+    A.m = a;
+    return A.v[imm];
+}
+
+static inline __m128i _mm_min_epi8_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi8(b, a);
+    a = _mm_and_si128(a, mask);
+    b = _mm_andnot_si128(mask, b);
+    return _mm_or_si128(a, b);
+}
+
 
 #ifdef PARASAIL_TABLE
 static inline void arr_store_si128(
@@ -35,14 +62,22 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 3);
-    array[1LL*(4*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 4);
-    array[1LL*(5*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 5);
-    array[1LL*(6*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 6);
-    array[1LL*(7*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 7);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  0);
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  1);
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  2);
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  3);
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  4);
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  5);
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  6);
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  7);
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  8);
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  9);
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 10);
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 11);
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 12);
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 13);
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 14);
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 15);
 }
 #endif
 
@@ -53,27 +88,35 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int16_t)_mm_extract_epi16(vH, 0);
-    col[1*seglen+t] = (int16_t)_mm_extract_epi16(vH, 1);
-    col[2*seglen+t] = (int16_t)_mm_extract_epi16(vH, 2);
-    col[3*seglen+t] = (int16_t)_mm_extract_epi16(vH, 3);
-    col[4*seglen+t] = (int16_t)_mm_extract_epi16(vH, 4);
-    col[5*seglen+t] = (int16_t)_mm_extract_epi16(vH, 5);
-    col[6*seglen+t] = (int16_t)_mm_extract_epi16(vH, 6);
-    col[7*seglen+t] = (int16_t)_mm_extract_epi16(vH, 7);
+    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  0);
+    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  1);
+    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  2);
+    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  3);
+    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  4);
+    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  5);
+    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  6);
+    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  7);
+    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  8);
+    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  9);
+    col[10*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 10);
+    col[11*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 11);
+    col[12*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 12);
+    col[13*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 13);
+    col[14*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 14);
+    col[15*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 15);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_table_striped_sse2_128_16
-#define PNAME parasail_sg_flags_table_striped_profile_sse2_128_16
+#define FNAME parasail_sg_flags_table_striped_sse2_128_8
+#define PNAME parasail_sg_flags_table_striped_profile_sse2_128_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_rowcol_striped_sse2_128_16
-#define PNAME parasail_sg_flags_rowcol_striped_profile_sse2_128_16
+#define FNAME parasail_sg_flags_rowcol_striped_sse2_128_8
+#define PNAME parasail_sg_flags_rowcol_striped_profile_sse2_128_8
 #else
-#define FNAME parasail_sg_flags_striped_sse2_128_16
-#define PNAME parasail_sg_flags_striped_profile_sse2_128_16
+#define FNAME parasail_sg_flags_striped_sse2_128_8
+#define PNAME parasail_sg_flags_striped_profile_sse2_128_8
 #endif
 #endif
 
@@ -99,7 +142,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_sse_128_16(s1, s1Len, matrix);
+    profile = parasail_profile_create_sse_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -130,12 +173,12 @@ parasail_result_t* PNAME(
     __m128i* restrict pvHStore = NULL;
     __m128i* restrict pvHLoad = NULL;
     __m128i* restrict pvE = NULL;
-    int16_t* restrict boundary = NULL;
+    int8_t* restrict boundary = NULL;
     __m128i vGapO;
     __m128i vGapE;
-    int16_t NEG_LIMIT = 0;
-    int16_t POS_LIMIT = 0;
-    int16_t score = 0;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
+    int8_t score = 0;
     __m128i vNegLimit;
     __m128i vPosLimit;
     __m128i vSaturationCheckMin;
@@ -146,7 +189,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile16.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -162,23 +205,23 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 8; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile = (__m128i*)profile->profile16.score;
-    vGapO = _mm_set1_epi16(open);
-    vGapE = _mm_set1_epi16(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    vProfile = (__m128i*)profile->profile8.score;
+    vGapO = _mm_set1_epi8(open);
+    vGapE = _mm_set1_epi8(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi16(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi16(POS_LIMIT);
+    vNegLimit = _mm_set1_epi8(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi8(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
-    vPosMask = _mm_cmpeq_epi16(_mm_set1_epi16(position),
-            _mm_set_epi16(0,1,2,3,4,5,6,7));
+    vPosMask = _mm_cmpeq_epi8(_mm_set1_epi8(position),
+            _mm_set_epi8(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -194,7 +237,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -210,7 +253,7 @@ parasail_result_t* PNAME(
     pvHStore = parasail_memalign___m128i(16, segLen);
     pvHLoad =  parasail_memalign___m128i(16, segLen);
     pvE      = parasail_memalign___m128i(16, segLen);
-    boundary = parasail_memalign_int16_t(16, s2Len+1);
+    boundary = parasail_memalign_int8_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -223,13 +266,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m128i_16_t h;
-            __m128i_16_t e;
+            __m128i_8_t h;
+            __m128i_8_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                h.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                e.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
             }
             _mm_store_si128(&pvHStore[index], h.m);
             _mm_store_si128(&pvE[index], e.m);
@@ -242,7 +285,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
+            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
         }
     }
 
@@ -254,7 +297,7 @@ parasail_result_t* PNAME(
         __m128i vF = vNegLimit;
 
         /* load final segment of pvHStore and shift left by 2 bytes */
-        __m128i vH = _mm_slli_si128(pvHStore[segLen - 1], 2);
+        __m128i vH = _mm_slli_si128(pvHStore[segLen - 1], 1);
 
         /* Correct part of the vProfile */
         const __m128i* vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -265,35 +308,35 @@ parasail_result_t* PNAME(
         pvHStore = pv;
 
         /* insert upper boundary condition */
-        vH = _mm_insert_epi16(vH, boundary[j], 0);
+        vH = _mm_insert_epi8_rpl(vH, boundary[j], 0);
 
         /* inner loop to process the query sequence */
         for (i=0; i<segLen; ++i) {
-            vH = _mm_adds_epi16(vH, _mm_load_si128(vP + i));
+            vH = _mm_adds_epi8(vH, _mm_load_si128(vP + i));
             vE = _mm_load_si128(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH = _mm_max_epi16(vH, vE);
-            vH = _mm_max_epi16(vH, vF);
+            vH = _mm_max_epi8_rpl(vH, vE);
+            vH = _mm_max_epi8_rpl(vH, vF);
             /* Save vH values. */
             _mm_store_si128(pvHStore + i, vH);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
-            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vE);
-            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm_min_epi8_rpl(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm_min_epi8_rpl(vSaturationCheckMin, vE);
+            vSaturationCheckMin = _mm_min_epi8_rpl(vSaturationCheckMin, vF);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
 
             /* Update vE value. */
-            vH = _mm_subs_epi16(vH, vGapO);
-            vE = _mm_subs_epi16(vE, vGapE);
-            vE = _mm_max_epi16(vE, vH);
+            vH = _mm_subs_epi8(vH, vGapO);
+            vE = _mm_subs_epi8(vE, vGapE);
+            vE = _mm_max_epi8_rpl(vE, vH);
             _mm_store_si128(pvE + i, vE);
 
             /* Update vF value. */
-            vF = _mm_subs_epi16(vF, vGapE);
-            vF = _mm_max_epi16(vF, vH);
+            vF = _mm_subs_epi8(vF, vGapE);
+            vF = _mm_max_epi8_rpl(vF, vH);
 
             /* Load the next vH. */
             vH = _mm_load_si128(pvHLoad + i);
@@ -303,22 +346,22 @@ parasail_result_t* PNAME(
          * then deletion, so don't update E(i, i), learn from SWPS3 */
         for (k=0; k<segWidth; ++k) {
             int64_t tmp = s2_beg ? -open : (boundary[j+1]-open);
-            int16_t tmp2 = tmp < INT16_MIN ? INT16_MIN : tmp;
-            vF = _mm_slli_si128(vF, 2);
-            vF = _mm_insert_epi16(vF, tmp2, 0);
+            int8_t tmp2 = tmp < INT8_MIN ? INT8_MIN : tmp;
+            vF = _mm_slli_si128(vF, 1);
+            vF = _mm_insert_epi8_rpl(vF, tmp2, 0);
             for (i=0; i<segLen; ++i) {
                 vH = _mm_load_si128(pvHStore + i);
-                vH = _mm_max_epi16(vH,vF);
+                vH = _mm_max_epi8_rpl(vH,vF);
                 _mm_store_si128(pvHStore + i, vH);
-                vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
+                vSaturationCheckMin = _mm_min_epi8_rpl(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vH);
 #ifdef PARASAIL_TABLE
                 arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-                vH = _mm_subs_epi16(vH, vGapO);
-                vF = _mm_subs_epi16(vF, vGapE);
-                if (! _mm_movemask_epi8(_mm_cmpgt_epi16(vF, vH))) goto end;
-                /*vF = _mm_max_epi16(vF, vH);*/
+                vH = _mm_subs_epi8(vH, vGapO);
+                vF = _mm_subs_epi8(vF, vGapE);
+                if (! _mm_movemask_epi8(_mm_cmpgt_epi8(vF, vH))) goto end;
+                /*vF = _mm_max_epi8_rpl(vF, vH);*/
             }
         }
 end:
@@ -326,16 +369,16 @@ end:
             /* extract vector containing last value from the column */
             __m128i vCompare;
             vH = _mm_load_si128(pvHStore + offset);
-            vCompare = _mm_and_si128(vPosMask, _mm_cmpgt_epi16(vH, vMaxH));
-            vMaxH = _mm_max_epi16(vH, vMaxH);
+            vCompare = _mm_and_si128(vPosMask, _mm_cmpgt_epi8(vH, vMaxH));
+            vMaxH = _mm_max_epi8_rpl(vH, vMaxH);
             if (_mm_movemask_epi8(vCompare)) {
                 end_ref = j;
             }
 #ifdef PARASAIL_ROWCOL
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 2);
+                vH = _mm_slli_si128(vH, 1);
             }
-            result->rowcols->score_row[j] = (int16_t) _mm_extract_epi16 (vH, 7);
+            result->rowcols->score_row[j] = (int8_t) _mm_extract_epi8_rpl (vH, 15);
 #endif
         }
     }
@@ -351,9 +394,9 @@ end:
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm_slli_si128(vMaxH, 2);
+            vMaxH = _mm_slli_si128(vMaxH, 1);
         }
-        score = (int16_t) _mm_extract_epi16(vMaxH, 7);
+        score = (int8_t) _mm_extract_epi8_rpl(vMaxH, 15);
         end_query = s1Len-1;
     }
 
@@ -361,7 +404,7 @@ end:
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int16_t *t = (int16_t*)pvHStore;
+        int8_t *t = (int8_t*)pvHStore;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -382,17 +425,17 @@ end:
         {
             __m128i vH = _mm_load_si128(pvHStore + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 2);
+                vH = _mm_slli_si128(vH, 1);
             }
-            score = (int16_t) _mm_extract_epi16 (vH, 7);
+            score = (int8_t) _mm_extract_epi8_rpl (vH, 15);
             end_ref = s2Len - 1;
             end_query = s1Len - 1;
         }
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
