@@ -16,7 +16,7 @@
 #include "parasail/internal_neon.h"
 
 #define SG_TRACE
-#define SG_SUFFIX _diag_neon_128_64
+#define SG_SUFFIX _diag_neon_128_16
 #include "sg_helper.h"
 
 
@@ -30,14 +30,32 @@ static inline void arr_store_si128(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int8_t)simde_mm_extract_epi64(vWH, 1);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int8_t)simde_mm_extract_epi16(vWH, 7);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int8_t)simde_mm_extract_epi64(vWH, 0);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int8_t)simde_mm_extract_epi16(vWH, 6);
+    }
+    if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
+        array[1LL*(i+2)*s2Len + (j-2)] = (int8_t)simde_mm_extract_epi16(vWH, 5);
+    }
+    if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
+        array[1LL*(i+3)*s2Len + (j-3)] = (int8_t)simde_mm_extract_epi16(vWH, 4);
+    }
+    if (0 <= i+4 && i+4 < s1Len && 0 <= j-4 && j-4 < s2Len) {
+        array[1LL*(i+4)*s2Len + (j-4)] = (int8_t)simde_mm_extract_epi16(vWH, 3);
+    }
+    if (0 <= i+5 && i+5 < s1Len && 0 <= j-5 && j-5 < s2Len) {
+        array[1LL*(i+5)*s2Len + (j-5)] = (int8_t)simde_mm_extract_epi16(vWH, 2);
+    }
+    if (0 <= i+6 && i+6 < s1Len && 0 <= j-6 && j-6 < s2Len) {
+        array[1LL*(i+6)*s2Len + (j-6)] = (int8_t)simde_mm_extract_epi16(vWH, 1);
+    }
+    if (0 <= i+7 && i+7 < s1Len && 0 <= j-7 && j-7 < s2Len) {
+        array[1LL*(i+7)*s2Len + (j-7)] = (int8_t)simde_mm_extract_epi16(vWH, 0);
     }
 }
 
-#define FNAME parasail_sg_flags_trace_diag_neon_128_64
+#define FNAME parasail_sg_flags_trace_diag_neon_128_16
 
 parasail_result_t* FNAME(
         const char * const restrict _s1, const int _s1Len,
@@ -51,22 +69,22 @@ parasail_result_t* FNAME(
     int32_t PAD2 = 0;
     int32_t s1Len_PAD = 0;
     int32_t s2Len_PAD = 0;
-    int64_t * restrict s1 = NULL;
-    int64_t * restrict s2B = NULL;
-    int64_t * restrict _H_pr = NULL;
-    int64_t * restrict _F_pr = NULL;
-    int64_t * restrict s2 = NULL;
-    int64_t * restrict H_pr = NULL;
-    int64_t * restrict F_pr = NULL;
+    int16_t * restrict s1 = NULL;
+    int16_t * restrict s2B = NULL;
+    int16_t * restrict _H_pr = NULL;
+    int16_t * restrict _F_pr = NULL;
+    int16_t * restrict s2 = NULL;
+    int16_t * restrict H_pr = NULL;
+    int16_t * restrict F_pr = NULL;
     parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t s1Len = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    int64_t NEG_LIMIT = 0;
-    int64_t POS_LIMIT = 0;
-    int64_t score = 0;
+    int16_t NEG_LIMIT = 0;
+    int16_t POS_LIMIT = 0;
+    int16_t score = 0;
     simde__m128i vNegLimit;
     simde__m128i vPosLimit;
     simde__m128i vSaturationCheckMin;
@@ -110,7 +128,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize stack variables */
-    N = 2; /* number of values in vector */
+    N = 8; /* number of values in vector */
     PAD = N-1;
     PAD2 = PAD*2;
     s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
@@ -120,41 +138,47 @@ parasail_result_t* FNAME(
     j = 0;
     end_query = s1Len-1;
     end_ref = s2Len-1;
-    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
+    POS_LIMIT = INT16_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = simde_mm_set1_epi64x(NEG_LIMIT);
-    vPosLimit = simde_mm_set1_epi64x(POS_LIMIT);
+    vNegLimit = simde_mm_set1_epi16(NEG_LIMIT);
+    vPosLimit = simde_mm_set1_epi16(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vNegInf = simde_mm_set1_epi64x(NEG_LIMIT);
-    vOpen = simde_mm_set1_epi64x(open);
-    vGap  = simde_mm_set1_epi64x(gap);
-    vOne = simde_mm_set1_epi64x(1);
-    vN = simde_mm_set1_epi64x(N);
-    vGapN = s1_beg ? simde_mm_set1_epi64x(0) : simde_mm_set1_epi64x(gap*N);
-    vNegOne = simde_mm_set1_epi64x(-1);
-    vI = simde_mm_set_epi64x(0,1);
-    vJreset = simde_mm_set_epi64x(0,-1);
+    vNegInf = simde_mm_set1_epi16(NEG_LIMIT);
+    vOpen = simde_mm_set1_epi16(open);
+    vGap  = simde_mm_set1_epi16(gap);
+    vOne = simde_mm_set1_epi16(1);
+    vN = simde_mm_set1_epi16(N);
+    vGapN = s1_beg ? simde_mm_set1_epi16(0) : simde_mm_set1_epi16(gap*N);
+    vNegOne = simde_mm_set1_epi16(-1);
+    vI = simde_mm_set_epi16(0,1,2,3,4,5,6,7);
+    vJreset = simde_mm_set_epi16(0,-1,-2,-3,-4,-5,-6,-7);
     vMaxHRow = vNegInf;
     vMaxHCol = vNegInf;
     vLastVal = vNegInf;
     vEndI = vNegInf;
     vEndJ = vNegInf;
-    vILimit = simde_mm_set1_epi64x(s1Len);
-    vILimit1 = simde_mm_sub_epi64(vILimit, vOne);
-    vJLimit = simde_mm_set1_epi64x(s2Len);
-    vJLimit1 = simde_mm_sub_epi64(vJLimit, vOne);
-    vIBoundary = s1_beg ? simde_mm_set1_epi64x(0) : simde_mm_set_epi64x(
+    vILimit = simde_mm_set1_epi16(s1Len);
+    vILimit1 = simde_mm_subs_epi16(vILimit, vOne);
+    vJLimit = simde_mm_set1_epi16(s2Len);
+    vJLimit1 = simde_mm_subs_epi16(vJLimit, vOne);
+    vIBoundary = s1_beg ? simde_mm_set1_epi16(0) : simde_mm_set_epi16(
             -open-0*gap,
-            -open-1*gap);
-    vTDiag = simde_mm_set1_epi64x(PARASAIL_DIAG);
-    vTIns = simde_mm_set1_epi64x(PARASAIL_INS);
-    vTDel = simde_mm_set1_epi64x(PARASAIL_DEL);
-    vTDiagE = simde_mm_set1_epi64x(PARASAIL_DIAG_E);
-    vTInsE = simde_mm_set1_epi64x(PARASAIL_INS_E);
-    vTDiagF = simde_mm_set1_epi64x(PARASAIL_DIAG_F);
-    vTDelF = simde_mm_set1_epi64x(PARASAIL_DEL_F);
+            -open-1*gap,
+            -open-2*gap,
+            -open-3*gap,
+            -open-4*gap,
+            -open-5*gap,
+            -open-6*gap,
+            -open-7*gap);
+    vTDiag = simde_mm_set1_epi16(PARASAIL_DIAG);
+    vTIns = simde_mm_set1_epi16(PARASAIL_INS);
+    vTDel = simde_mm_set1_epi16(PARASAIL_DEL);
+    vTDiagE = simde_mm_set1_epi16(PARASAIL_DIAG_E);
+    vTInsE = simde_mm_set1_epi16(PARASAIL_INS_E);
+    vTDiagF = simde_mm_set1_epi16(PARASAIL_DIAG_F);
+    vTDelF = simde_mm_set1_epi16(PARASAIL_DEL_F);
 
     /* initialize result */
     result = parasail_result_new_trace(s1Len, s2Len, 16, sizeof(int8_t));
@@ -163,16 +187,16 @@ parasail_result_t* FNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_DIAG
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
+        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
     result->flag |= s2_end ? PARASAIL_FLAG_SG_S2_END : 0;
 
     /* initialize heap variables */
-    s2B= parasail_memalign_int64_t(16, s2Len+PAD2);
-    _H_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
-    _F_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    s2B= parasail_memalign_int16_t(16, s2Len+PAD2);
+    _H_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
+    _F_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
     s2 = s2B+PAD; /* will allow later for negative indices */
     H_pr = _H_pr+PAD;
     F_pr = _F_pr+PAD;
@@ -184,7 +208,7 @@ parasail_result_t* FNAME(
 
     /* convert _s1 from char to int in range 0-23 */
     if (matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) {
-        s1 = parasail_memalign_int64_t(16, s1Len+PAD);
+        s1 = parasail_memalign_int16_t(16, s1Len+PAD);
         if (!s1) return NULL;
         for (i=0; i<s1Len; ++i) {
             s1[i] = matrix->mapper[(unsigned char)_s1[i]];
@@ -246,76 +270,88 @@ parasail_result_t* FNAME(
         simde__m128i vJ = vJreset;
         const int * const restrict matrow0 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+0] : ((i+0 >= s1Len) ? s1Len-1 : i+0))];
         const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
-        simde__m128i vIltLimit = simde_mm_cmplt_epi64(vI, vILimit);
-        simde__m128i vIeqLimit1 = simde_mm_cmpeq_epi64(vI, vILimit1);
-        vNH = simde_mm_srli_si128(vNH, 8);
-        vNH = simde_mm_insert_epi64(vNH, H_pr[-1], 1);
-        vWH = simde_mm_srli_si128(vWH, 8);
-        vWH = simde_mm_insert_epi64(vWH, s1_beg ? 0 : (-open - i*gap), 1);
+        const int * const restrict matrow2 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+2] : ((i+2 >= s1Len) ? s1Len-1 : i+2))];
+        const int * const restrict matrow3 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+3] : ((i+3 >= s1Len) ? s1Len-1 : i+3))];
+        const int * const restrict matrow4 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+4] : ((i+4 >= s1Len) ? s1Len-1 : i+4))];
+        const int * const restrict matrow5 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+5] : ((i+5 >= s1Len) ? s1Len-1 : i+5))];
+        const int * const restrict matrow6 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+6] : ((i+6 >= s1Len) ? s1Len-1 : i+6))];
+        const int * const restrict matrow7 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+7] : ((i+7 >= s1Len) ? s1Len-1 : i+7))];
+        simde__m128i vIltLimit = simde_mm_cmplt_epi16(vI, vILimit);
+        simde__m128i vIeqLimit1 = simde_mm_cmpeq_epi16(vI, vILimit1);
+        vNH = simde_mm_srli_si128(vNH, 2);
+        vNH = simde_mm_insert_epi16(vNH, H_pr[-1], 7);
+        vWH = simde_mm_srli_si128(vWH, 2);
+        vWH = simde_mm_insert_epi16(vWH, s1_beg ? 0 : (-open - i*gap), 7);
         H_pr[-1] = -open - (i+N)*gap;
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
             simde__m128i vMat;
             simde__m128i vNWH = vNH;
-            vNH = simde_mm_srli_si128(vWH, 8);
-            vNH = simde_mm_insert_epi64(vNH, H_pr[j], 1);
-            vF = simde_mm_srli_si128(vF, 8);
-            vF = simde_mm_insert_epi64(vF, F_pr[j], 1);
-            vF_opn = simde_mm_sub_epi64(vNH, vOpen);
-            vF_ext = simde_mm_sub_epi64(vF, vGap);
-            vF = simde_mm_max_epi64(vF_opn, vF_ext);
-            vE_opn = simde_mm_sub_epi64(vWH, vOpen);
-            vE_ext = simde_mm_sub_epi64(vE, vGap);
-            vE = simde_mm_max_epi64(vE_opn, vE_ext);
-            vMat = simde_mm_set_epi64x(
+            vNH = simde_mm_srli_si128(vWH, 2);
+            vNH = simde_mm_insert_epi16(vNH, H_pr[j], 7);
+            vF = simde_mm_srli_si128(vF, 2);
+            vF = simde_mm_insert_epi16(vF, F_pr[j], 7);
+            vF_opn = simde_mm_subs_epi16(vNH, vOpen);
+            vF_ext = simde_mm_subs_epi16(vF, vGap);
+            vF = simde_mm_max_epi16(vF_opn, vF_ext);
+            vE_opn = simde_mm_subs_epi16(vWH, vOpen);
+            vE_ext = simde_mm_subs_epi16(vE, vGap);
+            vE = simde_mm_max_epi16(vE_opn, vE_ext);
+            vMat = simde_mm_set_epi16(
                     matrow0[s2[j-0]],
-                    matrow1[s2[j-1]]
+                    matrow1[s2[j-1]],
+                    matrow2[s2[j-2]],
+                    matrow3[s2[j-3]],
+                    matrow4[s2[j-4]],
+                    matrow5[s2[j-5]],
+                    matrow6[s2[j-6]],
+                    matrow7[s2[j-7]]
                     );
-            vNWH = simde_mm_add_epi64(vNWH, vMat);
-            vWH = simde_mm_max_epi64(vNWH, vE);
-            vWH = simde_mm_max_epi64(vWH, vF);
+            vNWH = simde_mm_adds_epi16(vNWH, vMat);
+            vWH = simde_mm_max_epi16(vNWH, vE);
+            vWH = simde_mm_max_epi16(vWH, vF);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                simde__m128i cond = simde_mm_cmpeq_epi64(vJ,vNegOne);
+                simde__m128i cond = simde_mm_cmpeq_epi16(vJ,vNegOne);
                 vWH = simde_mm_blendv_epi8(vWH, vIBoundary, cond);
                 vF = simde_mm_blendv_epi8(vF, vNegInf, cond);
                 vE = simde_mm_blendv_epi8(vE, vNegInf, cond);
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vWH);
+                vSaturationCheckMin = simde_mm_min_epi16(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = simde_mm_max_epi16(vSaturationCheckMax, vWH);
             }
             /* trace table */
             {
-                simde__m128i case1 = simde_mm_cmpeq_epi64(vWH, vNWH);
-                simde__m128i case2 = simde_mm_cmpeq_epi64(vWH, vF);
+                simde__m128i case1 = simde_mm_cmpeq_epi16(vWH, vNWH);
+                simde__m128i case2 = simde_mm_cmpeq_epi16(vWH, vF);
                 simde__m128i vT = simde_mm_blendv_epi8(
                         simde_mm_blendv_epi8(vTIns, vTDel, case2),
                         vTDiag,
                         case1);
-                simde__m128i condE = simde_mm_cmpgt_epi64(vE_opn, vE_ext);
-                simde__m128i condF = simde_mm_cmpgt_epi64(vF_opn, vF_ext);
+                simde__m128i condE = simde_mm_cmpgt_epi16(vE_opn, vE_ext);
+                simde__m128i condF = simde_mm_cmpgt_epi16(vF_opn, vF_ext);
                 simde__m128i vET = simde_mm_blendv_epi8(vTInsE, vTDiagE, condE);
                 simde__m128i vFT = simde_mm_blendv_epi8(vTDelF, vTDiagF, condF);
                 vT = simde_mm_or_si128(vT, vET);
                 vT = simde_mm_or_si128(vT, vFT);
                 arr_store_si128(result->trace->trace_table, vT, i, s1Len, j, s2Len);
             }
-            H_pr[j-1] = (int64_t)simde_mm_extract_epi64(vWH,0);
-            F_pr[j-1] = (int64_t)simde_mm_extract_epi64(vF,0);
+            H_pr[j-7] = (int16_t)simde_mm_extract_epi16(vWH,0);
+            F_pr[j-7] = (int16_t)simde_mm_extract_epi16(vF,0);
             /* as minor diagonal vector passes across the i or j limit
              * boundary, extract the last value of the column or row */
             {
-                simde__m128i vJeqLimit1 = simde_mm_cmpeq_epi64(vJ, vJLimit1);
-                simde__m128i vJgtNegOne = simde_mm_cmpgt_epi64(vJ, vNegOne);
-                simde__m128i vJltLimit = simde_mm_cmplt_epi64(vJ, vJLimit);
+                simde__m128i vJeqLimit1 = simde_mm_cmpeq_epi16(vJ, vJLimit1);
+                simde__m128i vJgtNegOne = simde_mm_cmpgt_epi16(vJ, vNegOne);
+                simde__m128i vJltLimit = simde_mm_cmplt_epi16(vJ, vJLimit);
                 simde__m128i cond_j = simde_mm_and_si128(vIltLimit, vJeqLimit1);
                 simde__m128i cond_i = simde_mm_and_si128(vIeqLimit1,
                         simde_mm_and_si128(vJgtNegOne, vJltLimit));
-                simde__m128i cond_max_row = simde_mm_cmpgt_epi64(vWH, vMaxHRow);
-                simde__m128i cond_max_col = simde_mm_cmpgt_epi64(vWH, vMaxHCol);
+                simde__m128i cond_max_row = simde_mm_cmpgt_epi16(vWH, vMaxHRow);
+                simde__m128i cond_max_col = simde_mm_cmpgt_epi16(vWH, vMaxHCol);
                 simde__m128i cond_last_val = simde_mm_and_si128(vIeqLimit1, vJeqLimit1);
                 simde__m128i cond_all_row = simde_mm_and_si128(cond_max_row, cond_i);
                 simde__m128i cond_all_col = simde_mm_and_si128(cond_max_col, cond_j);
@@ -325,22 +361,22 @@ parasail_result_t* FNAME(
                 vEndI = simde_mm_blendv_epi8(vEndI, vI, cond_all_col);
                 vEndJ = simde_mm_blendv_epi8(vEndJ, vJ, cond_all_row);
             }
-            vJ = simde_mm_add_epi64(vJ, vOne);
+            vJ = simde_mm_adds_epi16(vJ, vOne);
         }
-        vI = simde_mm_add_epi64(vI, vN);
-        vIBoundary = simde_mm_sub_epi64(vIBoundary, vGapN);
+        vI = simde_mm_adds_epi16(vI, vN);
+        vIBoundary = simde_mm_subs_epi16(vIBoundary, vGapN);
     }
 
     /* alignment ending position */
     {
-        int64_t max_row = NEG_LIMIT;
-        int64_t max_col = NEG_LIMIT;
-        int64_t last_val = NEG_LIMIT;
-        int64_t *s = (int64_t*)&vMaxHRow;
-        int64_t *t = (int64_t*)&vMaxHCol;
-        int64_t *u = (int64_t*)&vLastVal;
-        int64_t *i = (int64_t*)&vEndI;
-        int64_t *j = (int64_t*)&vEndJ;
+        int16_t max_row = NEG_LIMIT;
+        int16_t max_col = NEG_LIMIT;
+        int16_t last_val = NEG_LIMIT;
+        int16_t *s = (int16_t*)&vMaxHRow;
+        int16_t *t = (int16_t*)&vMaxHCol;
+        int16_t *u = (int16_t*)&vLastVal;
+        int16_t *i = (int16_t*)&vEndI;
+        int16_t *j = (int16_t*)&vEndJ;
         int32_t k;
         for (k=0; k<N; ++k, ++s, ++t, ++u, ++i, ++j) {
             if (*t > max_col || (*t == max_col && *i < end_query)) {
@@ -381,8 +417,8 @@ parasail_result_t* FNAME(
     }
 
     if (simde_mm_movemask_epi8(simde_mm_or_si128(
-            simde_mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
-            simde_mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
+            simde_mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
+            simde_mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
