@@ -17,8 +17,8 @@
 #include "parasail/internal_altivec.h"
 
 #define SG_TRACE
-#define SG_SUFFIX _scan_altivec_128_32
-#define SG_SUFFIX_PROF _scan_profile_altivec_128_32
+#define SG_SUFFIX _scan_altivec_128_8
+#define SG_SUFFIX_PROF _scan_profile_altivec_128_8
 #include "sg_helper.h"
 
 
@@ -42,8 +42,8 @@ static inline vec128i arr_load(
     return _mm_load_si128(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_sg_flags_trace_scan_altivec_128_32
-#define PNAME parasail_sg_flags_trace_scan_profile_altivec_128_32
+#define FNAME parasail_sg_flags_trace_scan_altivec_128_8
+#define PNAME parasail_sg_flags_trace_scan_profile_altivec_128_8
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -67,7 +67,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_altivec_128_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_altivec_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -96,16 +96,16 @@ parasail_result_t* PNAME(
     int32_t position = 0;
     vec128i* restrict pvP = NULL;
     vec128i* restrict pvE = NULL;
-    int32_t* restrict boundary = NULL;
+    int8_t* restrict boundary = NULL;
     vec128i* restrict pvHt = NULL;
     vec128i* restrict pvH = NULL;
     vec128i* restrict pvGapper = NULL;
     vec128i vGapO;
     vec128i vGapE;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
     vec128i vZero;
-    int32_t score = 0;
+    int8_t score = 0;
     vec128i vNegLimit;
     vec128i vPosLimit;
     vec128i vSaturationCheckMin;
@@ -125,7 +125,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -141,35 +141,35 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 4; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP = (vec128i*)profile->profile32.score;
-    vGapO = _mm_set1_epi32(open);
-    vGapE = _mm_set1_epi32(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    pvP = (vec128i*)profile->profile8.score;
+    vGapO = _mm_set1_epi8(open);
+    vGapE = _mm_set1_epi8(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
     vZero = _mm_setzero_si128();
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm_set1_epi8(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi8(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
-    vPosMask = _mm_cmpeq_epi32(_mm_set1_epi32(position),
-            _mm_set_epi32(0,1,2,3));
+    vPosMask = _mm_cmpeq_epi8(_mm_set1_epi8(position),
+            _mm_set_epi8(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15));
     vNegInfFront = vZero;
-    vNegInfFront = _mm_insert_epi32(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm_add_epi32(vNegInfFront,
-            _mm_slli_si128(_mm_set1_epi32(-segLen*gap), 4));
-    vTIns  = _mm_set1_epi32(PARASAIL_INS);
-    vTDel  = _mm_set1_epi32(PARASAIL_DEL);
-    vTDiag = _mm_set1_epi32(PARASAIL_DIAG);
-    vTDiagE= _mm_set1_epi32(PARASAIL_DIAG_E);
-    vTInsE = _mm_set1_epi32(PARASAIL_INS_E);
-    vTDiagF= _mm_set1_epi32(PARASAIL_DIAG_F);
-    vTDelF = _mm_set1_epi32(PARASAIL_DEL_F);
+    vNegInfFront = _mm_insert_epi8(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm_adds_epi8(vNegInfFront,
+            _mm_slli_si128(_mm_set1_epi8(-segLen*gap), 1));
+    vTIns  = _mm_set1_epi8(PARASAIL_INS);
+    vTDel  = _mm_set1_epi8(PARASAIL_DEL);
+    vTDiag = _mm_set1_epi8(PARASAIL_DIAG);
+    vTDiagE= _mm_set1_epi8(PARASAIL_DIAG_E);
+    vTInsE = _mm_set1_epi8(PARASAIL_INS_E);
+    vTDiagF= _mm_set1_epi8(PARASAIL_DIAG_F);
+    vTDelF = _mm_set1_epi8(PARASAIL_DEL_F);
 
     /* initialize result */
     result = parasail_result_new_trace(segLen, s2Len, 16, sizeof(vec128i));
@@ -178,7 +178,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -186,7 +186,7 @@ parasail_result_t* PNAME(
 
     /* initialize heap variables */
     pvE = parasail_memalign_vec128i(16, segLen);
-    boundary = parasail_memalign_int32_t(16, s2Len+1);
+    boundary = parasail_memalign_int8_t(16, s2Len+1);
     pvHt= parasail_memalign_vec128i(16, segLen);
     pvH = parasail_memalign_vec128i(16, segLen);
     pvGapper = parasail_memalign_vec128i(16, segLen);
@@ -203,13 +203,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            vec128i_32_t h;
-            vec128i_32_t e;
+            vec128i_8_t h;
+            vec128i_8_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                h.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                e.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
             }
             _mm_store_si128(&pvH[index], h.m);
             _mm_store_si128(&pvE[index], e.m);
@@ -222,17 +222,17 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
+            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
         }
     }
 
     {
-        vec128i vGapper = _mm_sub_epi32(vZero,vGapO);
+        vec128i vGapper = _mm_subs_epi8(vZero,vGapO);
         for (i=segLen-1; i>=0; --i) {
             _mm_store_si128(pvGapper+i, vGapper);
-            vGapper = _mm_sub_epi32(vGapper, vGapE);
+            vGapper = _mm_subs_epi8(vGapper, vGapE);
             /* long queries and/or large penalties will break the pseudo prefix scan */
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vGapper);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vGapper);
         }
     }
 
@@ -260,27 +260,27 @@ parasail_result_t* PNAME(
         /* calculate Ht */
         /* calculate F and H first pass */
         vHp = _mm_load_si128(pvH+(segLen-1));
-        vHp = _mm_slli_si128(vHp, 4);
-        vHp = _mm_insert_epi32(vHp, boundary[j], 0);
+        vHp = _mm_slli_si128(vHp, 1);
+        vHp = _mm_insert_epi8(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = _mm_sub_epi32(vNegLimit, pvGapper[0]);
+        vHt = _mm_subs_epi8(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         for (i=0; i<segLen; ++i) {
             vH = _mm_load_si128(pvH+i);
             vE = _mm_load_si128(pvE+i);
             vW = _mm_load_si128(pvW+i);
             vGapper = _mm_load_si128(pvGapper+i);
-            vE_opn = _mm_sub_epi32(vH, vGapO);
-            vE_ext = _mm_sub_epi32(vE, vGapE);
-            case1 = _mm_cmpgt_epi32(vE_opn, vE_ext);
+            vE_opn = _mm_subs_epi8(vH, vGapO);
+            vE_ext = _mm_subs_epi8(vE, vGapE);
+            case1 = _mm_cmpgt_epi8(vE_opn, vE_ext);
             vET = _mm_blendv_epi8(vTInsE, vTDiagE, case1);
             arr_store(result->trace->trace_table, vET, i, segLen, j);
-            vE = _mm_max_epi32(vE_opn, vE_ext);
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vE);
-            vGapper = _mm_add_epi32(vHt, vGapper);
-            vF = _mm_max_epi32(vF, vGapper);
-            vHp = _mm_add_epi32(vHp, vW);
-            vHt = _mm_max_epi32(vE, vHp);
+            vE = _mm_max_epi8(vE_opn, vE_ext);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vE);
+            vGapper = _mm_adds_epi8(vHt, vGapper);
+            vF = _mm_max_epi8(vF, vGapper);
+            vHp = _mm_adds_epi8(vHp, vW);
+            vHt = _mm_max_epi8(vE, vHp);
             _mm_store_si128(pvE+i, vE);
             _mm_store_si128(pvHt+i, vHt);
             _mm_store_si128(pvH+i, vHp);
@@ -288,33 +288,33 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm_slli_si128(vHt, 4);
-        vHt = _mm_insert_epi32(vHt, boundary[j+1], 0);
+        vHt = _mm_slli_si128(vHt, 1);
+        vHt = _mm_insert_epi8(vHt, boundary[j+1], 0);
         vGapper = _mm_load_si128(pvGapper);
-        vGapper = _mm_add_epi32(vHt, vGapper);
-        vF = _mm_max_epi32(vF, vGapper);
+        vGapper = _mm_adds_epi8(vHt, vGapper);
+        vF = _mm_max_epi8(vF, vGapper);
         for (i=0; i<segWidth-2; ++i) {
-            vec128i vFt = _mm_slli_si128(vF, 4);
-            vFt = _mm_add_epi32(vFt, vSegLenXgap);
-            vF = _mm_max_epi32(vF, vFt);
+            vec128i vFt = _mm_slli_si128(vF, 1);
+            vFt = _mm_adds_epi8(vFt, vSegLenXgap);
+            vF = _mm_max_epi8(vF, vFt);
         }
 
         /* calculate final H */
-        vF = _mm_slli_si128(vF, 4);
-        vF = _mm_add_epi32(vF, vNegInfFront);
-        vH = _mm_max_epi32(vF, vHt);
+        vF = _mm_slli_si128(vF, 1);
+        vF = _mm_adds_epi8(vF, vNegInfFront);
+        vH = _mm_max_epi8(vF, vHt);
         for (i=0; i<segLen; ++i) {
             vET = arr_load(result->trace->trace_table, i, segLen, j);
             vHp = _mm_load_si128(pvH+i);
             vHt = _mm_load_si128(pvHt+i);
-            vF_opn = _mm_sub_epi32(vH, vGapO);
-            vF_ext = _mm_sub_epi32(vF, vGapE);
-            vF = _mm_max_epi32(vF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi32(vF_opn, vF_ext);
+            vF_opn = _mm_subs_epi8(vH, vGapO);
+            vF_ext = _mm_subs_epi8(vF, vGapE);
+            vF = _mm_max_epi8(vF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi8(vF_opn, vF_ext);
             vFT = _mm_blendv_epi8(vTDelF, vTDiagF, case1);
-            vH = _mm_max_epi32(vHt, vF);
-            case1 = _mm_cmpeq_epi32(vH, vHp);
-            case2 = _mm_cmpeq_epi32(vH, vF);
+            vH = _mm_max_epi8(vHt, vF);
+            case1 = _mm_cmpeq_epi8(vH, vHp);
+            case2 = _mm_cmpeq_epi8(vH, vF);
             vT = _mm_blendv_epi8(
                     _mm_blendv_epi8(vTIns, vTDel, case2),
                     vTDiag, case1);
@@ -322,17 +322,17 @@ parasail_result_t* PNAME(
             vT = _mm_or_si128(vT, vFT);
             arr_store(result->trace->trace_table, vT, i, segLen, j);
             _mm_store_si128(pvH+i, vH);
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vF);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vH);
         }
 
         /* extract vector containing last value from column */
         {
             vec128i vCompare;
             vH = _mm_load_si128(pvH + offset);
-            vCompare = _mm_and_si128(vPosMask, _mm_cmpgt_epi32(vH, vMaxH));
-            vMaxH = _mm_max_epi32(vH, vMaxH);
+            vCompare = _mm_and_si128(vPosMask, _mm_cmpgt_epi8(vH, vMaxH));
+            vMaxH = _mm_max_epi8(vH, vMaxH);
             if (_mm_movemask_epi8(vCompare)) {
                 end_ref = j;
             }
@@ -343,9 +343,9 @@ parasail_result_t* PNAME(
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm_slli_si128(vMaxH, 4);
+            vMaxH = _mm_slli_si128(vMaxH, 1);
         }
-        score = (int32_t) _mm_extract_epi32(vMaxH, 3);
+        score = (int8_t) _mm_extract_epi8(vMaxH, 15);
         end_query = s1Len-1;
     }
 
@@ -353,7 +353,7 @@ parasail_result_t* PNAME(
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int32_t *t = (int32_t*)pvH;
+        int8_t *t = (int8_t*)pvH;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -374,17 +374,17 @@ parasail_result_t* PNAME(
         {
             vec128i vH = _mm_load_si128(pvH + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 4);
+                vH = _mm_slli_si128(vH, 1);
             }
-            score = (int32_t) _mm_extract_epi32 (vH, 3);
+            score = (int8_t) _mm_extract_epi8 (vH, 15);
             end_ref = s2Len - 1;
             end_query = s1Len - 1;
         }
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
