@@ -14,7 +14,6 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
-#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -22,94 +21,17 @@
 #include "parasail/internal_sse.h"
 
 #define SG_TRACE
-#define SG_SUFFIX _scan_sse41_128_64
-#define SG_SUFFIX_PROF _scan_profile_sse41_128_64
+#define SG_SUFFIX _scan_sse2_128_16
+#define SG_SUFFIX_PROF _scan_profile_sse2_128_16
 #include "sg_helper.h"
 
 
-static inline __m128i _mm_cmpgt_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]>B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    A.v[1] = (A.v[1]>B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    return A.m;
+static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
+    a = _mm_andnot_si128(mask, a);
+    a = _mm_or_si128(a, _mm_and_si128(mask, b));
+    return a;
 }
 
-#if HAVE_SSE41_MM_INSERT_EPI64
-#define _mm_insert_epi64_rpl _mm_insert_epi64
-#else
-static inline __m128i _mm_insert_epi64_rpl(__m128i a, int64_t i, int imm) {
-    __m128i_64_t A;
-    A.m = a;
-    A.v[imm] = i;
-    return A.m;
-}
-#endif
-
-static inline __m128i _mm_max_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
-    return A.m;
-}
-
-#if HAVE_SSE2_MM_SET_EPI64X
-#define _mm_set_epi64x_rpl _mm_set_epi64x
-#else
-static inline __m128i _mm_set_epi64x_rpl(int64_t e1, int64_t e0) {
-    __m128i_64_t A;
-    A.v[0] = e0;
-    A.v[1] = e1;
-    return A.m;
-}
-#endif
-
-#if HAVE_SSE41_MM_EXTRACT_EPI64
-#define _mm_extract_epi64_rpl _mm_extract_epi64
-#else
-static inline int64_t _mm_extract_epi64_rpl(__m128i a, int imm) {
-    __m128i_64_t A;
-    A.m = a;
-    return A.v[imm];
-}
-#endif
-
-static inline __m128i _mm_min_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
-    return A.m;
-}
-
-static inline __m128i _mm_cmplt_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]<B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    A.v[1] = (A.v[1]<B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    return A.m;
-}
-
-#if HAVE_SSE2_MM_SET1_EPI64X
-#define _mm_set1_epi64x_rpl _mm_set1_epi64x
-#else
-static inline __m128i _mm_set1_epi64x_rpl(int64_t i) {
-    __m128i_64_t A;
-    A.v[0] = i;
-    A.v[1] = i;
-    return A.m;
-}
-#endif
-
 
 static inline void arr_store(
         __m128i *array,
@@ -130,8 +52,8 @@ static inline __m128i arr_load(
     return _mm_load_si128(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_sg_flags_trace_scan_sse41_128_64
-#define PNAME parasail_sg_flags_trace_scan_profile_sse41_128_64
+#define FNAME parasail_sg_flags_trace_scan_sse2_128_16
+#define PNAME parasail_sg_flags_trace_scan_profile_sse2_128_16
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -155,7 +77,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_sse_128_64(s1, s1Len, matrix);
+    profile = parasail_profile_create_sse_128_16(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -184,16 +106,16 @@ parasail_result_t* PNAME(
     int32_t position = 0;
     __m128i* restrict pvP = NULL;
     __m128i* restrict pvE = NULL;
-    int64_t* restrict boundary = NULL;
+    int16_t* restrict boundary = NULL;
     __m128i* restrict pvHt = NULL;
     __m128i* restrict pvH = NULL;
     __m128i* restrict pvGapper = NULL;
     __m128i vGapO;
     __m128i vGapE;
-    int64_t NEG_LIMIT = 0;
-    int64_t POS_LIMIT = 0;
+    int16_t NEG_LIMIT = 0;
+    int16_t POS_LIMIT = 0;
     __m128i vZero;
-    int64_t score = 0;
+    int16_t score = 0;
     __m128i vNegLimit;
     __m128i vPosLimit;
     __m128i vSaturationCheckMin;
@@ -213,7 +135,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->profile16.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -229,35 +151,35 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 2; /* number of values in vector unit */
+    segWidth = 8; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP = (__m128i*)profile->profile64.score;
-    vGapO = _mm_set1_epi64x_rpl(open);
-    vGapE = _mm_set1_epi64x_rpl(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    pvP = (__m128i*)profile->profile16.score;
+    vGapO = _mm_set1_epi16(open);
+    vGapE = _mm_set1_epi16(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
+    POS_LIMIT = INT16_MAX - matrix->max - 1;
     vZero = _mm_setzero_si128();
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi64x_rpl(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi64x_rpl(POS_LIMIT);
+    vNegLimit = _mm_set1_epi16(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi16(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
-    vPosMask = _mm_cmpeq_epi64(_mm_set1_epi64x_rpl(position),
-            _mm_set_epi64x_rpl(0,1));
+    vPosMask = _mm_cmpeq_epi16(_mm_set1_epi16(position),
+            _mm_set_epi16(0,1,2,3,4,5,6,7));
     vNegInfFront = vZero;
-    vNegInfFront = _mm_insert_epi64_rpl(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm_add_epi64(vNegInfFront,
-            _mm_slli_si128(_mm_set1_epi64x_rpl(-segLen*gap), 8));
-    vTIns  = _mm_set1_epi64x_rpl(PARASAIL_INS);
-    vTDel  = _mm_set1_epi64x_rpl(PARASAIL_DEL);
-    vTDiag = _mm_set1_epi64x_rpl(PARASAIL_DIAG);
-    vTDiagE= _mm_set1_epi64x_rpl(PARASAIL_DIAG_E);
-    vTInsE = _mm_set1_epi64x_rpl(PARASAIL_INS_E);
-    vTDiagF= _mm_set1_epi64x_rpl(PARASAIL_DIAG_F);
-    vTDelF = _mm_set1_epi64x_rpl(PARASAIL_DEL_F);
+    vNegInfFront = _mm_insert_epi16(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm_adds_epi16(vNegInfFront,
+            _mm_slli_si128(_mm_set1_epi16(-segLen*gap), 2));
+    vTIns  = _mm_set1_epi16(PARASAIL_INS);
+    vTDel  = _mm_set1_epi16(PARASAIL_DEL);
+    vTDiag = _mm_set1_epi16(PARASAIL_DIAG);
+    vTDiagE= _mm_set1_epi16(PARASAIL_DIAG_E);
+    vTInsE = _mm_set1_epi16(PARASAIL_INS_E);
+    vTDiagF= _mm_set1_epi16(PARASAIL_DIAG_F);
+    vTDelF = _mm_set1_epi16(PARASAIL_DEL_F);
 
     /* initialize result */
     result = parasail_result_new_trace(segLen, s2Len, 16, sizeof(__m128i));
@@ -266,7 +188,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
+        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -274,7 +196,7 @@ parasail_result_t* PNAME(
 
     /* initialize heap variables */
     pvE = parasail_memalign___m128i(16, segLen);
-    boundary = parasail_memalign_int64_t(16, s2Len+1);
+    boundary = parasail_memalign_int16_t(16, s2Len+1);
     pvHt= parasail_memalign___m128i(16, segLen);
     pvH = parasail_memalign___m128i(16, segLen);
     pvGapper = parasail_memalign___m128i(16, segLen);
@@ -291,13 +213,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m128i_64_t h;
-            __m128i_64_t e;
+            __m128i_16_t h;
+            __m128i_16_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                e.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
             }
             _mm_store_si128(&pvH[index], h.m);
             _mm_store_si128(&pvE[index], e.m);
@@ -310,17 +232,17 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
+            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
         }
     }
 
     {
-        __m128i vGapper = _mm_sub_epi64(vZero,vGapO);
+        __m128i vGapper = _mm_subs_epi16(vZero,vGapO);
         for (i=segLen-1; i>=0; --i) {
             _mm_store_si128(pvGapper+i, vGapper);
-            vGapper = _mm_sub_epi64(vGapper, vGapE);
+            vGapper = _mm_subs_epi16(vGapper, vGapE);
             /* long queries and/or large penalties will break the pseudo prefix scan */
-            vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vGapper);
+            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vGapper);
         }
     }
 
@@ -348,27 +270,27 @@ parasail_result_t* PNAME(
         /* calculate Ht */
         /* calculate F and H first pass */
         vHp = _mm_load_si128(pvH+(segLen-1));
-        vHp = _mm_slli_si128(vHp, 8);
-        vHp = _mm_insert_epi64_rpl(vHp, boundary[j], 0);
+        vHp = _mm_slli_si128(vHp, 2);
+        vHp = _mm_insert_epi16(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = _mm_sub_epi64(vNegLimit, pvGapper[0]);
+        vHt = _mm_subs_epi16(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         for (i=0; i<segLen; ++i) {
             vH = _mm_load_si128(pvH+i);
             vE = _mm_load_si128(pvE+i);
             vW = _mm_load_si128(pvW+i);
             vGapper = _mm_load_si128(pvGapper+i);
-            vE_opn = _mm_sub_epi64(vH, vGapO);
-            vE_ext = _mm_sub_epi64(vE, vGapE);
-            case1 = _mm_cmpgt_epi64_rpl(vE_opn, vE_ext);
-            vET = _mm_blendv_epi8(vTInsE, vTDiagE, case1);
+            vE_opn = _mm_subs_epi16(vH, vGapO);
+            vE_ext = _mm_subs_epi16(vE, vGapE);
+            case1 = _mm_cmpgt_epi16(vE_opn, vE_ext);
+            vET = _mm_blendv_epi8_rpl(vTInsE, vTDiagE, case1);
             arr_store(result->trace->trace_table, vET, i, segLen, j);
-            vE = _mm_max_epi64_rpl(vE_opn, vE_ext);
-            vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vE);
-            vGapper = _mm_add_epi64(vHt, vGapper);
-            vF = _mm_max_epi64_rpl(vF, vGapper);
-            vHp = _mm_add_epi64(vHp, vW);
-            vHt = _mm_max_epi64_rpl(vE, vHp);
+            vE = _mm_max_epi16(vE_opn, vE_ext);
+            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vE);
+            vGapper = _mm_adds_epi16(vHt, vGapper);
+            vF = _mm_max_epi16(vF, vGapper);
+            vHp = _mm_adds_epi16(vHp, vW);
+            vHt = _mm_max_epi16(vE, vHp);
             _mm_store_si128(pvE+i, vE);
             _mm_store_si128(pvHt+i, vHt);
             _mm_store_si128(pvH+i, vHp);
@@ -376,51 +298,51 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm_slli_si128(vHt, 8);
-        vHt = _mm_insert_epi64_rpl(vHt, boundary[j+1], 0);
+        vHt = _mm_slli_si128(vHt, 2);
+        vHt = _mm_insert_epi16(vHt, boundary[j+1], 0);
         vGapper = _mm_load_si128(pvGapper);
-        vGapper = _mm_add_epi64(vHt, vGapper);
-        vF = _mm_max_epi64_rpl(vF, vGapper);
+        vGapper = _mm_adds_epi16(vHt, vGapper);
+        vF = _mm_max_epi16(vF, vGapper);
         for (i=0; i<segWidth-2; ++i) {
-            __m128i vFt = _mm_slli_si128(vF, 8);
-            vFt = _mm_add_epi64(vFt, vSegLenXgap);
-            vF = _mm_max_epi64_rpl(vF, vFt);
+            __m128i vFt = _mm_slli_si128(vF, 2);
+            vFt = _mm_adds_epi16(vFt, vSegLenXgap);
+            vF = _mm_max_epi16(vF, vFt);
         }
 
         /* calculate final H */
-        vF = _mm_slli_si128(vF, 8);
-        vF = _mm_add_epi64(vF, vNegInfFront);
-        vH = _mm_max_epi64_rpl(vF, vHt);
+        vF = _mm_slli_si128(vF, 2);
+        vF = _mm_adds_epi16(vF, vNegInfFront);
+        vH = _mm_max_epi16(vF, vHt);
         for (i=0; i<segLen; ++i) {
             vET = arr_load(result->trace->trace_table, i, segLen, j);
             vHp = _mm_load_si128(pvH+i);
             vHt = _mm_load_si128(pvHt+i);
-            vF_opn = _mm_sub_epi64(vH, vGapO);
-            vF_ext = _mm_sub_epi64(vF, vGapE);
-            vF = _mm_max_epi64_rpl(vF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi64_rpl(vF_opn, vF_ext);
-            vFT = _mm_blendv_epi8(vTDelF, vTDiagF, case1);
-            vH = _mm_max_epi64_rpl(vHt, vF);
-            case1 = _mm_cmpeq_epi64(vH, vHp);
-            case2 = _mm_cmpeq_epi64(vH, vF);
-            vT = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vTIns, vTDel, case2),
+            vF_opn = _mm_subs_epi16(vH, vGapO);
+            vF_ext = _mm_subs_epi16(vF, vGapE);
+            vF = _mm_max_epi16(vF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi16(vF_opn, vF_ext);
+            vFT = _mm_blendv_epi8_rpl(vTDelF, vTDiagF, case1);
+            vH = _mm_max_epi16(vHt, vF);
+            case1 = _mm_cmpeq_epi16(vH, vHp);
+            case2 = _mm_cmpeq_epi16(vH, vF);
+            vT = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vTIns, vTDel, case2),
                     vTDiag, case1);
             vT = _mm_or_si128(vT, vET);
             vT = _mm_or_si128(vT, vFT);
             arr_store(result->trace->trace_table, vT, i, segLen, j);
             _mm_store_si128(pvH+i, vH);
-            vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vF);
-            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
         }
 
         /* extract vector containing last value from column */
         {
             __m128i vCompare;
             vH = _mm_load_si128(pvH + offset);
-            vCompare = _mm_and_si128(vPosMask, _mm_cmpgt_epi64_rpl(vH, vMaxH));
-            vMaxH = _mm_max_epi64_rpl(vH, vMaxH);
+            vCompare = _mm_and_si128(vPosMask, _mm_cmpgt_epi16(vH, vMaxH));
+            vMaxH = _mm_max_epi16(vH, vMaxH);
             if (_mm_movemask_epi8(vCompare)) {
                 end_ref = j;
             }
@@ -431,9 +353,9 @@ parasail_result_t* PNAME(
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm_slli_si128(vMaxH, 8);
+            vMaxH = _mm_slli_si128(vMaxH, 2);
         }
-        score = (int64_t) _mm_extract_epi64_rpl(vMaxH, 1);
+        score = (int16_t) _mm_extract_epi16(vMaxH, 7);
         end_query = s1Len-1;
     }
 
@@ -441,7 +363,7 @@ parasail_result_t* PNAME(
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int64_t *t = (int64_t*)pvH;
+        int16_t *t = (int16_t*)pvH;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -462,17 +384,17 @@ parasail_result_t* PNAME(
         {
             __m128i vH = _mm_load_si128(pvH + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 8);
+                vH = _mm_slli_si128(vH, 2);
             }
-            score = (int64_t) _mm_extract_epi64_rpl (vH, 1);
+            score = (int16_t) _mm_extract_epi16 (vH, 7);
             end_ref = s2Len - 1;
             end_query = s1Len - 1;
         }
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi64_rpl(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
