@@ -17,8 +17,8 @@
 #include "parasail/internal_altivec.h"
 
 #define SG_TRACE
-#define SG_SUFFIX _striped_altivec_128_16
-#define SG_SUFFIX_PROF _striped_profile_altivec_128_16
+#define SG_SUFFIX _striped_altivec_128_32
+#define SG_SUFFIX_PROF _striped_profile_altivec_128_32
 #include "sg_helper.h"
 
 #define SWAP(A,B) { vec128i* tmp = A; A = B; B = tmp; }
@@ -44,8 +44,8 @@ static inline vec128i arr_load(
     return _mm_load_si128(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_sg_flags_trace_striped_altivec_128_16
-#define PNAME parasail_sg_flags_trace_striped_profile_altivec_128_16
+#define FNAME parasail_sg_flags_trace_striped_altivec_128_32
+#define PNAME parasail_sg_flags_trace_striped_profile_altivec_128_32
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -69,7 +69,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_altivec_128_16(s1, s1Len, matrix);
+    profile = parasail_profile_create_altivec_128_32(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -103,12 +103,12 @@ parasail_result_t* PNAME(
     vec128i* restrict pvEaStore = NULL;
     vec128i* restrict pvEaLoad = NULL;
     vec128i* restrict pvHT = NULL;
-    int16_t* restrict boundary = NULL;
+    int32_t* restrict boundary = NULL;
     vec128i vGapO;
     vec128i vGapE;
-    int16_t NEG_LIMIT = 0;
-    int16_t POS_LIMIT = 0;
-    int16_t score = 0;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
+    int32_t score = 0;
     vec128i vNegLimit;
     vec128i vPosLimit;
     vec128i vSaturationCheckMin;
@@ -128,7 +128,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile16.score);
+    PARASAIL_CHECK_NULL(profile->profile32.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -144,32 +144,32 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 8; /* number of values in vector unit */
+    segWidth = 4; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile = (vec128i*)profile->profile16.score;
-    vGapO = _mm_set1_epi16(open);
-    vGapE = _mm_set1_epi16(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    vProfile = (vec128i*)profile->profile32.score;
+    vGapO = _mm_set1_epi32(open);
+    vGapE = _mm_set1_epi32(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi16(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi16(POS_LIMIT);
+    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi32(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
-    vPosMask = _mm_cmpeq_epi16(_mm_set1_epi16(position),
-            _mm_set_epi16(0,1,2,3,4,5,6,7));
-    vTIns  = _mm_set1_epi16(PARASAIL_INS);
-    vTDel  = _mm_set1_epi16(PARASAIL_DEL);
-    vTDiag = _mm_set1_epi16(PARASAIL_DIAG);
-    vTDiagE= _mm_set1_epi16(PARASAIL_DIAG_E);
-    vTInsE = _mm_set1_epi16(PARASAIL_INS_E);
-    vTDiagF= _mm_set1_epi16(PARASAIL_DIAG_F);
-    vTDelF = _mm_set1_epi16(PARASAIL_DEL_F);
-    vTMask = _mm_set1_epi16(PARASAIL_ZERO_MASK);
-    vFTMask= _mm_set1_epi16(PARASAIL_F_MASK);
+    vPosMask = _mm_cmpeq_epi32(_mm_set1_epi32(position),
+            _mm_set_epi32(0,1,2,3));
+    vTIns  = _mm_set1_epi32(PARASAIL_INS);
+    vTDel  = _mm_set1_epi32(PARASAIL_DEL);
+    vTDiag = _mm_set1_epi32(PARASAIL_DIAG);
+    vTDiagE= _mm_set1_epi32(PARASAIL_DIAG_E);
+    vTInsE = _mm_set1_epi32(PARASAIL_INS_E);
+    vTDiagF= _mm_set1_epi32(PARASAIL_DIAG_F);
+    vTDelF = _mm_set1_epi32(PARASAIL_DEL_F);
+    vTMask = _mm_set1_epi32(PARASAIL_ZERO_MASK);
+    vFTMask= _mm_set1_epi32(PARASAIL_F_MASK);
 
     /* initialize result */
     result = parasail_result_new_trace(segLen, s2Len, 16, sizeof(vec128i));
@@ -178,7 +178,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -191,7 +191,7 @@ parasail_result_t* PNAME(
     pvEaStore= parasail_memalign_vec128i(16, segLen);
     pvEaLoad = parasail_memalign_vec128i(16, segLen);
     pvHT     = parasail_memalign_vec128i(16, segLen);
-    boundary = parasail_memalign_int16_t(16, s2Len+1);
+    boundary = parasail_memalign_int32_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -207,13 +207,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            vec128i_16_t h;
-            vec128i_16_t e;
+            vec128i_32_t h;
+            vec128i_32_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                h.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                e.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
             }
             _mm_store_si128(&pvHStore[index], h.m);
             _mm_store_si128(&pvE[index], e.m);
@@ -227,7 +227,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
+            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
         }
     }
 
@@ -252,12 +252,12 @@ parasail_result_t* PNAME(
          * corrected in the Lazy_F loop. */
         vF = vNegLimit;
 
-        /* load final segment of pvHStore and shift left by 2 bytes */
+        /* load final segment of pvHStore and shift left by 4 bytes */
         vH = _mm_load_si128(&pvHStore[segLen - 1]);
-        vH = _mm_slli_si128(vH, 2);
+        vH = _mm_slli_si128(vH, 4);
 
         /* insert upper boundary condition */
-        vH = _mm_insert_epi16(vH, boundary[j], 0);
+        vH = _mm_insert_epi32(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -271,20 +271,20 @@ parasail_result_t* PNAME(
             vE = _mm_load_si128(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm_adds_epi16(vH, _mm_load_si128(vP + i));
-            vH = _mm_max_epi16(vH_dag, vE);
-            vH = _mm_max_epi16(vH, vF);
+            vH_dag = _mm_add_epi32(vH, _mm_load_si128(vP + i));
+            vH = _mm_max_epi32(vH_dag, vE);
+            vH = _mm_max_epi32(vH, vF);
             /* Save vH values. */
             _mm_store_si128(pvHStore + i, vH);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
-            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vE);
-            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vE);
+            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vF);
 
             {
                 vec128i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                vec128i case1 = _mm_cmpeq_epi16(vH, vH_dag);
-                vec128i case2 = _mm_cmpeq_epi16(vH, vF);
+                vec128i case1 = _mm_cmpeq_epi32(vH, vH_dag);
+                vec128i case2 = _mm_cmpeq_epi32(vH, vF);
                 vec128i vT = _mm_blendv_epi8(
                         _mm_blendv_epi8(vTIns, vTDel, case2),
                         vTDiag, case1);
@@ -293,30 +293,30 @@ parasail_result_t* PNAME(
                 arr_store(result->trace->trace_table, vT, i, segLen, j);
             }
 
-            vEF_opn = _mm_subs_epi16(vH, vGapO);
+            vEF_opn = _mm_sub_epi32(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm_subs_epi16(vE, vGapE);
-            vE = _mm_max_epi16(vEF_opn, vE_ext);
+            vE_ext = _mm_sub_epi32(vE, vGapE);
+            vE = _mm_max_epi32(vEF_opn, vE_ext);
             _mm_store_si128(pvE + i, vE);
             {
                 vec128i vEa = _mm_load_si128(pvEaLoad + i);
-                vec128i vEa_ext = _mm_subs_epi16(vEa, vGapE);
-                vEa = _mm_max_epi16(vEF_opn, vEa_ext);
+                vec128i vEa_ext = _mm_sub_epi32(vEa, vGapE);
+                vEa = _mm_max_epi32(vEF_opn, vEa_ext);
                 _mm_store_si128(pvEaStore + i, vEa);
                 if (j+1<s2Len) {
-                    vec128i cond = _mm_cmpgt_epi16(vEF_opn, vEa_ext);
+                    vec128i cond = _mm_cmpgt_epi32(vEF_opn, vEa_ext);
                     vec128i vT = _mm_blendv_epi8(vTInsE, vTDiagE, cond);
                     arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                 }
             }
 
             /* Update vF value. */
-            vF_ext = _mm_subs_epi16(vF, vGapE);
-            vF = _mm_max_epi16(vEF_opn, vF_ext);
+            vF_ext = _mm_sub_epi32(vF, vGapE);
+            vF = _mm_max_epi32(vEF_opn, vF_ext);
             if (i+1<segLen) {
                 vec128i vTAll = arr_load(result->trace->trace_table, i+1, segLen, j);
-                vec128i cond = _mm_cmpgt_epi16(vEF_opn, vF_ext);
+                vec128i cond = _mm_cmpgt_epi32(vEF_opn, vF_ext);
                 vec128i vT = _mm_blendv_epi8(vTDelF, vTDiagF, cond);
                 vT = _mm_or_si128(vT, vTAll);
                 arr_store(result->trace->trace_table, vT, i+1, segLen, j);
@@ -332,35 +332,35 @@ parasail_result_t* PNAME(
         vFa = vF;
         for (k=0; k<segWidth; ++k) {
             int64_t tmp = s2_beg ? -open : (boundary[j+1]-open);
-            int16_t tmp2 = tmp < INT16_MIN ? INT16_MIN : tmp;
+            int32_t tmp2 = tmp < INT32_MIN ? INT32_MIN : tmp;
             vec128i vHp = _mm_load_si128(&pvHLoad[segLen - 1]);
-            vHp = _mm_slli_si128(vHp, 2);
-            vHp = _mm_insert_epi16(vHp, boundary[j], 0);
-            vEF_opn = _mm_slli_si128(vEF_opn, 2);
-            vEF_opn = _mm_insert_epi16(vEF_opn, tmp2, 0);
-            vF_ext = _mm_slli_si128(vF_ext, 2);
-            vF_ext = _mm_insert_epi16(vF_ext, NEG_LIMIT, 0);
-            vF = _mm_slli_si128(vF, 2);
-            vF = _mm_insert_epi16(vF, tmp2, 0);
-            vFa_ext = _mm_slli_si128(vFa_ext, 2);
-            vFa_ext = _mm_insert_epi16(vFa_ext, NEG_LIMIT, 0);
-            vFa = _mm_slli_si128(vFa, 2);
-            vFa = _mm_insert_epi16(vFa, tmp2, 0);
+            vHp = _mm_slli_si128(vHp, 4);
+            vHp = _mm_insert_epi32(vHp, boundary[j], 0);
+            vEF_opn = _mm_slli_si128(vEF_opn, 4);
+            vEF_opn = _mm_insert_epi32(vEF_opn, tmp2, 0);
+            vF_ext = _mm_slli_si128(vF_ext, 4);
+            vF_ext = _mm_insert_epi32(vF_ext, NEG_LIMIT, 0);
+            vF = _mm_slli_si128(vF, 4);
+            vF = _mm_insert_epi32(vF, tmp2, 0);
+            vFa_ext = _mm_slli_si128(vFa_ext, 4);
+            vFa_ext = _mm_insert_epi32(vFa_ext, NEG_LIMIT, 0);
+            vFa = _mm_slli_si128(vFa, 4);
+            vFa = _mm_insert_epi32(vFa, tmp2, 0);
             for (i=0; i<segLen; ++i) {
                 vH = _mm_load_si128(pvHStore + i);
-                vH = _mm_max_epi16(vH,vF);
+                vH = _mm_max_epi32(vH,vF);
                 _mm_store_si128(pvHStore + i, vH);
-                vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
+                vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
                 {
                     vec128i vTAll;
                     vec128i vT;
                     vec128i case1;
                     vec128i case2;
                     vec128i cond;
-                    vHp = _mm_adds_epi16(vHp, _mm_load_si128(vP + i));
-                    case1 = _mm_cmpeq_epi16(vH, vHp);
-                    case2 = _mm_cmpeq_epi16(vH, vF);
+                    vHp = _mm_add_epi32(vHp, _mm_load_si128(vP + i));
+                    case1 = _mm_cmpeq_epi32(vH, vHp);
+                    case2 = _mm_cmpeq_epi32(vH, vF);
                     cond = _mm_andnot_si128(case1,case2);
                     vTAll = arr_load(result->trace->trace_table, i, segLen, j);
                     vT = _mm_load_si128(pvHT + i);
@@ -373,34 +373,34 @@ parasail_result_t* PNAME(
                 /* Update vF value. */
                 {
                     vec128i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                    vec128i cond = _mm_cmpgt_epi16(vEF_opn, vFa_ext);
+                    vec128i cond = _mm_cmpgt_epi32(vEF_opn, vFa_ext);
                     vec128i vT = _mm_blendv_epi8(vTDelF, vTDiagF, cond);
                     vTAll = _mm_and_si128(vTAll, vFTMask);
                     vTAll = _mm_or_si128(vTAll, vT);
                     arr_store(result->trace->trace_table, vTAll, i, segLen, j);
                 }
-                vEF_opn = _mm_subs_epi16(vH, vGapO);
-                vF_ext = _mm_subs_epi16(vF, vGapE);
+                vEF_opn = _mm_sub_epi32(vH, vGapO);
+                vF_ext = _mm_sub_epi32(vF, vGapE);
                 {
                     vec128i vEa = _mm_load_si128(pvEaLoad + i);
-                    vec128i vEa_ext = _mm_subs_epi16(vEa, vGapE);
-                    vEa = _mm_max_epi16(vEF_opn, vEa_ext);
+                    vec128i vEa_ext = _mm_sub_epi32(vEa, vGapE);
+                    vEa = _mm_max_epi32(vEF_opn, vEa_ext);
                     _mm_store_si128(pvEaStore + i, vEa);
                     if (j+1<s2Len) {
-                        vec128i cond = _mm_cmpgt_epi16(vEF_opn, vEa_ext);
+                        vec128i cond = _mm_cmpgt_epi32(vEF_opn, vEa_ext);
                         vec128i vT = _mm_blendv_epi8(vTInsE, vTDiagE, cond);
                         arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                     }
                 }
                 if (! _mm_movemask_epi8(
                             _mm_or_si128(
-                                _mm_cmpgt_epi16(vF_ext, vEF_opn),
-                                _mm_cmpeq_epi16(vF_ext, vEF_opn))))
+                                _mm_cmpgt_epi32(vF_ext, vEF_opn),
+                                _mm_cmpeq_epi32(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = _mm_max_epi16(vEF_opn, vF_ext);*/
+                /*vF = _mm_max_epi32(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                vFa_ext = _mm_subs_epi16(vFa, vGapE);
-                vFa = _mm_max_epi16(vEF_opn, vFa_ext);
+                vFa_ext = _mm_sub_epi32(vFa, vGapE);
+                vFa = _mm_max_epi32(vEF_opn, vFa_ext);
                 vHp = _mm_load_si128(pvHLoad + i);
             }
         }
@@ -409,8 +409,8 @@ end:
             /* extract vector containing last value from the column */
             vec128i vCompare;
             vH = _mm_load_si128(pvHStore + offset);
-            vCompare = _mm_and_si128(vPosMask, _mm_cmpgt_epi16(vH, vMaxH));
-            vMaxH = _mm_max_epi16(vH, vMaxH);
+            vCompare = _mm_and_si128(vPosMask, _mm_cmpgt_epi32(vH, vMaxH));
+            vMaxH = _mm_max_epi32(vH, vMaxH);
             if (_mm_movemask_epi8(vCompare)) {
                 end_ref = j;
             }
@@ -421,9 +421,9 @@ end:
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm_slli_si128(vMaxH, 2);
+            vMaxH = _mm_slli_si128(vMaxH, 4);
         }
-        score = (int16_t) _mm_extract_epi16(vMaxH, 7);
+        score = (int32_t) _mm_extract_epi32(vMaxH, 3);
         end_query = s1Len-1;
     }
 
@@ -431,7 +431,7 @@ end:
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int16_t *t = (int16_t*)pvHStore;
+        int32_t *t = (int32_t*)pvHStore;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -452,17 +452,17 @@ end:
         {
             vec128i vH = _mm_load_si128(pvHStore + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 2);
+                vH = _mm_slli_si128(vH, 4);
             }
-            score = (int16_t) _mm_extract_epi16 (vH, 7);
+            score = (int32_t) _mm_extract_epi32 (vH, 3);
             end_ref = s2Len - 1;
             end_query = s1Len - 1;
         }
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
