@@ -17,36 +17,36 @@
 #include "parasail/internal_avx.h"
 
 #define SG_TRACE
-#define SG_SUFFIX _striped_avx2_256_8
-#define SG_SUFFIX_PROF _striped_profile_avx2_256_8
+#define SG_SUFFIX _striped_avx2_256_16
+#define SG_SUFFIX_PROF _striped_profile_avx2_256_16
 #include "sg_helper.h"
 
 #define SWAP(A,B) { __m256i* tmp = A; A = B; B = tmp; }
 
 
-#if HAVE_AVX2_MM256_INSERT_EPI8
-#define _mm256_insert_epi8_rpl _mm256_insert_epi8
+#define _mm256_cmplt_epi16_rpl(a,b) _mm256_cmpgt_epi16(b,a)
+
+#if HAVE_AVX2_MM256_INSERT_EPI16
+#define _mm256_insert_epi16_rpl _mm256_insert_epi16
 #else
-static inline __m256i _mm256_insert_epi8_rpl(__m256i a, int8_t i, int imm) {
-    __m256i_8_t A;
+static inline __m256i _mm256_insert_epi16_rpl(__m256i a, int16_t i, int imm) {
+    __m256i_16_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_EXTRACT_EPI8
-#define _mm256_extract_epi8_rpl _mm256_extract_epi8
+#if HAVE_AVX2_MM256_EXTRACT_EPI16
+#define _mm256_extract_epi16_rpl _mm256_extract_epi16
 #else
-static inline int8_t _mm256_extract_epi8_rpl(__m256i a, int imm) {
-    __m256i_8_t A;
+static inline int16_t _mm256_extract_epi16_rpl(__m256i a, int imm) {
+    __m256i_16_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
-#define _mm256_cmplt_epi8_rpl(a,b) _mm256_cmpgt_epi8(b,a)
-
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
 
@@ -69,8 +69,8 @@ static inline __m256i arr_load(
     return _mm256_load_si256(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_sg_flags_trace_striped_avx2_256_8
-#define PNAME parasail_sg_flags_trace_striped_profile_avx2_256_8
+#define FNAME parasail_sg_flags_trace_striped_avx2_256_16
+#define PNAME parasail_sg_flags_trace_striped_profile_avx2_256_16
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -94,7 +94,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_avx_256_8(s1, s1Len, matrix);
+    profile = parasail_profile_create_avx_256_16(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -128,12 +128,12 @@ parasail_result_t* PNAME(
     __m256i* restrict pvEaStore = NULL;
     __m256i* restrict pvEaLoad = NULL;
     __m256i* restrict pvHT = NULL;
-    int8_t* restrict boundary = NULL;
+    int16_t* restrict boundary = NULL;
     __m256i vGapO;
     __m256i vGapE;
-    int8_t NEG_LIMIT = 0;
-    int8_t POS_LIMIT = 0;
-    int8_t score = 0;
+    int16_t NEG_LIMIT = 0;
+    int16_t POS_LIMIT = 0;
+    int16_t score = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -153,7 +153,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile8.score);
+    PARASAIL_CHECK_NULL(profile->profile16.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -169,32 +169,32 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 32; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile = (__m256i*)profile->profile8.score;
-    vGapO = _mm256_set1_epi8(open);
-    vGapE = _mm256_set1_epi8(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
-    POS_LIMIT = INT8_MAX - matrix->max - 1;
+    vProfile = (__m256i*)profile->profile16.score;
+    vGapO = _mm256_set1_epi16(open);
+    vGapE = _mm256_set1_epi16(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
+    POS_LIMIT = INT16_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm256_set1_epi8(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi8(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi16(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi16(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
-    vPosMask = _mm256_cmpeq_epi8(_mm256_set1_epi8(position),
-            _mm256_set_epi8(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31));
-    vTIns  = _mm256_set1_epi8(PARASAIL_INS);
-    vTDel  = _mm256_set1_epi8(PARASAIL_DEL);
-    vTDiag = _mm256_set1_epi8(PARASAIL_DIAG);
-    vTDiagE= _mm256_set1_epi8(PARASAIL_DIAG_E);
-    vTInsE = _mm256_set1_epi8(PARASAIL_INS_E);
-    vTDiagF= _mm256_set1_epi8(PARASAIL_DIAG_F);
-    vTDelF = _mm256_set1_epi8(PARASAIL_DEL_F);
-    vTMask = _mm256_set1_epi8(PARASAIL_ZERO_MASK);
-    vFTMask= _mm256_set1_epi8(PARASAIL_F_MASK);
+    vPosMask = _mm256_cmpeq_epi16(_mm256_set1_epi16(position),
+            _mm256_set_epi16(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15));
+    vTIns  = _mm256_set1_epi16(PARASAIL_INS);
+    vTDel  = _mm256_set1_epi16(PARASAIL_DEL);
+    vTDiag = _mm256_set1_epi16(PARASAIL_DIAG);
+    vTDiagE= _mm256_set1_epi16(PARASAIL_DIAG_E);
+    vTInsE = _mm256_set1_epi16(PARASAIL_INS_E);
+    vTDiagF= _mm256_set1_epi16(PARASAIL_DIAG_F);
+    vTDelF = _mm256_set1_epi16(PARASAIL_DEL_F);
+    vTMask = _mm256_set1_epi16(PARASAIL_ZERO_MASK);
+    vFTMask= _mm256_set1_epi16(PARASAIL_F_MASK);
 
     /* initialize result */
     result = parasail_result_new_trace(segLen, s2Len, 32, sizeof(__m256i));
@@ -203,7 +203,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_32;
+        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_16;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -216,7 +216,7 @@ parasail_result_t* PNAME(
     pvEaStore= parasail_memalign___m256i(32, segLen);
     pvEaLoad = parasail_memalign___m256i(32, segLen);
     pvHT     = parasail_memalign___m256i(32, segLen);
-    boundary = parasail_memalign_int8_t(32, s2Len+1);
+    boundary = parasail_memalign_int16_t(32, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -232,13 +232,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m256i_8_t h;
-            __m256i_8_t e;
+            __m256i_16_t h;
+            __m256i_16_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
+                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
+                e.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
             }
             _mm256_store_si256(&pvHStore[index], h.m);
             _mm256_store_si256(&pvE[index], e.m);
@@ -252,7 +252,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
+            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
         }
     }
 
@@ -277,12 +277,12 @@ parasail_result_t* PNAME(
          * corrected in the Lazy_F loop. */
         vF = vNegLimit;
 
-        /* load final segment of pvHStore and shift left by 1 bytes */
+        /* load final segment of pvHStore and shift left by 2 bytes */
         vH = _mm256_load_si256(&pvHStore[segLen - 1]);
-        vH = _mm256_slli_si256_rpl(vH, 1);
+        vH = _mm256_slli_si256_rpl(vH, 2);
 
         /* insert upper boundary condition */
-        vH = _mm256_insert_epi8_rpl(vH, boundary[j], 0);
+        vH = _mm256_insert_epi16_rpl(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -296,20 +296,20 @@ parasail_result_t* PNAME(
             vE = _mm256_load_si256(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm256_adds_epi8(vH, _mm256_load_si256(vP + i));
-            vH = _mm256_max_epi8(vH_dag, vE);
-            vH = _mm256_max_epi8(vH, vF);
+            vH_dag = _mm256_adds_epi16(vH, _mm256_load_si256(vP + i));
+            vH = _mm256_max_epi16(vH_dag, vE);
+            vH = _mm256_max_epi16(vH, vF);
             /* Save vH values. */
             _mm256_store_si256(pvHStore + i, vH);
-            vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vH);
-            vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vE);
-            vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vE);
+            vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vF);
 
             {
                 __m256i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                __m256i case1 = _mm256_cmpeq_epi8(vH, vH_dag);
-                __m256i case2 = _mm256_cmpeq_epi8(vH, vF);
+                __m256i case1 = _mm256_cmpeq_epi16(vH, vH_dag);
+                __m256i case2 = _mm256_cmpeq_epi16(vH, vF);
                 __m256i vT = _mm256_blendv_epi8(
                         _mm256_blendv_epi8(vTIns, vTDel, case2),
                         vTDiag, case1);
@@ -318,30 +318,30 @@ parasail_result_t* PNAME(
                 arr_store(result->trace->trace_table, vT, i, segLen, j);
             }
 
-            vEF_opn = _mm256_subs_epi8(vH, vGapO);
+            vEF_opn = _mm256_subs_epi16(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm256_subs_epi8(vE, vGapE);
-            vE = _mm256_max_epi8(vEF_opn, vE_ext);
+            vE_ext = _mm256_subs_epi16(vE, vGapE);
+            vE = _mm256_max_epi16(vEF_opn, vE_ext);
             _mm256_store_si256(pvE + i, vE);
             {
                 __m256i vEa = _mm256_load_si256(pvEaLoad + i);
-                __m256i vEa_ext = _mm256_subs_epi8(vEa, vGapE);
-                vEa = _mm256_max_epi8(vEF_opn, vEa_ext);
+                __m256i vEa_ext = _mm256_subs_epi16(vEa, vGapE);
+                vEa = _mm256_max_epi16(vEF_opn, vEa_ext);
                 _mm256_store_si256(pvEaStore + i, vEa);
                 if (j+1<s2Len) {
-                    __m256i cond = _mm256_cmpgt_epi8(vEF_opn, vEa_ext);
+                    __m256i cond = _mm256_cmpgt_epi16(vEF_opn, vEa_ext);
                     __m256i vT = _mm256_blendv_epi8(vTInsE, vTDiagE, cond);
                     arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                 }
             }
 
             /* Update vF value. */
-            vF_ext = _mm256_subs_epi8(vF, vGapE);
-            vF = _mm256_max_epi8(vEF_opn, vF_ext);
+            vF_ext = _mm256_subs_epi16(vF, vGapE);
+            vF = _mm256_max_epi16(vEF_opn, vF_ext);
             if (i+1<segLen) {
                 __m256i vTAll = arr_load(result->trace->trace_table, i+1, segLen, j);
-                __m256i cond = _mm256_cmpgt_epi8(vEF_opn, vF_ext);
+                __m256i cond = _mm256_cmpgt_epi16(vEF_opn, vF_ext);
                 __m256i vT = _mm256_blendv_epi8(vTDelF, vTDiagF, cond);
                 vT = _mm256_or_si256(vT, vTAll);
                 arr_store(result->trace->trace_table, vT, i+1, segLen, j);
@@ -357,35 +357,35 @@ parasail_result_t* PNAME(
         vFa = vF;
         for (k=0; k<segWidth; ++k) {
             int64_t tmp = s2_beg ? -open : (boundary[j+1]-open);
-            int8_t tmp2 = tmp < INT8_MIN ? INT8_MIN : tmp;
+            int16_t tmp2 = tmp < INT16_MIN ? INT16_MIN : tmp;
             __m256i vHp = _mm256_load_si256(&pvHLoad[segLen - 1]);
-            vHp = _mm256_slli_si256_rpl(vHp, 1);
-            vHp = _mm256_insert_epi8_rpl(vHp, boundary[j], 0);
-            vEF_opn = _mm256_slli_si256_rpl(vEF_opn, 1);
-            vEF_opn = _mm256_insert_epi8_rpl(vEF_opn, tmp2, 0);
-            vF_ext = _mm256_slli_si256_rpl(vF_ext, 1);
-            vF_ext = _mm256_insert_epi8_rpl(vF_ext, NEG_LIMIT, 0);
-            vF = _mm256_slli_si256_rpl(vF, 1);
-            vF = _mm256_insert_epi8_rpl(vF, tmp2, 0);
-            vFa_ext = _mm256_slli_si256_rpl(vFa_ext, 1);
-            vFa_ext = _mm256_insert_epi8_rpl(vFa_ext, NEG_LIMIT, 0);
-            vFa = _mm256_slli_si256_rpl(vFa, 1);
-            vFa = _mm256_insert_epi8_rpl(vFa, tmp2, 0);
+            vHp = _mm256_slli_si256_rpl(vHp, 2);
+            vHp = _mm256_insert_epi16_rpl(vHp, boundary[j], 0);
+            vEF_opn = _mm256_slli_si256_rpl(vEF_opn, 2);
+            vEF_opn = _mm256_insert_epi16_rpl(vEF_opn, tmp2, 0);
+            vF_ext = _mm256_slli_si256_rpl(vF_ext, 2);
+            vF_ext = _mm256_insert_epi16_rpl(vF_ext, NEG_LIMIT, 0);
+            vF = _mm256_slli_si256_rpl(vF, 2);
+            vF = _mm256_insert_epi16_rpl(vF, tmp2, 0);
+            vFa_ext = _mm256_slli_si256_rpl(vFa_ext, 2);
+            vFa_ext = _mm256_insert_epi16_rpl(vFa_ext, NEG_LIMIT, 0);
+            vFa = _mm256_slli_si256_rpl(vFa, 2);
+            vFa = _mm256_insert_epi16_rpl(vFa, tmp2, 0);
             for (i=0; i<segLen; ++i) {
                 vH = _mm256_load_si256(pvHStore + i);
-                vH = _mm256_max_epi8(vH,vF);
+                vH = _mm256_max_epi16(vH,vF);
                 _mm256_store_si256(pvHStore + i, vH);
-                vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vH);
+                vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vH);
                 {
                     __m256i vTAll;
                     __m256i vT;
                     __m256i case1;
                     __m256i case2;
                     __m256i cond;
-                    vHp = _mm256_adds_epi8(vHp, _mm256_load_si256(vP + i));
-                    case1 = _mm256_cmpeq_epi8(vH, vHp);
-                    case2 = _mm256_cmpeq_epi8(vH, vF);
+                    vHp = _mm256_adds_epi16(vHp, _mm256_load_si256(vP + i));
+                    case1 = _mm256_cmpeq_epi16(vH, vHp);
+                    case2 = _mm256_cmpeq_epi16(vH, vF);
                     cond = _mm256_andnot_si256(case1,case2);
                     vTAll = arr_load(result->trace->trace_table, i, segLen, j);
                     vT = _mm256_load_si256(pvHT + i);
@@ -398,34 +398,34 @@ parasail_result_t* PNAME(
                 /* Update vF value. */
                 {
                     __m256i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                    __m256i cond = _mm256_cmpgt_epi8(vEF_opn, vFa_ext);
+                    __m256i cond = _mm256_cmpgt_epi16(vEF_opn, vFa_ext);
                     __m256i vT = _mm256_blendv_epi8(vTDelF, vTDiagF, cond);
                     vTAll = _mm256_and_si256(vTAll, vFTMask);
                     vTAll = _mm256_or_si256(vTAll, vT);
                     arr_store(result->trace->trace_table, vTAll, i, segLen, j);
                 }
-                vEF_opn = _mm256_subs_epi8(vH, vGapO);
-                vF_ext = _mm256_subs_epi8(vF, vGapE);
+                vEF_opn = _mm256_subs_epi16(vH, vGapO);
+                vF_ext = _mm256_subs_epi16(vF, vGapE);
                 {
                     __m256i vEa = _mm256_load_si256(pvEaLoad + i);
-                    __m256i vEa_ext = _mm256_subs_epi8(vEa, vGapE);
-                    vEa = _mm256_max_epi8(vEF_opn, vEa_ext);
+                    __m256i vEa_ext = _mm256_subs_epi16(vEa, vGapE);
+                    vEa = _mm256_max_epi16(vEF_opn, vEa_ext);
                     _mm256_store_si256(pvEaStore + i, vEa);
                     if (j+1<s2Len) {
-                        __m256i cond = _mm256_cmpgt_epi8(vEF_opn, vEa_ext);
+                        __m256i cond = _mm256_cmpgt_epi16(vEF_opn, vEa_ext);
                         __m256i vT = _mm256_blendv_epi8(vTInsE, vTDiagE, cond);
                         arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                     }
                 }
                 if (! _mm256_movemask_epi8(
                             _mm256_or_si256(
-                                _mm256_cmpgt_epi8(vF_ext, vEF_opn),
-                                _mm256_cmpeq_epi8(vF_ext, vEF_opn))))
+                                _mm256_cmpgt_epi16(vF_ext, vEF_opn),
+                                _mm256_cmpeq_epi16(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = _mm256_max_epi8(vEF_opn, vF_ext);*/
+                /*vF = _mm256_max_epi16(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                vFa_ext = _mm256_subs_epi8(vFa, vGapE);
-                vFa = _mm256_max_epi8(vEF_opn, vFa_ext);
+                vFa_ext = _mm256_subs_epi16(vFa, vGapE);
+                vFa = _mm256_max_epi16(vEF_opn, vFa_ext);
                 vHp = _mm256_load_si256(pvHLoad + i);
             }
         }
@@ -434,8 +434,8 @@ end:
             /* extract vector containing last value from the column */
             __m256i vCompare;
             vH = _mm256_load_si256(pvHStore + offset);
-            vCompare = _mm256_and_si256(vPosMask, _mm256_cmpgt_epi8(vH, vMaxH));
-            vMaxH = _mm256_max_epi8(vH, vMaxH);
+            vCompare = _mm256_and_si256(vPosMask, _mm256_cmpgt_epi16(vH, vMaxH));
+            vMaxH = _mm256_max_epi16(vH, vMaxH);
             if (_mm256_movemask_epi8(vCompare)) {
                 end_ref = j;
             }
@@ -446,9 +446,9 @@ end:
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm256_slli_si256_rpl(vMaxH, 1);
+            vMaxH = _mm256_slli_si256_rpl(vMaxH, 2);
         }
-        score = (int8_t) _mm256_extract_epi8_rpl(vMaxH, 31);
+        score = (int16_t) _mm256_extract_epi16_rpl(vMaxH, 15);
         end_query = s1Len-1;
     }
 
@@ -456,7 +456,7 @@ end:
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int8_t *t = (int8_t*)pvHStore;
+        int16_t *t = (int16_t*)pvHStore;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -477,17 +477,17 @@ end:
         {
             __m256i vH = _mm256_load_si256(pvHStore + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm256_slli_si256_rpl(vH, 1);
+                vH = _mm256_slli_si256_rpl(vH, 2);
             }
-            score = (int8_t) _mm256_extract_epi8_rpl (vH, 31);
+            score = (int16_t) _mm256_extract_epi16_rpl (vH, 15);
             end_ref = s2Len - 1;
             end_query = s1Len - 1;
         }
     }
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi8_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi16_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
