@@ -17,8 +17,8 @@
 #include "parasail/internal_neon.h"
 
 #define SG_TRACE
-#define SG_SUFFIX _striped_neon_128_64
-#define SG_SUFFIX_PROF _striped_profile_neon_128_64
+#define SG_SUFFIX _striped_neon_128_32
+#define SG_SUFFIX_PROF _striped_profile_neon_128_32
 #include "sg_helper.h"
 
 #define SWAP(A,B) { simde__m128i* tmp = A; A = B; B = tmp; }
@@ -44,8 +44,8 @@ static inline simde__m128i arr_load(
     return simde_mm_load_si128(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_sg_flags_trace_striped_neon_128_64
-#define PNAME parasail_sg_flags_trace_striped_profile_neon_128_64
+#define FNAME parasail_sg_flags_trace_striped_neon_128_32
+#define PNAME parasail_sg_flags_trace_striped_profile_neon_128_32
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -69,7 +69,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_neon_128_64(s1, s1Len, matrix);
+    profile = parasail_profile_create_neon_128_32(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -103,12 +103,12 @@ parasail_result_t* PNAME(
     simde__m128i* restrict pvEaStore = NULL;
     simde__m128i* restrict pvEaLoad = NULL;
     simde__m128i* restrict pvHT = NULL;
-    int64_t* restrict boundary = NULL;
+    int32_t* restrict boundary = NULL;
     simde__m128i vGapO;
     simde__m128i vGapE;
-    int64_t NEG_LIMIT = 0;
-    int64_t POS_LIMIT = 0;
-    int64_t score = 0;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
+    int32_t score = 0;
     simde__m128i vNegLimit;
     simde__m128i vPosLimit;
     simde__m128i vSaturationCheckMin;
@@ -128,7 +128,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->profile32.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -144,32 +144,32 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 2; /* number of values in vector unit */
+    segWidth = 4; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile = (simde__m128i*)profile->profile64.score;
-    vGapO = simde_mm_set1_epi64x(open);
-    vGapE = simde_mm_set1_epi64x(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    vProfile = (simde__m128i*)profile->profile32.score;
+    vGapO = simde_mm_set1_epi32(open);
+    vGapE = simde_mm_set1_epi32(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = simde_mm_set1_epi64x(NEG_LIMIT);
-    vPosLimit = simde_mm_set1_epi64x(POS_LIMIT);
+    vNegLimit = simde_mm_set1_epi32(NEG_LIMIT);
+    vPosLimit = simde_mm_set1_epi32(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
-    vPosMask = simde_mm_cmpeq_epi64(simde_mm_set1_epi64x(position),
-            simde_mm_set_epi64x(0,1));
-    vTIns  = simde_mm_set1_epi64x(PARASAIL_INS);
-    vTDel  = simde_mm_set1_epi64x(PARASAIL_DEL);
-    vTDiag = simde_mm_set1_epi64x(PARASAIL_DIAG);
-    vTDiagE= simde_mm_set1_epi64x(PARASAIL_DIAG_E);
-    vTInsE = simde_mm_set1_epi64x(PARASAIL_INS_E);
-    vTDiagF= simde_mm_set1_epi64x(PARASAIL_DIAG_F);
-    vTDelF = simde_mm_set1_epi64x(PARASAIL_DEL_F);
-    vTMask = simde_mm_set1_epi64x(PARASAIL_ZERO_MASK);
-    vFTMask= simde_mm_set1_epi64x(PARASAIL_F_MASK);
+    vPosMask = simde_mm_cmpeq_epi32(simde_mm_set1_epi32(position),
+            simde_mm_set_epi32(0,1,2,3));
+    vTIns  = simde_mm_set1_epi32(PARASAIL_INS);
+    vTDel  = simde_mm_set1_epi32(PARASAIL_DEL);
+    vTDiag = simde_mm_set1_epi32(PARASAIL_DIAG);
+    vTDiagE= simde_mm_set1_epi32(PARASAIL_DIAG_E);
+    vTInsE = simde_mm_set1_epi32(PARASAIL_INS_E);
+    vTDiagF= simde_mm_set1_epi32(PARASAIL_DIAG_F);
+    vTDelF = simde_mm_set1_epi32(PARASAIL_DEL_F);
+    vTMask = simde_mm_set1_epi32(PARASAIL_ZERO_MASK);
+    vFTMask= simde_mm_set1_epi32(PARASAIL_F_MASK);
 
     /* initialize result */
     result = parasail_result_new_trace(segLen, s2Len, 16, sizeof(simde__m128i));
@@ -178,7 +178,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -191,7 +191,7 @@ parasail_result_t* PNAME(
     pvEaStore= parasail_memalign_simde__m128i(16, segLen);
     pvEaLoad = parasail_memalign_simde__m128i(16, segLen);
     pvHT     = parasail_memalign_simde__m128i(16, segLen);
-    boundary = parasail_memalign_int64_t(16, s2Len+1);
+    boundary = parasail_memalign_int32_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -211,9 +211,9 @@ parasail_result_t* PNAME(
             simde__m128i e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.i64[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                h.i32[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
                 tmp = tmp - open;
-                e.i64[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                e.i32[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
             }
             simde_mm_store_si128(&pvHStore[index], h);
             simde_mm_store_si128(&pvE[index], e);
@@ -227,7 +227,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
+            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
         }
     }
 
@@ -252,12 +252,12 @@ parasail_result_t* PNAME(
          * corrected in the Lazy_F loop. */
         vF = vNegLimit;
 
-        /* load final segment of pvHStore and shift left by 8 bytes */
+        /* load final segment of pvHStore and shift left by 4 bytes */
         vH = simde_mm_load_si128(&pvHStore[segLen - 1]);
-        vH = simde_mm_slli_si128(vH, 8);
+        vH = simde_mm_slli_si128(vH, 4);
 
         /* insert upper boundary condition */
-        vH = simde_mm_insert_epi64(vH, boundary[j], 0);
+        vH = simde_mm_insert_epi32(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -271,20 +271,20 @@ parasail_result_t* PNAME(
             vE = simde_mm_load_si128(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = simde_mm_add_epi64(vH, simde_mm_load_si128(vP + i));
-            vH = simde_mm_max_epi64(vH_dag, vE);
-            vH = simde_mm_max_epi64(vH, vF);
+            vH_dag = simde_mm_add_epi32(vH, simde_mm_load_si128(vP + i));
+            vH = simde_mm_max_epi32(vH_dag, vE);
+            vH = simde_mm_max_epi32(vH, vF);
             /* Save vH values. */
             simde_mm_store_si128(pvHStore + i, vH);
-            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vH);
-            vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vH);
-            vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vE);
-            vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vF);
+            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vH);
+            vSaturationCheckMin = simde_mm_min_epi32(vSaturationCheckMin, vH);
+            vSaturationCheckMin = simde_mm_min_epi32(vSaturationCheckMin, vE);
+            vSaturationCheckMin = simde_mm_min_epi32(vSaturationCheckMin, vF);
 
             {
                 simde__m128i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                simde__m128i case1 = simde_mm_cmpeq_epi64(vH, vH_dag);
-                simde__m128i case2 = simde_mm_cmpeq_epi64(vH, vF);
+                simde__m128i case1 = simde_mm_cmpeq_epi32(vH, vH_dag);
+                simde__m128i case2 = simde_mm_cmpeq_epi32(vH, vF);
                 simde__m128i vT = simde_mm_blendv_epi8(
                         simde_mm_blendv_epi8(vTIns, vTDel, case2),
                         vTDiag, case1);
@@ -293,30 +293,30 @@ parasail_result_t* PNAME(
                 arr_store(result->trace->trace_table, vT, i, segLen, j);
             }
 
-            vEF_opn = simde_mm_sub_epi64(vH, vGapO);
+            vEF_opn = simde_mm_sub_epi32(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = simde_mm_sub_epi64(vE, vGapE);
-            vE = simde_mm_max_epi64(vEF_opn, vE_ext);
+            vE_ext = simde_mm_sub_epi32(vE, vGapE);
+            vE = simde_mm_max_epi32(vEF_opn, vE_ext);
             simde_mm_store_si128(pvE + i, vE);
             {
                 simde__m128i vEa = simde_mm_load_si128(pvEaLoad + i);
-                simde__m128i vEa_ext = simde_mm_sub_epi64(vEa, vGapE);
-                vEa = simde_mm_max_epi64(vEF_opn, vEa_ext);
+                simde__m128i vEa_ext = simde_mm_sub_epi32(vEa, vGapE);
+                vEa = simde_mm_max_epi32(vEF_opn, vEa_ext);
                 simde_mm_store_si128(pvEaStore + i, vEa);
                 if (j+1<s2Len) {
-                    simde__m128i cond = simde_mm_cmpgt_epi64(vEF_opn, vEa_ext);
+                    simde__m128i cond = simde_mm_cmpgt_epi32(vEF_opn, vEa_ext);
                     simde__m128i vT = simde_mm_blendv_epi8(vTInsE, vTDiagE, cond);
                     arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                 }
             }
 
             /* Update vF value. */
-            vF_ext = simde_mm_sub_epi64(vF, vGapE);
-            vF = simde_mm_max_epi64(vEF_opn, vF_ext);
+            vF_ext = simde_mm_sub_epi32(vF, vGapE);
+            vF = simde_mm_max_epi32(vEF_opn, vF_ext);
             if (i+1<segLen) {
                 simde__m128i vTAll = arr_load(result->trace->trace_table, i+1, segLen, j);
-                simde__m128i cond = simde_mm_cmpgt_epi64(vEF_opn, vF_ext);
+                simde__m128i cond = simde_mm_cmpgt_epi32(vEF_opn, vF_ext);
                 simde__m128i vT = simde_mm_blendv_epi8(vTDelF, vTDiagF, cond);
                 vT = simde_mm_or_si128(vT, vTAll);
                 arr_store(result->trace->trace_table, vT, i+1, segLen, j);
@@ -332,35 +332,35 @@ parasail_result_t* PNAME(
         vFa = vF;
         for (k=0; k<segWidth; ++k) {
             int64_t tmp = s2_beg ? -open : (boundary[j+1]-open);
-            int64_t tmp2 = tmp < INT64_MIN ? INT64_MIN : tmp;
+            int32_t tmp2 = tmp < INT32_MIN ? INT32_MIN : tmp;
             simde__m128i vHp = simde_mm_load_si128(&pvHLoad[segLen - 1]);
-            vHp = simde_mm_slli_si128(vHp, 8);
-            vHp = simde_mm_insert_epi64(vHp, boundary[j], 0);
-            vEF_opn = simde_mm_slli_si128(vEF_opn, 8);
-            vEF_opn = simde_mm_insert_epi64(vEF_opn, tmp2, 0);
-            vF_ext = simde_mm_slli_si128(vF_ext, 8);
-            vF_ext = simde_mm_insert_epi64(vF_ext, NEG_LIMIT, 0);
-            vF = simde_mm_slli_si128(vF, 8);
-            vF = simde_mm_insert_epi64(vF, tmp2, 0);
-            vFa_ext = simde_mm_slli_si128(vFa_ext, 8);
-            vFa_ext = simde_mm_insert_epi64(vFa_ext, NEG_LIMIT, 0);
-            vFa = simde_mm_slli_si128(vFa, 8);
-            vFa = simde_mm_insert_epi64(vFa, tmp2, 0);
+            vHp = simde_mm_slli_si128(vHp, 4);
+            vHp = simde_mm_insert_epi32(vHp, boundary[j], 0);
+            vEF_opn = simde_mm_slli_si128(vEF_opn, 4);
+            vEF_opn = simde_mm_insert_epi32(vEF_opn, tmp2, 0);
+            vF_ext = simde_mm_slli_si128(vF_ext, 4);
+            vF_ext = simde_mm_insert_epi32(vF_ext, NEG_LIMIT, 0);
+            vF = simde_mm_slli_si128(vF, 4);
+            vF = simde_mm_insert_epi32(vF, tmp2, 0);
+            vFa_ext = simde_mm_slli_si128(vFa_ext, 4);
+            vFa_ext = simde_mm_insert_epi32(vFa_ext, NEG_LIMIT, 0);
+            vFa = simde_mm_slli_si128(vFa, 4);
+            vFa = simde_mm_insert_epi32(vFa, tmp2, 0);
             for (i=0; i<segLen; ++i) {
                 vH = simde_mm_load_si128(pvHStore + i);
-                vH = simde_mm_max_epi64(vH,vF);
+                vH = simde_mm_max_epi32(vH,vF);
                 simde_mm_store_si128(pvHStore + i, vH);
-                vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vH);
-                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vH);
+                vSaturationCheckMin = simde_mm_min_epi32(vSaturationCheckMin, vH);
+                vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vH);
                 {
                     simde__m128i vTAll;
                     simde__m128i vT;
                     simde__m128i case1;
                     simde__m128i case2;
                     simde__m128i cond;
-                    vHp = simde_mm_add_epi64(vHp, simde_mm_load_si128(vP + i));
-                    case1 = simde_mm_cmpeq_epi64(vH, vHp);
-                    case2 = simde_mm_cmpeq_epi64(vH, vF);
+                    vHp = simde_mm_add_epi32(vHp, simde_mm_load_si128(vP + i));
+                    case1 = simde_mm_cmpeq_epi32(vH, vHp);
+                    case2 = simde_mm_cmpeq_epi32(vH, vF);
                     cond = simde_mm_andnot_si128(case1,case2);
                     vTAll = arr_load(result->trace->trace_table, i, segLen, j);
                     vT = simde_mm_load_si128(pvHT + i);
@@ -373,34 +373,34 @@ parasail_result_t* PNAME(
                 /* Update vF value. */
                 {
                     simde__m128i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                    simde__m128i cond = simde_mm_cmpgt_epi64(vEF_opn, vFa_ext);
+                    simde__m128i cond = simde_mm_cmpgt_epi32(vEF_opn, vFa_ext);
                     simde__m128i vT = simde_mm_blendv_epi8(vTDelF, vTDiagF, cond);
                     vTAll = simde_mm_and_si128(vTAll, vFTMask);
                     vTAll = simde_mm_or_si128(vTAll, vT);
                     arr_store(result->trace->trace_table, vTAll, i, segLen, j);
                 }
-                vEF_opn = simde_mm_sub_epi64(vH, vGapO);
-                vF_ext = simde_mm_sub_epi64(vF, vGapE);
+                vEF_opn = simde_mm_sub_epi32(vH, vGapO);
+                vF_ext = simde_mm_sub_epi32(vF, vGapE);
                 {
                     simde__m128i vEa = simde_mm_load_si128(pvEaLoad + i);
-                    simde__m128i vEa_ext = simde_mm_sub_epi64(vEa, vGapE);
-                    vEa = simde_mm_max_epi64(vEF_opn, vEa_ext);
+                    simde__m128i vEa_ext = simde_mm_sub_epi32(vEa, vGapE);
+                    vEa = simde_mm_max_epi32(vEF_opn, vEa_ext);
                     simde_mm_store_si128(pvEaStore + i, vEa);
                     if (j+1<s2Len) {
-                        simde__m128i cond = simde_mm_cmpgt_epi64(vEF_opn, vEa_ext);
+                        simde__m128i cond = simde_mm_cmpgt_epi32(vEF_opn, vEa_ext);
                         simde__m128i vT = simde_mm_blendv_epi8(vTInsE, vTDiagE, cond);
                         arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                     }
                 }
                 if (! simde_mm_movemask_epi8(
                             simde_mm_or_si128(
-                                simde_mm_cmpgt_epi64(vF_ext, vEF_opn),
-                                simde_mm_cmpeq_epi64(vF_ext, vEF_opn))))
+                                simde_mm_cmpgt_epi32(vF_ext, vEF_opn),
+                                simde_mm_cmpeq_epi32(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = simde_mm_max_epi64(vEF_opn, vF_ext);*/
+                /*vF = simde_mm_max_epi32(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                vFa_ext = simde_mm_sub_epi64(vFa, vGapE);
-                vFa = simde_mm_max_epi64(vEF_opn, vFa_ext);
+                vFa_ext = simde_mm_sub_epi32(vFa, vGapE);
+                vFa = simde_mm_max_epi32(vEF_opn, vFa_ext);
                 vHp = simde_mm_load_si128(pvHLoad + i);
             }
         }
@@ -409,8 +409,8 @@ end:
             /* extract vector containing last value from the column */
             simde__m128i vCompare;
             vH = simde_mm_load_si128(pvHStore + offset);
-            vCompare = simde_mm_and_si128(vPosMask, simde_mm_cmpgt_epi64(vH, vMaxH));
-            vMaxH = simde_mm_max_epi64(vH, vMaxH);
+            vCompare = simde_mm_and_si128(vPosMask, simde_mm_cmpgt_epi32(vH, vMaxH));
+            vMaxH = simde_mm_max_epi32(vH, vMaxH);
             if (simde_mm_movemask_epi8(vCompare)) {
                 end_ref = j;
             }
@@ -421,9 +421,9 @@ end:
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = simde_mm_slli_si128(vMaxH, 8);
+            vMaxH = simde_mm_slli_si128(vMaxH, 4);
         }
-        score = (int64_t) simde_mm_extract_epi64(vMaxH, 1);
+        score = (int32_t) simde_mm_extract_epi32(vMaxH, 3);
         end_query = s1Len-1;
     }
 
@@ -431,7 +431,7 @@ end:
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int64_t *t = (int64_t*)pvHStore;
+        int32_t *t = (int32_t*)pvHStore;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -452,17 +452,17 @@ end:
         {
             simde__m128i vH = simde_mm_load_si128(pvHStore + offset);
             for (k=0; k<position; ++k) {
-                vH = simde_mm_slli_si128(vH, 8);
+                vH = simde_mm_slli_si128(vH, 4);
             }
-            score = (int64_t) simde_mm_extract_epi64 (vH, 1);
+            score = (int32_t) simde_mm_extract_epi32 (vH, 3);
             end_ref = s2Len - 1;
             end_query = s1Len - 1;
         }
     }
 
     if (simde_mm_movemask_epi8(simde_mm_or_si128(
-            simde_mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
-            simde_mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
+            simde_mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
+            simde_mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
