@@ -18,72 +18,37 @@
 #include "parasail/internal_avx.h"
 
 
-#if HAVE_AVX2_MM256_INSERT_EPI64
-#define _mm256_insert_epi64_rpl _mm256_insert_epi64
+#define _mm256_cmplt_epi16_rpl(a,b) _mm256_cmpgt_epi16(b,a)
+
+#if HAVE_AVX2_MM256_INSERT_EPI16
+#define _mm256_insert_epi16_rpl _mm256_insert_epi16
 #else
-static inline __m256i _mm256_insert_epi64_rpl(__m256i a, int64_t i, int imm) {
-    __m256i_64_t A;
+static inline __m256i _mm256_insert_epi16_rpl(__m256i a, int16_t i, int imm) {
+    __m256i_16_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_SET1_EPI64X
-#define _mm256_set1_epi64x_rpl _mm256_set1_epi64x
+#if HAVE_AVX2_MM256_EXTRACT_EPI16
+#define _mm256_extract_epi16_rpl _mm256_extract_epi16
 #else
-static inline __m256i _mm256_set1_epi64x_rpl(int64_t i) {
-    __m256i_64_t A;
-    A.v[0] = i;
-    A.v[1] = i;
-    A.v[2] = i;
-    A.v[3] = i;
-    return A.m;
-}
-#endif
-
-static inline __m256i _mm256_max_epi64_rpl(__m256i a, __m256i b) {
-    __m256i_64_t A;
-    __m256i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
-    A.v[2] = (A.v[2]>B.v[2]) ? A.v[2] : B.v[2];
-    A.v[3] = (A.v[3]>B.v[3]) ? A.v[3] : B.v[3];
-    return A.m;
-}
-
-#if HAVE_AVX2_MM256_EXTRACT_EPI64
-#define _mm256_extract_epi64_rpl _mm256_extract_epi64
-#else
-static inline int64_t _mm256_extract_epi64_rpl(__m256i a, int imm) {
-    __m256i_64_t A;
+static inline int16_t _mm256_extract_epi16_rpl(__m256i a, int imm) {
+    __m256i_16_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
-static inline __m256i _mm256_min_epi64_rpl(__m256i a, __m256i b) {
-    __m256i_64_t A;
-    __m256i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
-    A.v[2] = (A.v[2]<B.v[2]) ? A.v[2] : B.v[2];
-    A.v[3] = (A.v[3]<B.v[3]) ? A.v[3] : B.v[3];
-    return A.m;
-}
-
-#define _mm256_cmplt_epi64_rpl(a,b) _mm256_cmpgt_epi64(b,a)
-
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
-static inline int64_t _mm256_hmax_epi64_rpl(__m256i a) {
-    a = _mm256_max_epi64_rpl(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,0,0)));
-    a = _mm256_max_epi64_rpl(a, _mm256_slli_si256(a, 8));
-    return _mm256_extract_epi64_rpl(a, 3);
+static inline int16_t _mm256_hmax_epi16_rpl(__m256i a) {
+    a = _mm256_max_epi16(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,0,0)));
+    a = _mm256_max_epi16(a, _mm256_slli_si256(a, 8));
+    a = _mm256_max_epi16(a, _mm256_slli_si256(a, 4));
+    a = _mm256_max_epi16(a, _mm256_slli_si256(a, 2));
+    return _mm256_extract_epi16_rpl(a, 15);
 }
 
 
@@ -96,10 +61,22 @@ static inline void arr_store_si256(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 3);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  0);
+    array[1LL*( 1*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  1);
+    array[1LL*( 2*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  2);
+    array[1LL*( 3*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  3);
+    array[1LL*( 4*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  4);
+    array[1LL*( 5*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  5);
+    array[1LL*( 6*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  6);
+    array[1LL*( 7*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  7);
+    array[1LL*( 8*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  8);
+    array[1LL*( 9*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  9);
+    array[1LL*(10*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 10);
+    array[1LL*(11*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 11);
+    array[1LL*(12*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 12);
+    array[1LL*(13*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 13);
+    array[1LL*(14*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 14);
+    array[1LL*(15*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 15);
 }
 #endif
 
@@ -110,23 +87,35 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 0);
-    col[1*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 1);
-    col[2*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 2);
-    col[3*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 3);
+    col[ 0*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  0);
+    col[ 1*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  1);
+    col[ 2*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  2);
+    col[ 3*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  3);
+    col[ 4*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  4);
+    col[ 5*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  5);
+    col[ 6*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  6);
+    col[ 7*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  7);
+    col[ 8*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  8);
+    col[ 9*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  9);
+    col[10*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 10);
+    col[11*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 11);
+    col[12*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 12);
+    col[13*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 13);
+    col[14*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 14);
+    col[15*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 15);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_table_scan_avx2_256_64
-#define PNAME parasail_sw_table_scan_profile_avx2_256_64
+#define FNAME parasail_sw_table_scan_avx2_256_16
+#define PNAME parasail_sw_table_scan_profile_avx2_256_16
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_rowcol_scan_avx2_256_64
-#define PNAME parasail_sw_rowcol_scan_profile_avx2_256_64
+#define FNAME parasail_sw_rowcol_scan_avx2_256_16
+#define PNAME parasail_sw_rowcol_scan_profile_avx2_256_16
 #else
-#define FNAME parasail_sw_scan_avx2_256_64
-#define PNAME parasail_sw_scan_profile_avx2_256_64
+#define FNAME parasail_sw_scan_avx2_256_16
+#define PNAME parasail_sw_scan_profile_avx2_256_16
 #endif
 #endif
 
@@ -151,7 +140,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_avx_256_64(s1, s1Len, matrix);
+    profile = parasail_profile_create_avx_256_16(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -186,10 +175,10 @@ parasail_result_t* PNAME(
     __m256i* restrict pvGapper = NULL;
     __m256i vGapO;
     __m256i vGapE;
-    int64_t NEG_LIMIT = 0;
-    int64_t POS_LIMIT = 0;
+    int16_t NEG_LIMIT = 0;
+    int16_t POS_LIMIT = 0;
     __m256i vZero;
-    int64_t score = 0;
+    int16_t score = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -202,7 +191,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->profile16.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -217,29 +206,29 @@ parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 4; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
 #ifdef PARASAIL_ROWCOL
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
 #endif
-    pvP = (__m256i*)profile->profile64.score;
-    vGapO = _mm256_set1_epi64x_rpl(open);
-    vGapE = _mm256_set1_epi64x_rpl(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    pvP = (__m256i*)profile->profile16.score;
+    vGapO = _mm256_set1_epi16(open);
+    vGapE = _mm256_set1_epi16(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
+    POS_LIMIT = INT16_MAX - matrix->max - 1;
     vZero = _mm256_setzero_si256();
     score = NEG_LIMIT;
-    vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi16(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi16(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
     vMaxHUnit = vNegLimit;
     vNegInfFront = vZero;
-    vNegInfFront = _mm256_insert_epi64_rpl(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm256_add_epi64(vNegInfFront,
-            _mm256_slli_si256_rpl(_mm256_set1_epi64x_rpl(-segLen*gap), 8));
+    vNegInfFront = _mm256_insert_epi16_rpl(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm256_adds_epi16(vNegInfFront,
+            _mm256_slli_si256_rpl(_mm256_set1_epi16(-segLen*gap), 2));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -255,7 +244,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_SCAN
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_16;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -280,10 +269,10 @@ parasail_result_t* PNAME(
     parasail_memset___m256i(pvH, vZero, segLen);
     parasail_memset___m256i(pvE, vNegLimit, segLen);
     {
-        __m256i vGapper = _mm256_sub_epi64(vZero,vGapO);
+        __m256i vGapper = _mm256_subs_epi16(vZero,vGapO);
         for (i=segLen-1; i>=0; --i) {
             _mm256_store_si256(pvGapper+i, vGapper);
-            vGapper = _mm256_sub_epi64(vGapper, vGapE);
+            vGapper = _mm256_subs_epi16(vGapper, vGapE);
         }
     }
 
@@ -301,7 +290,7 @@ parasail_result_t* PNAME(
         /* calculate Ht */
         /* calculate F and H first pass */
         vHp = _mm256_load_si256(pvH+(segLen-1));
-        vHp = _mm256_slli_si256_rpl(vHp, 8);
+        vHp = _mm256_slli_si256_rpl(vHp, 2);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         vHt = vZero;
         vF = vNegLimit;
@@ -309,52 +298,52 @@ parasail_result_t* PNAME(
             vH = _mm256_load_si256(pvH+i);
             vE = _mm256_load_si256(pvE+i);
             vW = _mm256_load_si256(pvW+i);
-            vE = _mm256_max_epi64_rpl(
-                    _mm256_sub_epi64(vE, vGapE),
-                    _mm256_sub_epi64(vH, vGapO));
-            vHp = _mm256_add_epi64(vHp, vW);
-            vF = _mm256_max_epi64_rpl(vF, _mm256_add_epi64(vHt, pvGapper[i]));
-            vHt = _mm256_max_epi64_rpl(vE, vHp);
+            vE = _mm256_max_epi16(
+                    _mm256_subs_epi16(vE, vGapE),
+                    _mm256_subs_epi16(vH, vGapO));
+            vHp = _mm256_adds_epi16(vHp, vW);
+            vF = _mm256_max_epi16(vF, _mm256_adds_epi16(vHt, pvGapper[i]));
+            vHt = _mm256_max_epi16(vE, vHp);
             _mm256_store_si256(pvE+i, vE);
             _mm256_store_si256(pvHt+i, vHt);
             vHp = vH;
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm256_slli_si256_rpl(vHt, 8);
-        vF = _mm256_max_epi64_rpl(vF, _mm256_add_epi64(vHt, pvGapper[0]));
+        vHt = _mm256_slli_si256_rpl(vHt, 2);
+        vF = _mm256_max_epi16(vF, _mm256_adds_epi16(vHt, pvGapper[0]));
         for (i=0; i<segWidth-2; ++i) {
-            __m256i vFt = _mm256_slli_si256_rpl(vF, 8);
-            vFt = _mm256_add_epi64(vFt, vSegLenXgap);
-            vF = _mm256_max_epi64_rpl(vF, vFt);
+            __m256i vFt = _mm256_slli_si256_rpl(vF, 2);
+            vFt = _mm256_adds_epi16(vFt, vSegLenXgap);
+            vF = _mm256_max_epi16(vF, vFt);
         }
 
         /* calculate final H */
-        vF = _mm256_slli_si256_rpl(vF, 8);
-        vF = _mm256_add_epi64(vF, vNegInfFront);
-        vH = _mm256_max_epi64_rpl(vHt, vF);
-        vH = _mm256_max_epi64_rpl(vH, vZero);
+        vF = _mm256_slli_si256_rpl(vF, 2);
+        vF = _mm256_adds_epi16(vF, vNegInfFront);
+        vH = _mm256_max_epi16(vHt, vF);
+        vH = _mm256_max_epi16(vH, vZero);
         for (i=0; i<segLen; ++i) {
             vHt = _mm256_load_si256(pvHt+i);
-            vF = _mm256_max_epi64_rpl(
-                    _mm256_sub_epi64(vF, vGapE),
-                    _mm256_sub_epi64(vH, vGapO));
-            vH = _mm256_max_epi64_rpl(vHt, vF);
-            vH = _mm256_max_epi64_rpl(vH, vZero);
+            vF = _mm256_max_epi16(
+                    _mm256_subs_epi16(vF, vGapE),
+                    _mm256_subs_epi16(vH, vGapO));
+            vH = _mm256_max_epi16(vHt, vF);
+            vH = _mm256_max_epi16(vH, vZero);
             _mm256_store_si256(pvH+i, vH);
-            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vH);
 #ifdef PARASAIL_TABLE
             arr_store_si256(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-            vMaxH = _mm256_max_epi64_rpl(vH, vMaxH);
+            vMaxH = _mm256_max_epi16(vH, vMaxH);
         } 
 
         {
-            __m256i vCompare = _mm256_cmpgt_epi64(vMaxH, vMaxHUnit);
+            __m256i vCompare = _mm256_cmpgt_epi16(vMaxH, vMaxHUnit);
             if (_mm256_movemask_epi8(vCompare)) {
-                score = _mm256_hmax_epi64_rpl(vMaxH);
-                vMaxHUnit = _mm256_set1_epi64x_rpl(score);
+                score = _mm256_hmax_epi16_rpl(vMaxH);
+                vMaxHUnit = _mm256_set1_epi16(score);
                 end_ref = j;
                 (void)memcpy(pvHMax, pvH, sizeof(__m256i)*segLen);
             }
@@ -366,16 +355,16 @@ parasail_result_t* PNAME(
             int32_t k = 0;
             __m256i vH = _mm256_load_si256(pvH + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm256_slli_si256_rpl(vH, 8);
+                vH = _mm256_slli_si256_rpl(vH, 2);
             }
-            result->rowcols->score_row[j] = (int64_t) _mm256_extract_epi64_rpl (vH, 3);
+            result->rowcols->score_row[j] = (int16_t) _mm256_extract_epi16_rpl (vH, 15);
         }
 #endif
     }
 
     /* Trace the alignment ending position on read. */
     {
-        int64_t *t = (int64_t*)pvHMax;
+        int16_t *t = (int16_t*)pvHMax;
         int32_t column_len = segLen * segWidth;
         end_query = s1Len;
         for (i = 0; i<column_len; ++i, ++t) {
@@ -396,8 +385,8 @@ parasail_result_t* PNAME(
 #endif
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi16_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
