@@ -28,10 +28,22 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 3);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  0);
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  1);
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  2);
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  3);
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  4);
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  5);
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  6);
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  7);
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  8);
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  9);
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 10);
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 11);
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 12);
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 13);
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 14);
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 15);
 }
 #endif
 
@@ -42,23 +54,35 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int32_t)_mm_extract_epi32(vH, 0);
-    col[1*seglen+t] = (int32_t)_mm_extract_epi32(vH, 1);
-    col[2*seglen+t] = (int32_t)_mm_extract_epi32(vH, 2);
-    col[3*seglen+t] = (int32_t)_mm_extract_epi32(vH, 3);
+    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8(vH,  0);
+    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8(vH,  1);
+    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8(vH,  2);
+    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8(vH,  3);
+    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8(vH,  4);
+    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8(vH,  5);
+    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8(vH,  6);
+    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8(vH,  7);
+    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8(vH,  8);
+    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8(vH,  9);
+    col[10*seglen+t] = (int8_t)_mm_extract_epi8(vH, 10);
+    col[11*seglen+t] = (int8_t)_mm_extract_epi8(vH, 11);
+    col[12*seglen+t] = (int8_t)_mm_extract_epi8(vH, 12);
+    col[13*seglen+t] = (int8_t)_mm_extract_epi8(vH, 13);
+    col[14*seglen+t] = (int8_t)_mm_extract_epi8(vH, 14);
+    col[15*seglen+t] = (int8_t)_mm_extract_epi8(vH, 15);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_stats_table_scan_altivec_128_32
-#define PNAME parasail_sw_stats_table_scan_profile_altivec_128_32
+#define FNAME parasail_sw_stats_table_scan_altivec_128_8
+#define PNAME parasail_sw_stats_table_scan_profile_altivec_128_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_stats_rowcol_scan_altivec_128_32
-#define PNAME parasail_sw_stats_rowcol_scan_profile_altivec_128_32
+#define FNAME parasail_sw_stats_rowcol_scan_altivec_128_8
+#define PNAME parasail_sw_stats_rowcol_scan_profile_altivec_128_8
 #else
-#define FNAME parasail_sw_stats_scan_altivec_128_32
-#define PNAME parasail_sw_stats_scan_profile_altivec_128_32
+#define FNAME parasail_sw_stats_scan_altivec_128_8
+#define PNAME parasail_sw_stats_scan_profile_altivec_128_8
 #endif
 #endif
 
@@ -86,7 +110,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_altivec_128_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_altivec_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -132,14 +156,14 @@ parasail_result_t* PNAME(
     vec128i* restrict pvGapperL = NULL;
     vec128i vGapO;
     vec128i vGapE;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
     vec128i vZero;
     vec128i vOne;
-    int32_t score = 0;
-    int32_t matches = 0;
-    int32_t similar = 0;
-    int32_t length = 0;
+    int8_t score = 0;
+    int8_t matches = 0;
+    int8_t similar = 0;
+    int8_t length = 0;
     vec128i vNegLimit;
     vec128i vPosLimit;
     vec128i vSaturationCheckMin;
@@ -156,7 +180,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -171,27 +195,27 @@ parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 4; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
 #ifdef PARASAIL_ROWCOL
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
 #endif
-    pvP  = (vec128i*)profile->profile32.score;
-    pvPm = (vec128i*)profile->profile32.matches;
-    pvPs = (vec128i*)profile->profile32.similar;
-    vGapO = _mm_set1_epi32(open);
-    vGapE = _mm_set1_epi32(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    pvP  = (vec128i*)profile->profile8.score;
+    pvPm = (vec128i*)profile->profile8.matches;
+    pvPs = (vec128i*)profile->profile8.similar;
+    vGapO = _mm_set1_epi8(open);
+    vGapE = _mm_set1_epi8(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
     vZero = _mm_setzero_si128();
-    vOne = _mm_set1_epi32(1);
+    vOne = _mm_set1_epi8(1);
     score = NEG_LIMIT;
     matches = 0;
     similar = 0;
     length = 0;
-    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm_set1_epi8(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi8(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
@@ -199,11 +223,11 @@ parasail_result_t* PNAME(
     vMaxS = vNegLimit;
     vMaxL = vNegLimit;
     vMaxHUnit = vNegLimit;
-    vSegLen = _mm_slli_si128(_mm_set1_epi32(segLen), 4);
+    vSegLen = _mm_slli_si128(_mm_set1_epi8(segLen), 1);
     vNegInfFront = vZero;
-    vNegInfFront = _mm_insert_epi32(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm_add_epi32(vNegInfFront,
-            _mm_slli_si128(_mm_set1_epi32(-segLen*gap), 4));
+    vNegInfFront = _mm_insert_epi8(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm_adds_epi8(vNegInfFront,
+            _mm_slli_si128(_mm_set1_epi8(-segLen*gap), 1));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -220,7 +244,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -269,13 +293,13 @@ parasail_result_t* PNAME(
     parasail_memset_vec128i(pvES, vZero, segLen);
     parasail_memset_vec128i(pvEL, vZero, segLen);
     {
-        vec128i vGapper = _mm_sub_epi32(vZero,vGapO);
+        vec128i vGapper = _mm_subs_epi8(vZero,vGapO);
         vec128i vGapperL = vOne;
         for (i=segLen-1; i>=0; --i) {
             _mm_store_si128(pvGapper+i, vGapper);
             _mm_store_si128(pvGapperL+i, vGapperL);
-            vGapper = _mm_sub_epi32(vGapper, vGapE);
-            vGapperL = _mm_add_epi32(vGapperL, vOne);
+            vGapper = _mm_subs_epi8(vGapper, vGapE);
+            vGapperL = _mm_adds_epi8(vGapperL, vOne);
         }
     }
 
@@ -324,10 +348,10 @@ parasail_result_t* PNAME(
         vHpM = _mm_load_si128(pvHM+(segLen-1));
         vHpS = _mm_load_si128(pvHS+(segLen-1));
         vHpL = _mm_load_si128(pvHL+(segLen-1));
-        vHp = _mm_slli_si128(vHp, 4);
-        vHpM = _mm_slli_si128(vHpM, 4);
-        vHpS = _mm_slli_si128(vHpS, 4);
-        vHpL = _mm_slli_si128(vHpL, 4);
+        vHp = _mm_slli_si128(vHp, 1);
+        vHpM = _mm_slli_si128(vHpM, 1);
+        vHpS = _mm_slli_si128(vHpS, 1);
+        vHpL = _mm_slli_si128(vHpL, 1);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWM= pvPm+ matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWS= pvPs+ matrix->mapper[(unsigned char)s2[j]]*segLen;
@@ -350,30 +374,30 @@ parasail_result_t* PNAME(
             vWS = _mm_load_si128(pvWS+i);
             vGapper = _mm_load_si128(pvGapper+i);
             vGapperL = _mm_load_si128(pvGapperL+i);
-            vE_opn = _mm_sub_epi32(vH, vGapO);
-            vE_ext = _mm_sub_epi32(vE, vGapE);
-            case1 = _mm_cmpgt_epi32(vE_opn, vE_ext);
-            vE = _mm_max_epi32(vE_opn, vE_ext);
+            vE_opn = _mm_subs_epi8(vH, vGapO);
+            vE_ext = _mm_subs_epi8(vE, vGapE);
+            case1 = _mm_cmpgt_epi8(vE_opn, vE_ext);
+            vE = _mm_max_epi8(vE_opn, vE_ext);
             vEM = _mm_blendv_epi8(vEM, vHM, case1);
             vES = _mm_blendv_epi8(vES, vHS, case1);
             vEL = _mm_blendv_epi8(vEL, vHL, case1);
-            vEL = _mm_add_epi32(vEL, vOne);
-            vGapper = _mm_add_epi32(vHt, vGapper);
+            vEL = _mm_adds_epi8(vEL, vOne);
+            vGapper = _mm_adds_epi8(vHt, vGapper);
             case1 = _mm_or_si128(
-                    _mm_cmpgt_epi32(vF, vGapper),
-                    _mm_cmpeq_epi32(vF, vGapper));
-            vF = _mm_max_epi32(vF, vGapper);
+                    _mm_cmpgt_epi8(vF, vGapper),
+                    _mm_cmpeq_epi8(vF, vGapper));
+            vF = _mm_max_epi8(vF, vGapper);
             vFM = _mm_blendv_epi8(vHtM, vFM, case1);
             vFS = _mm_blendv_epi8(vHtS, vFS, case1);
             vFL = _mm_blendv_epi8(
-                    _mm_add_epi32(vHtL, vGapperL),
+                    _mm_adds_epi8(vHtL, vGapperL),
                     vFL, case1);
-            vHp = _mm_add_epi32(vHp, vW);
-            vHpM = _mm_add_epi32(vHpM, vWM);
-            vHpS = _mm_add_epi32(vHpS, vWS);
-            vHpL = _mm_add_epi32(vHpL, vOne);
-            case1 = _mm_cmpgt_epi32(vE, vHp);
-            vHt = _mm_max_epi32(vE, vHp);
+            vHp = _mm_adds_epi8(vHp, vW);
+            vHpM = _mm_adds_epi8(vHpM, vWM);
+            vHpS = _mm_adds_epi8(vHpS, vWS);
+            vHpL = _mm_adds_epi8(vHpL, vOne);
+            case1 = _mm_cmpgt_epi8(vE, vHp);
+            vHt = _mm_max_epi8(vE, vHp);
             vHtM = _mm_blendv_epi8(vHpM, vEM, case1);
             vHtS = _mm_blendv_epi8(vHpS, vES, case1);
             vHtL = _mm_blendv_epi8(vHpL, vEL, case1);
@@ -392,49 +416,49 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm_slli_si128(vHt, 4);
-        vHtM = _mm_slli_si128(vHtM, 4);
-        vHtS = _mm_slli_si128(vHtS, 4);
-        vHtL = _mm_slli_si128(vHtL, 4);
+        vHt = _mm_slli_si128(vHt, 1);
+        vHtM = _mm_slli_si128(vHtM, 1);
+        vHtS = _mm_slli_si128(vHtS, 1);
+        vHtL = _mm_slli_si128(vHtL, 1);
         vGapper = _mm_load_si128(pvGapper);
         vGapperL = _mm_load_si128(pvGapperL);
-        vGapper = _mm_add_epi32(vHt, vGapper);
+        vGapper = _mm_adds_epi8(vHt, vGapper);
         case1 = _mm_or_si128(
-                _mm_cmpgt_epi32(vGapper, vF),
-                _mm_cmpeq_epi32(vGapper, vF));
-        vF = _mm_max_epi32(vF, vGapper);
+                _mm_cmpgt_epi8(vGapper, vF),
+                _mm_cmpeq_epi8(vGapper, vF));
+        vF = _mm_max_epi8(vF, vGapper);
         vFM = _mm_blendv_epi8(vFM, vHtM, case1);
         vFS = _mm_blendv_epi8(vFS, vHtS, case1);
         vFL = _mm_blendv_epi8(
                 vFL,
-                _mm_add_epi32(vHtL, vGapperL),
+                _mm_adds_epi8(vHtL, vGapperL),
                 case1);
         for (i=0; i<segWidth-2; ++i) {
-            vec128i vFt = _mm_slli_si128(vF, 4);
-            vec128i vFtM = _mm_slli_si128(vFM, 4);
-            vec128i vFtS = _mm_slli_si128(vFS, 4);
-            vec128i vFtL = _mm_slli_si128(vFL, 4);
-            vFt = _mm_add_epi32(vFt, vSegLenXgap);
+            vec128i vFt = _mm_slli_si128(vF, 1);
+            vec128i vFtM = _mm_slli_si128(vFM, 1);
+            vec128i vFtS = _mm_slli_si128(vFS, 1);
+            vec128i vFtL = _mm_slli_si128(vFL, 1);
+            vFt = _mm_adds_epi8(vFt, vSegLenXgap);
             case1 = _mm_or_si128(
-                    _mm_cmpgt_epi32(vFt, vF),
-                    _mm_cmpeq_epi32(vFt, vF));
-            vF = _mm_max_epi32(vF, vFt);
+                    _mm_cmpgt_epi8(vFt, vF),
+                    _mm_cmpeq_epi8(vFt, vF));
+            vF = _mm_max_epi8(vF, vFt);
             vFM = _mm_blendv_epi8(vFM, vFtM, case1);
             vFS = _mm_blendv_epi8(vFS, vFtS, case1);
             vFL = _mm_blendv_epi8(
                     vFL,
-                    _mm_add_epi32(vFtL, vSegLen),
+                    _mm_adds_epi8(vFtL, vSegLen),
                     case1);
         }
 
         /* calculate final H */
-        vF = _mm_slli_si128(vF, 4);
-        vFM = _mm_slli_si128(vFM, 4);
-        vFS = _mm_slli_si128(vFS, 4);
-        vFL = _mm_slli_si128(vFL, 4);
-        vF = _mm_add_epi32(vF, vNegInfFront);
-        case1 = _mm_cmpgt_epi32(vF, vHt);
-        vH = _mm_max_epi32(vF, vHt);
+        vF = _mm_slli_si128(vF, 1);
+        vFM = _mm_slli_si128(vFM, 1);
+        vFS = _mm_slli_si128(vFS, 1);
+        vFL = _mm_slli_si128(vFL, 1);
+        vF = _mm_adds_epi8(vF, vNegInfFront);
+        case1 = _mm_cmpgt_epi8(vF, vHt);
+        vH = _mm_max_epi8(vF, vHt);
         vHM = _mm_blendv_epi8(vHtM, vFM, case1);
         vHS = _mm_blendv_epi8(vHtS, vFS, case1);
         vHL = _mm_blendv_epi8(vHtL, vFL, case1);
@@ -447,20 +471,20 @@ parasail_result_t* PNAME(
             vEM = _mm_load_si128(pvEM+i);
             vES = _mm_load_si128(pvES+i);
             vEL = _mm_load_si128(pvEL+i);
-            vF_opn = _mm_sub_epi32(vH, vGapO);
-            vF_ext = _mm_sub_epi32(vF, vGapE);
-            vF = _mm_max_epi32(vF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi32(vF_opn, vF_ext);
+            vF_opn = _mm_subs_epi8(vH, vGapO);
+            vF_ext = _mm_subs_epi8(vF, vGapE);
+            vF = _mm_max_epi8(vF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi8(vF_opn, vF_ext);
             vFM = _mm_blendv_epi8(vFM, vHM, case1);
             vFS = _mm_blendv_epi8(vFS, vHS, case1);
             vFL = _mm_blendv_epi8(vFL, vHL, case1);
-            vFL = _mm_add_epi32(vFL, vOne);
-            vH = _mm_max_epi32(vHp, vE);
-            vH = _mm_max_epi32(vH, vF);
-            vH = _mm_max_epi32(vH, vZero);
-            case1 = _mm_cmpeq_epi32(vH, vHp);
-            case2 = _mm_cmpeq_epi32(vH, vF);
-            case0 = _mm_cmpeq_epi32(vH, vZero);
+            vFL = _mm_adds_epi8(vFL, vOne);
+            vH = _mm_max_epi8(vHp, vE);
+            vH = _mm_max_epi8(vH, vF);
+            vH = _mm_max_epi8(vH, vZero);
+            case1 = _mm_cmpeq_epi8(vH, vHp);
+            case2 = _mm_cmpeq_epi8(vH, vF);
+            case0 = _mm_cmpeq_epi8(vH, vZero);
             vHM = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEM, vFM, case2),
                     vHpM, case1);
@@ -477,11 +501,11 @@ parasail_result_t* PNAME(
             _mm_store_si128(pvHM+i, vHM);
             _mm_store_si128(pvHS+i, vHS);
             _mm_store_si128(pvHL+i, vHL);
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
@@ -489,8 +513,8 @@ parasail_result_t* PNAME(
             arr_store_si128(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
 #endif
             {
-                vec128i cond_max = _mm_cmpgt_epi32(vH, vMaxH);
-                vMaxH = _mm_max_epi32(vH, vMaxH);
+                vec128i cond_max = _mm_cmpgt_epi8(vH, vMaxH);
+                vMaxH = _mm_max_epi8(vH, vMaxH);
                 vMaxM = _mm_blendv_epi8(vMaxM, vHM, cond_max);
                 vMaxS = _mm_blendv_epi8(vMaxS, vHS, cond_max);
                 vMaxL = _mm_blendv_epi8(vMaxL, vHL, cond_max);
@@ -498,10 +522,10 @@ parasail_result_t* PNAME(
         } 
 
         {
-            vec128i vCompare = _mm_cmpgt_epi32(vMaxH, vMaxHUnit);
+            vec128i vCompare = _mm_cmpgt_epi8(vMaxH, vMaxHUnit);
             if (_mm_movemask_epi8(vCompare)) {
-                score = _mm_hmax_epi32(vMaxH);
-                vMaxHUnit = _mm_set1_epi32(score);
+                score = _mm_hmax_epi8(vMaxH);
+                vMaxHUnit = _mm_set1_epi8(score);
                 end_ref = j;
                 (void)memcpy(pvHMax, pvH, sizeof(vec128i)*segLen);
                 (void)memcpy(pvHMMax, pvHM, sizeof(vec128i)*segLen);
@@ -519,25 +543,25 @@ parasail_result_t* PNAME(
             vec128i vHS = _mm_load_si128(pvHS + offset);
             vec128i vHL = _mm_load_si128(pvHL + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 4);
-                vHM = _mm_slli_si128(vHM, 4);
-                vHS = _mm_slli_si128(vHS, 4);
-                vHL = _mm_slli_si128(vHL, 4);
+                vH = _mm_slli_si128(vH, 1);
+                vHM = _mm_slli_si128(vHM, 1);
+                vHS = _mm_slli_si128(vHS, 1);
+                vHL = _mm_slli_si128(vHL, 1);
             }
-            result->stats->rowcols->score_row[j] = (int32_t) _mm_extract_epi32 (vH, 3);
-            result->stats->rowcols->matches_row[j] = (int32_t) _mm_extract_epi32 (vHM, 3);
-            result->stats->rowcols->similar_row[j] = (int32_t) _mm_extract_epi32 (vHS, 3);
-            result->stats->rowcols->length_row[j] = (int32_t) _mm_extract_epi32 (vHL, 3);
+            result->stats->rowcols->score_row[j] = (int8_t) _mm_extract_epi8 (vH, 15);
+            result->stats->rowcols->matches_row[j] = (int8_t) _mm_extract_epi8 (vHM, 15);
+            result->stats->rowcols->similar_row[j] = (int8_t) _mm_extract_epi8 (vHS, 15);
+            result->stats->rowcols->length_row[j] = (int8_t) _mm_extract_epi8 (vHL, 15);
         }
 #endif
     }
 
     /* Trace the alignment ending position on read. */
     {
-        int32_t *t = (int32_t*)pvHMax;
-        int32_t *m = (int32_t*)pvHMMax;
-        int32_t *s = (int32_t*)pvHSMax;
-        int32_t *l = (int32_t*)pvHLMax;
+        int8_t *t = (int8_t*)pvHMax;
+        int8_t *m = (int8_t*)pvHMMax;
+        int8_t *s = (int8_t*)pvHSMax;
+        int8_t *l = (int8_t*)pvHLMax;
         int32_t column_len = segLen * segWidth;
         end_query = s1Len;
         for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
@@ -567,8 +591,8 @@ parasail_result_t* PNAME(
 #endif
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
