@@ -28,8 +28,10 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)simde_mm_extract_epi64(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)simde_mm_extract_epi64(vH, 1);
+    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 3);
 }
 #endif
 
@@ -40,21 +42,23 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int64_t)simde_mm_extract_epi64(vH, 0);
-    col[1*seglen+t] = (int64_t)simde_mm_extract_epi64(vH, 1);
+    col[0*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 0);
+    col[1*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 1);
+    col[2*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 2);
+    col[3*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 3);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_stats_table_scan_neon_128_64
-#define PNAME parasail_sw_stats_table_scan_profile_neon_128_64
+#define FNAME parasail_sw_stats_table_scan_neon_128_32
+#define PNAME parasail_sw_stats_table_scan_profile_neon_128_32
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_stats_rowcol_scan_neon_128_64
-#define PNAME parasail_sw_stats_rowcol_scan_profile_neon_128_64
+#define FNAME parasail_sw_stats_rowcol_scan_neon_128_32
+#define PNAME parasail_sw_stats_rowcol_scan_profile_neon_128_32
 #else
-#define FNAME parasail_sw_stats_scan_neon_128_64
-#define PNAME parasail_sw_stats_scan_profile_neon_128_64
+#define FNAME parasail_sw_stats_scan_neon_128_32
+#define PNAME parasail_sw_stats_scan_profile_neon_128_32
 #endif
 #endif
 
@@ -63,9 +67,31 @@ parasail_result_t* FNAME(
         const char * const restrict s2, const int s2Len,
         const int open, const int gap, const parasail_matrix_t *matrix)
 {
-    parasail_profile_t *profile = parasail_profile_create_stats_neon_128_64(s1, s1Len, matrix);
-    parasail_result_t *result = PNAME(profile, s2, s2Len, open, gap);
+    /* declare local variables */
+    parasail_profile_t *profile = NULL;
+    parasail_result_t *result = NULL;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+    PARASAIL_CHECK_NULL(matrix);
+    if (matrix->type == PARASAIL_MATRIX_TYPE_PSSM) {
+        PARASAIL_CHECK_NULL_PSSM_STATS(s1);
+    }
+    else {
+        PARASAIL_CHECK_NULL(s1);
+        PARASAIL_CHECK_GT0(s1Len);
+    }
+
+    /* initialize local variables */
+    profile = parasail_profile_create_stats_neon_128_32(s1, s1Len, matrix);
+    if (!profile) return NULL;
+    result = PNAME(profile, s2, s2Len, open, gap);
+
     parasail_profile_free(profile);
+
     return result;
 }
 
@@ -74,69 +100,165 @@ parasail_result_t* PNAME(
         const char * const restrict s2, const int s2Len,
         const int open, const int gap)
 {
+    /* declare local variables */
     int32_t i = 0;
     int32_t j = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    const int s1Len = profile->s1Len;
-    const parasail_matrix_t *matrix = profile->matrix;
-    const int32_t segWidth = 2; /* number of values in vector unit */
-    const int32_t segLen = (s1Len + segWidth - 1) / segWidth;
-    simde__m128i* const restrict pvP  = (simde__m128i*)profile->profile64.score;
-    simde__m128i* const restrict pvPm = (simde__m128i*)profile->profile64.matches;
-    simde__m128i* const restrict pvPs = (simde__m128i*)profile->profile64.similar;
-    simde__m128i* const restrict pvE  = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvEM = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvES = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvEL = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvH  = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvHM = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvHS = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvHL = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvHMax  = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvHMMax = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvHSMax = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvHLMax = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvGapper = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvGapperL = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i vGapO = simde_mm_set1_epi64x(open);
-    simde__m128i vGapE = simde_mm_set1_epi64x(gap);
-    const int64_t NEG_LIMIT = (-open < matrix->min ?
-        INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    const int64_t POS_LIMIT = INT64_MAX - matrix->max - 1;
-    simde__m128i vZero = simde_mm_setzero_si128();
-    simde__m128i vOne = simde_mm_set1_epi64x(1);
-    int64_t score = NEG_LIMIT;
-    int64_t matches = 0;
-    int64_t similar = 0;
-    int64_t length = 0;
-    simde__m128i vNegLimit = simde_mm_set1_epi64x(NEG_LIMIT);
-    simde__m128i vPosLimit = simde_mm_set1_epi64x(POS_LIMIT);
-    simde__m128i vSaturationCheckMin = vPosLimit;
-    simde__m128i vSaturationCheckMax = vNegLimit;
-    simde__m128i vMaxH = vNegLimit;
-    simde__m128i vMaxM = vNegLimit;
-    simde__m128i vMaxS = vNegLimit;
-    simde__m128i vMaxL = vNegLimit;
-    simde__m128i vMaxHUnit = vNegLimit;
-    simde__m128i vNegInfFront = vZero;
+    int32_t s1Len = 0;
+    const parasail_matrix_t *matrix = NULL;
+    int32_t segWidth = 0;
+    int32_t segLen = 0;
+#ifdef PARASAIL_ROWCOL
+    int32_t offset = 0;
+    int32_t position = 0;
+#endif
+    simde__m128i* restrict pvP = NULL;
+    simde__m128i* restrict pvPm = NULL;
+    simde__m128i* restrict pvPs = NULL;
+    simde__m128i* restrict pvE = NULL;
+    simde__m128i* restrict pvEM = NULL;
+    simde__m128i* restrict pvES = NULL;
+    simde__m128i* restrict pvEL = NULL;
+    simde__m128i* restrict pvH = NULL;
+    simde__m128i* restrict pvHM = NULL;
+    simde__m128i* restrict pvHS = NULL;
+    simde__m128i* restrict pvHL = NULL;
+    simde__m128i* restrict pvHMax = NULL;
+    simde__m128i* restrict pvHMMax = NULL;
+    simde__m128i* restrict pvHSMax = NULL;
+    simde__m128i* restrict pvHLMax = NULL;
+    simde__m128i* restrict pvGapper = NULL;
+    simde__m128i* restrict pvGapperL = NULL;
+    simde__m128i vGapO;
+    simde__m128i vGapE;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
+    simde__m128i vZero;
+    simde__m128i vOne;
+    int32_t score = 0;
+    int32_t matches = 0;
+    int32_t similar = 0;
+    int32_t length = 0;
+    simde__m128i vNegLimit;
+    simde__m128i vPosLimit;
+    simde__m128i vSaturationCheckMin;
+    simde__m128i vSaturationCheckMax;
+    simde__m128i vMaxH;
+    simde__m128i vMaxM;
+    simde__m128i vMaxS;
+    simde__m128i vMaxL;
+    simde__m128i vMaxHUnit;
+    simde__m128i vNegInfFront;
     simde__m128i vSegLenXgap;
-    simde__m128i vSegLen = simde_mm_slli_si128(simde_mm_set1_epi64x(segLen), 8);
+    simde__m128i vSegLen;
+    parasail_result_t *result = NULL;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(profile);
+    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->matrix);
+    PARASAIL_CHECK_GT0(profile->s1Len);
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+
+    /* initialize stack variables */
+    i = 0;
+    j = 0;
+    end_query = 0;
+    end_ref = 0;
+    s1Len = profile->s1Len;
+    matrix = profile->matrix;
+    segWidth = 4; /* number of values in vector unit */
+    segLen = (s1Len + segWidth - 1) / segWidth;
+#ifdef PARASAIL_ROWCOL
+    offset = (s1Len - 1) % segLen;
+    position = (segWidth - 1) - (s1Len - 1) / segLen;
+#endif
+    pvP  = (simde__m128i*)profile->profile32.score;
+    pvPm = (simde__m128i*)profile->profile32.matches;
+    pvPs = (simde__m128i*)profile->profile32.similar;
+    vGapO = simde_mm_set1_epi32(open);
+    vGapE = simde_mm_set1_epi32(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    vZero = simde_mm_setzero_si128();
+    vOne = simde_mm_set1_epi32(1);
+    score = NEG_LIMIT;
+    matches = 0;
+    similar = 0;
+    length = 0;
+    vNegLimit = simde_mm_set1_epi32(NEG_LIMIT);
+    vPosLimit = simde_mm_set1_epi32(POS_LIMIT);
+    vSaturationCheckMin = vPosLimit;
+    vSaturationCheckMax = vNegLimit;
+    vMaxH = vNegLimit;
+    vMaxM = vNegLimit;
+    vMaxS = vNegLimit;
+    vMaxL = vNegLimit;
+    vMaxHUnit = vNegLimit;
+    vSegLen = simde_mm_slli_si128(simde_mm_set1_epi32(segLen), 4);
+    vNegInfFront = vZero;
+    vNegInfFront = simde_mm_insert_epi32(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = simde_mm_add_epi32(vNegInfFront,
+            simde_mm_slli_si128(simde_mm_set1_epi32(-segLen*gap), 4));
+
+    /* initialize result */
 #ifdef PARASAIL_TABLE
-    parasail_result_t *result = parasail_result_new_table3(segLen*segWidth, s2Len);
+    result = parasail_result_new_table3(segLen*segWidth, s2Len);
 #else
 #ifdef PARASAIL_ROWCOL
-    parasail_result_t *result = parasail_result_new_rowcol3(segLen*segWidth, s2Len);
-    const int32_t offset = (s1Len - 1) % segLen;
-    const int32_t position = (segWidth - 1) - (s1Len - 1) / segLen;
+    result = parasail_result_new_rowcol3(segLen*segWidth, s2Len);
 #else
-    parasail_result_t *result = parasail_result_new_stats();
+    result = parasail_result_new_stats();
 #endif
+#endif
+    if (!result) return NULL;
+
+    /* set known flags */
+    result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_SCAN
+        | PARASAIL_FLAG_STATS
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+#ifdef PARASAIL_TABLE
+    result->flag |= PARASAIL_FLAG_TABLE;
+#endif
+#ifdef PARASAIL_ROWCOL
+    result->flag |= PARASAIL_FLAG_ROWCOL;
 #endif
 
-    vNegInfFront = simde_mm_insert_epi64(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = simde_mm_add_epi64(vNegInfFront,
-            simde_mm_slli_si128(simde_mm_set1_epi64x(-segLen*gap), 8));
+    /* initialize heap variables */
+    pvE  = parasail_memalign_simde__m128i(16, segLen);
+    pvEM = parasail_memalign_simde__m128i(16, segLen);
+    pvES = parasail_memalign_simde__m128i(16, segLen);
+    pvEL = parasail_memalign_simde__m128i(16, segLen);
+    pvH  = parasail_memalign_simde__m128i(16, segLen);
+    pvHM = parasail_memalign_simde__m128i(16, segLen);
+    pvHS = parasail_memalign_simde__m128i(16, segLen);
+    pvHL = parasail_memalign_simde__m128i(16, segLen);
+    pvHMax  = parasail_memalign_simde__m128i(16, segLen);
+    pvHMMax = parasail_memalign_simde__m128i(16, segLen);
+    pvHSMax = parasail_memalign_simde__m128i(16, segLen);
+    pvHLMax = parasail_memalign_simde__m128i(16, segLen);
+    pvGapper = parasail_memalign_simde__m128i(16, segLen);
+    pvGapperL = parasail_memalign_simde__m128i(16, segLen);
+
+    /* validate heap variables */
+    if (!pvE) return NULL;
+    if (!pvEM) return NULL;
+    if (!pvES) return NULL;
+    if (!pvEL) return NULL;
+    if (!pvH) return NULL;
+    if (!pvHM) return NULL;
+    if (!pvHS) return NULL;
+    if (!pvHL) return NULL;
+    if (!pvHMax) return NULL;
+    if (!pvHMMax) return NULL;
+    if (!pvHSMax) return NULL;
+    if (!pvHLMax) return NULL;
+    if (!pvGapper) return NULL;
+    if (!pvGapperL) return NULL;
 
     parasail_memset_simde__m128i(pvH, vZero, segLen);
     parasail_memset_simde__m128i(pvHM, vZero, segLen);
@@ -147,13 +269,13 @@ parasail_result_t* PNAME(
     parasail_memset_simde__m128i(pvES, vZero, segLen);
     parasail_memset_simde__m128i(pvEL, vZero, segLen);
     {
-        simde__m128i vGapper = simde_mm_sub_epi64(vZero,vGapO);
+        simde__m128i vGapper = simde_mm_sub_epi32(vZero,vGapO);
         simde__m128i vGapperL = vOne;
         for (i=segLen-1; i>=0; --i) {
             simde_mm_store_si128(pvGapper+i, vGapper);
             simde_mm_store_si128(pvGapperL+i, vGapperL);
-            vGapper = simde_mm_sub_epi64(vGapper, vGapE);
-            vGapperL = simde_mm_add_epi64(vGapperL, vOne);
+            vGapper = simde_mm_sub_epi32(vGapper, vGapE);
+            vGapperL = simde_mm_add_epi32(vGapperL, vOne);
         }
     }
 
@@ -202,10 +324,10 @@ parasail_result_t* PNAME(
         vHpM = simde_mm_load_si128(pvHM+(segLen-1));
         vHpS = simde_mm_load_si128(pvHS+(segLen-1));
         vHpL = simde_mm_load_si128(pvHL+(segLen-1));
-        vHp = simde_mm_slli_si128(vHp, 8);
-        vHpM = simde_mm_slli_si128(vHpM, 8);
-        vHpS = simde_mm_slli_si128(vHpS, 8);
-        vHpL = simde_mm_slli_si128(vHpL, 8);
+        vHp = simde_mm_slli_si128(vHp, 4);
+        vHpM = simde_mm_slli_si128(vHpM, 4);
+        vHpS = simde_mm_slli_si128(vHpS, 4);
+        vHpL = simde_mm_slli_si128(vHpL, 4);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWM= pvPm+ matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWS= pvPs+ matrix->mapper[(unsigned char)s2[j]]*segLen;
@@ -228,30 +350,30 @@ parasail_result_t* PNAME(
             vWS = simde_mm_load_si128(pvWS+i);
             vGapper = simde_mm_load_si128(pvGapper+i);
             vGapperL = simde_mm_load_si128(pvGapperL+i);
-            vE_opn = simde_mm_sub_epi64(vH, vGapO);
-            vE_ext = simde_mm_sub_epi64(vE, vGapE);
-            case1 = simde_mm_cmpgt_epi64(vE_opn, vE_ext);
-            vE = simde_mm_max_epi64(vE_opn, vE_ext);
+            vE_opn = simde_mm_sub_epi32(vH, vGapO);
+            vE_ext = simde_mm_sub_epi32(vE, vGapE);
+            case1 = simde_mm_cmpgt_epi32(vE_opn, vE_ext);
+            vE = simde_mm_max_epi32(vE_opn, vE_ext);
             vEM = simde_mm_blendv_epi8(vEM, vHM, case1);
             vES = simde_mm_blendv_epi8(vES, vHS, case1);
             vEL = simde_mm_blendv_epi8(vEL, vHL, case1);
-            vEL = simde_mm_add_epi64(vEL, vOne);
-            vGapper = simde_mm_add_epi64(vHt, vGapper);
+            vEL = simde_mm_add_epi32(vEL, vOne);
+            vGapper = simde_mm_add_epi32(vHt, vGapper);
             case1 = simde_mm_or_si128(
-                    simde_mm_cmpgt_epi64(vF, vGapper),
-                    simde_mm_cmpeq_epi64(vF, vGapper));
-            vF = simde_mm_max_epi64(vF, vGapper);
+                    simde_mm_cmpgt_epi32(vF, vGapper),
+                    simde_mm_cmpeq_epi32(vF, vGapper));
+            vF = simde_mm_max_epi32(vF, vGapper);
             vFM = simde_mm_blendv_epi8(vHtM, vFM, case1);
             vFS = simde_mm_blendv_epi8(vHtS, vFS, case1);
             vFL = simde_mm_blendv_epi8(
-                    simde_mm_add_epi64(vHtL, vGapperL),
+                    simde_mm_add_epi32(vHtL, vGapperL),
                     vFL, case1);
-            vHp = simde_mm_add_epi64(vHp, vW);
-            vHpM = simde_mm_add_epi64(vHpM, vWM);
-            vHpS = simde_mm_add_epi64(vHpS, vWS);
-            vHpL = simde_mm_add_epi64(vHpL, vOne);
-            case1 = simde_mm_cmpgt_epi64(vE, vHp);
-            vHt = simde_mm_max_epi64(vE, vHp);
+            vHp = simde_mm_add_epi32(vHp, vW);
+            vHpM = simde_mm_add_epi32(vHpM, vWM);
+            vHpS = simde_mm_add_epi32(vHpS, vWS);
+            vHpL = simde_mm_add_epi32(vHpL, vOne);
+            case1 = simde_mm_cmpgt_epi32(vE, vHp);
+            vHt = simde_mm_max_epi32(vE, vHp);
             vHtM = simde_mm_blendv_epi8(vHpM, vEM, case1);
             vHtS = simde_mm_blendv_epi8(vHpS, vES, case1);
             vHtL = simde_mm_blendv_epi8(vHpL, vEL, case1);
@@ -270,49 +392,49 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = simde_mm_slli_si128(vHt, 8);
-        vHtM = simde_mm_slli_si128(vHtM, 8);
-        vHtS = simde_mm_slli_si128(vHtS, 8);
-        vHtL = simde_mm_slli_si128(vHtL, 8);
+        vHt = simde_mm_slli_si128(vHt, 4);
+        vHtM = simde_mm_slli_si128(vHtM, 4);
+        vHtS = simde_mm_slli_si128(vHtS, 4);
+        vHtL = simde_mm_slli_si128(vHtL, 4);
         vGapper = simde_mm_load_si128(pvGapper);
         vGapperL = simde_mm_load_si128(pvGapperL);
-        vGapper = simde_mm_add_epi64(vHt, vGapper);
+        vGapper = simde_mm_add_epi32(vHt, vGapper);
         case1 = simde_mm_or_si128(
-                simde_mm_cmpgt_epi64(vGapper, vF),
-                simde_mm_cmpeq_epi64(vGapper, vF));
-        vF = simde_mm_max_epi64(vF, vGapper);
+                simde_mm_cmpgt_epi32(vGapper, vF),
+                simde_mm_cmpeq_epi32(vGapper, vF));
+        vF = simde_mm_max_epi32(vF, vGapper);
         vFM = simde_mm_blendv_epi8(vFM, vHtM, case1);
         vFS = simde_mm_blendv_epi8(vFS, vHtS, case1);
         vFL = simde_mm_blendv_epi8(
                 vFL,
-                simde_mm_add_epi64(vHtL, vGapperL),
+                simde_mm_add_epi32(vHtL, vGapperL),
                 case1);
         for (i=0; i<segWidth-2; ++i) {
-            simde__m128i vFt = simde_mm_slli_si128(vF, 8);
-            simde__m128i vFtM = simde_mm_slli_si128(vFM, 8);
-            simde__m128i vFtS = simde_mm_slli_si128(vFS, 8);
-            simde__m128i vFtL = simde_mm_slli_si128(vFL, 8);
-            vFt = simde_mm_add_epi64(vFt, vSegLenXgap);
+            simde__m128i vFt = simde_mm_slli_si128(vF, 4);
+            simde__m128i vFtM = simde_mm_slli_si128(vFM, 4);
+            simde__m128i vFtS = simde_mm_slli_si128(vFS, 4);
+            simde__m128i vFtL = simde_mm_slli_si128(vFL, 4);
+            vFt = simde_mm_add_epi32(vFt, vSegLenXgap);
             case1 = simde_mm_or_si128(
-                    simde_mm_cmpgt_epi64(vFt, vF),
-                    simde_mm_cmpeq_epi64(vFt, vF));
-            vF = simde_mm_max_epi64(vF, vFt);
+                    simde_mm_cmpgt_epi32(vFt, vF),
+                    simde_mm_cmpeq_epi32(vFt, vF));
+            vF = simde_mm_max_epi32(vF, vFt);
             vFM = simde_mm_blendv_epi8(vFM, vFtM, case1);
             vFS = simde_mm_blendv_epi8(vFS, vFtS, case1);
             vFL = simde_mm_blendv_epi8(
                     vFL,
-                    simde_mm_add_epi64(vFtL, vSegLen),
+                    simde_mm_add_epi32(vFtL, vSegLen),
                     case1);
         }
 
         /* calculate final H */
-        vF = simde_mm_slli_si128(vF, 8);
-        vFM = simde_mm_slli_si128(vFM, 8);
-        vFS = simde_mm_slli_si128(vFS, 8);
-        vFL = simde_mm_slli_si128(vFL, 8);
-        vF = simde_mm_add_epi64(vF, vNegInfFront);
-        case1 = simde_mm_cmpgt_epi64(vF, vHt);
-        vH = simde_mm_max_epi64(vF, vHt);
+        vF = simde_mm_slli_si128(vF, 4);
+        vFM = simde_mm_slli_si128(vFM, 4);
+        vFS = simde_mm_slli_si128(vFS, 4);
+        vFL = simde_mm_slli_si128(vFL, 4);
+        vF = simde_mm_add_epi32(vF, vNegInfFront);
+        case1 = simde_mm_cmpgt_epi32(vF, vHt);
+        vH = simde_mm_max_epi32(vF, vHt);
         vHM = simde_mm_blendv_epi8(vHtM, vFM, case1);
         vHS = simde_mm_blendv_epi8(vHtS, vFS, case1);
         vHL = simde_mm_blendv_epi8(vHtL, vFL, case1);
@@ -325,20 +447,20 @@ parasail_result_t* PNAME(
             vEM = simde_mm_load_si128(pvEM+i);
             vES = simde_mm_load_si128(pvES+i);
             vEL = simde_mm_load_si128(pvEL+i);
-            vF_opn = simde_mm_sub_epi64(vH, vGapO);
-            vF_ext = simde_mm_sub_epi64(vF, vGapE);
-            vF = simde_mm_max_epi64(vF_opn, vF_ext);
-            case1 = simde_mm_cmpgt_epi64(vF_opn, vF_ext);
+            vF_opn = simde_mm_sub_epi32(vH, vGapO);
+            vF_ext = simde_mm_sub_epi32(vF, vGapE);
+            vF = simde_mm_max_epi32(vF_opn, vF_ext);
+            case1 = simde_mm_cmpgt_epi32(vF_opn, vF_ext);
             vFM = simde_mm_blendv_epi8(vFM, vHM, case1);
             vFS = simde_mm_blendv_epi8(vFS, vHS, case1);
             vFL = simde_mm_blendv_epi8(vFL, vHL, case1);
-            vFL = simde_mm_add_epi64(vFL, vOne);
-            vH = simde_mm_max_epi64(vHp, vE);
-            vH = simde_mm_max_epi64(vH, vF);
-            vH = simde_mm_max_epi64(vH, vZero);
-            case1 = simde_mm_cmpeq_epi64(vH, vHp);
-            case2 = simde_mm_cmpeq_epi64(vH, vF);
-            case0 = simde_mm_cmpeq_epi64(vH, vZero);
+            vFL = simde_mm_add_epi32(vFL, vOne);
+            vH = simde_mm_max_epi32(vHp, vE);
+            vH = simde_mm_max_epi32(vH, vF);
+            vH = simde_mm_max_epi32(vH, vZero);
+            case1 = simde_mm_cmpeq_epi32(vH, vHp);
+            case2 = simde_mm_cmpeq_epi32(vH, vF);
+            case0 = simde_mm_cmpeq_epi32(vH, vZero);
             vHM = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vEM, vFM, case2),
                     vHpM, case1);
@@ -355,11 +477,11 @@ parasail_result_t* PNAME(
             simde_mm_store_si128(pvHM+i, vHM);
             simde_mm_store_si128(pvHS+i, vHS);
             simde_mm_store_si128(pvHL+i, vHL);
-            vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vH);
-            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vH);
-            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = simde_mm_min_epi32(vSaturationCheckMin, vH);
+            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vH);
+            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
@@ -367,8 +489,8 @@ parasail_result_t* PNAME(
             arr_store_si128(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
 #endif
             {
-                simde__m128i cond_max = simde_mm_cmpgt_epi64(vH, vMaxH);
-                vMaxH = simde_mm_max_epi64(vH, vMaxH);
+                simde__m128i cond_max = simde_mm_cmpgt_epi32(vH, vMaxH);
+                vMaxH = simde_mm_max_epi32(vH, vMaxH);
                 vMaxM = simde_mm_blendv_epi8(vMaxM, vHM, cond_max);
                 vMaxS = simde_mm_blendv_epi8(vMaxS, vHS, cond_max);
                 vMaxL = simde_mm_blendv_epi8(vMaxL, vHL, cond_max);
@@ -376,10 +498,10 @@ parasail_result_t* PNAME(
         } 
 
         {
-            simde__m128i vCompare = simde_mm_cmpgt_epi64(vMaxH, vMaxHUnit);
+            simde__m128i vCompare = simde_mm_cmpgt_epi32(vMaxH, vMaxHUnit);
             if (simde_mm_movemask_epi8(vCompare)) {
-                score = simde_mm_hmax_epi64(vMaxH);
-                vMaxHUnit = simde_mm_set1_epi64x(score);
+                score = simde_mm_hmax_epi32(vMaxH);
+                vMaxHUnit = simde_mm_set1_epi32(score);
                 end_ref = j;
                 (void)memcpy(pvHMax, pvH, sizeof(simde__m128i)*segLen);
                 (void)memcpy(pvHMMax, pvHM, sizeof(simde__m128i)*segLen);
@@ -397,25 +519,25 @@ parasail_result_t* PNAME(
             simde__m128i vHS = simde_mm_load_si128(pvHS + offset);
             simde__m128i vHL = simde_mm_load_si128(pvHL + offset);
             for (k=0; k<position; ++k) {
-                vH = simde_mm_slli_si128(vH, 8);
-                vHM = simde_mm_slli_si128(vHM, 8);
-                vHS = simde_mm_slli_si128(vHS, 8);
-                vHL = simde_mm_slli_si128(vHL, 8);
+                vH = simde_mm_slli_si128(vH, 4);
+                vHM = simde_mm_slli_si128(vHM, 4);
+                vHS = simde_mm_slli_si128(vHS, 4);
+                vHL = simde_mm_slli_si128(vHL, 4);
             }
-            result->stats->rowcols->score_row[j] = (int64_t) simde_mm_extract_epi64 (vH, 1);
-            result->stats->rowcols->matches_row[j] = (int64_t) simde_mm_extract_epi64 (vHM, 1);
-            result->stats->rowcols->similar_row[j] = (int64_t) simde_mm_extract_epi64 (vHS, 1);
-            result->stats->rowcols->length_row[j] = (int64_t) simde_mm_extract_epi64 (vHL, 1);
+            result->stats->rowcols->score_row[j] = (int32_t) simde_mm_extract_epi32 (vH, 3);
+            result->stats->rowcols->matches_row[j] = (int32_t) simde_mm_extract_epi32 (vHM, 3);
+            result->stats->rowcols->similar_row[j] = (int32_t) simde_mm_extract_epi32 (vHS, 3);
+            result->stats->rowcols->length_row[j] = (int32_t) simde_mm_extract_epi32 (vHL, 3);
         }
 #endif
     }
 
     /* Trace the alignment ending position on read. */
     {
-        int64_t *t = (int64_t*)pvHMax;
-        int64_t *m = (int64_t*)pvHMMax;
-        int64_t *s = (int64_t*)pvHSMax;
-        int64_t *l = (int64_t*)pvHLMax;
+        int32_t *t = (int32_t*)pvHMax;
+        int32_t *m = (int32_t*)pvHMMax;
+        int32_t *s = (int32_t*)pvHSMax;
+        int32_t *l = (int32_t*)pvHLMax;
         int32_t column_len = segLen * segWidth;
         end_query = s1Len;
         for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
@@ -445,8 +567,8 @@ parasail_result_t* PNAME(
 #endif
 
     if (simde_mm_movemask_epi8(simde_mm_or_si128(
-            simde_mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
-            simde_mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
+            simde_mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
+            simde_mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
@@ -462,15 +584,6 @@ parasail_result_t* PNAME(
     result->stats->matches = matches;
     result->stats->similar = similar;
     result->stats->length = length;
-    result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_SCAN
-        | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
-#ifdef PARASAIL_TABLE
-    result->flag |= PARASAIL_FLAG_TABLE;
-#endif
-#ifdef PARASAIL_ROWCOL
-    result->flag |= PARASAIL_FLAG_ROWCOL;
-#endif
 
     parasail_free(pvGapperL);
     parasail_free(pvGapper);
