@@ -15,6 +15,7 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
+#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -22,45 +23,12 @@
 #include "parasail/internal_sse.h"
 
 
-static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
-    a = _mm_andnot_si128(mask, a);
-    a = _mm_or_si128(a, _mm_and_si128(mask, b));
-    return a;
-}
-
-static inline __m128i _mm_insert_epi8_rpl(__m128i a, int8_t i, const int imm) {
-    __m128i_8_t A;
-    A.m = a;
-    A.v[imm] = i;
-    return A.m;
-}
-
-static inline __m128i _mm_max_epi8_rpl(__m128i a, __m128i b) {
-    __m128i mask = _mm_cmpgt_epi8(a, b);
-    a = _mm_and_si128(a, mask);
-    b = _mm_andnot_si128(mask, b);
-    return _mm_or_si128(a, b);
-}
-
-static inline int8_t _mm_extract_epi8_rpl(__m128i a, const int imm) {
-    __m128i_8_t A;
-    A.m = a;
-    return A.v[imm];
-}
-
-static inline __m128i _mm_min_epi8_rpl(__m128i a, __m128i b) {
-    __m128i mask = _mm_cmpgt_epi8(b, a);
-    a = _mm_and_si128(a, mask);
-    b = _mm_andnot_si128(mask, b);
-    return _mm_or_si128(a, b);
-}
-
 static inline int8_t _mm_hmax_epi8_rpl(__m128i a) {
-    a = _mm_max_epi8_rpl(a, _mm_srli_si128(a, 8));
-    a = _mm_max_epi8_rpl(a, _mm_srli_si128(a, 4));
-    a = _mm_max_epi8_rpl(a, _mm_srli_si128(a, 2));
-    a = _mm_max_epi8_rpl(a, _mm_srli_si128(a, 1));
-    return _mm_extract_epi8_rpl(a, 0);
+    a = _mm_max_epi8(a, _mm_srli_si128(a, 8));
+    a = _mm_max_epi8(a, _mm_srli_si128(a, 4));
+    a = _mm_max_epi8(a, _mm_srli_si128(a, 2));
+    a = _mm_max_epi8(a, _mm_srli_si128(a, 1));
+    return _mm_extract_epi8(a, 0);
 }
 
 
@@ -73,22 +41,22 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  0);
-    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  1);
-    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  2);
-    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  3);
-    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  4);
-    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  5);
-    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  6);
-    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  7);
-    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  8);
-    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  9);
-    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 10);
-    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 11);
-    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 12);
-    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 13);
-    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 14);
-    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 15);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  0);
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  1);
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  2);
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  3);
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  4);
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  5);
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  6);
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  7);
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  8);
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  9);
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 10);
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 11);
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 12);
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 13);
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 14);
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 15);
 }
 #endif
 
@@ -99,35 +67,35 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  0);
-    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  1);
-    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  2);
-    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  3);
-    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  4);
-    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  5);
-    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  6);
-    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  7);
-    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  8);
-    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  9);
-    col[10*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 10);
-    col[11*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 11);
-    col[12*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 12);
-    col[13*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 13);
-    col[14*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 14);
-    col[15*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 15);
+    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8(vH,  0);
+    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8(vH,  1);
+    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8(vH,  2);
+    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8(vH,  3);
+    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8(vH,  4);
+    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8(vH,  5);
+    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8(vH,  6);
+    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8(vH,  7);
+    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8(vH,  8);
+    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8(vH,  9);
+    col[10*seglen+t] = (int8_t)_mm_extract_epi8(vH, 10);
+    col[11*seglen+t] = (int8_t)_mm_extract_epi8(vH, 11);
+    col[12*seglen+t] = (int8_t)_mm_extract_epi8(vH, 12);
+    col[13*seglen+t] = (int8_t)_mm_extract_epi8(vH, 13);
+    col[14*seglen+t] = (int8_t)_mm_extract_epi8(vH, 14);
+    col[15*seglen+t] = (int8_t)_mm_extract_epi8(vH, 15);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_stats_table_scan_sse2_128_8
-#define PNAME parasail_sw_stats_table_scan_profile_sse2_128_8
+#define FNAME parasail_sw_stats_table_scan_sse41_128_8
+#define PNAME parasail_sw_stats_table_scan_profile_sse41_128_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_stats_rowcol_scan_sse2_128_8
-#define PNAME parasail_sw_stats_rowcol_scan_profile_sse2_128_8
+#define FNAME parasail_sw_stats_rowcol_scan_sse41_128_8
+#define PNAME parasail_sw_stats_rowcol_scan_profile_sse41_128_8
 #else
-#define FNAME parasail_sw_stats_scan_sse2_128_8
-#define PNAME parasail_sw_stats_scan_profile_sse2_128_8
+#define FNAME parasail_sw_stats_scan_sse41_128_8
+#define PNAME parasail_sw_stats_scan_profile_sse41_128_8
 #endif
 #endif
 
@@ -136,9 +104,31 @@ parasail_result_t* FNAME(
         const char * const restrict s2, const int s2Len,
         const int open, const int gap, const parasail_matrix_t *matrix)
 {
-    parasail_profile_t *profile = parasail_profile_create_stats_sse_128_8(s1, s1Len, matrix);
-    parasail_result_t *result = PNAME(profile, s2, s2Len, open, gap);
+    /* declare local variables */
+    parasail_profile_t *profile = NULL;
+    parasail_result_t *result = NULL;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+    PARASAIL_CHECK_NULL(matrix);
+    if (matrix->type == PARASAIL_MATRIX_TYPE_PSSM) {
+        PARASAIL_CHECK_NULL_PSSM_STATS(s1);
+    }
+    else {
+        PARASAIL_CHECK_NULL(s1);
+        PARASAIL_CHECK_GT0(s1Len);
+    }
+
+    /* initialize local variables */
+    profile = parasail_profile_create_stats_sse_128_8(s1, s1Len, matrix);
+    if (!profile) return NULL;
+    result = PNAME(profile, s2, s2Len, open, gap);
+
     parasail_profile_free(profile);
+
     return result;
 }
 
@@ -147,70 +137,166 @@ parasail_result_t* PNAME(
         const char * const restrict s2, const int s2Len,
         const int open, const int gap)
 {
+    /* declare local variables */
     int32_t i = 0;
     int32_t j = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    const int s1Len = profile->s1Len;
-    const parasail_matrix_t *matrix = profile->matrix;
-    const int32_t segWidth = 16; /* number of values in vector unit */
-    const int32_t segLen = (s1Len + segWidth - 1) / segWidth;
-    __m128i* const restrict pvP  = (__m128i*)profile->profile8.score;
-    __m128i* const restrict pvPm = (__m128i*)profile->profile8.matches;
-    __m128i* const restrict pvPs = (__m128i*)profile->profile8.similar;
-    __m128i* const restrict pvE  = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvEM = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvES = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvEL = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvH  = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvHM = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvHS = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvHL = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvHMax  = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvHMMax = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvHSMax = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvHLMax = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvGapper = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvGapperL = parasail_memalign___m128i(16, segLen);
-    __m128i vGapO = _mm_set1_epi8(open);
-    __m128i vGapE = _mm_set1_epi8(gap);
-    const int8_t NEG_LIMIT = (-open < matrix->min ?
-        INT8_MIN + open : INT8_MIN - matrix->min) + 1;
-    const int8_t POS_LIMIT = INT8_MAX - matrix->max - 1;
-    __m128i vZero = _mm_setzero_si128();
-    __m128i vOne = _mm_set1_epi8(1);
-    int8_t score = NEG_LIMIT;
+    int32_t s1Len = 0;
+    const parasail_matrix_t *matrix = NULL;
+    int32_t segWidth = 0;
+    int32_t segLen = 0;
+#ifdef PARASAIL_ROWCOL
+    int32_t offset = 0;
+    int32_t position = 0;
+#endif
+    __m128i* restrict pvP = NULL;
+    __m128i* restrict pvPm = NULL;
+    __m128i* restrict pvPs = NULL;
+    __m128i* restrict pvE = NULL;
+    __m128i* restrict pvEM = NULL;
+    __m128i* restrict pvES = NULL;
+    __m128i* restrict pvEL = NULL;
+    __m128i* restrict pvH = NULL;
+    __m128i* restrict pvHM = NULL;
+    __m128i* restrict pvHS = NULL;
+    __m128i* restrict pvHL = NULL;
+    __m128i* restrict pvHMax = NULL;
+    __m128i* restrict pvHMMax = NULL;
+    __m128i* restrict pvHSMax = NULL;
+    __m128i* restrict pvHLMax = NULL;
+    __m128i* restrict pvGapper = NULL;
+    __m128i* restrict pvGapperL = NULL;
+    __m128i vGapO;
+    __m128i vGapE;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
+    __m128i vZero;
+    __m128i vOne;
+    int8_t score = 0;
     int8_t matches = 0;
     int8_t similar = 0;
     int8_t length = 0;
-    __m128i vNegLimit = _mm_set1_epi8(NEG_LIMIT);
-    __m128i vPosLimit = _mm_set1_epi8(POS_LIMIT);
-    __m128i vSaturationCheckMin = vPosLimit;
-    __m128i vSaturationCheckMax = vNegLimit;
-    __m128i vMaxH = vNegLimit;
-    __m128i vMaxM = vNegLimit;
-    __m128i vMaxS = vNegLimit;
-    __m128i vMaxL = vNegLimit;
-    __m128i vMaxHUnit = vNegLimit;
-    __m128i vNegInfFront = vZero;
+    __m128i vNegLimit;
+    __m128i vPosLimit;
+    __m128i vSaturationCheckMin;
+    __m128i vSaturationCheckMax;
+    __m128i vMaxH;
+    __m128i vMaxM;
+    __m128i vMaxS;
+    __m128i vMaxL;
+    __m128i vMaxHUnit;
+    __m128i vNegInfFront;
     __m128i vSegLenXgap;
-    __m128i vSegLen = _mm_slli_si128(_mm_set1_epi8(segLen), 1);
-#ifdef PARASAIL_TABLE
-    parasail_result_t *result = parasail_result_new_table3(segLen*segWidth, s2Len);
-#else
-#ifdef PARASAIL_ROWCOL
-    parasail_result_t *result = parasail_result_new_rowcol3(segLen*segWidth, s2Len);
-    const int32_t offset = (s1Len - 1) % segLen;
-    const int32_t position = (segWidth - 1) - (s1Len - 1) / segLen;
-#else
-    parasail_result_t *result = parasail_result_new_stats();
-#endif
-#endif
+    __m128i vSegLen;
+    parasail_result_t *result = NULL;
 
-    vNegInfFront = _mm_insert_epi8_rpl(vNegInfFront, NEG_LIMIT, 0);
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(profile);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
+    PARASAIL_CHECK_NULL(profile->matrix);
+    PARASAIL_CHECK_GT0(profile->s1Len);
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+
+    /* initialize stack variables */
+    i = 0;
+    j = 0;
+    end_query = 0;
+    end_ref = 0;
+    s1Len = profile->s1Len;
+    matrix = profile->matrix;
+    segWidth = 16; /* number of values in vector unit */
+    segLen = (s1Len + segWidth - 1) / segWidth;
+#ifdef PARASAIL_ROWCOL
+    offset = (s1Len - 1) % segLen;
+    position = (segWidth - 1) - (s1Len - 1) / segLen;
+#endif
+    pvP  = (__m128i*)profile->profile8.score;
+    pvPm = (__m128i*)profile->profile8.matches;
+    pvPs = (__m128i*)profile->profile8.similar;
+    vGapO = _mm_set1_epi8(open);
+    vGapE = _mm_set1_epi8(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
+    vZero = _mm_setzero_si128();
+    vOne = _mm_set1_epi8(1);
+    score = NEG_LIMIT;
+    matches = 0;
+    similar = 0;
+    length = 0;
+    vNegLimit = _mm_set1_epi8(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi8(POS_LIMIT);
+    vSaturationCheckMin = vPosLimit;
+    vSaturationCheckMax = vNegLimit;
+    vMaxH = vNegLimit;
+    vMaxM = vNegLimit;
+    vMaxS = vNegLimit;
+    vMaxL = vNegLimit;
+    vMaxHUnit = vNegLimit;
+    vSegLen = _mm_slli_si128(_mm_set1_epi8(segLen), 1);
+    vNegInfFront = vZero;
+    vNegInfFront = _mm_insert_epi8(vNegInfFront, NEG_LIMIT, 0);
     vSegLenXgap = _mm_adds_epi8(vNegInfFront,
             _mm_slli_si128(_mm_set1_epi8(-segLen*gap), 1));
 
+    /* initialize result */
+#ifdef PARASAIL_TABLE
+    result = parasail_result_new_table3(segLen*segWidth, s2Len);
+#else
+#ifdef PARASAIL_ROWCOL
+    result = parasail_result_new_rowcol3(segLen*segWidth, s2Len);
+#else
+    result = parasail_result_new_stats();
+#endif
+#endif
+    if (!result) return NULL;
+
+    /* set known flags */
+    result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_SCAN
+        | PARASAIL_FLAG_STATS
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
+#ifdef PARASAIL_TABLE
+    result->flag |= PARASAIL_FLAG_TABLE;
+#endif
+#ifdef PARASAIL_ROWCOL
+    result->flag |= PARASAIL_FLAG_ROWCOL;
+#endif
+
+    /* initialize heap variables */
+    pvE  = parasail_memalign___m128i(16, segLen);
+    pvEM = parasail_memalign___m128i(16, segLen);
+    pvES = parasail_memalign___m128i(16, segLen);
+    pvEL = parasail_memalign___m128i(16, segLen);
+    pvH  = parasail_memalign___m128i(16, segLen);
+    pvHM = parasail_memalign___m128i(16, segLen);
+    pvHS = parasail_memalign___m128i(16, segLen);
+    pvHL = parasail_memalign___m128i(16, segLen);
+    pvHMax  = parasail_memalign___m128i(16, segLen);
+    pvHMMax = parasail_memalign___m128i(16, segLen);
+    pvHSMax = parasail_memalign___m128i(16, segLen);
+    pvHLMax = parasail_memalign___m128i(16, segLen);
+    pvGapper = parasail_memalign___m128i(16, segLen);
+    pvGapperL = parasail_memalign___m128i(16, segLen);
+
+    /* validate heap variables */
+    if (!pvE) return NULL;
+    if (!pvEM) return NULL;
+    if (!pvES) return NULL;
+    if (!pvEL) return NULL;
+    if (!pvH) return NULL;
+    if (!pvHM) return NULL;
+    if (!pvHS) return NULL;
+    if (!pvHL) return NULL;
+    if (!pvHMax) return NULL;
+    if (!pvHMMax) return NULL;
+    if (!pvHSMax) return NULL;
+    if (!pvHLMax) return NULL;
+    if (!pvGapper) return NULL;
+    if (!pvGapperL) return NULL;
+
     parasail_memset___m128i(pvH, vZero, segLen);
     parasail_memset___m128i(pvHM, vZero, segLen);
     parasail_memset___m128i(pvHS, vZero, segLen);
@@ -304,19 +390,19 @@ parasail_result_t* PNAME(
             vE_opn = _mm_subs_epi8(vH, vGapO);
             vE_ext = _mm_subs_epi8(vE, vGapE);
             case1 = _mm_cmpgt_epi8(vE_opn, vE_ext);
-            vE = _mm_max_epi8_rpl(vE_opn, vE_ext);
-            vEM = _mm_blendv_epi8_rpl(vEM, vHM, case1);
-            vES = _mm_blendv_epi8_rpl(vES, vHS, case1);
-            vEL = _mm_blendv_epi8_rpl(vEL, vHL, case1);
+            vE = _mm_max_epi8(vE_opn, vE_ext);
+            vEM = _mm_blendv_epi8(vEM, vHM, case1);
+            vES = _mm_blendv_epi8(vES, vHS, case1);
+            vEL = _mm_blendv_epi8(vEL, vHL, case1);
             vEL = _mm_adds_epi8(vEL, vOne);
             vGapper = _mm_adds_epi8(vHt, vGapper);
             case1 = _mm_or_si128(
                     _mm_cmpgt_epi8(vF, vGapper),
                     _mm_cmpeq_epi8(vF, vGapper));
-            vF = _mm_max_epi8_rpl(vF, vGapper);
-            vFM = _mm_blendv_epi8_rpl(vHtM, vFM, case1);
-            vFS = _mm_blendv_epi8_rpl(vHtS, vFS, case1);
-            vFL = _mm_blendv_epi8_rpl(
+            vF = _mm_max_epi8(vF, vGapper);
+            vFM = _mm_blendv_epi8(vHtM, vFM, case1);
+            vFS = _mm_blendv_epi8(vHtS, vFS, case1);
+            vFL = _mm_blendv_epi8(
                     _mm_adds_epi8(vHtL, vGapperL),
                     vFL, case1);
             vHp = _mm_adds_epi8(vHp, vW);
@@ -324,10 +410,10 @@ parasail_result_t* PNAME(
             vHpS = _mm_adds_epi8(vHpS, vWS);
             vHpL = _mm_adds_epi8(vHpL, vOne);
             case1 = _mm_cmpgt_epi8(vE, vHp);
-            vHt = _mm_max_epi8_rpl(vE, vHp);
-            vHtM = _mm_blendv_epi8_rpl(vHpM, vEM, case1);
-            vHtS = _mm_blendv_epi8_rpl(vHpS, vES, case1);
-            vHtL = _mm_blendv_epi8_rpl(vHpL, vEL, case1);
+            vHt = _mm_max_epi8(vE, vHp);
+            vHtM = _mm_blendv_epi8(vHpM, vEM, case1);
+            vHtS = _mm_blendv_epi8(vHpS, vES, case1);
+            vHtL = _mm_blendv_epi8(vHpL, vEL, case1);
             _mm_store_si128(pvE+i, vE);
             _mm_store_si128(pvEM+i, vEM);
             _mm_store_si128(pvES+i, vES);
@@ -353,10 +439,10 @@ parasail_result_t* PNAME(
         case1 = _mm_or_si128(
                 _mm_cmpgt_epi8(vGapper, vF),
                 _mm_cmpeq_epi8(vGapper, vF));
-        vF = _mm_max_epi8_rpl(vF, vGapper);
-        vFM = _mm_blendv_epi8_rpl(vFM, vHtM, case1);
-        vFS = _mm_blendv_epi8_rpl(vFS, vHtS, case1);
-        vFL = _mm_blendv_epi8_rpl(
+        vF = _mm_max_epi8(vF, vGapper);
+        vFM = _mm_blendv_epi8(vFM, vHtM, case1);
+        vFS = _mm_blendv_epi8(vFS, vHtS, case1);
+        vFL = _mm_blendv_epi8(
                 vFL,
                 _mm_adds_epi8(vHtL, vGapperL),
                 case1);
@@ -369,10 +455,10 @@ parasail_result_t* PNAME(
             case1 = _mm_or_si128(
                     _mm_cmpgt_epi8(vFt, vF),
                     _mm_cmpeq_epi8(vFt, vF));
-            vF = _mm_max_epi8_rpl(vF, vFt);
-            vFM = _mm_blendv_epi8_rpl(vFM, vFtM, case1);
-            vFS = _mm_blendv_epi8_rpl(vFS, vFtS, case1);
-            vFL = _mm_blendv_epi8_rpl(
+            vF = _mm_max_epi8(vF, vFt);
+            vFM = _mm_blendv_epi8(vFM, vFtM, case1);
+            vFS = _mm_blendv_epi8(vFS, vFtS, case1);
+            vFL = _mm_blendv_epi8(
                     vFL,
                     _mm_adds_epi8(vFtL, vSegLen),
                     case1);
@@ -385,10 +471,10 @@ parasail_result_t* PNAME(
         vFL = _mm_slli_si128(vFL, 1);
         vF = _mm_adds_epi8(vF, vNegInfFront);
         case1 = _mm_cmpgt_epi8(vF, vHt);
-        vH = _mm_max_epi8_rpl(vF, vHt);
-        vHM = _mm_blendv_epi8_rpl(vHtM, vFM, case1);
-        vHS = _mm_blendv_epi8_rpl(vHtS, vFS, case1);
-        vHL = _mm_blendv_epi8_rpl(vHtL, vFL, case1);
+        vH = _mm_max_epi8(vF, vHt);
+        vHM = _mm_blendv_epi8(vHtM, vFM, case1);
+        vHS = _mm_blendv_epi8(vHtS, vFS, case1);
+        vHL = _mm_blendv_epi8(vHtL, vFL, case1);
         for (i=0; i<segLen; ++i) {
             vHp = _mm_load_si128(pvH+i);
             vHpM = _mm_load_si128(pvHM+i);
@@ -400,39 +486,39 @@ parasail_result_t* PNAME(
             vEL = _mm_load_si128(pvEL+i);
             vF_opn = _mm_subs_epi8(vH, vGapO);
             vF_ext = _mm_subs_epi8(vF, vGapE);
-            vF = _mm_max_epi8_rpl(vF_opn, vF_ext);
+            vF = _mm_max_epi8(vF_opn, vF_ext);
             case1 = _mm_cmpgt_epi8(vF_opn, vF_ext);
-            vFM = _mm_blendv_epi8_rpl(vFM, vHM, case1);
-            vFS = _mm_blendv_epi8_rpl(vFS, vHS, case1);
-            vFL = _mm_blendv_epi8_rpl(vFL, vHL, case1);
+            vFM = _mm_blendv_epi8(vFM, vHM, case1);
+            vFS = _mm_blendv_epi8(vFS, vHS, case1);
+            vFL = _mm_blendv_epi8(vFL, vHL, case1);
             vFL = _mm_adds_epi8(vFL, vOne);
-            vH = _mm_max_epi8_rpl(vHp, vE);
-            vH = _mm_max_epi8_rpl(vH, vF);
-            vH = _mm_max_epi8_rpl(vH, vZero);
+            vH = _mm_max_epi8(vHp, vE);
+            vH = _mm_max_epi8(vH, vF);
+            vH = _mm_max_epi8(vH, vZero);
             case1 = _mm_cmpeq_epi8(vH, vHp);
             case2 = _mm_cmpeq_epi8(vH, vF);
             case0 = _mm_cmpeq_epi8(vH, vZero);
-            vHM = _mm_blendv_epi8_rpl(
-                    _mm_blendv_epi8_rpl(vEM, vFM, case2),
+            vHM = _mm_blendv_epi8(
+                    _mm_blendv_epi8(vEM, vFM, case2),
                     vHpM, case1);
-            vHM = _mm_blendv_epi8_rpl(vHM, vZero, case0);
-            vHS = _mm_blendv_epi8_rpl(
-                    _mm_blendv_epi8_rpl(vES, vFS, case2),
+            vHM = _mm_blendv_epi8(vHM, vZero, case0);
+            vHS = _mm_blendv_epi8(
+                    _mm_blendv_epi8(vES, vFS, case2),
                     vHpS, case1);
-            vHS = _mm_blendv_epi8_rpl(vHS, vZero, case0);
-            vHL = _mm_blendv_epi8_rpl(
-                    _mm_blendv_epi8_rpl(vEL, vFL, case2),
+            vHS = _mm_blendv_epi8(vHS, vZero, case0);
+            vHL = _mm_blendv_epi8(
+                    _mm_blendv_epi8(vEL, vFL, case2),
                     vHpL, case1);
-            vHL = _mm_blendv_epi8_rpl(vHL, vZero, case0);
+            vHL = _mm_blendv_epi8(vHL, vZero, case0);
             _mm_store_si128(pvH+i, vH);
             _mm_store_si128(pvHM+i, vHM);
             _mm_store_si128(pvHS+i, vHS);
             _mm_store_si128(pvHL+i, vHL);
-            vSaturationCheckMin = _mm_min_epi8_rpl(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
@@ -441,10 +527,10 @@ parasail_result_t* PNAME(
 #endif
             {
                 __m128i cond_max = _mm_cmpgt_epi8(vH, vMaxH);
-                vMaxH = _mm_max_epi8_rpl(vH, vMaxH);
-                vMaxM = _mm_blendv_epi8_rpl(vMaxM, vHM, cond_max);
-                vMaxS = _mm_blendv_epi8_rpl(vMaxS, vHS, cond_max);
-                vMaxL = _mm_blendv_epi8_rpl(vMaxL, vHL, cond_max);
+                vMaxH = _mm_max_epi8(vH, vMaxH);
+                vMaxM = _mm_blendv_epi8(vMaxM, vHM, cond_max);
+                vMaxS = _mm_blendv_epi8(vMaxS, vHS, cond_max);
+                vMaxL = _mm_blendv_epi8(vMaxL, vHL, cond_max);
             }
         } 
 
@@ -475,10 +561,10 @@ parasail_result_t* PNAME(
                 vHS = _mm_slli_si128(vHS, 1);
                 vHL = _mm_slli_si128(vHL, 1);
             }
-            result->stats->rowcols->score_row[j] = (int8_t) _mm_extract_epi8_rpl (vH, 15);
-            result->stats->rowcols->matches_row[j] = (int8_t) _mm_extract_epi8_rpl (vHM, 15);
-            result->stats->rowcols->similar_row[j] = (int8_t) _mm_extract_epi8_rpl (vHS, 15);
-            result->stats->rowcols->length_row[j] = (int8_t) _mm_extract_epi8_rpl (vHL, 15);
+            result->stats->rowcols->score_row[j] = (int8_t) _mm_extract_epi8 (vH, 15);
+            result->stats->rowcols->matches_row[j] = (int8_t) _mm_extract_epi8 (vHM, 15);
+            result->stats->rowcols->similar_row[j] = (int8_t) _mm_extract_epi8 (vHS, 15);
+            result->stats->rowcols->length_row[j] = (int8_t) _mm_extract_epi8 (vHL, 15);
         }
 #endif
     }
@@ -535,15 +621,6 @@ parasail_result_t* PNAME(
     result->stats->matches = matches;
     result->stats->similar = similar;
     result->stats->length = length;
-    result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_SCAN
-        | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
-#ifdef PARASAIL_TABLE
-    result->flag |= PARASAIL_FLAG_TABLE;
-#endif
-#ifdef PARASAIL_ROWCOL
-    result->flag |= PARASAIL_FLAG_ROWCOL;
-#endif
 
     parasail_free(pvGapperL);
     parasail_free(pvGapper);
