@@ -28,14 +28,22 @@ static inline void arr_store_si128(
         int32_t dlen,
         int32_t bias)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 0) - bias;
-    array[1LL*(1*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 1) - bias;
-    array[1LL*(2*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 2) - bias;
-    array[1LL*(3*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 3) - bias;
-    array[1LL*(4*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 4) - bias;
-    array[1LL*(5*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 5) - bias;
-    array[1LL*(6*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 6) - bias;
-    array[1LL*(7*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 7) - bias;
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  0) - bias;
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  1) - bias;
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  2) - bias;
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  3) - bias;
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  4) - bias;
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  5) - bias;
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  6) - bias;
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  7) - bias;
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  8) - bias;
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  9) - bias;
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 10) - bias;
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 11) - bias;
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 12) - bias;
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 13) - bias;
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 14) - bias;
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 15) - bias;
 }
 #endif
 
@@ -47,27 +55,35 @@ static inline void arr_store_col(
         int32_t seglen,
         int32_t bias)
 {
-    col[0*seglen+t] = (int16_t)_mm_extract_epi16(vH, 0) - bias;
-    col[1*seglen+t] = (int16_t)_mm_extract_epi16(vH, 1) - bias;
-    col[2*seglen+t] = (int16_t)_mm_extract_epi16(vH, 2) - bias;
-    col[3*seglen+t] = (int16_t)_mm_extract_epi16(vH, 3) - bias;
-    col[4*seglen+t] = (int16_t)_mm_extract_epi16(vH, 4) - bias;
-    col[5*seglen+t] = (int16_t)_mm_extract_epi16(vH, 5) - bias;
-    col[6*seglen+t] = (int16_t)_mm_extract_epi16(vH, 6) - bias;
-    col[7*seglen+t] = (int16_t)_mm_extract_epi16(vH, 7) - bias;
+    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8(vH,  0) - bias;
+    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8(vH,  1) - bias;
+    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8(vH,  2) - bias;
+    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8(vH,  3) - bias;
+    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8(vH,  4) - bias;
+    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8(vH,  5) - bias;
+    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8(vH,  6) - bias;
+    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8(vH,  7) - bias;
+    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8(vH,  8) - bias;
+    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8(vH,  9) - bias;
+    col[10*seglen+t] = (int8_t)_mm_extract_epi8(vH, 10) - bias;
+    col[11*seglen+t] = (int8_t)_mm_extract_epi8(vH, 11) - bias;
+    col[12*seglen+t] = (int8_t)_mm_extract_epi8(vH, 12) - bias;
+    col[13*seglen+t] = (int8_t)_mm_extract_epi8(vH, 13) - bias;
+    col[14*seglen+t] = (int8_t)_mm_extract_epi8(vH, 14) - bias;
+    col[15*seglen+t] = (int8_t)_mm_extract_epi8(vH, 15) - bias;
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_table_striped_altivec_128_16
-#define PNAME parasail_sw_table_striped_profile_altivec_128_16
+#define FNAME parasail_sw_table_striped_altivec_128_8
+#define PNAME parasail_sw_table_striped_profile_altivec_128_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_rowcol_striped_altivec_128_16
-#define PNAME parasail_sw_rowcol_striped_profile_altivec_128_16
+#define FNAME parasail_sw_rowcol_striped_altivec_128_8
+#define PNAME parasail_sw_rowcol_striped_profile_altivec_128_8
 #else
-#define FNAME parasail_sw_striped_altivec_128_16
-#define PNAME parasail_sw_striped_profile_altivec_128_16
+#define FNAME parasail_sw_striped_altivec_128_8
+#define PNAME parasail_sw_striped_profile_altivec_128_8
 #endif
 #endif
 
@@ -92,7 +108,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_altivec_128_16(s1, s1Len, matrix);
+    profile = parasail_profile_create_altivec_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -128,19 +144,19 @@ parasail_result_t* PNAME(
     vec128i vGapO;
     vec128i vGapE;
     vec128i vZero;
-    int16_t bias = 0;
-    int16_t score = 0;
+    int8_t bias = 0;
+    int8_t score = 0;
     vec128i vBias;
     vec128i vMaxH;
     vec128i vMaxHUnit;
-    int16_t maxp = 0;
+    int8_t maxp = 0;
     vec128i insert_mask;
-    /*int16_t stop = 0;*/
+    /*int8_t stop = 0;*/
     parasail_result_t *result = NULL;
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile16.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -156,26 +172,26 @@ parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 8; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
 #ifdef PARASAIL_ROWCOL
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
 #endif
-    vProfile = (vec128i*)profile->profile16.score;
-    vGapO = _mm_set1_epi16(open);
-    vGapE = _mm_set1_epi16(gap);
-    vZero = _mm_set1_epi16(0);
-    bias = INT16_MIN;
+    vProfile = (vec128i*)profile->profile8.score;
+    vGapO = _mm_set1_epi8(open);
+    vGapE = _mm_set1_epi8(gap);
+    vZero = _mm_set1_epi8(0);
+    bias = INT8_MIN;
     score = bias;
-    vBias = _mm_set1_epi16(bias);
+    vBias = _mm_set1_epi8(bias);
     vMaxH = vBias;
     vMaxHUnit = vBias;
-    maxp = INT16_MAX - (int16_t)(matrix->max+1);
-    insert_mask = _mm_cmpgt_epi16(
-            _mm_set_epi16(0,0,0,0,0,0,0,1),
+    maxp = INT8_MAX - (int8_t)(matrix->max+1);
+    insert_mask = _mm_cmpgt_epi8(
+            _mm_set_epi8(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1),
             vZero);
-    /*stop = profile->stop == INT32_MAX ?  INT16_MAX : (int16_t)profile->stop-bias;*/
+    /*stop = profile->stop == INT32_MAX ?  INT8_MAX : (int8_t)profile->stop-bias;*/
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -191,7 +207,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_STRIPED
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -227,8 +243,8 @@ parasail_result_t* PNAME(
          * corrected in the Lazy_F loop.  */
         vF = vBias;
 
-        /* load final segment of pvHStore and shift left by 2 bytes */
-        vH = _mm_slli_si128(pvHStore[segLen - 1], 2);
+        /* load final segment of pvHStore and shift left by 1 bytes */
+        vH = _mm_slli_si128(pvHStore[segLen - 1], 1);
         vH = _mm_blendv_epi8(vH, vBias, insert_mask);
 
         /* Correct part of the vProfile */
@@ -250,28 +266,28 @@ parasail_result_t* PNAME(
 
         /* inner loop to process the query sequence */
         for (i=0; i<segLen; ++i) {
-            vH = _mm_adds_epi16(vH, _mm_load_si128(vP + i));
+            vH = _mm_adds_epi8(vH, _mm_load_si128(vP + i));
             vE = _mm_load_si128(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH = _mm_max_epi16(vH, vE);
-            vH = _mm_max_epi16(vH, vF);
+            vH = _mm_max_epi8(vH, vE);
+            vH = _mm_max_epi8(vH, vF);
             /* Save vH values. */
             _mm_store_si128(pvHStore + i, vH);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len, bias);
 #endif
-            vMaxH = _mm_max_epi16(vH, vMaxH);
+            vMaxH = _mm_max_epi8(vH, vMaxH);
 
             /* Update vE value. */
-            vH = _mm_subs_epi16(vH, vGapO);
-            vE = _mm_subs_epi16(vE, vGapE);
-            vE = _mm_max_epi16(vE, vH);
+            vH = _mm_subs_epi8(vH, vGapO);
+            vE = _mm_subs_epi8(vE, vGapE);
+            vE = _mm_max_epi8(vE, vH);
             _mm_store_si128(pvE + i, vE);
 
             /* Update vF value. */
-            vF = _mm_subs_epi16(vF, vGapE);
-            vF = _mm_max_epi16(vF, vH);
+            vF = _mm_subs_epi8(vF, vGapE);
+            vF = _mm_max_epi8(vF, vH);
 
             /* Load the next vH. */
             vH = _mm_load_si128(pvHLoad + i);
@@ -280,20 +296,20 @@ parasail_result_t* PNAME(
         /* Lazy_F loop: has been revised to disallow adjecent insertion and
          * then deletion, so don't update E(i, i), learn from SWPS3 */
         for (k=0; k<segWidth; ++k) {
-            vF = _mm_slli_si128(vF, 2);
+            vF = _mm_slli_si128(vF, 1);
             vF = _mm_blendv_epi8(vF, vBias, insert_mask);
             for (i=0; i<segLen; ++i) {
                 vH = _mm_load_si128(pvHStore + i);
-                vH = _mm_max_epi16(vH,vF);
+                vH = _mm_max_epi8(vH,vF);
                 _mm_store_si128(pvHStore + i, vH);
 #ifdef PARASAIL_TABLE
                 arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len, bias);
 #endif
-                vMaxH = _mm_max_epi16(vH, vMaxH);
-                vH = _mm_subs_epi16(vH, vGapO);
-                vF = _mm_subs_epi16(vF, vGapE);
-                if (! _mm_movemask_epi8(_mm_cmpgt_epi16(vF, vH))) goto end;
-                /*vF = _mm_max_epi16(vF, vH);*/
+                vMaxH = _mm_max_epi8(vH, vMaxH);
+                vH = _mm_subs_epi8(vH, vGapO);
+                vF = _mm_subs_epi8(vF, vGapE);
+                if (! _mm_movemask_epi8(_mm_cmpgt_epi8(vF, vH))) goto end;
+                /*vF = _mm_max_epi8(vF, vH);*/
             }
         }
 end:
@@ -305,22 +321,22 @@ end:
         {
             vH = _mm_load_si128(pvHStore + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 2);
+                vH = _mm_slli_si128(vH, 1);
             }
-            result->rowcols->score_row[j] = (int16_t) _mm_extract_epi16 (vH, 7) - bias;
+            result->rowcols->score_row[j] = (int8_t) _mm_extract_epi8 (vH, 15) - bias;
         }
 #endif
 
         {
-            vec128i vCompare = _mm_cmpgt_epi16(vMaxH, vMaxHUnit);
+            vec128i vCompare = _mm_cmpgt_epi8(vMaxH, vMaxHUnit);
             if (_mm_movemask_epi8(vCompare)) {
-                score = _mm_hmax_epi16(vMaxH);
+                score = _mm_hmax_epi8(vMaxH);
                 /* if score has potential to overflow, abort early */
                 if (score > maxp) {
                     result->flag |= PARASAIL_FLAG_SATURATED;
                     break;
                 }
-                vMaxHUnit = _mm_set1_epi16(score);
+                vMaxHUnit = _mm_set1_epi8(score);
                 end_ref = j;
             }
         }
@@ -335,12 +351,12 @@ end:
     }
 #endif
 
-    if (score == INT16_MAX) {
+    if (score == INT8_MAX) {
         result->flag |= PARASAIL_FLAG_SATURATED;
     }
 
     if (parasail_result_is_saturated(result)) {
-        score = INT16_MAX;
+        score = INT8_MAX;
         end_query = 0;
         end_ref = 0;
     }
@@ -359,7 +375,7 @@ end:
         }
         /* Trace the alignment ending position on read. */
         {
-            int16_t *t = (int16_t*)pvHMax;
+            int8_t *t = (int8_t*)pvHMax;
             int32_t column_len = segLen * segWidth;
             end_query = s1Len - 1;
             for (i = 0; i<column_len; ++i, ++t) {
