@@ -26,20 +26,14 @@ static inline void arr_store_si128(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int8_t)_mm_extract_epi32(vWH, 3);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int8_t)_mm_extract_epi64(vWH, 1);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int8_t)_mm_extract_epi32(vWH, 2);
-    }
-    if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
-        array[1LL*(i+2)*s2Len + (j-2)] = (int8_t)_mm_extract_epi32(vWH, 1);
-    }
-    if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
-        array[1LL*(i+3)*s2Len + (j-3)] = (int8_t)_mm_extract_epi32(vWH, 0);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int8_t)_mm_extract_epi64(vWH, 0);
     }
 }
 
-#define FNAME parasail_sw_trace_diag_altivec_128_32
+#define FNAME parasail_sw_trace_diag_altivec_128_64
 
 parasail_result_t* FNAME(
         const char * const restrict _s1, const int _s1Len,
@@ -52,22 +46,22 @@ parasail_result_t* FNAME(
     int32_t PAD2 = 0;
     int32_t s1Len_PAD = 0;
     int32_t s2Len_PAD = 0;
-    int32_t * restrict s1 = NULL;
-    int32_t * restrict s2B = NULL;
-    int32_t * restrict _H_pr = NULL;
-    int32_t * restrict _F_pr = NULL;
-    int32_t * restrict s2 = NULL;
-    int32_t * restrict H_pr = NULL;
-    int32_t * restrict F_pr = NULL;
+    int64_t * restrict s1 = NULL;
+    int64_t * restrict s2B = NULL;
+    int64_t * restrict _H_pr = NULL;
+    int64_t * restrict _F_pr = NULL;
+    int64_t * restrict s2 = NULL;
+    int64_t * restrict H_pr = NULL;
+    int64_t * restrict F_pr = NULL;
     parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t s1Len = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
-    int32_t score = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
+    int64_t score = 0;
     vec128i vNegLimit;
     vec128i vPosLimit;
     vec128i vSaturationCheckMin;
@@ -108,7 +102,7 @@ parasail_result_t* FNAME(
     }
         
     /* initialize stack variables */
-    N = 4; /* number of values in vector */
+    N = 2; /* number of values in vector */
     PAD = N-1;
     PAD2 = PAD*2;
     s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
@@ -118,36 +112,36 @@ parasail_result_t* FNAME(
     j = 0;
     end_query = 0;
     end_ref = 0;
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm_set1_epi64(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi64(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vNegInf = _mm_set1_epi32(NEG_LIMIT);
-    vNegInf0 = _mm_srli_si128(vNegInf, 4); /* shift in a 0 */
-    vOpen = _mm_set1_epi32(open);
-    vGap  = _mm_set1_epi32(gap);
-    vZero = _mm_set1_epi32(0);
-    vOne = _mm_set1_epi32(1);
-    vN = _mm_set1_epi32(N);
-    vNegOne = _mm_set1_epi32(-1);
-    vI = _mm_set_epi32(0,1,2,3);
-    vJreset = _mm_set_epi32(0,-1,-2,-3);
+    vNegInf = _mm_set1_epi64(NEG_LIMIT);
+    vNegInf0 = _mm_srli_si128(vNegInf, 8); /* shift in a 0 */
+    vOpen = _mm_set1_epi64(open);
+    vGap  = _mm_set1_epi64(gap);
+    vZero = _mm_set1_epi64(0);
+    vOne = _mm_set1_epi64(1);
+    vN = _mm_set1_epi64(N);
+    vNegOne = _mm_set1_epi64(-1);
+    vI = _mm_set_epi64(0,1);
+    vJreset = _mm_set_epi64(0,-1);
     vMaxH = vNegInf;
     vEndI = vNegInf;
     vEndJ = vNegInf;
-    vILimit = _mm_set1_epi32(s1Len);
-    vJLimit = _mm_set1_epi32(s2Len);
-    vTDiag = _mm_set1_epi32(PARASAIL_DIAG);
-    vTIns = _mm_set1_epi32(PARASAIL_INS);
-    vTDel = _mm_set1_epi32(PARASAIL_DEL);
-    vTZero = _mm_set1_epi32(PARASAIL_ZERO);
-    vTDiagE = _mm_set1_epi32(PARASAIL_DIAG_E);
-    vTInsE = _mm_set1_epi32(PARASAIL_INS_E);
-    vTDiagF = _mm_set1_epi32(PARASAIL_DIAG_F);
-    vTDelF = _mm_set1_epi32(PARASAIL_DEL_F);
+    vILimit = _mm_set1_epi64(s1Len);
+    vJLimit = _mm_set1_epi64(s2Len);
+    vTDiag = _mm_set1_epi64(PARASAIL_DIAG);
+    vTIns = _mm_set1_epi64(PARASAIL_INS);
+    vTDel = _mm_set1_epi64(PARASAIL_DEL);
+    vTZero = _mm_set1_epi64(PARASAIL_ZERO);
+    vTDiagE = _mm_set1_epi64(PARASAIL_DIAG_E);
+    vTInsE = _mm_set1_epi64(PARASAIL_INS_E);
+    vTDiagF = _mm_set1_epi64(PARASAIL_DIAG_F);
+    vTDelF = _mm_set1_epi64(PARASAIL_DEL_F);
 
     /* initialize result */
     result = parasail_result_new_trace(s1Len, s2Len, 16, sizeof(int8_t));
@@ -156,12 +150,12 @@ parasail_result_t* FNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_DIAG
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
 
     /* initialize heap variables */
-    s2B= parasail_memalign_int32_t(16, s2Len+PAD2);
-    _H_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    _F_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    s2B= parasail_memalign_int64_t(16, s2Len+PAD2);
+    _H_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _F_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
     s2 = s2B+PAD; /* will allow later for negative indices */
     H_pr = _H_pr+PAD;
     F_pr = _F_pr+PAD;
@@ -173,7 +167,7 @@ parasail_result_t* FNAME(
 
     /* convert _s1 from char to int in range 0-23 */
     if (matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) {
-        s1 = parasail_memalign_int32_t(16, s1Len+PAD);
+        s1 = parasail_memalign_int64_t(16, s1Len+PAD);
         if (!s1) return NULL;
         for (i=0; i<s1Len; ++i) {
             s1[i] = matrix->mapper[(unsigned char)_s1[i]];
@@ -226,76 +220,72 @@ parasail_result_t* FNAME(
         vec128i vJ = vJreset;
         const int * const restrict matrow0 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+0] : ((i+0 >= s1Len) ? s1Len-1 : i+0))];
         const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
-        const int * const restrict matrow2 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+2] : ((i+2 >= s1Len) ? s1Len-1 : i+2))];
-        const int * const restrict matrow3 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+3] : ((i+3 >= s1Len) ? s1Len-1 : i+3))];
-        vec128i vIltLimit = _mm_cmplt_epi32(vI, vILimit);
+        vec128i vIltLimit = _mm_cmplt_epi64(vI, vILimit);
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
             vec128i vMat;
             vec128i vNWH = vNH;
-            vNH = _mm_srli_si128(vWH, 4);
-            vNH = _mm_insert_epi32(vNH, H_pr[j], 3);
-            vF = _mm_srli_si128(vF, 4);
-            vF = _mm_insert_epi32(vF, F_pr[j], 3);
-            vF_opn = _mm_sub_epi32(vNH, vOpen);
-            vF_ext = _mm_sub_epi32(vF, vGap);
-            vF = _mm_max_epi32(vF_opn, vF_ext);
-            vE_opn = _mm_sub_epi32(vWH, vOpen);
-            vE_ext = _mm_sub_epi32(vE, vGap);
-            vE = _mm_max_epi32(vE_opn, vE_ext);
-            vMat = _mm_set_epi32(
+            vNH = _mm_srli_si128(vWH, 8);
+            vNH = _mm_insert_epi64(vNH, H_pr[j], 1);
+            vF = _mm_srli_si128(vF, 8);
+            vF = _mm_insert_epi64(vF, F_pr[j], 1);
+            vF_opn = _mm_sub_epi64(vNH, vOpen);
+            vF_ext = _mm_sub_epi64(vF, vGap);
+            vF = _mm_max_epi64(vF_opn, vF_ext);
+            vE_opn = _mm_sub_epi64(vWH, vOpen);
+            vE_ext = _mm_sub_epi64(vE, vGap);
+            vE = _mm_max_epi64(vE_opn, vE_ext);
+            vMat = _mm_set_epi64(
                     matrow0[s2[j-0]],
-                    matrow1[s2[j-1]],
-                    matrow2[s2[j-2]],
-                    matrow3[s2[j-3]]
+                    matrow1[s2[j-1]]
                     );
-            vNWH = _mm_add_epi32(vNWH, vMat);
-            vNWH = _mm_max_epi32(vNWH, vZero);
-            vWH = _mm_max_epi32(vNWH, vE);
-            vWH = _mm_max_epi32(vWH, vF);
+            vNWH = _mm_add_epi64(vNWH, vMat);
+            vNWH = _mm_max_epi64(vNWH, vZero);
+            vWH = _mm_max_epi64(vNWH, vE);
+            vWH = _mm_max_epi64(vWH, vF);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                vec128i cond = _mm_cmpeq_epi32(vJ,vNegOne);
+                vec128i cond = _mm_cmpeq_epi64(vJ,vNegOne);
                 vWH = _mm_andnot_si128(cond, vWH);
                 vF = _mm_blendv_epi8(vF, vNegInf, cond);
                 vE = _mm_blendv_epi8(vE, vNegInf, cond);
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWH);
+                vSaturationCheckMin = _mm_min_epi64(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vWH);
             }
             /* trace table */
             {
-                vec128i cond_zero = _mm_cmpeq_epi32(vWH, vZero);
-                vec128i case1 = _mm_cmpeq_epi32(vWH, vNWH);
-                vec128i case2 = _mm_cmpeq_epi32(vWH, vF);
+                vec128i cond_zero = _mm_cmpeq_epi64(vWH, vZero);
+                vec128i case1 = _mm_cmpeq_epi64(vWH, vNWH);
+                vec128i case2 = _mm_cmpeq_epi64(vWH, vF);
                 vec128i vT = _mm_blendv_epi8(
                         _mm_blendv_epi8(vTIns, vTDel, case2),
                         _mm_blendv_epi8(vTDiag, vTZero, cond_zero),
                         case1);
-                vec128i condE = _mm_cmpgt_epi32(vE_opn, vE_ext);
-                vec128i condF = _mm_cmpgt_epi32(vF_opn, vF_ext);
+                vec128i condE = _mm_cmpgt_epi64(vE_opn, vE_ext);
+                vec128i condF = _mm_cmpgt_epi64(vF_opn, vF_ext);
                 vec128i vET = _mm_blendv_epi8(vTInsE, vTDiagE, condE);
                 vec128i vFT = _mm_blendv_epi8(vTDelF, vTDiagF, condF);
                 vT = _mm_or_si128(vT, vET);
                 vT = _mm_or_si128(vT, vFT);
                 arr_store_si128(result->trace->trace_table, vT, i, s1Len, j, s2Len);
             }
-            H_pr[j-3] = (int32_t)_mm_extract_epi32(vWH,0);
-            F_pr[j-3] = (int32_t)_mm_extract_epi32(vF,0);
+            H_pr[j-1] = (int64_t)_mm_extract_epi64(vWH,0);
+            F_pr[j-1] = (int64_t)_mm_extract_epi64(vF,0);
             /* as minor diagonal vector passes across table, extract
              * max values within the i,j bounds */
             {
                 vec128i cond_valid_J = _mm_and_si128(
-                        _mm_cmpgt_epi32(vJ, vNegOne),
-                        _mm_cmplt_epi32(vJ, vJLimit));
+                        _mm_cmpgt_epi64(vJ, vNegOne),
+                        _mm_cmplt_epi64(vJ, vJLimit));
                 vec128i cond_valid_IJ = _mm_and_si128(cond_valid_J, vIltLimit);
-                vec128i cond_eq = _mm_cmpeq_epi32(vWH, vMaxH);
-                vec128i cond_max = _mm_cmpgt_epi32(vWH, vMaxH);
+                vec128i cond_eq = _mm_cmpeq_epi64(vWH, vMaxH);
+                vec128i cond_max = _mm_cmpgt_epi64(vWH, vMaxH);
                 vec128i cond_all = _mm_and_si128(cond_max, cond_valid_IJ);
-                vec128i cond_Jlt = _mm_cmplt_epi32(vJ, vEndJ);
+                vec128i cond_Jlt = _mm_cmplt_epi64(vJ, vEndJ);
                 vMaxH = _mm_blendv_epi8(vMaxH, vWH, cond_all);
                 vEndI = _mm_blendv_epi8(vEndI, vI, cond_all);
                 vEndJ = _mm_blendv_epi8(vEndJ, vJ, cond_all);
@@ -304,16 +294,16 @@ parasail_result_t* FNAME(
                 vEndI = _mm_blendv_epi8(vEndI, vI, cond_all);
                 vEndJ = _mm_blendv_epi8(vEndJ, vJ, cond_all);
             }
-            vJ = _mm_add_epi32(vJ, vOne);
+            vJ = _mm_add_epi64(vJ, vOne);
         }
-        vI = _mm_add_epi32(vI, vN);
+        vI = _mm_add_epi64(vI, vN);
     }
 
     /* alignment ending position */
     {
-        int32_t *t = (int32_t*)&vMaxH;
-        int32_t *i = (int32_t*)&vEndI;
-        int32_t *j = (int32_t*)&vEndJ;
+        int64_t *t = (int64_t*)&vMaxH;
+        int64_t *i = (int64_t*)&vEndI;
+        int64_t *j = (int64_t*)&vEndJ;
         int32_t k;
         for (k=0; k<N; ++k, ++t, ++i, ++j) {
             if (*t > score) {
@@ -335,8 +325,8 @@ parasail_result_t* FNAME(
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
