@@ -15,7 +15,6 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
-#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -23,12 +22,43 @@
 #include "parasail/internal_sse.h"
 
 
-static inline int8_t _mm_hmax_epi8_rpl(__m128i a) {
-    a = _mm_max_epi8(a, _mm_srli_si128(a, 8));
-    a = _mm_max_epi8(a, _mm_srli_si128(a, 4));
-    a = _mm_max_epi8(a, _mm_srli_si128(a, 2));
-    a = _mm_max_epi8(a, _mm_srli_si128(a, 1));
-    return _mm_extract_epi8(a, 0);
+static inline int32_t _mm_extract_epi32_rpl(__m128i a, const int imm) {
+    __m128i_32_t A;
+    A.m = a;
+    return A.v[imm];
+}
+
+static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
+    a = _mm_andnot_si128(mask, a);
+    a = _mm_or_si128(a, _mm_and_si128(mask, b));
+    return a;
+}
+
+static inline __m128i _mm_insert_epi32_rpl(__m128i a, int32_t i, const int imm) {
+    __m128i_32_t A;
+    A.m = a;
+    A.v[imm] = i;
+    return A.m;
+}
+
+static inline __m128i _mm_max_epi32_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi32(a, b);
+    a = _mm_and_si128(a, mask);
+    b = _mm_andnot_si128(mask, b);
+    return _mm_or_si128(a, b);
+}
+
+static inline __m128i _mm_min_epi32_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi32(b, a);
+    a = _mm_and_si128(a, mask);
+    b = _mm_andnot_si128(mask, b);
+    return _mm_or_si128(a, b);
+}
+
+static inline int32_t _mm_hmax_epi32_rpl(__m128i a) {
+    a = _mm_max_epi32_rpl(a, _mm_srli_si128(a, 8));
+    a = _mm_max_epi32_rpl(a, _mm_srli_si128(a, 4));
+    return _mm_extract_epi32_rpl(a, 0);
 }
 
 
@@ -51,8 +81,8 @@ static inline __m128i arr_load(
     return _mm_load_si128(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_sw_trace_scan_sse41_128_8
-#define PNAME parasail_sw_trace_scan_profile_sse41_128_8
+#define FNAME parasail_sw_trace_scan_sse2_128_32
+#define PNAME parasail_sw_trace_scan_profile_sse2_128_32
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -75,7 +105,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_sse_128_8(s1, s1Len, matrix);
+    profile = parasail_profile_create_sse_128_32(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -106,10 +136,10 @@ parasail_result_t* PNAME(
     __m128i* restrict pvGapper = NULL;
     __m128i vGapO;
     __m128i vGapE;
-    int8_t NEG_LIMIT = 0;
-    int8_t POS_LIMIT = 0;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
     __m128i vZero;
-    int8_t score = 0;
+    int32_t score = 0;
     __m128i vNegLimit;
     __m128i vPosLimit;
     __m128i vSaturationCheckMin;
@@ -130,7 +160,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile8.score);
+    PARASAIL_CHECK_NULL(profile->profile32.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -145,33 +175,33 @@ parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 16; /* number of values in vector unit */
+    segWidth = 4; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
-    pvP = (__m128i*)profile->profile8.score;
-    vGapO = _mm_set1_epi8(open);
-    vGapE = _mm_set1_epi8(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
-    POS_LIMIT = INT8_MAX - matrix->max - 1;
+    pvP = (__m128i*)profile->profile32.score;
+    vGapO = _mm_set1_epi32(open);
+    vGapE = _mm_set1_epi32(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
     vZero = _mm_setzero_si128();
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi8(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi8(POS_LIMIT);
+    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi32(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
     vMaxHUnit = vNegLimit;
     vNegInfFront = vZero;
-    vNegInfFront = _mm_insert_epi8(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm_adds_epi8(vNegInfFront,
-            _mm_slli_si128(_mm_set1_epi8(-segLen*gap), 1));
-    vTZero = _mm_set1_epi8(PARASAIL_ZERO);
-    vTIns  = _mm_set1_epi8(PARASAIL_INS);
-    vTDel  = _mm_set1_epi8(PARASAIL_DEL);
-    vTDiag = _mm_set1_epi8(PARASAIL_DIAG);
-    vTDiagE = _mm_set1_epi8(PARASAIL_DIAG_E);
-    vTInsE = _mm_set1_epi8(PARASAIL_INS_E);
-    vTDiagF = _mm_set1_epi8(PARASAIL_DIAG_F);
-    vTDelF = _mm_set1_epi8(PARASAIL_DEL_F);
+    vNegInfFront = _mm_insert_epi32_rpl(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm_add_epi32(vNegInfFront,
+            _mm_slli_si128(_mm_set1_epi32(-segLen*gap), 4));
+    vTZero = _mm_set1_epi32(PARASAIL_ZERO);
+    vTIns  = _mm_set1_epi32(PARASAIL_INS);
+    vTDel  = _mm_set1_epi32(PARASAIL_DEL);
+    vTDiag = _mm_set1_epi32(PARASAIL_DIAG);
+    vTDiagE = _mm_set1_epi32(PARASAIL_DIAG_E);
+    vTInsE = _mm_set1_epi32(PARASAIL_INS_E);
+    vTDiagF = _mm_set1_epi32(PARASAIL_DIAG_F);
+    vTDelF = _mm_set1_epi32(PARASAIL_DEL_F);
 
 
     /* initialize result */
@@ -181,7 +211,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
 
     /* initialize heap variables */
     pvE    = parasail_memalign___m128i(16, segLen);
@@ -200,10 +230,10 @@ parasail_result_t* PNAME(
     parasail_memset___m128i(pvH, vZero, segLen);
     parasail_memset___m128i(pvE, vNegLimit, segLen);
     {
-        __m128i vGapper = _mm_subs_epi8(vZero,vGapO);
+        __m128i vGapper = _mm_sub_epi32(vZero,vGapO);
         for (i=segLen-1; i>=0; --i) {
             _mm_store_si128(pvGapper+i, vGapper);
-            vGapper = _mm_subs_epi8(vGapper, vGapE);
+            vGapper = _mm_sub_epi32(vGapper, vGapE);
         }
     }
 
@@ -232,7 +262,7 @@ parasail_result_t* PNAME(
         /* calculate Ht */
         /* calculate F and H first pass */
         vHp = _mm_load_si128(pvH+(segLen-1));
-        vHp = _mm_slli_si128(vHp, 1);
+        vHp = _mm_slli_si128(vHp, 4);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         vHt = vZero;
         vF = vNegLimit;
@@ -241,16 +271,16 @@ parasail_result_t* PNAME(
             vE = _mm_load_si128(pvE+i);
             vW = _mm_load_si128(pvW+i);
             vGapper = _mm_load_si128(pvGapper+i);
-            vE_opn = _mm_subs_epi8(vH, vGapO);
-            vE_ext = _mm_subs_epi8(vE, vGapE);
-            case1 = _mm_cmpgt_epi8(vE_opn, vE_ext);
-            vET = _mm_blendv_epi8(vTInsE, vTDiagE, case1);
+            vE_opn = _mm_sub_epi32(vH, vGapO);
+            vE_ext = _mm_sub_epi32(vE, vGapE);
+            case1 = _mm_cmpgt_epi32(vE_opn, vE_ext);
+            vET = _mm_blendv_epi8_rpl(vTInsE, vTDiagE, case1);
             arr_store(result->trace->trace_table, vET, i, segLen, j);
-            vE = _mm_max_epi8(vE_opn, vE_ext);
-            vGapper = _mm_adds_epi8(vHt, vGapper);
-            vF = _mm_max_epi8(vF, vGapper);
-            vHp = _mm_adds_epi8(vHp, vW);
-            vHt = _mm_max_epi8(vE, vHp);
+            vE = _mm_max_epi32_rpl(vE_opn, vE_ext);
+            vGapper = _mm_add_epi32(vHt, vGapper);
+            vF = _mm_max_epi32_rpl(vF, vGapper);
+            vHp = _mm_add_epi32(vHp, vW);
+            vHt = _mm_max_epi32_rpl(vE, vHp);
             _mm_store_si128(pvE+i, vE);
             _mm_store_si128(pvHt+i, vHt);
             _mm_store_si128(pvH+i, vHp);
@@ -258,55 +288,55 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm_slli_si128(vHt, 1);
+        vHt = _mm_slli_si128(vHt, 4);
         vGapper = _mm_load_si128(pvGapper);
-        vGapper = _mm_adds_epi8(vHt, vGapper);
-        vF = _mm_max_epi8(vF, vGapper);
+        vGapper = _mm_add_epi32(vHt, vGapper);
+        vF = _mm_max_epi32_rpl(vF, vGapper);
         for (i=0; i<segWidth-2; ++i) {
-            __m128i vFt = _mm_slli_si128(vF, 1);
-            vFt = _mm_adds_epi8(vFt, vSegLenXgap);
-            vF = _mm_max_epi8(vF, vFt);
+            __m128i vFt = _mm_slli_si128(vF, 4);
+            vFt = _mm_add_epi32(vFt, vSegLenXgap);
+            vF = _mm_max_epi32_rpl(vF, vFt);
         }
 
         /* calculate final H */
-        vF = _mm_slli_si128(vF, 1);
-        vF = _mm_adds_epi8(vF, vNegInfFront);
-        vH = _mm_max_epi8(vF, vHt);
-        vH = _mm_max_epi8(vH, vZero);
+        vF = _mm_slli_si128(vF, 4);
+        vF = _mm_add_epi32(vF, vNegInfFront);
+        vH = _mm_max_epi32_rpl(vF, vHt);
+        vH = _mm_max_epi32_rpl(vH, vZero);
         for (i=0; i<segLen; ++i) {
             vET = arr_load(result->trace->trace_table, i, segLen, j);
             vHp = _mm_load_si128(pvH+i);
             vHt = _mm_load_si128(pvHt+i);
-            vF_opn = _mm_subs_epi8(vH, vGapO);
-            vF_ext = _mm_subs_epi8(vF, vGapE);
-            vF = _mm_max_epi8(vF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi8(vF_opn, vF_ext);
-            vFT = _mm_blendv_epi8(vTDelF, vTDiagF, case1);
-            vH = _mm_max_epi8(vHt, vF);
-            vH = _mm_max_epi8(vH, vZero);
-            case0 = _mm_cmpeq_epi8(vH, vZero);
-            case1 = _mm_cmpeq_epi8(vH, vHp);
-            case2 = _mm_cmpeq_epi8(vH, vF);
-            vT = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vTIns, vTDel, case2),
+            vF_opn = _mm_sub_epi32(vH, vGapO);
+            vF_ext = _mm_sub_epi32(vF, vGapE);
+            vF = _mm_max_epi32_rpl(vF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi32(vF_opn, vF_ext);
+            vFT = _mm_blendv_epi8_rpl(vTDelF, vTDiagF, case1);
+            vH = _mm_max_epi32_rpl(vHt, vF);
+            vH = _mm_max_epi32_rpl(vH, vZero);
+            case0 = _mm_cmpeq_epi32(vH, vZero);
+            case1 = _mm_cmpeq_epi32(vH, vHp);
+            case2 = _mm_cmpeq_epi32(vH, vF);
+            vT = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vTIns, vTDel, case2),
                     vTDiag, case1);
-            vT = _mm_blendv_epi8(vT, vTZero, case0);
+            vT = _mm_blendv_epi8_rpl(vT, vTZero, case0);
             vT = _mm_or_si128(vT, vET);
             vT = _mm_or_si128(vT, vFT);
             arr_store(result->trace->trace_table, vT, i, segLen, j);
             _mm_store_si128(pvH+i, vH);
-            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm_min_epi32_rpl(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm_max_epi32_rpl(vSaturationCheckMax, vH);
             {
-                vMaxH = _mm_max_epi8(vH, vMaxH);
+                vMaxH = _mm_max_epi32_rpl(vH, vMaxH);
             }
         } 
 
         {
-            __m128i vCompare = _mm_cmpgt_epi8(vMaxH, vMaxHUnit);
+            __m128i vCompare = _mm_cmpgt_epi32(vMaxH, vMaxHUnit);
             if (_mm_movemask_epi8(vCompare)) {
-                score = _mm_hmax_epi8_rpl(vMaxH);
-                vMaxHUnit = _mm_set1_epi8(score);
+                score = _mm_hmax_epi32_rpl(vMaxH);
+                vMaxHUnit = _mm_set1_epi32(score);
                 end_ref = j;
                 (void)memcpy(pvHMax, pvH, sizeof(__m128i)*segLen);
             }
@@ -315,7 +345,7 @@ parasail_result_t* PNAME(
 
     /* Trace the alignment ending position on read. */
     {
-        int8_t *t = (int8_t*)pvHMax;
+        int32_t *t = (int32_t*)pvHMax;
         int32_t column_len = segLen * segWidth;
         end_query = s1Len;
         for (i = 0; i<column_len; ++i, ++t) {
@@ -329,8 +359,8 @@ parasail_result_t* PNAME(
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
