@@ -19,38 +19,60 @@
 #define SWAP(A,B) { __m256i* tmp = A; A = B; B = tmp; }
 #define SWAP3(A,B,C) { __m256i* tmp = A; A = B; B = C; C = tmp; }
 
-#define NEG_INF INT8_MIN
+#define NEG_INF (INT64_MIN/(int64_t)(2))
 
-#if HAVE_AVX2_MM256_EXTRACT_EPI8
-#define _mm256_extract_epi8_rpl _mm256_extract_epi8
+#if HAVE_AVX2_MM256_EXTRACT_EPI64
+#define _mm256_extract_epi64_rpl _mm256_extract_epi64
 #else
-static inline int8_t _mm256_extract_epi8_rpl(__m256i a, int imm) {
-    __m256i_8_t A;
+static inline int64_t _mm256_extract_epi64_rpl(__m256i a, int imm) {
+    __m256i_64_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
-#if HAVE_AVX2_MM256_INSERT_EPI8
-#define _mm256_insert_epi8_rpl _mm256_insert_epi8
+#if HAVE_AVX2_MM256_INSERT_EPI64
+#define _mm256_insert_epi64_rpl _mm256_insert_epi64
 #else
-static inline __m256i _mm256_insert_epi8_rpl(__m256i a, int8_t i, int imm) {
-    __m256i_8_t A;
+static inline __m256i _mm256_insert_epi64_rpl(__m256i a, int64_t i, int imm) {
+    __m256i_64_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
+#if HAVE_AVX2_MM256_SET1_EPI64X
+#define _mm256_set1_epi64x_rpl _mm256_set1_epi64x
+#else
+static inline __m256i _mm256_set1_epi64x_rpl(int64_t i) {
+    __m256i_64_t A;
+    A.v[0] = i;
+    A.v[1] = i;
+    A.v[2] = i;
+    A.v[3] = i;
+    return A.m;
+}
+#endif
+
+static inline __m256i _mm256_max_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]>B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]>B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
-static inline int8_t _mm256_hmax_epi8_rpl(__m256i a) {
-    a = _mm256_max_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,0,0)));
-    a = _mm256_max_epi8(a, _mm256_slli_si256(a, 8));
-    a = _mm256_max_epi8(a, _mm256_slli_si256(a, 4));
-    a = _mm256_max_epi8(a, _mm256_slli_si256(a, 2));
-    a = _mm256_max_epi8(a, _mm256_slli_si256(a, 1));
-    return _mm256_extract_epi8_rpl(a, 31);
+static inline int64_t _mm256_hmax_epi64_rpl(__m256i a) {
+    a = _mm256_max_epi64_rpl(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,0,0)));
+    a = _mm256_max_epi64_rpl(a, _mm256_slli_si256(a, 8));
+    return _mm256_extract_epi64_rpl(a, 3);
 }
 
 
@@ -73,17 +95,36 @@ static inline __m256i arr_load(
     return _mm256_load_si256(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_sw_trace_striped_avx2_256_8
-#define PNAME parasail_sw_trace_striped_profile_avx2_256_8
+#define FNAME parasail_sw_trace_striped_avx2_256_64
+#define PNAME parasail_sw_trace_striped_profile_avx2_256_64
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
         const char * const restrict s2, const int s2Len,
         const int open, const int gap, const parasail_matrix_t *matrix)
 {
-    parasail_profile_t *profile = parasail_profile_create_avx_256_8(s1, s1Len, matrix);
-    parasail_result_t *result = PNAME(profile, s2, s2Len, open, gap);
+    /* declare local variables */
+    parasail_profile_t *profile = NULL;
+    parasail_result_t *result = NULL;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+    PARASAIL_CHECK_NULL(matrix);
+    if (matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) {
+        PARASAIL_CHECK_NULL(s1);
+        PARASAIL_CHECK_GT0(s1Len);
+    }
+
+    /* initialize local variables */
+    profile = parasail_profile_create_avx_256_64(s1, s1Len, matrix);
+    if (!profile) return NULL;
+    result = PNAME(profile, s2, s2Len, open, gap);
+
     parasail_profile_free(profile);
+
     return result;
 }
 
@@ -92,46 +133,113 @@ parasail_result_t* PNAME(
         const char * const restrict s2, const int s2Len,
         const int open, const int gap)
 {
+    /* declare local variables */
     int32_t i = 0;
     int32_t j = 0;
     int32_t k = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    const int s1Len = profile->s1Len;
-    const parasail_matrix_t *matrix = profile->matrix;
-    const int32_t segWidth = 32; /* number of values in vector unit */
-    const int32_t segLen = (s1Len + segWidth - 1) / segWidth;
-    __m256i* const restrict vProfile = (__m256i*)profile->profile8.score;
-    __m256i* restrict pvHStore = parasail_memalign___m256i(32, segLen);
-    __m256i* restrict pvHLoad =  parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvE = parasail_memalign___m256i(32, segLen);
-    __m256i* restrict pvEaStore = parasail_memalign___m256i(32, segLen);
-    __m256i* restrict pvEaLoad = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvHT = parasail_memalign___m256i(32, segLen);
-    __m256i* restrict pvHMax = parasail_memalign___m256i(32, segLen);
-    __m256i vGapO = _mm256_set1_epi8(open);
-    __m256i vGapE = _mm256_set1_epi8(gap);
-    __m256i vZero = _mm256_setzero_si256();
-    int8_t score = NEG_INF;
-    __m256i vMaxH = vZero;
-    __m256i vMaxHUnit = vZero;
-    int8_t maxp = INT8_MAX - (int8_t)(matrix->max+1);
-    parasail_result_t *result = parasail_result_new_trace(segLen, s2Len, 32, sizeof(__m256i));
-    __m256i vTZero = _mm256_set1_epi8(PARASAIL_ZERO);
-    __m256i vTIns  = _mm256_set1_epi8(PARASAIL_INS);
-    __m256i vTDel  = _mm256_set1_epi8(PARASAIL_DEL);
-    __m256i vTDiag = _mm256_set1_epi8(PARASAIL_DIAG);
-    __m256i vTDiagE = _mm256_set1_epi8(PARASAIL_DIAG_E);
-    __m256i vTInsE = _mm256_set1_epi8(PARASAIL_INS_E);
-    __m256i vTDiagF = _mm256_set1_epi8(PARASAIL_DIAG_F);
-    __m256i vTDelF = _mm256_set1_epi8(PARASAIL_DEL_F);
-    __m256i vTMask = _mm256_set1_epi8(PARASAIL_ZERO_MASK);
-    __m256i vFTMask = _mm256_set1_epi8(PARASAIL_F_MASK);
+    int32_t s1Len = 0;
+    const parasail_matrix_t *matrix = NULL;
+    int32_t segWidth = 0;
+    int32_t segLen = 0;
+    __m256i* restrict vProfile = NULL;
+    __m256i* restrict pvHStore = NULL;
+    __m256i* restrict pvHLoad = NULL;
+    __m256i* restrict pvE = NULL;
+    __m256i* restrict pvEaStore = NULL;
+    __m256i* restrict pvEaLoad = NULL;
+    __m256i* restrict pvHT = NULL;
+    __m256i* restrict pvHMax = NULL;
+    __m256i vGapO;
+    __m256i vGapE;
+    __m256i vZero;
+    int64_t score = 0;
+    __m256i vMaxH;
+    __m256i vMaxHUnit;
+    int64_t maxp = 0;
+    parasail_result_t *result = NULL;
+    __m256i vTZero;
+    __m256i vTIns;
+    __m256i vTDel;
+    __m256i vTDiag;
+    __m256i vTDiagE;
+    __m256i vTInsE;
+    __m256i vTDiagF;
+    __m256i vTDelF;
+    __m256i vTMask;
+    __m256i vFTMask;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(profile);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->matrix);
+    PARASAIL_CHECK_GT0(profile->s1Len);
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+
+    /* initialize stack variables */
+    i = 0;
+    j = 0;
+    k = 0;
+    end_query = 0;
+    end_ref = 0;
+    s1Len = profile->s1Len;
+    matrix = profile->matrix;
+    segWidth = 4; /* number of values in vector unit */
+    segLen = (s1Len + segWidth - 1) / segWidth;
+    vProfile = (__m256i*)profile->profile64.score;
+    vGapO = _mm256_set1_epi64x_rpl(open);
+    vGapE = _mm256_set1_epi64x_rpl(gap);
+    vZero = _mm256_setzero_si256();
+    score = NEG_INF;
+    vMaxH = vZero;
+    vMaxHUnit = vZero;
+    maxp = INT64_MAX - (int64_t)(matrix->max+1);
+    vTZero = _mm256_set1_epi64x_rpl(PARASAIL_ZERO);
+    vTIns  = _mm256_set1_epi64x_rpl(PARASAIL_INS);
+    vTDel  = _mm256_set1_epi64x_rpl(PARASAIL_DEL);
+    vTDiag = _mm256_set1_epi64x_rpl(PARASAIL_DIAG);
+    vTDiagE= _mm256_set1_epi64x_rpl(PARASAIL_DIAG_E);
+    vTInsE = _mm256_set1_epi64x_rpl(PARASAIL_INS_E);
+    vTDiagF= _mm256_set1_epi64x_rpl(PARASAIL_DIAG_F);
+    vTDelF = _mm256_set1_epi64x_rpl(PARASAIL_DEL_F);
+    vTMask = _mm256_set1_epi64x_rpl(PARASAIL_ZERO_MASK);
+    vFTMask= _mm256_set1_epi64x_rpl(PARASAIL_F_MASK);
+
+    /* initialize result */
+    result = parasail_result_new_trace(segLen, s2Len, 32, sizeof(__m256i));
+    if (!result) return NULL;
+
+    /* set known flags */
+    result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_STRIPED
+        | PARASAIL_FLAG_TRACE
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
+
+    /* initialize heap variables */
+    pvHStore = parasail_memalign___m256i(32, segLen);
+    pvHLoad =  parasail_memalign___m256i(32, segLen);
+    pvE = parasail_memalign___m256i(32, segLen);
+    pvEaStore = parasail_memalign___m256i(32, segLen);
+    pvEaLoad = parasail_memalign___m256i(32, segLen);
+    pvHT = parasail_memalign___m256i(32, segLen);
+    pvHMax = parasail_memalign___m256i(32, segLen);
+
+    /* validate heap variables */
+    if (!pvHStore) return NULL;
+    if (!pvHLoad) return NULL;
+    if (!pvE) return NULL;
+    if (!pvEaStore) return NULL;
+    if (!pvEaLoad) return NULL;
+    if (!pvHT) return NULL;
+    if (!pvHMax) return NULL;
 
     /* initialize H and E */
     parasail_memset___m256i(pvHStore, vZero, segLen);
-    parasail_memset___m256i(pvE, _mm256_set1_epi8(-open), segLen);
-    parasail_memset___m256i(pvEaStore, _mm256_set1_epi8(-open), segLen);
+    parasail_memset___m256i(pvE, _mm256_set1_epi64x_rpl(-open), segLen);
+    parasail_memset___m256i(pvEaStore, _mm256_set1_epi64x_rpl(-open), segLen);
 
     for (i=0; i<segLen; ++i) {
         arr_store(result->trace->trace_table, vTDiagE, i, segLen, 0);
@@ -152,11 +260,11 @@ parasail_result_t* PNAME(
 
         /* Initialize F value to 0.  Any errors to vH values will be
          * corrected in the Lazy_F loop. */
-        vF = _mm256_subs_epi8(vZero,vGapO);
+        vF = _mm256_sub_epi64(vZero,vGapO);
 
-        /* load final segment of pvHStore and shift left by 1 bytes */
+        /* load final segment of pvHStore and shift left by 8 bytes */
         vH = _mm256_load_si256(&pvHStore[segLen - 1]);
-        vH = _mm256_slli_si256_rpl(vH, 1);
+        vH = _mm256_slli_si256_rpl(vH, 8);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -177,18 +285,18 @@ parasail_result_t* PNAME(
             vE = _mm256_load_si256(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm256_adds_epi8(vH, _mm256_load_si256(vP + i));
-            vH_dag = _mm256_max_epi8(vH_dag, vZero);
-            vH = _mm256_max_epi8(vH_dag, vE);
-            vH = _mm256_max_epi8(vH, vF);
+            vH_dag = _mm256_add_epi64(vH, _mm256_load_si256(vP + i));
+            vH_dag = _mm256_max_epi64_rpl(vH_dag, vZero);
+            vH = _mm256_max_epi64_rpl(vH_dag, vE);
+            vH = _mm256_max_epi64_rpl(vH, vF);
             /* Save vH values. */
             _mm256_store_si256(pvHStore + i, vH);
 
             {
                 __m256i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                __m256i cond_zero = _mm256_cmpeq_epi8(vH, vZero);
-                __m256i case1 = _mm256_cmpeq_epi8(vH, vH_dag);
-                __m256i case2 = _mm256_cmpeq_epi8(vH, vF);
+                __m256i cond_zero = _mm256_cmpeq_epi64(vH, vZero);
+                __m256i case1 = _mm256_cmpeq_epi64(vH, vH_dag);
+                __m256i case2 = _mm256_cmpeq_epi64(vH, vF);
                 __m256i vT = _mm256_blendv_epi8(
                         _mm256_blendv_epi8(vTIns, vTDel, case2),
                         _mm256_blendv_epi8(vTDiag, vTZero, cond_zero),
@@ -197,31 +305,31 @@ parasail_result_t* PNAME(
                 vT = _mm256_or_si256(vT, vTAll);
                 arr_store(result->trace->trace_table, vT, i, segLen, j);
             }
-            vMaxH = _mm256_max_epi8(vH, vMaxH);
-            vEF_opn = _mm256_subs_epi8(vH, vGapO);
+            vMaxH = _mm256_max_epi64_rpl(vH, vMaxH);
+            vEF_opn = _mm256_sub_epi64(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm256_subs_epi8(vE, vGapE);
-            vE = _mm256_max_epi8(vEF_opn, vE_ext);
+            vE_ext = _mm256_sub_epi64(vE, vGapE);
+            vE = _mm256_max_epi64_rpl(vEF_opn, vE_ext);
             _mm256_store_si256(pvE + i, vE);
             {
                 __m256i vEa = _mm256_load_si256(pvEaLoad + i);
-                __m256i vEa_ext = _mm256_subs_epi8(vEa, vGapE);
-                vEa = _mm256_max_epi8(vEF_opn, vEa_ext);
+                __m256i vEa_ext = _mm256_sub_epi64(vEa, vGapE);
+                vEa = _mm256_max_epi64_rpl(vEF_opn, vEa_ext);
                 _mm256_store_si256(pvEaStore + i, vEa);
                 if (j+1<s2Len) {
-                    __m256i cond = _mm256_cmpgt_epi8(vEF_opn, vEa_ext);
+                    __m256i cond = _mm256_cmpgt_epi64(vEF_opn, vEa_ext);
                     __m256i vT = _mm256_blendv_epi8(vTInsE, vTDiagE, cond);
                     arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                 }
             }
 
             /* Update vF value. */
-            vF_ext = _mm256_subs_epi8(vF, vGapE);
-            vF = _mm256_max_epi8(vEF_opn, vF_ext);
+            vF_ext = _mm256_sub_epi64(vF, vGapE);
+            vF = _mm256_max_epi64_rpl(vEF_opn, vF_ext);
             if (i+1<segLen) {
                 __m256i vTAll = arr_load(result->trace->trace_table, i+1, segLen, j);
-                __m256i cond = _mm256_cmpgt_epi8(vEF_opn, vF_ext);
+                __m256i cond = _mm256_cmpgt_epi64(vEF_opn, vF_ext);
                 __m256i vT = _mm256_blendv_epi8(vTDelF, vTDiagF, cond);
                 vT = _mm256_or_si256(vT, vTAll);
                 arr_store(result->trace->trace_table, vT, i+1, segLen, j);
@@ -237,20 +345,20 @@ parasail_result_t* PNAME(
         vFa = vF;
         for (k=0; k<segWidth; ++k) {
             __m256i vHp = _mm256_load_si256(&pvHLoad[segLen - 1]);
-            vHp = _mm256_slli_si256_rpl(vHp, 1);
-            vEF_opn = _mm256_slli_si256_rpl(vEF_opn, 1);
-            vEF_opn = _mm256_insert_epi8_rpl(vEF_opn, -open, 0);
-            vF_ext = _mm256_slli_si256_rpl(vF_ext, 1);
-            vF_ext = _mm256_insert_epi8_rpl(vF_ext, NEG_INF, 0);
-            vF = _mm256_slli_si256_rpl(vF, 1);
-            vF = _mm256_insert_epi8_rpl(vF, -open, 0);
-            vFa_ext = _mm256_slli_si256_rpl(vFa_ext, 1);
-            vFa_ext = _mm256_insert_epi8_rpl(vFa_ext, NEG_INF, 0);
-            vFa = _mm256_slli_si256_rpl(vFa, 1);
-            vFa = _mm256_insert_epi8_rpl(vFa, -open, 0);
+            vHp = _mm256_slli_si256_rpl(vHp, 8);
+            vEF_opn = _mm256_slli_si256_rpl(vEF_opn, 8);
+            vEF_opn = _mm256_insert_epi64_rpl(vEF_opn, -open, 0);
+            vF_ext = _mm256_slli_si256_rpl(vF_ext, 8);
+            vF_ext = _mm256_insert_epi64_rpl(vF_ext, NEG_INF, 0);
+            vF = _mm256_slli_si256_rpl(vF, 8);
+            vF = _mm256_insert_epi64_rpl(vF, -open, 0);
+            vFa_ext = _mm256_slli_si256_rpl(vFa_ext, 8);
+            vFa_ext = _mm256_insert_epi64_rpl(vFa_ext, NEG_INF, 0);
+            vFa = _mm256_slli_si256_rpl(vFa, 8);
+            vFa = _mm256_insert_epi64_rpl(vFa, -open, 0);
             for (i=0; i<segLen; ++i) {
                 vH = _mm256_load_si256(pvHStore + i);
-                vH = _mm256_max_epi8(vH,vF);
+                vH = _mm256_max_epi64_rpl(vH,vF);
                 _mm256_store_si256(pvHStore + i, vH);
                 {
                     __m256i vTAll;
@@ -258,10 +366,10 @@ parasail_result_t* PNAME(
                     __m256i case1;
                     __m256i case2;
                     __m256i cond;
-                    vHp = _mm256_adds_epi8(vHp, _mm256_load_si256(vP + i));
-                    vHp = _mm256_max_epi8(vHp, vZero);
-                    case1 = _mm256_cmpeq_epi8(vH, vHp);
-                    case2 = _mm256_cmpeq_epi8(vH, vF);
+                    vHp = _mm256_add_epi64(vHp, _mm256_load_si256(vP + i));
+                    vHp = _mm256_max_epi64_rpl(vHp, vZero);
+                    case1 = _mm256_cmpeq_epi64(vH, vHp);
+                    case2 = _mm256_cmpeq_epi64(vH, vF);
                     cond = _mm256_andnot_si256(case1,case2);
                     vTAll = arr_load(result->trace->trace_table, i, segLen, j);
                     vT = _mm256_load_si256(pvHT + i);
@@ -271,38 +379,38 @@ parasail_result_t* PNAME(
                     vTAll = _mm256_or_si256(vTAll, vT);
                     arr_store(result->trace->trace_table, vTAll, i, segLen, j);
                 }
-                vMaxH = _mm256_max_epi8(vH, vMaxH);
+                vMaxH = _mm256_max_epi64_rpl(vH, vMaxH);
                 /* Update vF value. */
                 {
                     __m256i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                    __m256i cond = _mm256_cmpgt_epi8(vEF_opn, vFa_ext);
+                    __m256i cond = _mm256_cmpgt_epi64(vEF_opn, vFa_ext);
                     __m256i vT = _mm256_blendv_epi8(vTDelF, vTDiagF, cond);
                     vTAll = _mm256_and_si256(vTAll, vFTMask);
                     vTAll = _mm256_or_si256(vTAll, vT);
                     arr_store(result->trace->trace_table, vTAll, i, segLen, j);
                 }
-                vEF_opn = _mm256_subs_epi8(vH, vGapO);
-                vF_ext = _mm256_subs_epi8(vF, vGapE);
+                vEF_opn = _mm256_sub_epi64(vH, vGapO);
+                vF_ext = _mm256_sub_epi64(vF, vGapE);
                 {
                     __m256i vEa = _mm256_load_si256(pvEaLoad + i);
-                    __m256i vEa_ext = _mm256_subs_epi8(vEa, vGapE);
-                    vEa = _mm256_max_epi8(vEF_opn, vEa_ext);
+                    __m256i vEa_ext = _mm256_sub_epi64(vEa, vGapE);
+                    vEa = _mm256_max_epi64_rpl(vEF_opn, vEa_ext);
                     _mm256_store_si256(pvEaStore + i, vEa);
                     if (j+1<s2Len) {
-                        __m256i cond = _mm256_cmpgt_epi8(vEF_opn, vEa_ext);
+                        __m256i cond = _mm256_cmpgt_epi64(vEF_opn, vEa_ext);
                         __m256i vT = _mm256_blendv_epi8(vTInsE, vTDiagE, cond);
                         arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                     }
                 }
                 if (! _mm256_movemask_epi8(
                             _mm256_or_si256(
-                                _mm256_cmpgt_epi8(vF_ext, vEF_opn),
-                                _mm256_cmpeq_epi8(vF_ext, vEF_opn))))
+                                _mm256_cmpgt_epi64(vF_ext, vEF_opn),
+                                _mm256_cmpeq_epi64(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = _mm256_max_epi8(vEF_opn, vF_ext);*/
+                /*vF = _mm256_max_epi64_rpl(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                vFa_ext = _mm256_subs_epi8(vFa, vGapE);
-                vFa = _mm256_max_epi8(vEF_opn, vFa_ext);
+                vFa_ext = _mm256_sub_epi64(vFa, vGapE);
+                vFa = _mm256_max_epi64_rpl(vEF_opn, vFa_ext);
                 vHp = _mm256_load_si256(pvHLoad + i);
             }
         }
@@ -311,15 +419,15 @@ end:
         }
 
         {
-            __m256i vCompare = _mm256_cmpgt_epi8(vMaxH, vMaxHUnit);
+            __m256i vCompare = _mm256_cmpgt_epi64(vMaxH, vMaxHUnit);
             if (_mm256_movemask_epi8(vCompare)) {
-                score = _mm256_hmax_epi8_rpl(vMaxH);
+                score = _mm256_hmax_epi64_rpl(vMaxH);
                 /* if score has potential to overflow, abort early */
                 if (score > maxp) {
                     result->flag |= PARASAIL_FLAG_SATURATED;
                     break;
                 }
-                vMaxHUnit = _mm256_set1_epi8(score);
+                vMaxHUnit = _mm256_set1_epi64x_rpl(score);
                 end_ref = j;
             }
         }
@@ -327,7 +435,7 @@ end:
         /*if (score == stop) break;*/
     }
 
-    if (score == INT8_MAX) {
+    if (score == INT64_MAX) {
         result->flag |= PARASAIL_FLAG_SATURATED;
     }
 
@@ -347,7 +455,7 @@ end:
         }
         /* Trace the alignment ending position on read. */
         {
-            int8_t *t = (int8_t*)pvHMax;
+            int64_t *t = (int64_t*)pvHMax;
             int32_t column_len = segLen * segWidth;
             end_query = s1Len - 1;
             for (i = 0; i<column_len; ++i, ++t) {
@@ -364,9 +472,6 @@ end:
     result->score = score;
     result->end_query = end_query;
     result->end_ref = end_ref;
-    result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_STRIPED
-        | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_32;
 
     parasail_free(pvHMax);
     parasail_free(pvHT);
